/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "EyeTrack.h"
#include "AudioCaptureDlg.h"

#include <sstream>
#include ".\audiocapturedlg.h"

// CAudioCaptureDlg dialog

IMPLEMENT_DYNAMIC(CAudioCaptureDlg, CDialog)

CAudioCaptureDlg::CAudioCaptureDlg(CWnd* pParent /*=NULL*/)
	: CDialog(CAudioCaptureDlg::IDD, pParent) 
{}

CAudioCaptureDlg::~CAudioCaptureDlg()
{}

BEGIN_MESSAGE_MAP(CAudioCaptureDlg, CDialog)
	ON_BN_CLICKED(IDC_BUTTON_BROWSE_AUDIO_PATH, OnBnClickedBrowseAudioPath)
	ON_CBN_SELCHANGE(IDC_COMBO_AUDIO_CHANNELS, OnChangeItem)
	ON_CBN_SELCHANGE(IDC_COMBO_AUDIO_SAMPLE_RATE, OnChangeItem)
	ON_CBN_SELCHANGE(IDC_COMBO_AUDIO_SAMPLE_SIZE, OnChangeItem)
	ON_EN_CHANGE(IDC_EDIT_AUDIO_LENGTH, OnChangeItem)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_EN_CHANGE(IDC_EDIT_AUDIO_PATH, OnEnChangeEditAudioPath)
END_MESSAGE_MAP()

void CAudioCaptureDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

const AudioCaptureData &CAudioCaptureDlg::GetCaptureData() const
{
	return(_audio);
}

void CAudioCaptureDlg::OnBnClickedBrowseAudioPath()
{
	const int max = 200;
	char buf[max];

	GetDlgItem(IDC_EDIT_AUDIO_PATH)->GetWindowText(buf, max);

	CFileDialog dlg(TRUE, 0, buf, OFN_HIDEREADONLY, 0, this, 0);

	if(dlg.DoModal() == IDOK) {
		_audio.path = dlg.GetPathName().GetBuffer();
		GetDlgItem(IDC_EDIT_AUDIO_PATH)->SetWindowText(_audio.path.c_str());
	}
}

void CAudioCaptureDlg::OnChangeItem()
{
	const int max = 20;
	std::ostringstream os;
	CComboBox *combo;
	float audioBufSz;
	char buf[max];

	GetDlgItem(IDC_EDIT_AUDIO_LENGTH)->GetWindowText(buf, max);
	audioBufSz = static_cast<float>(atof(buf));  //length of buffer in seconds

	combo = (CComboBox*)(GetDlgItem(IDC_COMBO_AUDIO_CHANNELS));
	if(combo->GetCurSel() == 1)                  //multiply by number of channels
		audioBufSz *= 2;

	combo = (CComboBox*)(GetDlgItem(IDC_COMBO_AUDIO_SAMPLE_RATE));
	switch(combo->GetCurSel()) {                 //multiply by sampling rate
	case 0: audioBufSz *= 8000.0;   break;
	case 1: audioBufSz *= 11025.0;  break;
	case 2: audioBufSz *= 16000.0;  break;
	case 3: audioBufSz *= 22050.0;  break;
	case 4: audioBufSz *= 32000.0;  break;
	case 5: audioBufSz *= 44100.0;  break;
	} 

	combo = (CComboBox*)(GetDlgItem(IDC_COMBO_AUDIO_SAMPLE_SIZE));
	if(combo->GetCurSel() == 1)                  //multiply by sample size (bytes)
		audioBufSz *= 2;

	audioBufSz /= 1048576.0;                     //convert to mega bytes
	audioBufSz = int(audioBufSz*100) / 100.0f;   //round to two decimal places
	os<<audioBufSz;                              //convert to text

	GetDlgItem(IDC_EDIT_AUDIO_BUFFER_SIZE)->SetWindowText(os.str().c_str());
}

BOOL CAudioCaptureDlg::OnInitDialog()
{
	CComboBox *combo;
	char buf[10];

	CDialog::OnInitDialog();

	combo = (CComboBox*)(GetDlgItem(IDC_COMBO_AUDIO_CHANNELS));
	if(_audio.nChannels == 2) combo->SetCurSel(1);
	else combo->SetCurSel(0);

	combo = (CComboBox*)(GetDlgItem(IDC_COMBO_AUDIO_SAMPLE_RATE));
	switch(_audio.sampleRate) {
	case 11025: combo->SetCurSel(1);   break;
	case 16000: combo->SetCurSel(2);   break;
	case 22050: combo->SetCurSel(3);   break;
	case 32000: combo->SetCurSel(4);   break;
	case 44100: combo->SetCurSel(5);   break;
	default:    combo->SetCurSel(0);
	}

	combo = (CComboBox*)(GetDlgItem(IDC_COMBO_AUDIO_SAMPLE_SIZE));
	if(_audio.sampleSize == 16) combo->SetCurSel(1);
	else combo->SetCurSel(0);

	GetDlgItem(IDC_EDIT_AUDIO_LENGTH)->SetWindowText(_itoa(_audio.length, buf, 10)); 
	GetDlgItem(IDC_EDIT_AUDIO_PATH)->SetWindowText(_audio.path.c_str());

	OnChangeItem();

	return TRUE;
}

void CAudioCaptureDlg::OnOK()
{
	const int max = 200;
	CComboBox *combo;
	char buf[max];

	if(GetDlgItem(IDC_EDIT_AUDIO_LENGTH)->GetWindowTextLength() < 1) {
		MessageBox("Maximum length (time) of audio capture required.", "Dialog Incomplete");
		return;
	}

	if(GetDlgItem(IDC_EDIT_AUDIO_PATH)->GetWindowTextLength() < 1) {
		MessageBox("Storage path for captured audio files required.", "Dialog Incomplete");
		return;
	}

	GetDlgItem(IDC_EDIT_AUDIO_PATH)->GetWindowText(buf, max);
	_audio.path = buf;

	GetDlgItem(IDC_EDIT_AUDIO_LENGTH)->GetWindowText(buf, max);
	_audio.length = atoi(buf);

	combo = (CComboBox*)(GetDlgItem(IDC_COMBO_AUDIO_CHANNELS));
	_audio.nChannels = combo->GetCurSel() + 1;

	combo = (CComboBox*)(GetDlgItem(IDC_COMBO_AUDIO_SAMPLE_RATE));
	switch(combo->GetCurSel()) {
	case 0: _audio.sampleRate = 8000;   break;
	case 1: _audio.sampleRate = 11025;  break;
	case 2: _audio.sampleRate = 16000;  break;
	case 3: _audio.sampleRate = 22050;  break;
	case 4: _audio.sampleRate = 32000;  break;
	case 5: _audio.sampleRate = 44100;  break;
	} 

	combo = (CComboBox*)(GetDlgItem(IDC_COMBO_AUDIO_SAMPLE_SIZE));
	if(combo->GetCurSel() == 0) _audio.sampleSize = 8;
	else _audio.sampleSize = 16;

	CDialog::OnOK();
}

void CAudioCaptureDlg::SetCaptureData(const AudioCaptureData &format)
{
	_audio.path = format.path;
	_audio.length = format.length;
	_audio.nChannels = format.nChannels;
	_audio.sampleRate = format.sampleRate;
	_audio.sampleSize = format.sampleSize;
}

void CAudioCaptureDlg::OnBnClickedOk()
{
	// TODO: Add your control notification handler code here
	OnOK();
}

void CAudioCaptureDlg::OnEnChangeEditAudioPath()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}
