/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

#include <string>
#include "stdafx.h"
#include "Mmreg.h"
#include "dsound.h"

/****************************************************************************/

struct AudioRenderData {
	AudioRenderData();

	const AudioRenderData& operator=(const AudioRenderData &rhs);

	std::string  path;
	unsigned int startDelay;
	unsigned int stopDelay;
};

/****************************************************************************/

class CAudioRender {
public:
	enum PollResult {
		RENDER_START,
		RENDER_STOP,
		RENDER_UNCHANGED
	};

public:
	CAudioRender();
	~CAudioRender();

	PollResult Poll(unsigned int time);

	unsigned int AudioOffTime() const;
	unsigned int AudioOnTime() const;
	unsigned int AudioStartDelay() const;
	unsigned int AudioStopDelay() const;
	unsigned int StopTime() const;

	bool Initialize(const AudioRenderData &data);

	void Start(unsigned int time);
	void Stop();

private:
	DSBUFFERDESC        _dsbd;                 //Direct Sound rendering
	LPDIRECTSOUND8      _pDS;
	LPDIRECTSOUNDBUFFER _pDSB;
	WAVEFORMATEX        _waveRenderFmt;

	AudioRenderData     _data;                 //Audio time data
	unsigned int        _audioOnTime;
	unsigned int        _audioOffTime;
	unsigned int        _length;
	unsigned int        _stopTime;
	bool                _isPlaying;
};
