/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	**This class implements an interface into DirectInput8, allowing the program
	to read directly from the hardware keyboard and mouse buffers, instead of
	waiting	for the (much slower) windows messaging system. The implementation
	used here is a simple polling approach.  The functions return the current
	state of the mouse and keyboard. No queue of events is kept.
*/

#pragma once

#define DIRECTINPUT_VERSION 0x0800
#include <DInput.h>
#include <list>

class CDirectInput {
public:
	CDirectInput();
	~CDirectInput();

	const CPoint &GetCursorLocation();           //return current location of mouse cursor

	int KeyDown();                               //return the code for pressed key (if any)

	bool Initialize();                           //prepare system for direct input
	bool IsLButtonDown(CPoint &pt);              //true if left mouse button pressed (pt assigned cursor location)
	bool IsRButtonDown(CPoint &pt);              //true if right mouse button pressed (pt assigned cursor location)
	bool IsReady() const;                        //true if direct input is conected and working

	void Terminate();                            //shut down direct input system

private:
	static const int KEY_BUF_SIZE   = 256;
	static const int MOUSE_BUF_SIZE = 100;

	LPDIRECTINPUT8       _pDI;
	LPDIRECTINPUTDEVICE8 _pKeyboard;
	LPDIRECTINPUTDEVICE8 _pMouse;
	CPoint               _cursorLoc;
	bool                 _ready;
};
