/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "DisplayData.h"

MultiRect::MultiRect() : r1(0, 0, 0, 0), r2(0, 0, 0, 0), r3(0, 0, 0, 0)
{}

/*const rectVector &rectVector::operator=(const rectVector &r)
{
	for (unsigned int i=0; i<r.size(); i++){
		this.push_back(r[i]);
	}
	return(*this);
}*/

const ToggleData &ToggleData::operator=(const ToggleData &td){
	delay = td.delay;
	duration = td.duration;
	region = td.region;
	delay2 = td.delay2;
	duration2 = td.duration2;
	wordIndex = td.wordIndex;
	hasRegion = td.hasRegion;
	return(*this);
}

ToggleData::ToggleData(CRect r1)
: region(r1), delay(0), duration(0), wordIndex(0), hasRegion(true)
{
	
}

ToggleData::ToggleData(CRect r1, unsigned int delay, unsigned int duration)
: region(r1), delay(delay), duration(duration), wordIndex(0), hasRegion(true)
{}

ToggleData::ToggleData(CRect r1, unsigned int del, unsigned int dur, unsigned int del2, unsigned int dur2)
:region(r1), delay(del), duration(dur), delay2(del2), duration2(dur2), wordIndex(0), hasRegion(true)
{}



ToggleData::ToggleData()
: region(0,0,0,0)
{
}

const TimedRectPair &TimedRectPair::operator=(const TimedRectPair &trp){
	r1 = trp.r1;
	r2data = trp.r2data;
	return (*this);
}

TimedRectPair::TimedRectPair(CRect r1, TOGGLEDATA td)
: r1(r1), r2data(td), written(false)
{}

TimedRectPair::TimedRectPair()
:r1(0,0,0,0), r2data(), written(false)
{}

const MultiRect &MultiRect::operator=(const MultiRect &mr)
{	r1 = mr.r1;
	r2 = mr.r2;
	r3 = mr.r3;
	return(*this);
}

const RectPair &RectPair::operator=(const RectPair &rp)
{
	region1 = rp.region1;
	region2 = rp.region2;
	return (*this);
}

RectPair::RectPair(CRect r1, CRect r2)
: region1(r1), region2(r2)
{}

RectPair::RectPair()
: region1(0,0,0,0), region2(0,0,0,0)
{}

RectPair::~RectPair()
{}


const windowType &windowType::operator=(const windowType &wt)
{
	useWordTriggers = wt.useWordTriggers;
	useWordTargets = wt.useWordTargets;
	return (*this);
}

windowType::windowType()
: useWordTriggers(FALSE), useWordTargets(FALSE)
{}

windowType::~windowType()
{}

const imagePos &imagePos::operator=(const imagePos &ip)
{
	fileName = ip.fileName;
	upperLeft = ip.upperLeft;
	return(*this);
}

imagePos::imagePos()
: fileName(""), upperLeft(-1,-1)
{}

imagePos::imagePos(std::string fName, CPoint pt)
{
	fileName = fName;
	upperLeft = pt;
}

imagePos::~imagePos()
{}

DisplayData::DisplayData()
  : gcRect(0, 0, 0, 0), isFixedDisplay(true), isTextDisplay(true), isMovingWindow(false), isPairedRegionChange(false), isTimedPairedRegionChange(false), images1(NULL), font2start(0), font2end(0)
  /*region(0), target(0), wordTriggers(0), wordTargets(0), wordTargetRange(0), multiTarget(0),
  charTriggers(0), charTargets(0)*/
{}


DisplayData::DisplayData(const DisplayData &dd)
{
	display1       = dd.display1;
	display2       = dd.display2;
	display3		= dd.display3;
	font2start		 = dd.font2start;
	font2end		= dd.font2end;
	region         = dd.region;
	target		   = dd.target;
	regionPairs		= dd.regionPairs;
	timedRectPairs	= dd.timedRectPairs;
	wordTriggers	= dd.wordTriggers;
	wordTargets		= dd.wordTargets;
	wordTargetRange = dd.wordTargetRange;
	multiTarget		= dd.multiTarget;
	charTriggers	= dd.charTriggers;
	charTargets		= dd.charTargets;
	gcRect         = dd.gcRect;
	isFixedDisplay = dd.isFixedDisplay;
	isScotoma		= dd.isScotoma;
	isMovingWindow = dd.isMovingWindow;
	isTextDisplay  = dd.isTextDisplay;
	windowSize		= dd.windowSize;
	windowRange		= dd.windowRange;
	movWindowType	= dd.movWindowType;
	isPairedRegionChange = dd.isPairedRegionChange;
	isTimedPairedRegionChange = dd.isTimedPairedRegionChange;
	toggleWord		= dd.toggleWord;
	toggleWords		= dd.toggleWords;
	toggleRegions	= dd.toggleRegions;
	images1			= dd.images1;
	images2			= dd.images2;
	images3			= dd.images3;
}

DisplayData::~DisplayData()
{}

/*{
	while(!region.empty()) {                 //delete all sequences
		CRect r = region.back();
		region.pop_back();
		delete r;
	}
	
	while(!target.empty()) {
		CRect r = target.back();
		target.pop_back();
		delete r;
	}
	while(!multiTarget.empty()){
		rectVector rv = multiTarget.back();
		multiTarget.pop_back();
		while (!rv.empty()){
			CRect r = rv.back();
			rv.pop_back();
			delete r;
		}
		delete &rv;
	}

	while(!wordTargetRange.empty()){
		CPoint p = wordTargetRange.back();
		wordTargetRange.pop_back();
		delete &p;
	}

}*/

const DisplayData &DisplayData::operator=(const DisplayData &rhs)
{
	display1       = rhs.display1;
	display2       = rhs.display2;
	display3		= rhs.display3;
	font2start		= rhs.font2start;
	font2end		= rhs.font2end;
	region         = rhs.region;
	target		   = rhs.target;
	regionPairs		= rhs.regionPairs;
	timedRectPairs = rhs.timedRectPairs;
	wordTriggers   = rhs.wordTriggers;
	wordTargets    = rhs.wordTargets;
	wordTargetRange = rhs.wordTargetRange;
	multiTarget		= rhs.multiTarget;
	charTriggers   = rhs.charTriggers;
	charTargets	   = rhs.charTargets;
	gcRect         = rhs.gcRect;
	isFixedDisplay = rhs.isFixedDisplay;
	isScotoma		= rhs.isScotoma;
	isMovingWindow = rhs.isMovingWindow;
	isTextDisplay  = rhs.isTextDisplay;
	windowSize		= rhs.windowSize;
	windowRange		= rhs.windowRange;
	movWindowType	= rhs.movWindowType;
	isPairedRegionChange = rhs.isPairedRegionChange;
	isTimedPairedRegionChange = rhs.isTimedPairedRegionChange;
	toggleWord		= rhs.toggleWord;
	toggleWords		= rhs.toggleWords;
	toggleRegions		= rhs.toggleRegions;
	images1			= rhs.images1;
	images2			= rhs.images2;
	images3			= rhs.images3;
	return(*this);
}

/*****************************************************************************/

TextFormat::TextFormat()
  : fontHt(0), lineSpace(0), xOffset(0), yOffset(0), rendering(0), spacingDiff(100)
{}

const TextFormat &TextFormat::operator=(const TextFormat &fmt)
{
	fontName  = fmt.fontName;
	fontHt    = fmt.fontHt;
	fontWt	  = fmt.fontWt;
	italic		= fmt.italic;
	lineSpace = fmt.lineSpace;
	xOffset   = fmt.xOffset;
	yOffset   = fmt.yOffset;
	rendering = fmt.rendering;
	spacingDiff = fmt.spacingDiff;
	return(*this);
}
