/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

#include <string>
#include <vector>

typedef struct ToggleData{
	ToggleData();
	ToggleData(CRect r1);
	ToggleData(CRect r1, unsigned int d1, unsigned int d2);
	ToggleData(CRect r1, unsigned int d1, unsigned int d2, unsigned int d3, unsigned int d4);
	int wordIndex;
	bool hasRegion;
	unsigned int delay;
	unsigned int duration;
	unsigned int delay2;
	unsigned int duration2;
	CRect region;

	const ToggleData &operator=(const ToggleData &rp);
} TOGGLEDATA;

typedef std::vector<CRect> rectVector;

/*struct tPair {
	tPair();
	CRect trigger;

	const tPair &operator=(const tPair &p);
};*/
typedef struct RectPair{
	RectPair(CRect r1, CRect r2);
	RectPair();
	~RectPair();

	CRect region1;
	CRect region2;

	const RectPair &operator=(const RectPair &rp);
} RECTPAIR;

typedef struct TimedRectPair{
	TimedRectPair(CRect r1, ToggleData td);
	TimedRectPair();

	CRect r1;
	TOGGLEDATA r2data;
	bool written;

	const TimedRectPair &operator=(const TimedRectPair &trp);
} TIMED_RECT_PAIR;

typedef struct FlexRectPair{
	FlexRectPair(CRect r1, CRect r2);
	FlexRectPair();
	~FlexRectPair();

	CRect r1;
	CRect r2;

	unsigned int t1; // delay after fixation in r1, that r2 will be changed from stim1 to stim2
	unsigned int t2; // amount of time that stim2 will be displayed in r2 until r2 reverts to stim1
					// t2 is timed from trigger fixation NOT FROM BOUNDARY CROSSING TIME!

	const FlexRectPair &operator=(const FlexRectPair &rp);
}FLEXRECTPAIR;

typedef struct imagePos{
	imagePos(std::string fName, CPoint pt);
	imagePos();
	~imagePos();

	std::string fileName;
	CPoint upperLeft;
	const imagePos &operator=(const imagePos &ip);
}IMAGEPOS;

struct MultiRect {
	MultiRect();
	CRect r1;
	CRect r2;
	CRect r3;

	const MultiRect &operator=(const MultiRect &mr);
};

typedef struct windowType {
	windowType();
	~windowType();
	int useWordTriggers;
	int useWordTargets;

	const windowType &operator=(const windowType &wt);
}_WINDOW_TYPE;

struct DisplayData {                        //visual display information
	DisplayData();
	DisplayData(const DisplayData &dd);
	~DisplayData();

	const DisplayData &operator=(const DisplayData &rhs);

	std::string        display1;
	std::string        display2;
	std::string			display3;
	int					font2start;
	int					font2end;
	std::vector<CRect> region;
	std::vector<CRect> target;
	std::vector<RECTPAIR>   regionPairs;
	std::vector<TIMED_RECT_PAIR> timedRectPairs;
	std::vector<rectVector> multiTarget;
	std::vector<int>   wordTriggers;
	std::vector<int>   wordTargets;
	std::vector<CPoint> wordTargetRange;
	std::vector<int>   charTriggers;			// MW
	std::vector<int>   charTargets;		// MW
	CPoint				windowSize;
	CPoint				windowRange;
	CRect              gcRect;
	bool               isFixedDisplay;
	bool			   isScotoma;
	bool			   isMovingWindow;
	bool               isTextDisplay;
	bool				isPairedRegionChange;
	bool				isTimedPairedRegionChange;
	_WINDOW_TYPE		movWindowType;
	CPoint				toggleWord;
	std::vector<TOGGLEDATA>	toggleWords;
	std::vector<TOGGLEDATA>	toggleRegions;
	std::vector<IMAGEPOS> images1;
	std::vector<IMAGEPOS> images2;
	std::vector<IMAGEPOS> images3;
};

/*****************************************************************************/

struct TextFormat {                         //inline text formatting information
	TextFormat();
	const TextFormat &operator=(const TextFormat &fmt);

	std::string fontName;
	int         fontHt;
	int			fontWt;
	BYTE		italic;
	int         lineSpace;
	int         xOffset;
	int         yOffset;
	int			spacingDiff;
	BYTE        rendering;
};
