/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include <sstream>
#include "EyeTrack.h"
#include "EditExperimentDlg.h"
#include ".\editexperimentdlg.h"



// CEditExperimentDlg dialog

IMPLEMENT_DYNAMIC(CEditExperimentDlg, CDialog)
CEditExperimentDlg::CEditExperimentDlg(CWnd* pParent)
  : CDialog(CEditExperimentDlg::IDD, pParent),
    _backColor(0xFFFFFF), _foreColor(0x0), _nConds(0), _nExpts(0),
	_filterMode(Tokenizer::NONE_SELECTED), _calibrationType(Tokenizer::HV9)
{
	for (int i=0; i<20; i++)
		_expC[i] = 0;
}

CEditExperimentDlg::~CEditExperimentDlg()
{
}

void CEditExperimentDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_EDIT_EXP_COND, expCond);
}


BEGIN_MESSAGE_MAP(CEditExperimentDlg, CDialog)
	ON_BN_CLICKED(IDC_BUTTON_CHANGE_BACKGROUND_COLOR, OnBnClickedChangeBackColor)
	ON_BN_CLICKED(IDC_BUTTON_CHANGE_TEXT_FOREGROUND, OnBnClickedChangeTextForeColor)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_WM_PAINT()
	ON_EN_CHANGE(IDC_EDIT_EXPERIMENT_HEADER, OnEnChangeEditExperimentHeader)
	ON_BN_CLICKED(IDC_LINK_FILTER_OFF, OnChangeFilterMode)
	ON_BN_CLICKED(IDC_LINK_FILTER_ON, OnChangeFilterMode)
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
	ON_BN_CLICKED(IDC_CALIBRATION_H3, OnChangeCalibrationType)
	ON_BN_CLICKED(IDC_CALIBRATION_HV9, OnChangeCalibrationType)
	ON_BN_CLICKED(IDC_CALIBRATION_HV13, OnChangeCalibrationType)
	ON_EN_CHANGE(IDC_EDIT_EXPERIMENT_CONDITIONS, OnEnChangeEditExperimentConditions)
	ON_EN_CHANGE(IDC_EDIT_EXPERIMENT_EXPTS, OnEnChangeEditExperimentExpts)
	ON_EN_CHANGE(IDC_EDIT_EXPERIMENT_GAZE_DELTA_THRESHOLD, OnEnChangeEditExperimentGazeDeltaThreshold)
	ON_BN_CLICKED(IDC_LCD_DISPLAY_TYPE,  OnChangeDisplayType)
	ON_BN_CLICKED(IDC_CRT_DISPLAY_TYPE, OnChangeDisplayType)
	ON_EN_CHANGE(IDC_EDIT_EXP_COND, OnEnChangeEditExpCond)
END_MESSAGE_MAP()

COLORREF CEditExperimentDlg::GetBackground() const
{
	return(_backColor);
}

COLORREF CEditExperimentDlg::GetForeground() const
{
	return(_foreColor);
}

const std::string &CEditExperimentDlg::GetHeader() const
{
	return(_header);
}

Tokenizer::FilterMode CEditExperimentDlg::GetFilterMode() 
{
	return(_filterMode);
}

Tokenizer::CalibrationType CEditExperimentDlg::GetCalibrationType()
{
	return(_calibrationType);
}

Tokenizer::DisplayType CEditExperimentDlg::GetDisplayType()
{
	return(_displayType);
}

int CEditExperimentDlg::GetNConditions() const
{
	return(_nConds);
}

int CEditExperimentDlg::GetNExperiments() const
{
	return(_nExpts);
}

unsigned int CEditExperimentDlg::GetDCThreshold() const
{
	return(_dcThreshold);
}

void CEditExperimentDlg::OnBnClickedChangeBackColor()
{
	CColorDialog dlg(_backColor, 0, this);

	if(dlg.DoModal() == IDOK) {
		_backColor = dlg.GetColor();
		Invalidate();
		RedrawWindow();
	}
}

void CEditExperimentDlg::OnBnClickedChangeTextForeColor()
{
	CColorDialog dlg(_foreColor, 0, this);

	if(dlg.DoModal() == IDOK) {
		_foreColor = dlg.GetColor();
		Invalidate();
		RedrawWindow();
	}
}

BOOL CEditExperimentDlg::OnInitDialog()
{
	char buf[10];
	int fmodeOff = (_filterMode == Tokenizer::FILTER_OFF) ? BST_CHECKED : BST_UNCHECKED;
	int fmodeOn  = (_filterMode == Tokenizer::FILTER_ON)  ? BST_CHECKED : BST_UNCHECKED;


	int calibH3  = (_calibrationType == Tokenizer::H3) ? BST_CHECKED : BST_UNCHECKED;
	int calibHV9 = (_calibrationType == Tokenizer::HV9) ? BST_CHECKED : BST_UNCHECKED;
	int calibHV13 = (_calibrationType == Tokenizer::HV13) ? BST_CHECKED : BST_UNCHECKED;

	int dispLCD = (_displayType == Tokenizer::LCD) ? BST_CHECKED : BST_UNCHECKED;
	int dispCRT = (_displayType == Tokenizer::CRT) ? BST_CHECKED : BST_UNCHECKED;


	CDialog::OnInitDialog();

	GetDlgItem(IDC_EDIT_EXPERIMENT_HEADER)->SetWindowText(_header.c_str());
	_itoa(_nConds, buf, 10);
	GetDlgItem(IDC_EDIT_EXPERIMENT_CONDITIONS)->SetWindowText(buf);
	_itoa(_nExpts, buf, 10);
	GetDlgItem(IDC_EDIT_EXPERIMENT_EXPTS)->SetWindowText(buf);
	((CButton*)GetDlgItem(IDC_LINK_FILTER_ON))->SetCheck(fmodeOn);
	((CButton*)GetDlgItem(IDC_LINK_FILTER_OFF))->SetCheck(fmodeOff);
	((CButton*)GetDlgItem(IDC_CALIBRATION_H3))->SetCheck(calibH3);
	((CButton*)GetDlgItem(IDC_CALIBRATION_HV9))->SetCheck(calibHV9);
	((CButton*)GetDlgItem(IDC_CALIBRATION_HV13))->SetCheck(calibHV13);
	((CButton*)GetDlgItem(IDC_LCD_DISPLAY_TYPE))->SetCheck(dispLCD);
	((CButton*)GetDlgItem(IDC_CRT_DISPLAY_TYPE))->SetCheck(dispCRT);
	_itoa(_dcThreshold, buf, 10);
	GetDlgItem(IDC_EDIT_EXPERIMENT_GAZE_DELTA_THRESHOLD)->SetWindowText(buf);

	std::ostringstream os;

	char buffer[10];
	
	for (int i=0; i<20; i++){
		if ( this->_expC[i] != 0 )
			os<<itoa(this->_expC[i], buffer, 10)<<" ";
	}
	GetDlgItem(IDC_EDIT_EXP_COND)->SetWindowText(os.str().c_str());

	m_ToolTip= new CToolTipCtrl;
	m_ToolTip->Create(this);
	m_ToolTip->AddTool(GetDlgItem(IDC_LCD_DISPLAY_TYPE), "Choose this option for most Display Change, Preview, and Moving Window experiments.");
	m_ToolTip->AddTool(GetDlgItem(IDC_CRT_DISPLAY_TYPE), "Choose this option for more precise priming duration.");
	m_ToolTip->AddTool(GetDlgItem(IDC_EDIT_EXP_COND), "Complete this field if your script contains multiple experiments with different number of conditions in each");
	m_ToolTip->AddTool(GetDlgItem(IDC_EDIT_EXPERIMENT_GAZE_DELTA_THRESHOLD), "Normal range is 2 to 4 pixels, lower choice will result in more flicker near boundary crossings, higher will increase DC response delay.");
	m_ToolTip->AddTool(GetDlgItem(IDC_LINK_FILTER_ON), "Choose this option if higher precision fixation and saccade times are of importance.");
	m_ToolTip->AddTool(GetDlgItem(IDC_LINK_FILTER_OFF), "Choose this option for most experiments. Filter off allows for higher transfer rates between subject and experimenter PC.");
	m_ToolTip->Activate(TRUE);	

	return TRUE;
}

BOOL CEditExperimentDlg::PreTranslateMessage(MSG* pMsg)
{
    // TODO: Add your specialized code here and/or call the base class

	/*switch (pMsg->message){
		case CB_SETCURSEL:
			break;
	}*/
    if(m_ToolTip != NULL)
       m_ToolTip->RelayEvent(pMsg);


    return CDialog::PreTranslateMessage(pMsg);
}

void CEditExperimentDlg::OnBnClickedOk()
{
	CString str;
	using namespace std;
	std::ostringstream missing;
	UpdateData(TRUE);

	missing << "The following fields are required:" <<std::endl;

	GetDlgItem(IDC_EDIT_EXPERIMENT_CONDITIONS)->GetWindowText(str);
	_nConds = atoi(str.GetBuffer());
	if(str.IsEmpty() || _nConds < 1) {
		MessageBox("Experiment must have at least one condition type.", "Dialog Incomplete", MB_OK | MB_ICONSTOP);
		return;
	}

	GetDlgItem(IDC_EDIT_EXPERIMENT_EXPTS)->GetWindowText(str);
	_nExpts = atoi(str.GetBuffer());
	if(str.IsEmpty() || _nExpts < 1) {
		MessageBox("Script must contain at least one experiment.", "Dialog Incomplete", MB_OK | MB_ICONSTOP);
		return;
	}

	GetDlgItem(IDC_EDIT_EXPERIMENT_GAZE_DELTA_THRESHOLD)->GetWindowText(str);
	_dcThreshold = atoi(str.GetBuffer());
	if(str.IsEmpty()) {
		MessageBox("Script must define display update threshold.", "Dialog Incomplete", MB_OK | MB_ICONSTOP);
		return;
	}

	GetDlgItem(IDC_EDIT_EXPERIMENT_HEADER)->GetWindowText(str);
	_header = str.GetBuffer();


#ifdef DEBUG
	if (_filterMode == Tokenizer::FILTER_ON){
		//MessageBox("Link Filtering enabled.", "Dialog Incomplete", MB_OK | MB_ICONSTOP);
		//return;
		OnOK();
	}

	else if (_filterMode == Tokenizer::FILTER_OFF){
		//MessageBox("Link Filtering disabled.", "Dialog Incomplete", MB_OK | MB_ICONSTOP);		
		//return;
		OnOK();
	}
	else
#endif // DEBUG
	if (_filterMode == Tokenizer::NONE_SELECTED){
		MessageBox("Please select link filtering mode.", "Dialog Incomplete", MB_OK | MB_ICONSTOP);
		return;
	}

	if(_expC[0] != 0){
		int sum = 0;
		for(int i=0; i<_nExpts; i++)
			sum += _expC[i];
		if (sum != this->_nConds){
		MessageBox("Experiment must have at least one condition type.", "Dialog Incomplete", MB_OK | MB_ICONSTOP);
		return;
		}
	}
	OnOK();
}

void CEditExperimentDlg::OnPaint()
{
	CStatic *preview = (CStatic*)GetDlgItem(IDC_STATIC_TEXT_PROPERTIES);
	CPaintDC dc(this);                           // device context for painting
	CPaintDC sDC(preview);
	CBrush backBrush(_backColor);
	CRect rect;
	
	sDC.SelectObject(backBrush);
	sDC.SetBkColor(_backColor);
	sDC.SetTextColor(_foreColor);

	preview->GetClientRect(rect);
	rect.InflateRect(1, 1);
	sDC.Rectangle(rect);

	rect.DeflateRect(1, 1);
	sDC.DrawText("Foreground\nText\nSample", 22, rect, DT_CENTER | DT_WORDBREAK);
}

void CEditExperimentDlg::SetColors(COLORREF back, COLORREF fore)
{
	_backColor = back;
	_foreColor = fore;
}

void CEditExperimentDlg::SetConditions(int nConditions, int nExperiments)
{
	_nConds = nConditions;
	_nExpts = nExperiments;
}

void CEditExperimentDlg::SetHeader(const std::string &header)
{
	_header = header;
}


void CEditExperimentDlg::SetFilterMode(const Tokenizer::FilterMode fMode)
{
	_filterMode = fMode;
}

void CEditExperimentDlg::SetCalibrationType(const Tokenizer::CalibrationType cType)
{
	_calibrationType = cType;
}

void CEditExperimentDlg::SetDisplayType(const Tokenizer::DisplayType dType)
{
	_displayType = dType;
}

void CEditExperimentDlg::SetDCThreshold(const unsigned int dcThreshold)
{
	_dcThreshold = dcThreshold;
}


void CEditExperimentDlg::OnEnChangeEditExperimentHeader()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}


void CEditExperimentDlg::OnChangeFilterMode()
{
	int selection = GetCheckedRadioButton(IDC_LINK_FILTER_ON, 
										  IDC_LINK_FILTER_OFF);

	switch(selection) {
		case IDC_LINK_FILTER_ON: 
			_filterMode = Tokenizer::FILTER_ON;
			break;
		case IDC_LINK_FILTER_OFF:
			_filterMode = Tokenizer::FILTER_OFF;
			break;
	}
}

void CEditExperimentDlg::OnChangeDisplayType()
{
	int selection = GetCheckedRadioButton(IDC_LCD_DISPLAY_TYPE,
										  IDC_CRT_DISPLAY_TYPE);

	switch(selection) {
		case IDC_LCD_DISPLAY_TYPE:
			_displayType = Tokenizer::LCD;
			break;
		case IDC_CRT_DISPLAY_TYPE:
			_displayType = Tokenizer::CRT;
			break;
		default:
			_displayType = Tokenizer::CRT;
			break;
	}
}

void CEditExperimentDlg::OnBnClickedCancel()
{
	// TODO: Add your control notification handler code here
	OnCancel();
}

void CEditExperimentDlg::OnChangeCalibrationType()
{
	int selection = GetCheckedRadioButton(IDC_CALIBRATION_H3,
										  IDC_CALIBRATION_HV13);

	switch(selection) {
		case IDC_CALIBRATION_H3:
			_calibrationType = Tokenizer::H3;
			break;
		case IDC_CALIBRATION_HV9:
			_calibrationType = Tokenizer::HV9;
			break;
		case IDC_CALIBRATION_HV13:
			_calibrationType = Tokenizer::HV13;
			break;
		default:
			_calibrationType = Tokenizer::HV9;
			break;
	}
}	


void CEditExperimentDlg::OnEnChangeEditExperimentConditions()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here

}

void CEditExperimentDlg::OnEnChangeEditExperimentExpts()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
	
}

void CEditExperimentDlg::OnBnClickedRadio2()
{
	// TODO: Add your control notification handler code here
}

void CEditExperimentDlg::OnEnChangeEditExperimentGazeDeltaThreshold()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}

void CEditExperimentDlg::OnBnClickedLcdDisplayType()
{
	// TODO: Add your control notification handler code here
}

void CEditExperimentDlg::OnLbnSelchangeListExpCond()
{
	// TODO: Add your control notification handler code here
}

void CEditExperimentDlg::OnCbnSelchangeComboExpCond()
{
	// TODO: Add your control notification handler code here
}

void CEditExperimentDlg::OnEnChangeEditExpCond()
{
	CString buff;
	GetDlgItem(IDC_EDIT_EXP_COND)->GetWindowText(buff);

	int next;
	next = getNextInt(buff);
	
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here

}

int CEditExperimentDlg::getNextInt(CString buf){
	int pos = 0;
	for (int i=0; i< _nExpts; i++){
 		int next = atoi(buf.Tokenize(" ", pos));
		if (next == 0){
			for(int j = i; j< _nExpts; j++)
				_expC[j] = 0;
			break;
		}
		_expC[i] = next;
	}
	return 0;
}

void CEditExperimentDlg::getExpConditions(int buf[20]) const{
	for (int i = 0; i<20; i++)
		buf[i] = _expC[i];
	return;
}

void CEditExperimentDlg::SetExpConditions(int  buf[20]){
	for (int i = 0; i<20; i++)
		_expC[i] = buf[i];
	return;
}
