/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

#include "resource.h"
#include "afxwin.h"
#include "experiment.hpp" // TODO:: cf. EditTrialTypeDlg.h
#include <string>

// CEditExperimentDlg dialog

class CEditExperimentDlg : public CDialog
{
	DECLARE_DYNAMIC(CEditExperimentDlg)

public:
	CEditExperimentDlg(CWnd* pParent);
	virtual ~CEditExperimentDlg();

	enum { IDD = IDD_EDITEXPERIMENTDLG };

	const std::string &GetHeader() const;
	Tokenizer::FilterMode GetFilterMode();
	Tokenizer::CalibrationType GetCalibrationType();
	Tokenizer::DisplayType GetDisplayType();
	unsigned int GetDCThreshold() const;

	COLORREF GetBackground() const;
	COLORREF GetForeground() const;

	int GetNConditions() const;
	int GetNExperiments() const;
	void getExpConditions(int buf[20]) const;

	virtual BOOL OnInitDialog();

	afx_msg void OnBnClickedChangeBackColor();
	afx_msg void OnBnClickedChangeTextForeColor();
	afx_msg void OnBnClickedOk();
	afx_msg void OnPaint();

	void SetColors(COLORREF back, COLORREF fore);
	void SetConditions(int nConditions, int nExperiments);
	void SetHeader(const std::string &header);
	void SetFilterMode(const Tokenizer::FilterMode fMode);
	void SetCalibrationType(const Tokenizer::CalibrationType cType);
	void SetDisplayType(const Tokenizer::DisplayType dType);
//	void SetWindowThreshold(unsigned int wThreshold);
	void SetDCThreshold(unsigned int dcThreshold);
	void SetExpConditions(int expc[20]);

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	CToolTipCtrl *m_ToolTip;
	DECLARE_MESSAGE_MAP()

private:
	CComboBox expCond;
	std::string _header;
	COLORREF _backColor;
	COLORREF _foreColor;
	int _nConds;
	int _nExpts;
	Tokenizer::FilterMode _filterMode;
	Tokenizer::CalibrationType _calibrationType;
	Tokenizer::DisplayType _displayType;
	unsigned int _dcThreshold;
	int getNextInt(CString buff);
	int _expC[20];

public:
	afx_msg void OnBnClickedRadioTrialPropertyExperimental();
	afx_msg void OnBnClickedRadioTrialPropertyExperimental2();
	afx_msg void OnEnChangeEditExperimentHeader();
	afx_msg void OnChangeFilterMode();
	afx_msg void OnBnClickedCancel();
	afx_msg void OnChangeCalibrationType();
	afx_msg void OnEnChangeEditExperimentConditions();
	afx_msg void OnEnChangeEditExperimentExpts();
	afx_msg void OnBnClickedRadio2();
	afx_msg void OnEnChangeEditExperimentGazeDeltaThreshold();
	afx_msg void OnChangeDisplayType();
	afx_msg void OnBnClickedLcdDisplayType();
	afx_msg void OnLbnSelchangeListExpCond();
	afx_msg void OnCbnSelchangeComboExpCond();
	afx_msg void OnEnChangeEditExpCond();
	BOOL PreTranslateMessage(MSG* pMsg);
};
