// EditSequenceDialog.cpp : implementation file
//

#include "stdafx.h"
#include "EyeTrack.h"
#include "EditSequenceDialog.h"
#include ".\editsequencedialog.h"


IMPLEMENT_DYNAMIC(CEditSequenceDialog, CDialog)
CEditSequenceDialog::CEditSequenceDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CEditSequenceDialog::IDD, pParent), 
	  _indepId(), _seqId(), _length(0), _altLength(0)
{}

CEditSequenceDialog::~CEditSequenceDialog()
{}

void CEditSequenceDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COMBO_INDEPENDENT_TRIALS, m_comboIndependent);
	DDX_Control(pDX, IDC_LIST_DEPENDENT_TRIALS, m_listDependent);
	DDX_Control(pDX, IDC_LIST_SEQUENCE_TRIALS, m_listSequence);
	DDX_Control(pDX, IDC_LIST_SEQUENCE_TRIALS2, m_listAltSequence);
	//DDX_Control(pDX, IDC_LIST_TRIALTYPE_BUTTONS, mListButtons);
	DDX_Control(pDX, IDC_COMBO_ALT_SELECT_BUTTON, m_listButtons);
}

BEGIN_MESSAGE_MAP(CEditSequenceDialog, CDialog)
	ON_BN_CLICKED(IDC_BUTTON_ADD_DEPENDENT, OnBnClickedAddDependent)
	ON_BN_CLICKED(IDC_BUTTON_REMOVE_FROM_SEQUENCE, OnBnClickedRemoveFromSequence)
	ON_CBN_SELCHANGE(IDC_COMBO_INDEPENDENT_TRIALS, OnCbnSelchangeIndependentTrial)
	ON_LBN_DBLCLK(IDC_LIST_DEPENDENT_TRIALS, OnBnClickedAddDependent)
	ON_LBN_DBLCLK(IDC_LIST_SEQUENCE_TRIALS, OnBnClickedRemoveFromSequence)
	ON_NOTIFY(UDN_DELTAPOS, IDC_SPIN_SEQUENCE_TRIALS, OnDeltaposSequenceTrial)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDC_BUTTON_ADD_DEPENDENT2, OnBnClickedButtonAddDependent2)
	ON_BN_CLICKED(IDC_BUTTON_REMOVE_FROM_SEQUENCE2, OnBnClickedButtonRemoveFromSequence2)
	ON_LBN_SELCHANGE(IDC_LIST_TRIALTYPE_BUTTONS, OnLbnSelchangeListTrialtypeButtons)
	ON_CBN_SELCHANGE(IDC_COMBO_ALT_SELECT_BUTTON, OnCbnSelchangeComboAltSelectButton)
	ON_LBN_SELCHANGE(IDC_LIST_DEPENDENT_TRIALS, OnLbnSelchangeListDependentTrials)
	ON_LBN_SELCHANGE(IDC_LIST_SEQUENCE_TRIALS, OnLbnSelchangeListSequenceTrials)
	ON_BN_CLICKED(IDC_CHECK_ENABLE_ALT_SEQUENCE, OnBnClickedCheckEnableAltSequence)
END_MESSAGE_MAP()


Sequence CEditSequenceDialog::GetSequence() const
{
	if (!_altSequence.empty() && _altEnabled)
		return(Sequence(_sequence, _altSequence, _seqId.idString(), (Tokenizer::ButtonType)_altSelectButton));
	else
		return(Sequence(_sequence, _seqId.idString()));
}
void CEditSequenceDialog::OnBnClickedButtonAddDependent2()
{
	char buf[100];
	int pos = m_listDependent.GetCurSel();

	if(pos != LB_ERR) {                          //make sure selection is valid
		std::string tName;

		m_listDependent.GetText(pos, buf);
		tName = buf;

		for(int i=0; i  < _altLength; ++i) {
			m_listAltSequence.GetText(i, buf);
			if(tName.compare(buf) == 0) {        //make sure selection not already in sequence
				MessageBox("A dependent trial may appear in a sequence at most once.", "Sequence Error");
				return;
			}
		}
		m_listAltSequence.AddString(tName.c_str()); //add it to sequence

		++_altLength;
		GetDlgItem(IDC_STATIC_SEQUENCE_LENGTH)->SetWindowText(_itoa(_length, buf, 10));
	}
}

void CEditSequenceDialog::OnBnClickedAddDependent()
{
	char buf[100];
	int pos = m_listDependent.GetCurSel();

	if(pos != LB_ERR) {                          //make sure selection is valid
		std::string tName;

		m_listDependent.GetText(pos, buf);
		tName = buf;

		for(int i=0; i  < _length; ++i) {
			m_listSequence.GetText(i, buf);
			if(tName.compare(buf) == 0) {        //make sure selection not already in sequence
				MessageBox("A dependent trial may appear in a sequence at most once.", "Sequence Error");
				return;
			}
		}
		m_listSequence.AddString(tName.c_str()); //add it to sequence

		++_length;
		GetDlgItem(IDC_STATIC_SEQUENCE_LENGTH)->SetWindowText(_itoa(_length, buf, 10));
	}
}


void CEditSequenceDialog::OnBnClickedRemoveFromSequence()
{
	char buf[100];
	int pos = m_listSequence.GetCurSel();

	if(pos != LB_ERR) {                          //make sure selection is valid
		m_listSequence.GetText(pos, buf);

		if(_indepId.idString().compare(buf) == 0) //prevent removal of independent
			MessageBox("Cannot remove independent trial.\nSelect an alternative from the list\nof independent trials.", "Sequence Error");
		else {                                   //okay, remove it
			m_listSequence.DeleteString(pos);
			--_length;
			GetDlgItem(IDC_STATIC_SEQUENCE_LENGTH)->SetWindowText(_itoa(_length, buf, 10));
		}
	}
}

void CEditSequenceDialog::OnCbnSelchangeIndependentTrial()
{
	std::string indepId = _indepId.idString();
	char buf[100];
	int pos = m_comboIndependent.GetCurSel();

	if(pos != CB_ERR) {                          //make sure new selection is valid
		std::string ident;

		m_comboIndependent.GetLBText(pos, buf);
		ident = buf;

		if(_indepId.condition() == 0) {          //nothing selected previously
			m_listSequence.AddString(buf);       //add to back of sequence
			++_length;
			if(_altEnabled){
				m_listAltSequence.AddString(buf);
				++_altLength;
			}
			GetDlgItem(IDC_STATIC_SEQUENCE_LENGTH)->SetWindowText(_itoa(_length, buf, 10));
		}
		else {                                   //something was selected
			pos = m_listSequence.FindString(-1, indepId.c_str());
			m_listSequence.DeleteString(pos);    //remove and replace
			m_listSequence.InsertString(pos, buf);

			if(_altEnabled){
				m_listAltSequence.DeleteString(pos);
				if (_altLength== 0)
					++_altLength;
				m_listAltSequence.InsertString(pos, buf);
			}
		}

		_indepId = TrialId(ident);               //make the new identifier
		_seqId = TrialId(_indepId.type(), _indepId.condition(), _indepId.item(), 0, true);
		GetDlgItem(IDC_STATIC_SEQUENCE_ID)->SetWindowText(_seqId.idString().c_str());
	}
	else {                                       //invalid selection, reselect the old
		pos = m_comboIndependent.FindString(-1, indepId.c_str());
		m_comboIndependent.SetCurSel(pos);
	}
}

void CEditSequenceDialog::OnDeltaposSequenceTrial(NMHDR *pNMHDR, LRESULT *pResult)
{
	LPNMUPDOWN pNMUpDown = reinterpret_cast<LPNMUPDOWN>(pNMHDR);
	CString temp;
	int oldPos = m_listSequence.GetCurSel();
	int newPos = oldPos + pNMUpDown->iDelta;
	
	if(oldPos != LB_ERR) {
		m_listSequence.GetText(oldPos, temp);

		if(newPos < oldPos) ++oldPos;
		else ++newPos;

		m_listSequence.InsertString(newPos, temp);
		m_listSequence.SetCurSel(newPos);
		m_listSequence.DeleteString(oldPos);
	}

	*pResult = 0;
}

BOOL CEditSequenceDialog::OnInitDialog()
{
	int pos;
	char buf[10];
	_altEnabled = false;


	CDialog::OnInitDialog();

	for(IdentifierIter i = _independent.begin(); i != _independent.end(); ++i)
		m_comboIndependent.AddString(i->idString().c_str());

	for(IdentifierIter i = _dependent.begin(); i != _dependent.end(); ++i)
		m_listDependent.AddString(i->idString().c_str());

	for(IdentifierIter i = _sequence.begin(); i != _sequence.end(); ++i) {
		const TrialId &id = *i;

		++_length;
		m_listSequence.AddString(id.idString().c_str());
		if(!id.isDependent())                    //this is the independent trial
			_indepId = id;
	}

		for(IdentifierIter i = _altSequence.begin(); i != _altSequence.end(); ++i) {
			const TrialId &id = *i;

			++_altLength;
			m_listAltSequence.AddString(id.idString().c_str());
			if(!id.isDependent())                    //this is the independent trial
				_indepId = id;
		}


	if (_altLength > 0){
		_altEnabled = true;
		((CButton *)GetDlgItem(IDC_CHECK_ENABLE_ALT_SEQUENCE))->SetCheck(BST_CHECKED);
		OnBnClickedCheckEnableAltSequence();
	}
	                                             //select the independent in the drop-down
	pos = m_comboIndependent.FindString(-1, _indepId.idString().c_str());
	m_comboIndependent.SetCurSel(pos);
                                                 //set the sequence name and length
	GetDlgItem(IDC_STATIC_SEQUENCE_ID)->SetWindowText(_seqId.idString().c_str());
	GetDlgItem(IDC_STATIC_SEQUENCE_LENGTH)->SetWindowText(_itoa(_length, buf, 10));

	updateButtonList();

	/*if(_altEnabled){
		((CButton *)GetDlgItem(IDC_CHECK_ENABLE_ALT_SEQUENCE))->SetCheck(BST_CHECKED);
	}*/
	return TRUE; 
}

void CEditSequenceDialog::OnOK()
{
	char buf[100];

	if(_length < 2) {
		MessageBox("Trial sequences must have at least two trials.", "Sequence Error");
		return;
	}

	if(_indepId.condition() == 0) {
		MessageBox("Trial sequences must have exactly one independent trial.", "Sequence Error");
		return;
	}

	_sequence.clear();
	for(int i=0; i < _length; ++i) {
		m_listSequence.GetText(i, buf);
		_sequence.push_back(TrialId(buf));
	}
	_altSequence.clear();
	for(int i=0; i < _altLength; ++i) {
		m_listAltSequence.GetText(i, buf);
		_altSequence.push_back(TrialId(buf));
	}

	if(_altLength > 0 && _length != _altLength){
		MessageBox("Alt sequence must have same length and same starting trial as main sequence.", "Sequence Error");
		return;
	}
	CDialog::OnOK();
}

void CEditSequenceDialog::SetSequence(Sequence &sequence)
{
	_sequence.clear();
	sequence.resetTrialPtr();

	while(!sequence.atEnd()){
		_sequence.push_back(sequence.nextTrialId());
		if(sequence.altLength() > 0)
		_altSequence.push_back(sequence.nextAltId());
	}

/*	while(!sequence.atAltEnd()){
		_altSequence.push_back(sequence.nextAltId());
	}*/
	_seqId = sequence.identifier();
	_altSelectButton = sequence.altSelectButton();
}

void CEditSequenceDialog::SetTrials(const IdentifierList &indep, const IdentifierList &dep)
{
	_independent = indep;
	_dependent = dep;
}

void CEditSequenceDialog::OnBnClickedOk()
{
	// TODO: Add your control notification handler code here
	OnOK();
}


void CEditSequenceDialog::OnBnClickedButtonRemoveFromSequence2()
{
	char buf[100];
	int pos = m_listAltSequence.GetCurSel();

	if(pos != LB_ERR) {                          //make sure selection is valid
		m_listAltSequence.GetText(pos, buf);

		if(_indepId.idString().compare(buf) == 0) //prevent removal of independent
			MessageBox("Cannot remove independent trial.\nSelect an alternative from the list\nof independent trials.", "Sequence Error");
		else {                                   //okay, remove it
			m_listAltSequence.DeleteString(pos);
			--_altLength;
			//GetDlgItem(IDC_STATIC_SEQUENCE_LENGTH)->SetWindowText(_itoa(_length, buf, 10));
		}
	}
}


void CEditSequenceDialog::updateButtonList()
{
	//if(_controlType == Tokenizer::FIVE_BUTTON){
	if(true){
	/*	mListButtons.ResetContent();
		mListButtons.AddString("(1) Blue");          //load buttons into list box
		mListButtons.AddString("(2) Red");
		mListButtons.AddString("(3) Green");
		mListButtons.AddString("(4) Yellow");
		mListButtons.AddString("(5) Purple");
		mListButtons.AddString("Mouse Click");*/

		m_listButtons.ResetContent();
		m_listButtons.AddString("(1) Blue");          //load buttons into list box
		m_listButtons.AddString("(2) Red");
		m_listButtons.AddString("(3) Green");
		m_listButtons.AddString("(4) Yellow");
		m_listButtons.AddString("(5) Purple");
		m_listButtons.AddString("Mouse Click");
	}

	/*else{
		mListButtons.ResetContent();
		mListButtons.AddString("(1) Y");
		mListButtons.AddString("(2) X");
		mListButtons.AddString("(3) B");
		mListButtons.AddString("(4) A");
		mListButtons.AddString("(5) Toggle");
		mListButtons.AddString("(6) Left Trigger");
		mListButtons.AddString("(7) Right Trigger");
		mListButtons.AddString("Mouse Click");
	}*/

//	mListButtons.SetSel(_altSelectButton-1);
	m_listButtons.SetCurSel(_altSelectButton-1);

	return;
}
void CEditSequenceDialog::OnLbnSelchangeListTrialtypeButtons()
{
	// TODO: Add your control notification handler code here
}

void CEditSequenceDialog::OnCbnSelchangeComboAltSelectButton()
{
	_altSelectButton = m_listButtons.GetCurSel()+1;
			_altEnabled = true;
		m_listButtons.EnableWindow(TRUE);
		GetDlgItem(IDC_COMBO_ALT_SELECT_BUTTON)->EnableWindow(TRUE);
		m_listAltSequence.EnableWindow(TRUE);

}

void CEditSequenceDialog::OnBnClickedCheckEnableAltSequence()
{
	CButton *btn = (CButton*)GetDlgItem(IDC_CHECK_ENABLE_ALT_SEQUENCE);

	if(btn->GetCheck() == BST_CHECKED){
		_altEnabled = true;
		m_listButtons.EnableWindow(TRUE);
		GetDlgItem(IDC_COMBO_ALT_SELECT_BUTTON)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_ADD_DEPENDENT2)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_REMOVE_FROM_SEQUENCE2)->EnableWindow(TRUE);
		m_listAltSequence.EnableWindow(TRUE);
	}
	else {
		_altEnabled = false;
		m_listButtons.EnableWindow(FALSE);
		m_listAltSequence.EnableWindow(FALSE);
		m_listAltSequence.ResetContent();
		GetDlgItem(IDC_COMBO_ALT_SELECT_BUTTON)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_ADD_DEPENDENT2)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_REMOVE_FROM_SEQUENCE2)->EnableWindow(FALSE);

	}
}

void CEditSequenceDialog::OnLbnSelchangeListDependentTrials()
{
	// TODO: Add your control notification handler code here
}

void CEditSequenceDialog::OnLbnSelchangeListSequenceTrials()
{
	// TODO: Add your control notification handler code here
}
