/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "EyeTrack.h"
#include "EditTrialAudioDialog.h"
#include ".\edittrialaudiodialog.h"


// CEditTrialAudioDialog dialog

IMPLEMENT_DYNAMIC(CEditTrialAudioDialog, CDialog)
CEditTrialAudioDialog::CEditTrialAudioDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CEditTrialAudioDialog::IDD, pParent), _data(), _isEnabled(false)
{}

CEditTrialAudioDialog::~CEditTrialAudioDialog()
{}

void CEditTrialAudioDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CEditTrialAudioDialog, CDialog)
	ON_BN_CLICKED(IDC_CHECK_TRIAL_AUDIO_ENABLE, OnBnClickedAudioEnable)
	ON_EN_KILLFOCUS(IDC_EDIT_TRIAL_AUDIO_FILE, OnEnKillfocusEditAudioFile)
	ON_EN_KILLFOCUS(IDC_EDIT_TRIAL_AUDIO_START, OnEnKillfocusEditStartDelay)
	ON_EN_KILLFOCUS(IDC_EDIT_TRIAL_AUDIO_STOP, OnEnKillfocusEditStopDelay)
	ON_BN_CLICKED(IDC_BUTTON_TRIAL_AUDIO_BROWSE, OnBnClickedBrowse)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_AUDIO_START, OnEnChangeEditTrialAudioStart)
END_MESSAGE_MAP()


void CEditTrialAudioDialog::OnBnClickedAudioEnable()
{
	CButton *button = (CButton*)GetDlgItem(IDC_CHECK_TRIAL_AUDIO_ENABLE);

	if(button->GetCheck() == BST_CHECKED) {
		GetDlgItem(IDC_BUTTON_TRIAL_AUDIO_BROWSE)->EnableWindow();
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_FILE)->EnableWindow();
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_START)->EnableWindow();
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_STOP)->EnableWindow();
		GetDlgItem(IDC_STATIC_TRIAL_AUDIO_FILE)->EnableWindow();
		GetDlgItem(IDC_STATIC_TRIAL_AUDIO_START)->EnableWindow();
		GetDlgItem(IDC_STATIC_TRIAL_AUDIO_STOP)->EnableWindow();
		_isEnabled = true;
	}
	else {
		GetDlgItem(IDC_BUTTON_TRIAL_AUDIO_BROWSE)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_FILE)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_FILE)->SetWindowText("");
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_START)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_START)->SetWindowText("0");
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_STOP)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_STOP)->SetWindowText("0");
		GetDlgItem(IDC_STATIC_TRIAL_AUDIO_FILE)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_AUDIO_START)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_AUDIO_STOP)->EnableWindow(FALSE);

		_data.path.clear();
		_data.startDelay = 0;
		_data.stopDelay = 0;
		_isEnabled = false;
	}
}

void CEditTrialAudioDialog::OnBnClickedBrowse()
{
	char filterExt[] = "Wave Files (*.wav)|*.wav|All Files (*.*)|*.*||";
	CFileDialog fileDlg(true, "wav", NULL, OFN_FILEMUSTEXIST, filterExt, this);

	if(fileDlg.DoModal() == IDOK) {
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_FILE)->SetWindowText(fileDlg.GetPathName());
		_data.path = fileDlg.GetPathName().GetBuffer();
	}
}

void CEditTrialAudioDialog::OnEnKillfocusEditAudioFile()
{
	char buf[250];

	GetDlgItem(IDC_EDIT_TRIAL_AUDIO_FILE)->GetWindowText(buf, 250);
	_data.path = buf;
}

void CEditTrialAudioDialog::OnEnKillfocusEditStartDelay()
{
	char buf[20];

	GetDlgItem(IDC_EDIT_TRIAL_AUDIO_START)->GetWindowText(buf, 20);
	_data.startDelay = atoi(buf);
}

void CEditTrialAudioDialog::OnEnKillfocusEditStopDelay()
{
	char buf[20];

	GetDlgItem(IDC_EDIT_TRIAL_AUDIO_STOP)->GetWindowText(buf, 20);
	_data.stopDelay = atoi(buf);
}

BOOL CEditTrialAudioDialog::OnInitDialog()
{
	CDialog::OnInitDialog();

	if(_data.path.empty()) {
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_START)->SetWindowText("0");
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_STOP)->SetWindowText("0");
	}
	else {
		char buf[20];

		CheckDlgButton(IDC_CHECK_TRIAL_AUDIO_ENABLE, BST_CHECKED);
		OnBnClickedAudioEnable();

		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_FILE)->SetWindowText(_data.path.c_str());
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_START)->SetWindowText(_itoa(_data.startDelay, buf, 10));
		GetDlgItem(IDC_EDIT_TRIAL_AUDIO_STOP)->SetWindowText(_itoa(_data.stopDelay, buf, 10));
	}

	return TRUE;
}

void CEditTrialAudioDialog::OnOK()
{
	if(_isEnabled) {
		if(_data.path.empty()) {
			MessageBox("Wave file name is required when audio playback is enabled.", "Property Error");
			GetDlgItem(IDC_EDIT_TRIAL_AUDIO_FILE)->SetFocus();
			return;
		}
	}

	CDialog::OnOK();
}

AudioRenderData &CEditTrialAudioDialog::RenderData()
{
	return(_data);
}
void CEditTrialAudioDialog::OnEnChangeEditTrialAudioStart()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}
