/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include <sstream>

#include "EyeTrack.h"
#include "EditTrialDialog.h"
#include "EditTrialDisplayDialog.h"
#include "EditTrialImagesDialog.h"
#include "TrialWnd.h"
#include ".\edittrialdisplaydialog.h"

IMPLEMENT_DYNAMIC(CEditTrialDisplayDialog, CDialog)
CEditTrialDisplayDialog::CEditTrialDisplayDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CEditTrialDisplayDialog::IDD, pParent), 
	  _dd(), _allowGCRect(false), _useStimulus1(true), _useStimulus(1)
{
	_dd.isTextDisplay = false;
}

CEditTrialDisplayDialog::~CEditTrialDisplayDialog()
{}

void CEditTrialDisplayDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CEditTrialDisplayDialog, CDialog)
	ON_BN_CLICKED(IDC_BUTTON_TRIAL_DISPLAY_BROWSE1, OnBnClickedBrowseDisplay1)
	ON_BN_CLICKED(IDC_BUTTON_TRIAL_DISPLAY_BROWSE2, OnBnClickedBrowseDisplay2)
	ON_BN_CLICKED(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_ADD, OnBnClickedAddRegion)
	ON_BN_CLICKED(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_REMOVE, OnBnClickedRemoveRegion)
	ON_BN_CLICKED(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_UPDATE, OnBnClickedUpdateRegion)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_DISPLAY_CHANGE_NONE, OnBnClickedDisplayChangeNone)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_DISPLAY_CHANGE_REGION, OnBnClickedDisplayChangeRegion)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_DISPLAY_PREVIEW1, OnBnClickedDisplayPreview1)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_DISPLAY_PREVIEW2, OnBnClickedDisplayPreview2)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_DISPLAY_TYPE_IMAGE, OnBnClickedDisplayTypeImage)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_DISPLAY_TYPE_TEXT, OnBnClickedDisplayTypeText)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1, OnEnChangeDCRect)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1, OnEnChangeDCRect)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2, OnEnChangeDCRect)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2, OnEnChangeDCRect)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_DISPLAY_GCX1, OnEnChangeGCRect)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_DISPLAY_GCY1, OnEnChangeGCRect)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_DISPLAY_GCX2, OnEnChangeGCRect)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_DISPLAY_GCY2, OnEnChangeGCRect)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_DISPLAY_STIMULUS1, OnEnChangeStimulus1)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2, OnEnChangeStimulus2)
	ON_LBN_SELCHANGE(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS, OnLbnSelchangeRegion)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_DISPLAY_CHANGE_MOVING, OnBnClickedDisplayChangeMovingWindow)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_DISPLAY_CHANGE_MW_WITH_PREVIEW, OnBnClickedDisplayChangeMovingWindowPreview)
	ON_BN_CLICKED(IDC_STATIC_TRIAL_DISPLAY_GCRECT, OnBnClickedStaticTrialDisplayGcrect)
	ON_STN_CLICKED(IDC_STATIC_TRIAL_DISPLAY_GCX3, OnStnClickedStaticTrialDisplayGcx3)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_CHAR, OnBnClickedRadioTrialMovWindTriggerChar)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_WORD, OnBnClickedRadioTrialMovWindTriggerWord)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_MOV_WIND_TARGET_CHAR, OnBnClickedRadioTrialMovWindTargetChar)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_MOV_WIND_TARGET_WORD, OnBnClickedRadioTrialMovWindTargetWord)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_MOV_WIND_TRAIL_SIZE, OnEnChangeTrialMovWindSize)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_MOV_WIND_LEAD_SIZE, OnEnChangeTrialMovWindSize)
	ON_BN_CLICKED(IDC_STATIC_TRIAL_DISPLAY_CHANGE_REGIONS, OnBnClickedStaticTrialDisplayChangeRegions)
	ON_BN_CLICKED(IDC_DC_DISTINCT_REGIONS, OnBnClickedDcDistinctRegions)
	ON_EN_CHANGE(IDC_TARGET_X1, OnEnChangeDCRect)
	ON_EN_CHANGE(IDC_TARGET_X2, OnEnChangeDCRect)
	ON_EN_CHANGE(IDC_TARGET_Y1, OnEnChangeDCRect)
	ON_EN_CHANGE(IDC_TARGET_Y2, OnEnChangeDCRect)
	ON_STN_CLICKED(IDC_TARGET_X1_CAPTION, OnStnClickedTargetX1Caption)
	ON_EN_CHANGE(IDC_EDIT_MW_PREVIEW_WORD_INDEX, OnEnChangeEditMwPreviewWordIndex)
	ON_LBN_SELCHANGE(IDC_LIST_TRIAL_MW_PREVIEW_WORDS, OnLbnSelchangePreviewWords)
	ON_BN_CLICKED(IDC_WINDOW_PREVIEW_ADD, OnBnClickedWindowPreviewAdd)
	ON_BN_CLICKED(IDC_WINDOW_PREVIEW_UPDATE, OnBnClickedWindowPreviewUpdate)
	ON_BN_CLICKED(IDC_WINDOW_PREVIEW_REMOVE, OnBnClickedWindowPreviewRemove)
	ON_EN_CHANGE(IDC_PAIRED_REGION_ONSET_DELAY, OnEnChangePairedRegionOnsetDelay)
	ON_BN_CLICKED(IDC_CHECK_SET_DELAY, OnBnClickedCheckSetDelay)
	ON_BN_CLICKED(IDC_BUTTON_DISPLAY_TRiAL_ADD_IMAGES_1, OnBnClickedButtonDisplayTrialAddImages1)
	ON_BN_CLICKED(IDC_BUTTON_DISPLAY_TRIAL_ADD_IMAGES_2, OnBnClickedButtonDisplayTrialAddImages2)
	ON_BN_CLICKED(IDCANCEL, OnBnClickedCancel)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_DISPLAY_PREVIEW3, OnBnClickedRadioTrialDisplayPreview3)
	ON_EN_CHANGE(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3, OnEnChangeEditTrialDisplayStimulus3)
	ON_BN_CLICKED(IDC_BUTTON_DISPLAY_TRIAL_ADD_IMAGES_3, OnBnClickedButtonDisplayTrialAddImages3)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_DISPLAY_CHANGE_SCOTOMA, OnBnClickedDisplayChangeScotoma)
END_MESSAGE_MAP()

void CEditTrialDisplayDialog::EnableGCRect()
{
	_allowGCRect = true;
}

BOOL CEditTrialDisplayDialog::OnInitDialog()
{
	m_ToolTip= new CToolTipCtrl;
	m_ToolTip->Create(this);
	
	
	m_ToolTip->AddTool(GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_PREVIEW3), "This content will be shown when conditions of Onset2 and Delay2 are met below.");
	m_ToolTip->AddTool(GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_CHANGE_SCOTOMA), "Use the rect. coord fields to define where the scotoma is, relative to gaze position. Gaze is (0,0). Lower right quad is positive.");
	m_ToolTip->AddTool(GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_TYPE_TEXT), "Choose this option if trial has either text only, text and images, or coordinate locked images with no scaling");
	m_ToolTip->AddTool(GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_TYPE_IMAGE), "Choose this option if your image bitmaps should be scaled to full screen.");
	m_ToolTip->AddTool(GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY), "Number of ms after event that bitmap associated with this region in stim 2 will be shown.");
	m_ToolTip->AddTool(GetDlgItem(IDC_PAIRED_REGION_DURATION), "Number of ms after event that bitmap associated with this region will revert to stim 1.");
	m_ToolTip->AddTool(GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY_2), "Number of ms after event that bitmap associated with this region in stim 3 will be shown.");
	m_ToolTip->AddTool(GetDlgItem(IDC_PAIRED_REGION_DURATION_2), "Number of ms after event that bitmap associated with this region will revert from stim 3 to stim 2. (note: Set this to the length of trial if no revert of stim 3)");
	m_ToolTip->AddTool(GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_TRAIL_SIZE), "Number of words (or character) behind the currently fixated word (or character) that are displayed as part of a moving window");
	m_ToolTip->AddTool(GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_LEAD_SIZE), "Number of words (or character) beyond the currently fixated word (or character) that are displayed as part of a moving window");
	m_ToolTip->AddTool(GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCX1), "Upper left corner is (0,0) grid increases down and right.");
	m_ToolTip->AddTool(GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCX2), "Upper left corner is (0,0) grid increases down and right.");
	m_ToolTip->AddTool(GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCY1), "Upper left corner is (0,0) grid increases down and right.");
	m_ToolTip->AddTool(GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCY2), "Upper left corner is (0,0) grid increases down and right.");

	m_ToolTip->AddTool(GetDlgItem(IDC_TRIGGER_CAPTION), "Set coordinates of the region that listens for gaze position.");
	m_ToolTip->AddTool(GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1), "Set coordinates of the region that listens for gaze position. Upper left corner is (0,0) grid increases down and right.");
	m_ToolTip->AddTool(GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2), "Set coordinates of the region that listens for gaze position. Upper left corner is (0,0) grid increases down and right.");
	m_ToolTip->AddTool(GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1), "Set coordinates of the region that listens for gaze position. Upper left corner is (0,0) grid increases down and right.");
	m_ToolTip->AddTool(GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2), "Set coordinates of the region that listens for gaze position. Upper left corner is (0,0) grid increases down and right.");
	
	m_ToolTip->AddTool(GetDlgItem(IDC_TARGET_X1), "Set coordinates of the region that updates during display change. Upper left corner is (0,0) grid increases down and right.");
	m_ToolTip->AddTool(GetDlgItem(IDC_TARGET_X2), "Set coordinates of the region that updates during display change. Upper left corner is (0,0) grid increases down and right.");
	m_ToolTip->AddTool(GetDlgItem(IDC_TARGET_Y1), "Set coordinates of the region that updates during display change. Upper left corner is (0,0) grid increases down and right.");
	m_ToolTip->AddTool(GetDlgItem(IDC_TARGET_Y2), "Set coordinates of the region that updates during display change. Upper left corner is (0,0) grid increases down and right.");
	
	m_ToolTip->AddTool(GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_CHANGE_NONE), "Simple static display.");
	m_ToolTip->AddTool(GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_CHANGE_REGION), "Define single or paired regions that change based upon predefined parameters.");
	m_ToolTip->AddTool(GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_CHANGE_MOVING), "Define a range of words (or characters) that are visible around the current gaze position. Requires appropriate masking text in Stimulus 2.");
	m_ToolTip->AddTool(GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_CHANGE_MW_WITH_PREVIEW), "Special case moving window that shows a preview (of n ms) of some set of words to the right of the window before crossing boundary.");
	m_ToolTip->AddTool(GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_CHANGE_SCOTOMA), "Define a rectangular mask of arbitrary size and relative position that follows the gaze.");

	m_ToolTip->AddTool(GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY), "ms");
	m_ToolTip->AddTool(GetDlgItem(IDC_PAIRED_REGION_DURATION), "ms");
	m_ToolTip->AddTool(GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY_2), "ms");
	m_ToolTip->AddTool(GetDlgItem(IDC_PAIRED_REGION_DURATION_2), "ms");

	m_ToolTip->Activate(TRUE);	
	return TRUE;
}

BOOL CEditTrialDisplayDialog::PreTranslateMessage(MSG* pMsg)
{
    if(m_ToolTip != NULL)
       m_ToolTip->RelayEvent(pMsg);


    return CDialog::PreTranslateMessage(pMsg);
}

void CEditTrialDisplayDialog::EnableText(const TextFormat &tf)
{
	_tf = tf;
	GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_TYPE_TEXT)->EnableWindow(TRUE);
}

std::string CEditTrialDisplayDialog::FormatForDisplay(const std::string &text)
{
	std::string str(text);
	std::string::size_type idx = str.find("\r\n");

	while(idx != std::string::npos) {            //replace carriage return & newline with just newline
		str.replace(idx, 1, "\n");
		str.erase(idx, 1);
		idx = str.find("\r\n", idx);
	}

	idx = str.find("\\_");
	while(idx != std::string::npos) {            //replace "\a" marker with single '\a' character
		str[idx] = '\a';
		str.erase(++idx, 1);
		idx = str.find("\\_", idx);
	}
	return(str);
}

std::string CEditTrialDisplayDialog::FormatForEditBox(const std::string &text)
{
	std::string str(text);
	std::string::size_type idx = str.find("\n");

	while(idx != std::string::npos) {            //replace '\n' with "\r\n"
		str.insert(idx, "\r");
		idx = str.find("\n", idx+2);
	}

	idx = str.find("\a");
	while(idx != std::string::npos) {            //replace '\a' marker with "\_"
		str[idx] = '\\';
		str.insert(++idx, "_");
		idx = str.find("\a");
	}
	return(str);
}

const DisplayData &CEditTrialDisplayDialog::GetDisplayData() const
{
	return(_dd);
}

void CEditTrialDisplayDialog::Initialize(const DisplayData &data)
{
	//_dd = data;
	if(data.isTextDisplay) {                     //set display type text or image
		std::string s1 = FormatForEditBox(data.display1);
		std::string s2 = FormatForEditBox(data.display2);
		std::string s3 = FormatForEditBox(data.display3);

		OnBnClickedDisplayTypeText();
		CheckRadioButton(IDC_RADIO_TRIAL_DISPLAY_TYPE_IMAGE, 
						 IDC_RADIO_TRIAL_DISPLAY_TYPE_TEXT, 
						 IDC_RADIO_TRIAL_DISPLAY_TYPE_TEXT);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS1)->SetWindowText(s1.c_str());
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->SetWindowText(s2.c_str());
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3)->SetWindowText(s3.c_str());
	}
	else {
		OnBnClickedDisplayTypeImage();
		CheckRadioButton(IDC_RADIO_TRIAL_DISPLAY_TYPE_IMAGE, 
						 IDC_RADIO_TRIAL_DISPLAY_TYPE_TEXT, 
						 IDC_RADIO_TRIAL_DISPLAY_TYPE_IMAGE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS1)->SetWindowText(data.display1.c_str());
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->SetWindowText(data.display2.c_str());
	}

	if(data.isFixedDisplay) {                    //set change type none or region
		OnBnClickedDisplayChangeNone();
		CheckRadioButton(IDC_RADIO_TRIAL_DISPLAY_CHANGE_NONE, 
						 IDC_RADIO_TRIAL_DISPLAY_CHANGE_MW_WITH_PREVIEW, 
						 IDC_RADIO_TRIAL_DISPLAY_CHANGE_NONE);

	}
	else if (data.isMovingWindow) {
		if(data.toggleWords.empty()){
			OnBnClickedDisplayChangeMovingWindow();
			CheckRadioButton(IDC_RADIO_TRIAL_DISPLAY_CHANGE_NONE, 
							 IDC_RADIO_TRIAL_DISPLAY_CHANGE_MW_WITH_PREVIEW, 
							 IDC_RADIO_TRIAL_DISPLAY_CHANGE_MOVING);
		}
		else{
			OnBnClickedDisplayChangeMovingWindowPreview();
			CheckRadioButton(IDC_RADIO_TRIAL_DISPLAY_CHANGE_NONE,
							 IDC_RADIO_TRIAL_DISPLAY_CHANGE_MW_WITH_PREVIEW,
							 IDC_RADIO_TRIAL_DISPLAY_CHANGE_MW_WITH_PREVIEW);
		}

		if (data.movWindowType.useWordTargets == TRUE){
			OnBnClickedRadioTrialMovWindTargetWord();
			CheckRadioButton(	IDC_RADIO_TRIAL_MOV_WIND_TARGET_CHAR,
								IDC_RADIO_TRIAL_MOV_WIND_TARGET_WORD,
								IDC_RADIO_TRIAL_MOV_WIND_TARGET_WORD);
		}
		else if (data.movWindowType.useWordTargets == FALSE) {
			OnBnClickedRadioTrialMovWindTargetChar();
			CheckRadioButton(	IDC_RADIO_TRIAL_MOV_WIND_TARGET_CHAR,
								IDC_RADIO_TRIAL_MOV_WIND_TARGET_WORD,
								IDC_RADIO_TRIAL_MOV_WIND_TARGET_CHAR);
		}

		if (data.movWindowType.useWordTriggers == TRUE){
			OnBnClickedRadioTrialMovWindTriggerWord();
			CheckRadioButton(	IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_CHAR,
								IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_WORD,
								IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_WORD);
		}
		else if (data.movWindowType.useWordTargets == FALSE){
			OnBnClickedRadioTrialMovWindTriggerChar();
			CheckRadioButton(	IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_CHAR,
								IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_WORD,
								IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_CHAR);
		}
		char buf[10];
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_TRAIL_SIZE)->SetWindowText(_itoa(data.windowSize.x, buf, 10));
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_LEAD_SIZE)->SetWindowText(_itoa(data.windowSize.y, buf, 10));
	
	
	}
	else if (data.isPairedRegionChange){
		CListBox &lb = *((CListBox*)GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS));
		_dd.isPairedRegionChange = true;
		OnBnClickedDisplayChangeRegion();

		CheckRadioButton(IDC_RADIO_TRIAL_DISPLAY_CHANGE_NONE, 
						 IDC_RADIO_TRIAL_DISPLAY_CHANGE_MW_WITH_PREVIEW, 
						 IDC_RADIO_TRIAL_DISPLAY_CHANGE_REGION);

		CheckRadioButton(IDC_DC_DISTINCT_REGIONS,IDC_DC_DISTINCT_REGIONS,IDC_DC_DISTINCT_REGIONS);

		GetDlgItem(IDC_CHECK_SET_DELAY)->EnableWindow(TRUE);

		for(unsigned int i =0; i < data.regionPairs.size(); ++i){
			const CRect &r1 = data.regionPairs[i].region1;
			const CRect &r2 = data.regionPairs[i].region2;
			std::ostringstream os;
			os << '('<<r1.top<<' '<<r1.left<<' '<<r1.bottom<<' '<<r1.right<<") -> ("<<r2.top<<' '<<r2.left<<' '<<r2.bottom<<' '<<r2.right<<')';
			lb.AddString(os.str().c_str());
		}
	}

	else if (data.isTimedPairedRegionChange){
				CListBox &lb = *((CListBox*)GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS));
		_dd.isPairedRegionChange = true;
		OnBnClickedDisplayChangeRegion();

		CheckRadioButton(IDC_RADIO_TRIAL_DISPLAY_CHANGE_NONE, 
						 IDC_RADIO_TRIAL_DISPLAY_CHANGE_MW_WITH_PREVIEW, 
						 IDC_RADIO_TRIAL_DISPLAY_CHANGE_REGION);

		CheckRadioButton(IDC_DC_DISTINCT_REGIONS,IDC_DC_DISTINCT_REGIONS,IDC_DC_DISTINCT_REGIONS);
		CheckDlgButton(IDC_CHECK_SET_DELAY, TRUE);
		GetDlgItem(IDC_CHECK_SET_DELAY)->EnableWindow(TRUE);

		GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY)->EnableWindow(TRUE);
		//GetDlgItem(IDC_STATIC_ONSET_DELAY)->EnableWindow(TRUE);
		GetDlgItem(IDC_PAIRED_REGION_DURATION)->EnableWindow(TRUE);
		//GetDlgItem(IDC_STATIC_REVERT_DELAY)->EnableWindow(TRUE);

		if(!data.display3.empty() || !data.images3.empty()){
			GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY_2)->EnableWindow(TRUE);
			GetDlgItem(IDC_PAIRED_REGION_DURATION_2)->EnableWindow(TRUE);
		}

		for(unsigned int i =0; i < data.timedRectPairs.size(); ++i){
			const CRect &r1 = data.timedRectPairs.at(i).r1;
			const CRect &r2 = data.timedRectPairs.at(i).r2data.region;
			std::ostringstream os;
			os << '('<<r1.top<<' '<<r1.left<<' '<<r1.bottom<<' '<<r1.right<<") -> ("<<r2.top<<' '<<r2.left<<' '<<r2.bottom<<' '<<r2.right<<')';
			lb.AddString(os.str().c_str());
		}
	}

	else {
		CListBox &lb = *((CListBox*)GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS));

		if(data.isScotoma){
			OnBnClickedDisplayChangeScotoma();
			CheckRadioButton(IDC_RADIO_TRIAL_DISPLAY_CHANGE_NONE, 
						 IDC_RADIO_TRIAL_DISPLAY_CHANGE_SCOTOMA, 
						 IDC_RADIO_TRIAL_DISPLAY_CHANGE_SCOTOMA);
		}
		else{
		  OnBnClickedDisplayChangeRegion();
		  CheckRadioButton(IDC_RADIO_TRIAL_DISPLAY_CHANGE_NONE, 
						 IDC_RADIO_TRIAL_DISPLAY_CHANGE_MW_WITH_PREVIEW, 
						 IDC_RADIO_TRIAL_DISPLAY_CHANGE_REGION);
		}
		for(unsigned int i=0; i < data.region.size(); ++i) {
			const CRect &r = data.region[i];
			std::ostringstream os;
			os<<'('<<r.top<<' '<<r.left<<' '<<r.bottom<<' '<<r.right<<')';
			lb.AddString(os.str().c_str());
		}
	}

	if(!data.toggleWords.empty()){
		CListBox &lb2 = *((CListBox*)GetDlgItem(IDC_LIST_TRIAL_MW_PREVIEW_WORDS));

		for (unsigned int i=0; i < data.toggleWords.size(); i++) {
			CPoint p;
			p.x = data.toggleWords.at(i).wordIndex;
			p.y = data.toggleWords.at(i).duration;
			std::ostringstream os2;
			os2<<'('<<p.x<<", "<<p.y<<')';
			//os<<'4'<<' '<<"200";
			lb2.AddString(os2.str().c_str());
		}
	}

	                                             //set preview to stimulus 1
	CheckRadioButton(IDC_RADIO_TRIAL_DISPLAY_PREVIEW1, 
					 IDC_RADIO_TRIAL_DISPLAY_PREVIEW3, 
					 IDC_RADIO_TRIAL_DISPLAY_PREVIEW1);

	if(_allowGCRect) {                           //set gaze-contingent rect (if necessary)
		char buf[10];

		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCX1)->SetWindowText(_itoa(data.gcRect.left, buf, 10));
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCY1)->SetWindowText(_itoa(data.gcRect.top, buf, 10));
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCX2)->SetWindowText(_itoa(data.gcRect.right, buf, 10));
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCY2)->SetWindowText(_itoa(data.gcRect.bottom, buf, 10));
	}
	else {
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCX1)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCY1)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCX2)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCY2)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_GCX1)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_GCY1)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_GCX2)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_GCY2)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_GCRECT)->EnableWindow(FALSE);
	}

	_dd = data;
	UpdateData(FALSE);
	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::OnBnClickedWindowPreviewAdd()
{
	std::vector<TOGGLEDATA>::iterator iter = 0;

	iter = _dd.toggleWords.begin();

	std::ostringstream os;

	CListBox &lb = *((CListBox*) GetDlgItem(IDC_LIST_TRIAL_MW_PREVIEW_WORDS));

	CPoint p = ReadPreviewMetrics();

	TOGGLEDATA td;
	td.wordIndex = p.x;
	td.duration = p.y;

	_dd.toggleWords.push_back(td);

	os<<'('<<p.x<<", "<<p.y<<')';
	lb.AddString(os.str().c_str());
	lb.SetSel(-1);

	return;

}

void CEditTrialDisplayDialog::OnBnClickedAddRegion()
{

	CPoint ul = ((CTrialWnd*)GetParent())->getUpperLeft();
	CPoint lr = ((CTrialWnd*)GetParent())->getLowerRight();

	char buf[10];
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->SetWindowText(_itoa(ul.x, buf, 10));
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->SetWindowText(_itoa(ul.y, buf, 10));
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->SetWindowText(_itoa(lr.x, buf, 10));
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->SetWindowText(_itoa(lr.y, buf, 10));

	const int MAX_X = ::GetSystemMetrics(SM_CXSCREEN);
	const int MAX_Y = ::GetSystemMetrics(SM_CYSCREEN);

	unsigned int delay, duration, delay2, duration2;

	std::vector<CRect>::iterator iter = 0;
	std::vector<RectPair>::iterator pairIter = 0;
	bool a = _dd.isPairedRegionChange || _dd.isTimedPairedRegionChange;
	if(!a) 
		iter = _dd.region.begin();
	else 
		pairIter = _dd.regionPairs.begin();

	std::ostringstream os;
	CListBox &lb = *((CListBox*)GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS));
	CRect r, r1, r2;
	//CRect r = ReadDCRect();
	if (!a)
		r = ReadDCRect();
	else
		ReadDCRects(&r1,&r2);
	if(_dd.isTimedPairedRegionChange)
		ReadDCTimes(&delay, &duration, &delay2, &duration2);

	if(!a){
		if(_dd.isScotoma){ // then allow any rectangle
		}
	    //make sure r is contained within the screen 
		else if(r.top < 0 || r.left < 0 || r.bottom > MAX_Y || r.right > MAX_X) {
			MessageBox("New rectangle must fit entirely within bounds of screen.", "Display Change Error");
			return;
		}

		if(r.IsRectEmpty()) {                        //make sure new rect not empty
			MessageBox("Cannot add region with zero area.", "Display Change Error");
			return;
		}
                                                 //ensure r does not overlap existing rects
		for(unsigned int i=0; i < _dd.region.size(); ++i) {
			CRect tmp = r & _dd.region[i];

			if(!tmp.IsRectEmpty()) {
				MessageBox("New rectangle must not overlap existing rectangles.", "Display Change Error");
				return;
			}
		}
		_dd.region.push_back(r);                     //add rect to display and list box
		os<<'('<<r.top<<' '<<r.left<<' '<<r.bottom<<' '<<r.right<<')';
	}

	else if(!_dd.isTimedPairedRegionChange){
		if(r1.top < 0 || r1.left < 0 || r1.bottom > MAX_Y || r1.right > MAX_X
			|| r2.top < 0 || r2.left < 0 || r2.bottom > MAX_Y || r2.right > MAX_X) {
				MessageBox("Updated rectangles must fit entirely within the bounds of screen.", "Display Change Error");
				return;
		}
		if(r1.IsRectEmpty()){
			MessageBox("Cannot add region with zero area.", "Display Change Error");
			return;
		}

		if(!_dd.isTimedPairedRegionChange){
			for(unsigned int i=0; i < _dd.regionPairs.size(); ++i) {
				CRect tmp = r1 & _dd.regionPairs.at(i).region1;	
				if (!tmp.IsRectEmpty()){
					MessageBox("Updated rectangle must not overlap any other rectangles.", "Display Change Error");
					return;
				}
			}
		}
			else {// isTimedPaired
				for(unsigned int i=0; i < _dd.timedRectPairs.size(); ++i) {
					CRect tmp = r1 & _dd.timedRectPairs.at(i).r1;	
					if(!tmp.IsRectEmpty()) {
						MessageBox("Updated rectangle must not overlap any other rectangles.", "Display Change Error");
						return;
					}
				}
			}

		_dd.regionPairs.push_back(RectPair(r1,r2));
		os << '('<<r1.top<<' '<<r1.left<<' '<<r1.bottom<<' '<<r1.right<<") -> ("<<r2.top<<' '<<r2.left<<' '<<r2.bottom<<' '<<r2.right<<')'; 
	}

	else { // not single and not untimed paired
		if(r1.top < 0 || r1.left < 0 || r1.bottom > MAX_Y || r1.right > MAX_X
			|| r2.top < 0 || r2.left < 0 || r2.bottom > MAX_Y || r2.right > MAX_X) {
				MessageBox("Updated rectangles must fit entirely within the bounds of screen.", "Display Change Error");
				return;
			}
			if(r1.IsRectEmpty()){
				MessageBox("Cannot add region with zero area.", "Display Change Error");
				return;
			}

			if(!_dd.isTimedPairedRegionChange){
				for(unsigned int i=0; i < _dd.regionPairs.size(); ++i) {
					CRect tmp = r1 & _dd.regionPairs.at(i).region1;	
					if(!tmp.IsRectEmpty()) {
						MessageBox("Updated rectangle must not overlap any other rectangles.", "Display Change Error");
						return;
					}
				}
			}
			else {// isTimedPaired
				for(unsigned int i=0; i < _dd.timedRectPairs.size(); ++i) {
					CRect tmp = r1 & _dd.timedRectPairs.at(i).r1;	
					if(!tmp.IsRectEmpty()) {
						MessageBox("Updated rectangle must not overlap any other rectangles.", "Display Change Error");
						return;
					}
				}
			}

			if(delay2 == 0 && duration2 == 0){
				_dd.timedRectPairs.push_back(TimedRectPair(r1,ToggleData(r2,delay,duration)));
				os << '('<<r1.top<<' '<<r1.left<<' '<<r1.bottom<<' '<<r1.right<<") -> ("<<r2.top<<' '<<r2.left<<' '<<r2.bottom<<' '<<r2.right<<')';
			}
			else{
				_dd.timedRectPairs.push_back(TimedRectPair(r1, ToggleData(r2,delay,duration,delay2,duration2)));
				os << '('<<r1.top<<' '<<r1.left<<' '<<r1.bottom<<' '<<r1.right<<") -> ("<<r2.top<<' '<<r2.left<<' '<<r2.bottom<<' '<<r2.right<<')';
			}

	}
	lb.AddString(os.str().c_str());
	lb.SetSel(-1);

	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::OnBnClickedBrowseDisplay1()
{
	char filterExt[] = "Bitmap Files (*.bmp)|*.bmp|All Files (*.*)|*.*||";
	CFileDialog fileDlg(true, "bmp", NULL, OFN_FILEMUSTEXIST, filterExt, this);

	if(fileDlg.DoModal() == IDOK) {
		_dd.display1 = fileDlg.GetPathName().GetBuffer();
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS1)->SetWindowText(_dd.display1.c_str());
	}

	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::OnBnClickedBrowseDisplay2()
{
	char filterExt[] = "Bitmap Files (*.bmp)|*.bmp|All Files (*.*)|*.*||";
	CFileDialog fileDlg(true, "bmp", NULL, OFN_FILEMUSTEXIST, filterExt, this);

	if(fileDlg.DoModal() == IDOK) {
		_dd.display2 = fileDlg.GetPathName().GetBuffer();
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->SetWindowText(_dd.display2.c_str());
	}

	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::resetDCdata()
{
		GetDlgItem(IDC_DC_DISTINCT_REGIONS)->EnableWindow(FALSE);

			GetDlgItem(IDC_TARGET_CAPTION_BOX)->EnableWindow(FALSE);
			GetDlgItem(IDC_TARGET_X1)->EnableWindow(FALSE);
			GetDlgItem(IDC_TARGET_X2)->EnableWindow(FALSE);
			GetDlgItem(IDC_TARGET_Y1)->EnableWindow(FALSE);
			GetDlgItem(IDC_TARGET_Y2)->EnableWindow(FALSE);

			GetDlgItem(IDC_TARGET_X1)->SetWindowText("");
			GetDlgItem(IDC_TARGET_X2)->SetWindowText("");
			GetDlgItem(IDC_TARGET_Y1)->SetWindowText("");
			GetDlgItem(IDC_TARGET_Y2)->SetWindowText("");

			GetDlgItem(IDC_TARGET_X1_CAPTION)->EnableWindow(FALSE);
			GetDlgItem(IDC_TARGET_X2_CAPTION)->EnableWindow(FALSE);
			GetDlgItem(IDC_TARGET_Y1_CAPTION)->EnableWindow(FALSE);
			GetDlgItem(IDC_TARGET_Y2_CAPTION)->EnableWindow(FALSE);

}

void CEditTrialDisplayDialog::OnBnClickedDisplayChangeNone()
{
	if(_dd.isFixedDisplay == false) {

		resetDCdata();
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->SetWindowText("");
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_STIMULUS2)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE2)->EnableWindow(FALSE);

		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3)->SetWindowText("");
//		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE3)->EnableWindow(FALSE);

		CheckRadioButton(IDC_RADIO_TRIAL_DISPLAY_PREVIEW1, 
						 IDC_RADIO_TRIAL_DISPLAY_PREVIEW3, 
						 IDC_RADIO_TRIAL_DISPLAY_PREVIEW1);
		_useStimulus1 = true;
		GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_PREVIEW2)->EnableWindow(FALSE);

		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_REGIONS)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_X1)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_X2)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_Y1)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_Y2)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->SetWindowText("");
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->SetWindowText("");
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->SetWindowText("");
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->SetWindowText("");
		GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS)->EnableWindow(FALSE);
		((CListBox*)GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS))->ResetContent();
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_ADD)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_UPDATE)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_REMOVE)->EnableWindow(FALSE);

		GetDlgItem(IDC_STATIC_MOVING_WINDOW_PROPERTIES)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_MOVING_WINDOW_TRIGGER)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_CHAR)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_WORD)->EnableWindow(FALSE);

		GetDlgItem(IDC_STATIC_MOVING_WINDOW_TYPE)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TARGET_CHAR)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TARGET_WORD)->EnableWindow(FALSE);
		
		GetDlgItem(IDC_STATIC_MOV_WIND_SIZE)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_MOV_WIND_TRAIL)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_MOV_WIND_LEAD)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_TRAIL_SIZE)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_LEAD_SIZE)->EnableWindow(FALSE);

		// Moving window with preview extras
		GetDlgItem(IDC_STATIC_MW_PREVIEW_PROPERTIES)->EnableWindow(TRUE);

		_dd.display2.clear();
		_dd.isFixedDisplay = true;
		_dd.isScotoma = false;
		_dd.region.clear();
		_dd.regionPairs.clear();

		UpdateTrialWnd();
	}
}

void CEditTrialDisplayDialog::OnBnClickedDisplayChangeRegion()
{
	//if(_dd.isFixedDisplay == true || _dd.isMovingWindow == true) {
		GetDlgItem(IDC_DC_DISTINCT_REGIONS)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_STIMULUS2)->EnableWindow(TRUE);
		EnableThree(TRUE);
		EnableRadio3(TRUE);

		/*EnableRadio3(TRUE);
		if(!_dd.display3.empty() || !_dd.images3.empty())
			EnableThree(TRUE);*/

		if(_dd.isTextDisplay == false)
			GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE2)->EnableWindow(TRUE);

		GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_PREVIEW2)->EnableWindow(TRUE);

		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_REGIONS)->EnableWindow(TRUE);
		GetDlgItem(IDC_TRIGGER_CAPTION)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_X1)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_X2)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_Y1)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_Y2)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->EnableWindow(TRUE);
	
		if(_dd.isPairedRegionChange == true){
			GetDlgItem(IDC_TARGET_CAPTION_BOX)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_X1)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_X2)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_Y1)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_Y2)->EnableWindow(TRUE);

			GetDlgItem(IDC_TARGET_X1_CAPTION)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_X2_CAPTION)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_Y1_CAPTION)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_Y2_CAPTION)->EnableWindow(TRUE);
		}

		if(_dd.isTimedPairedRegionChange == true){
			GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY)->EnableWindow(TRUE);
			GetDlgItem(IDC_PAIRED_REGION_DURATION)->EnableWindow(TRUE);
		}



		GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_ADD)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_UPDATE)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_REMOVE)->EnableWindow(TRUE);

		GetDlgItem(IDC_STATIC_MOVING_WINDOW_PROPERTIES)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_MOVING_WINDOW_TRIGGER)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_CHAR)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_WORD)->EnableWindow(FALSE);

		GetDlgItem(IDC_STATIC_MOVING_WINDOW_TYPE)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TARGET_CHAR)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TARGET_WORD)->EnableWindow(FALSE);
		
		GetDlgItem(IDC_STATIC_MOV_WIND_SIZE)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_MOV_WIND_TRAIL)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_MOV_WIND_LEAD)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_TRAIL_SIZE)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_LEAD_SIZE)->EnableWindow(FALSE);

		// Moving window with preview extras
		GetDlgItem(IDC_STATIC_MW_PREVIEW_PROPERTIES)->EnableWindow(FALSE);

		_dd.isFixedDisplay = false;
		_dd.isScotoma = false;

		UpdateTrialWnd();
	//}
}


void CEditTrialDisplayDialog::OnBnClickedDisplayChangeScotoma()
{
	//if(_dd.isFixedDisplay == true || _dd.isMovingWindow == true) {
		GetDlgItem(IDC_DC_DISTINCT_REGIONS)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_STIMULUS2)->EnableWindow(TRUE);
		EnableThree(TRUE);
		EnableRadio3(TRUE);

		/*EnableRadio3(TRUE);
		if(!_dd.display3.empty() || !_dd.images3.empty())
			EnableThree(TRUE);*/

		if(_dd.isTextDisplay == false)
			GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE2)->EnableWindow(TRUE);

		GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_PREVIEW2)->EnableWindow(TRUE);

		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_REGIONS)->EnableWindow(TRUE);
		GetDlgItem(IDC_TRIGGER_CAPTION)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_X1)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_X2)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_Y1)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_Y2)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->EnableWindow(TRUE);
	
		if(_dd.isPairedRegionChange == true){
			GetDlgItem(IDC_TARGET_CAPTION_BOX)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_X1)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_X2)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_Y1)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_Y2)->EnableWindow(TRUE);

			GetDlgItem(IDC_TARGET_X1_CAPTION)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_X2_CAPTION)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_Y1_CAPTION)->EnableWindow(TRUE);
			GetDlgItem(IDC_TARGET_Y2_CAPTION)->EnableWindow(TRUE);
		}

		if(_dd.isTimedPairedRegionChange == true){
			GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY)->EnableWindow(TRUE);
			GetDlgItem(IDC_PAIRED_REGION_DURATION)->EnableWindow(TRUE);
		}



		GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_ADD)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_UPDATE)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_REMOVE)->EnableWindow(TRUE);

		GetDlgItem(IDC_STATIC_MOVING_WINDOW_PROPERTIES)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_MOVING_WINDOW_TRIGGER)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_CHAR)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_WORD)->EnableWindow(FALSE);

		GetDlgItem(IDC_STATIC_MOVING_WINDOW_TYPE)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TARGET_CHAR)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TARGET_WORD)->EnableWindow(FALSE);
		
		GetDlgItem(IDC_STATIC_MOV_WIND_SIZE)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_MOV_WIND_TRAIL)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_MOV_WIND_LEAD)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_TRAIL_SIZE)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_LEAD_SIZE)->EnableWindow(FALSE);

		// Moving window with preview extras
		GetDlgItem(IDC_STATIC_MW_PREVIEW_PROPERTIES)->EnableWindow(FALSE);

		_dd.isFixedDisplay = false;
		_dd.isScotoma = true;  // add some dummy text so we can have images only
		if(_dd.display1.length() == 0)
			_dd.display1 = " ";
		if(_dd.display2.length() == 0)
			_dd.display2 = " ";

		UpdateTrialWnd();
	//}
}

void CEditTrialDisplayDialog::OnBnClickedDisplayChangeMovingWindow()
{
		resetDCdata();
	// TODO: Add your control notification handler code here
	//if(_dd.isMovingWindow == false) {
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_STIMULUS2)->EnableWindow(TRUE);
		if(_dd.isTextDisplay == false)
			GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE2)->EnableWindow(TRUE);

		GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_PREVIEW2)->EnableWindow(TRUE);

		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_REGIONS)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_X1)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_X2)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_Y1)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_Y2)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->SetWindowText("");
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->SetWindowText("");
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->SetWindowText("");
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->SetWindowText("");
		GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS)->EnableWindow(FALSE);
		GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS)->EnableWindow(FALSE);
		((CListBox*)GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS))->ResetContent();
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_ADD)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_UPDATE)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_REMOVE)->EnableWindow(FALSE);
		
		GetDlgItem(IDC_STATIC_MOVING_WINDOW_PROPERTIES)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_MOVING_WINDOW_TRIGGER)->EnableWindow(TRUE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_CHAR)->EnableWindow(TRUE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_WORD)->EnableWindow(TRUE);
		
		GetDlgItem(IDC_STATIC_MOVING_WINDOW_TYPE)->EnableWindow(TRUE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TARGET_CHAR)->EnableWindow(TRUE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TARGET_WORD)->EnableWindow(TRUE);
		
		GetDlgItem(IDC_STATIC_MOV_WIND_SIZE)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_MOV_WIND_TRAIL)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_MOV_WIND_LEAD)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_TRAIL_SIZE)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_LEAD_SIZE)->EnableWindow(TRUE);

		// Moving window with preview extras
		GetDlgItem(IDC_STATIC_MW_PREVIEW_PROPERTIES)->EnableWindow(FALSE);

		_dd.isFixedDisplay = false;
		_dd.isMovingWindow = true;
		_dd.isScotoma = false;
		OnEnChangeStimulus1();
		UpdateTrialWnd();
	//}
}

void CEditTrialDisplayDialog::OnBnClickedDisplayChangeMovingWindowPreview()
{
		resetDCdata();
	// TODO: Add your control notification handler code here
	//if(_dd.isMovingWindow == false) {
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_STIMULUS2)->EnableWindow(TRUE);
		if(_dd.isTextDisplay == false)
			GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE2)->EnableWindow(TRUE);

		GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_PREVIEW2)->EnableWindow(TRUE);

		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_REGIONS)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_X1)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_X2)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_Y1)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_TRIAL_DISPLAY_CHANGE_Y2)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->SetWindowText("");
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->SetWindowText("");
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->SetWindowText("");
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->SetWindowText("");
		GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS)->EnableWindow(FALSE);
		GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS)->EnableWindow(FALSE);
		((CListBox*)GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS))->ResetContent();
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_ADD)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_UPDATE)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_CHANGE_REMOVE)->EnableWindow(FALSE);
		
		GetDlgItem(IDC_STATIC_MOVING_WINDOW_PROPERTIES)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_MOVING_WINDOW_TRIGGER)->EnableWindow(TRUE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_CHAR)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_WORD)->EnableWindow(TRUE);
		CheckRadioButton(IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_CHAR,
						 IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_WORD,
							IDC_RADIO_TRIAL_MOV_WIND_TRIGGER_WORD);
		OnBnClickedRadioTrialMovWindTriggerWord();
		
		GetDlgItem(IDC_STATIC_MOVING_WINDOW_TYPE)->EnableWindow(TRUE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TARGET_CHAR)->EnableWindow(FALSE);
		GetDlgItem(IDC_RADIO_TRIAL_MOV_WIND_TARGET_WORD)->EnableWindow(TRUE);
		CheckRadioButton(IDC_RADIO_TRIAL_MOV_WIND_TARGET_CHAR,
						 IDC_RADIO_TRIAL_MOV_WIND_TARGET_WORD,
						 IDC_RADIO_TRIAL_MOV_WIND_TARGET_WORD);
		OnBnClickedRadioTrialMovWindTargetWord();

		
		GetDlgItem(IDC_STATIC_MOV_WIND_SIZE)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_MOV_WIND_TRAIL)->EnableWindow(FALSE);
		GetDlgItem(IDC_STATIC_MOV_WIND_LEAD)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_TRAIL_SIZE)->EnableWindow(FALSE);
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_TRAIL_SIZE)->SetWindowText("0");
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_LEAD_SIZE)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_LEAD_SIZE)->SetWindowText("0");

		// Moving window with preview extras
		GetDlgItem(IDC_STATIC_MW_PREVIEW_PROPERTIES)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_WORD_INDEX)->EnableWindow(TRUE);
		GetDlgItem(IDC_STATIC_PREVIEW_DURATION)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_MW_PREVIEW_WORD_INDEX)->EnableWindow(TRUE);
		GetDlgItem(IDC_EDIT_MW_PREVIEW_DURATION)->EnableWindow(TRUE);
		GetDlgItem(IDC_LIST_TRIAL_MW_PREVIEW_WORDS)->EnableWindow(TRUE);
		GetDlgItem(IDC_WINDOW_PREVIEW_ADD)->EnableWindow(TRUE);
		GetDlgItem(IDC_WINDOW_PREVIEW_UPDATE)->EnableWindow(TRUE);
		GetDlgItem(IDC_WINDOW_PREVIEW_REMOVE)->EnableWindow(TRUE);

		_dd.isFixedDisplay = false;
		_dd.isMovingWindow = true;
		_dd.isScotoma = false;
		OnEnChangeStimulus1();
		UpdateTrialWnd();
	//}
}
void CEditTrialDisplayDialog::OnBnClickedDisplayPreview1()
{
	_useStimulus1 = true;
	_useStimulus = 1;
	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::OnBnClickedDisplayPreview2()
{
	_useStimulus1 = false;
	_useStimulus = 2;
	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::OnBnClickedDisplayTypeImage()
{
	if(_dd.isTextDisplay == true) {
		//CRect r1(10, 107, 356, 128), r2(10, 190, 356, 211);
		CRect r1(10, 130, 356, 151), r2(10, 180, 356, 201);
		CRect r3(10, 230, 356, 251);

		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE1)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE1)->ShowWindow(SW_SHOW);

		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS1)->MoveWindow(&r1, TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS1)->SetWindowText("");

		if(_dd.isFixedDisplay == false)
			GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE2)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE2)->ShowWindow(SW_SHOW);

		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE3)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE3)->ShowWindow(SW_SHOW);

		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->MoveWindow(&r2, TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->SetWindowText("");

		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3)->MoveWindow(&r3, TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3)->SetWindowText("");
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_DISPLAY_TRIAL_ADD_IMAGES_3)->EnableWindow(FALSE);



		_dd.display1.clear();
		_dd.display2.clear();
		_dd.isTextDisplay = false;

		UpdateTrialWnd();
	}
}

void CEditTrialDisplayDialog::OnBnClickedDisplayTypeText()
{
	if(_dd.isTextDisplay == false) {
		//CRect r1(10, 107, 449, 168), r2(10, 190, 449, 251);
		//CRect r1(10, 130, 449, 191), r2(10, 213, 449, 274);
		CRect r1(10, 130, 449, 165), r2(10, 180, 449, 215);
		CRect r3(10, 230, 449, 265);

		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE1)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE1)->ShowWindow(SW_HIDE);

		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE2)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE2)->ShowWindow(SW_HIDE);

		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE3)->EnableWindow(FALSE);
		GetDlgItem(IDC_BUTTON_TRIAL_DISPLAY_BROWSE3)->ShowWindow(SW_HIDE);

		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS1)->MoveWindow(&r1, TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS1)->SetWindowText("");

		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->MoveWindow(&r2, TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->SetWindowText("");

		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3)->MoveWindow(&r3, TRUE);
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3)->SetWindowText("");
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3)->EnableWindow(TRUE);
		GetDlgItem(IDC_BUTTON_DISPLAY_TRIAL_ADD_IMAGES_3)->EnableWindow(TRUE);

		_dd.display1.clear();
		_dd.display2.clear();
		_dd.isTextDisplay = true;

		UpdateTrialWnd();
	}
}

void CEditTrialDisplayDialog::OnBnClickedWindowPreviewRemove()
{
	std::vector<TOGGLEDATA>::iterator iter =0;
	iter = _dd.toggleWords.begin();

	CListBox &lb = *((CListBox*)GetDlgItem(IDC_LIST_TRIAL_MW_PREVIEW_WORDS));

	int idx = lb.GetCurSel();

	if (idx == LB_ERR){
		return;
	}
	
	lb.DeleteString(idx);

	for(int i = 0; i <idx; ++i, ++iter);
	_dd.toggleWords.erase(iter);
	GetDlgItem(IDC_EDIT_MW_PREVIEW_WORD_INDEX)->SetWindowText("");
	GetDlgItem(IDC_EDIT_MW_PREVIEW_DURATION)->SetWindowText("");

	return;
}


void CEditTrialDisplayDialog::OnBnClickedRemoveRegion()
{
	std::vector<CRect>::iterator iter = 0;
	std::vector<RectPair>::iterator pairIter = 0;
	std::vector<TIMED_RECT_PAIR>::iterator timedPairIter = 0;
	bool a = _dd.isPairedRegionChange || _dd.isTimedPairedRegionChange;
	if(!a) 
		iter = _dd.region.begin();
	else if (!_dd.isTimedPairedRegionChange)
		pairIter = _dd.regionPairs.begin();
	else
		timedPairIter = _dd.timedRectPairs.begin();
	CListBox &lb = *((CListBox*)GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS));



	int idx = lb.GetCurSel();

	if(idx == LB_ERR)
		return;

	lb.DeleteString(idx);
	if(!a){
		for(int i=0; i < idx; ++i, ++iter);
		 _dd.region.erase(iter);
	}
	else {
		if(!_dd.isTimedPairedRegionChange){		
			for(int i = 0; i< idx; ++i, ++pairIter);
			_dd.regionPairs.erase(pairIter);
		}
		else{
			for(int i = 0; i<idx; ++i, ++timedPairIter);
			_dd.timedRectPairs.erase(timedPairIter);
		}
		GetDlgItem(IDC_TARGET_X1)->SetWindowText("");
		GetDlgItem(IDC_TARGET_X2)->SetWindowText("");
		GetDlgItem(IDC_TARGET_Y1)->SetWindowText("");
		GetDlgItem(IDC_TARGET_Y2)->SetWindowText("");
		GetDlgItem(IDC_PAIRED_REGION_DURATION)->SetWindowText("");
		GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY)->SetWindowText("");
	}

	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->SetWindowText("");
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->SetWindowText("");
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->SetWindowText("");
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->SetWindowText("");



	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::OnBnClickedWindowPreviewUpdate()
{

	std::ostringstream os;
	CListBox &lb = *((CListBox*)GetDlgItem(IDC_LIST_TRIAL_MW_PREVIEW_WORDS));

	CPoint p = ReadPreviewMetrics();
	int items = lb.GetCount();
	int idx = lb.GetCurSel();

	if(idx == LB_ERR) {
		MessageBox("Select a word to update.", "Moving Window Preview Error");
		return;
	}

	TOGGLEDATA td;
	td.wordIndex = p.x;
	td.duration = p.y;

	_dd.toggleWords.at(idx) = td;

	os<<'('<<td.wordIndex<<", "<<td.duration<<')';

	lb.DeleteString(idx);
	lb.InsertString(idx, os.str().c_str());
	lb.SetSel(-1);

	return;
}

void CEditTrialDisplayDialog::OnBnClickedUpdateRegion()
{
	//int single = FALSE, paired = FALSE;

	CPoint ul = ((CTrialWnd*)GetParent())->getUpperLeft();
	CPoint lr = ((CTrialWnd*)GetParent())->getLowerRight();

	char buf[10];
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->SetWindowText(_itoa(ul.x, buf, 10));
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->SetWindowText(_itoa(ul.y, buf, 10));
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->SetWindowText(_itoa(lr.x, buf, 10));
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->SetWindowText(_itoa(lr.y, buf, 10));



	bool paired = _dd.isPairedRegionChange || _dd.isTimedPairedRegionChange;
	//if (!_dd.isFixedDisplay && !_dd.isPairedRegionChange){
	//	single = TRUE;
	//	paired = FALSE;
	//}
	//e/lse if ( _dd.isPairedRegionChange){
	//	single = FALSE;
	//	paired = TRUE;
	//}

	const int MAX_X = ::GetSystemMetrics(SM_CXSCREEN);
	const int MAX_Y = ::GetSystemMetrics(SM_CYSCREEN);
	std::ostringstream os;
	CListBox &lb = *((CListBox*)GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS));

	CRect r, r1, r2;
	if (!paired)
		r = ReadDCRect();
	else
		ReadDCRects(&r1,&r2);

	unsigned int delay, duration, delay2, duration2;
	if(_dd.isTimedPairedRegionChange)
		ReadDCTimes(&delay, &duration, &delay2, &duration2);

	int nItems = lb.GetCount();
	int idx = lb.GetCurSel();

	if(idx == LB_ERR) {                          //must select rect to update
		MessageBox("Select a region to update.", "Display Change Error");
		return;
	}

	if( !paired ){

		if (_dd.isScotoma){ // then we can allow any position rectangle
		}
		//make sure r is contained within the screen 
		else if(r.top < 0 || r.left < 0 || r.bottom > MAX_Y || r.right > MAX_X) {
			MessageBox("Updated rectangle must fit entirely within bounds of screen.", "Display Change Error");
			return;
		}
	
		if(r.IsRectEmpty()) {                        //make sure new rect not empty
			MessageBox("Cannot add region with zero area.", "Display Change Error");
			return;
		}
 
		//ensure r does not overlap existing rects
		for(unsigned int i=0; i < _dd.region.size(); ++i) {
			CRect tmp = r & _dd.region[i];
	
			if(i != idx && !tmp.IsRectEmpty()) {
				MessageBox("Updated rectangle must not overlap any other rectangles.", "Display Change Error");
				return;
			}
		}
	}
	else{
		if(r1.top < 0 || r1.left < 0 || r1.bottom > MAX_Y || r1.right > MAX_X
			|| r2.top < 0 || r2.left < 0 || r2.bottom > MAX_Y || r2.right > MAX_X) {
			MessageBox("Updated rectangles must fit entirely within the bounds of screen.", "Display Change Error");
			return;
			}
		if(r1.IsRectEmpty()){
			MessageBox("Cannot add region with zero area.", "Display Change Error");
			return;
		}

		if(!_dd.isTimedPairedRegionChange){
			for(unsigned int i=0; i < _dd.regionPairs.size(); ++i) {
				CRect tmp = r1 & _dd.regionPairs.at(i).region1;	
				if(i != idx && !tmp.IsRectEmpty()) {
					MessageBox("Updated rectangle must not overlap any other rectangles.", "Display Change Error");
					return;
				}
			}
		}
		else {// isTimedPaired
			for(unsigned int i=0; i < _dd.timedRectPairs.size(); ++i) {
				CRect tmp = r1 & _dd.timedRectPairs.at(i).r1;	
				if(i != idx && !tmp.IsRectEmpty()) {
					MessageBox("Updated rectangle must not overlap any other rectangles.", "Display Change Error");
					return;
				}
			}
		}
	}


	if (!paired){
		_dd.region[idx] = r;
		os<<'('<<r.top<<' '<<r.left<<' '<<r.bottom<<' '<<r.right<<')';
	}
	else if (!_dd.isTimedPairedRegionChange) { 
		_dd.regionPairs[idx].region1 = r1;
		_dd.regionPairs[idx].region2 = r2;
		os << '('<<r1.top<<' '<<r1.left<<' '<<r1.bottom<<' '<<r1.right<<") -> ("<<r2.top<<' '<<r2.left<<' '<<r2.bottom<<' '<<r2.right<<')';
	}
	
	else {
		_dd.timedRectPairs.at(idx).r1 = r1;
		if(delay2 != 0 || duration2 != 0)
			_dd.timedRectPairs.at(idx).r2data = TOGGLEDATA(r2,delay,duration,delay2,duration2);
		else
		_dd.timedRectPairs.at(idx).r2data = TOGGLEDATA(r2,delay,duration);
		os << '('<<r1.top<<' '<<r1.left<<' '<<r1.bottom<<' '<<r1.right<<") -> ("<<r2.top<<' '<<r2.left<<' '<<r2.bottom<<' '<<r2.right<<')';
	}

	lb.DeleteString(idx);
	lb.InsertString(idx, os.str().c_str());
	lb.SetSel(-1);

	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::OnCancel()
{
	GetParent()->DestroyWindow();
}

void CEditTrialDisplayDialog::OnEnChangeDCRect()
{
	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::OnEnChangeGCRect()
{                                                //called if any of the GC coords changed
	const int MAX_LEN = 10;
	int len;
	char buf[MAX_LEN+1];

	len = GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCX1)->GetWindowText(buf, MAX_LEN);
	if(len == 0) return;
	else _dd.gcRect.left = atoi(buf);

	len = GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCY1)->GetWindowText(buf, MAX_LEN);
	if(len == 0) return;
	else _dd.gcRect.top = atoi(buf);

	len = GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCX2)->GetWindowText(buf, MAX_LEN);
	if(len == 0) return;
	else _dd.gcRect.right = atoi(buf);

	len = GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_GCY2)->GetWindowText(buf, MAX_LEN);
	if(len == 0) return;
	else _dd.gcRect.bottom = atoi(buf);

	UpdateTrialWnd();
}

unsigned int countTokens(std::string str){
	unsigned int nTokens = 0;
	size_t pos1=0, pos1a=0, pos1b=0, pos2=0;
	do{
		pos1 = str.find( ' ', pos2);
		pos1a = str.find('\n', pos2);
		if (pos1 < pos1a){
			pos1b = pos1;
			pos2 = str.find_first_not_of(' ', pos1);
		}
		else {
			pos1b = pos1a;
			pos2 = str.find_first_not_of('\n', pos1a);
		}
		//pos2 = str.find_first_not_of(' ', pos1);
		//pos2a = str.find_first_not_of('\n', pos1a);
		//if(pos1 != std::string.npos && pos2 != std::string.npos) nTokens++;
		nTokens++;
		}while(pos1b != std::string.npos && pos2 != std::string.npos);

	return nTokens;
}


	

void CEditTrialDisplayDialog::OnEnChangeStimulus1()
{                                                //called if stimulus 1 changed
	const int MAX_LEN = 5000;
	char buf[MAX_LEN+1];

	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS1)->GetWindowText(buf, MAX_LEN);
	_dd.display1 = FormatForDisplay(buf);
	if(_dd.isMovingWindow){
		std::string txt(buf);
		unsigned int tokens;
		tokens = countTokens(txt);
		_dd.windowRange.x = 0;
		_dd.windowRange.y = tokens-1;
	}
		
	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::OnEnChangeStimulus2()
{                                                //called if stimulus 2 changed
	const int MAX_LEN = 5000;
	char buf[MAX_LEN+1];

	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS2)->GetWindowText(buf, MAX_LEN);
	_dd.display2 = FormatForDisplay(buf);
	UpdateTrialWnd();
}





void CEditTrialDisplayDialog::OnLbnSelchangeRegion()
{                                                //called when display change region selected
	CListBox &lb = *((CListBox*)GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS));
	int idx = lb.GetCurSel();
	//CRect r = _dd.region[idx];
	char buf[25];

	if(idx == LB_ERR)
		return;

	if (!_dd.isPairedRegionChange && !_dd.isTimedPairedRegionChange){
		CRect r = _dd.region[idx];

		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->SetWindowText(_itoa(r.left, buf, 10));
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->SetWindowText(_itoa(r.top, buf, 10));
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->SetWindowText(_itoa(r.right, buf, 10));
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->SetWindowText(_itoa(r.bottom, buf, 10));
	}
	
	else { // dd.isPairedRegionChange || dd.isTimedPairedRegionChange
		CRect r1,r2;
		unsigned int delay=0, dur=0;
		if(!_dd.isTimedPairedRegionChange){
			r1 = _dd.regionPairs[idx].region1;
			r2 = _dd.regionPairs[idx].region2;
		}
		else {
			r1 = _dd.timedRectPairs[idx].r1;
			r2 = _dd.timedRectPairs[idx].r2data.region;
			delay = _dd.timedRectPairs[idx].r2data.delay;
			dur = _dd.timedRectPairs[idx].r2data.duration;
		}

		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->SetWindowText(_itoa(r1.left, buf, 10));
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->SetWindowText(_itoa(r1.top, buf, 10));
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->SetWindowText(_itoa(r1.right, buf, 10));
		GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->SetWindowText(_itoa(r1.bottom, buf, 10));

		GetDlgItem(IDC_TARGET_X1)->SetWindowText(_itoa(r2.left,buf,10));
		GetDlgItem(IDC_TARGET_Y1)->SetWindowText(_itoa(r2.top,buf,10));
		GetDlgItem(IDC_TARGET_X2)->SetWindowText(_itoa(r2.right,buf,10));
		GetDlgItem(IDC_TARGET_Y2)->SetWindowText(_itoa(r2.bottom,buf,10));

		if(_dd.isTimedPairedRegionChange){
			GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY)->SetWindowText(_itoa(delay,buf,10));
			GetDlgItem(IDC_PAIRED_REGION_DURATION)->SetWindowText(_itoa(dur,buf,10));
		}

		if(_dd.isTimedPairedRegionChange && !(_dd.display3.empty() && _dd.images3.empty())){
			unsigned int delay2=0, dur2=0;
			delay2 = _dd.timedRectPairs[idx].r2data.delay2;
			dur2 = _dd.timedRectPairs[idx].r2data.duration2;
			GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY_2)->SetWindowText(_itoa(delay2,buf,10));
			GetDlgItem(IDC_PAIRED_REGION_DURATION_2)->SetWindowText(_itoa(dur2,buf,10));
		}

	}
			
	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::OnOK()
{
	CWnd *parent = GetParent();

	if(_dd.display1.empty()) {
		MessageBox("Stimulus 1 must be specified in all trials.",
				   "Display Properties Error", MB_OK);
		return;
	}

	if(!_dd.isFixedDisplay) {
		if(_dd.display2.empty() && _dd.images2.empty() ) {
			MessageBox("Stimulus 2 must be specified in display change trials.",
					   "Display Properties Error", MB_OK);
			return;
		}

		if(_dd.region.size() == 0 && _dd.regionPairs.size() == 0 && !_dd.isMovingWindow && _dd.timedRectPairs.size() == 0) {
			MessageBox("Display change trials must specify at\nleast one display change region.",
					   "Display Properties Error", MB_OK);
			return;
		}
	}

	if(_dd.isMovingWindow || _dd.isPairedRegionChange || _dd.isTimedPairedRegionChange || !_dd.isFixedDisplay ) {
		if(_dd.display2.empty() && _dd.images2.empty()) {
			MessageBox("Stimulus 2 must be specified in display change trials.",
					   "Display Properties Error", MB_OK);
			return;
		}
	}


	((CEditTrialDialog*)(parent->GetParent()))->OnEditDisplayDialogOkay();
	parent->DestroyWindow();
}

void CEditTrialDisplayDialog::PostNcDestroy()
{
	CDialog::PostNcDestroy();
	delete this;
}

CRect CEditTrialDisplayDialog::ReadDCRect()
{
	const int MAX_LEN = 10;
	CRect r;
	char buf[MAX_LEN+1];
                                                 //read rect data from dialog controls
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->GetWindowText(buf, MAX_LEN);
	r.left = atoi(buf);
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->GetWindowText(buf, MAX_LEN);
	r.top = atoi(buf);
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->GetWindowText(buf, MAX_LEN);
	r.right = atoi(buf);
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->GetWindowText(buf, MAX_LEN);
	r.bottom = atoi(buf);
	r.NormalizeRect();

	return(r);
}

void CEditTrialDisplayDialog::ReadDCTimes(unsigned int *del, unsigned int *dur, unsigned int *del2, unsigned int *dur2)
{
	const int MAX_LEN = 10;
	char buf[MAX_LEN+1];

	GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY)->GetWindowText(buf, MAX_LEN);
	*del = static_cast<unsigned int>(atoi(buf));
	GetDlgItem(IDC_PAIRED_REGION_DURATION)->GetWindowText(buf, MAX_LEN);
	*dur = static_cast<unsigned int>(atoi(buf));
	if(!_dd.display3.empty() && !_dd.images3.empty()){
		GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY_2)->GetWindowText(buf, MAX_LEN);
		*del2 = static_cast<unsigned int>(atoi(buf));
		GetDlgItem(IDC_PAIRED_REGION_DURATION_2)->GetWindowText(buf, MAX_LEN);
		*dur2 = static_cast<unsigned int>(atoi(buf));
	}
	else 
		*del2 = 0; *dur2 = 0;
	
	
	return;
}

void CEditTrialDisplayDialog::ReadDCRects(CRect *r1, CRect* r2)
{
	const int MAX_LEN = 10;
	char buf[MAX_LEN+1];

	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->GetWindowText(buf, MAX_LEN);
	r1->left = atoi(buf);
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->GetWindowText(buf, MAX_LEN);
	r1->top = atoi(buf);
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X2)->GetWindowText(buf, MAX_LEN);
	r1->right = atoi(buf);
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y2)->GetWindowText(buf, MAX_LEN);
	r1->bottom = atoi(buf);

	GetDlgItem(IDC_TARGET_X1)->GetWindowText(buf, MAX_LEN);
	r2->left = atoi(buf);
	GetDlgItem(IDC_TARGET_Y1)->GetWindowText(buf, MAX_LEN);
	r2->top = atoi(buf);
	GetDlgItem(IDC_TARGET_X2)->GetWindowText(buf, MAX_LEN);
	r2->right = atoi(buf);
	GetDlgItem(IDC_TARGET_Y2)->GetWindowText(buf, MAX_LEN);
	r2->bottom = atoi(buf);

	r1->NormalizeRect();
	r2->NormalizeRect();

	return;
}

CPoint CEditTrialDisplayDialog::ReadPreviewMetrics()
{
	const int MAX_LEN = 10;
	char buf[MAX_LEN+1];

	CPoint p;
	GetDlgItem(IDC_EDIT_MW_PREVIEW_WORD_INDEX)->GetWindowText(buf, MAX_LEN);
	p.x = atoi(buf);
	GetDlgItem(IDC_EDIT_MW_PREVIEW_DURATION)->GetWindowText(buf, MAX_LEN);
	p.y = atoi(buf);

	return(p);
}


void CEditTrialDisplayDialog::UpdateTrialWnd()
{
	CTrialWnd &wnd = *((CTrialWnd*)GetParent());

	if(_dd.isTextDisplay) {
		wnd.DrawBackground("");
		/*if(_useStimulus1)
			wnd.DrawText(_dd.display1, _tf, _dd, false);
		else 
			wnd.DrawText(_dd.display2, _tf, _dd, true);*/
		if(_useStimulus == 1)
			wnd.DrawText(_dd.display1, _tf, _dd, 1, 100);
		else if(_useStimulus ==2 )
			wnd.DrawText(_dd.display2, _tf, _dd, 2, 100);
		else if(_useStimulus ==3 )
			wnd.DrawText(_dd.display3, _tf, _dd, 3, 100);
	}
	else if(_useStimulus1)
		wnd.DrawBackground(_dd.display1);
	else 
		wnd.DrawBackground(_dd.display2);

	if(_allowGCRect && !_dd.gcRect.IsRectEmpty())
		wnd.DrawGCRect(_dd.gcRect);

	if(_dd.isFixedDisplay == false) {
		CListBox &lb = *((CListBox*)GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS));
		CRect r,r1,r2;
		unsigned int idx = lb.GetCurSel();
		if(!_dd.isPairedRegionChange && !_dd.isTimedPairedRegionChange){
			r = ReadDCRect();
			for(unsigned int i=0; i < _dd.region.size(); ++i)
				wnd.DrawDCRect(_dd.region[i], false);
			wnd.DrawDCRect(r, true);
		}
		else {
			ReadDCRects(&r1,&r2);
			if (_dd.isTimedPairedRegionChange){
				for( unsigned int i=0; i< _dd.timedRectPairs.size(); ++i){
					wnd.DrawDCRect(_dd.timedRectPairs.at(i).r1, false);
					wnd.DrawDCRect(_dd.timedRectPairs.at(i).r2data.region, false);
				}
			}
			else{
				for(unsigned int i=0; i < _dd.regionPairs.size(); ++i){
					wnd.DrawDCRect(_dd.regionPairs[i].region1, false);
					wnd.DrawDCRect(_dd.regionPairs[i].region2, false);
				}
			}
			wnd.DrawDCRect(r1, true);
			wnd.DrawDCRect(r2, true);
		}
	}

	wnd.Invalidate(FALSE);
}

void CEditTrialDisplayDialog::OnBnClickedOk()
{
	// if MW 
	// check disp 1 matches disp 2
	// save # words to script

	// TODO: Add your control notification handler code here
	OnOK();
}

void CEditTrialDisplayDialog::OnBnClickedStaticTrialDisplayGcrect()
{
	// TODO: Add your control notification handler code here
}



void CEditTrialDisplayDialog::OnStnClickedStaticTrialDisplayGcx3()
{
	// TODO: Add your control notification handler code here
}


void CEditTrialDisplayDialog::OnBnClickedRadioTrialMovWindTriggerChar()
{
	_dd.movWindowType.useWordTriggers = FALSE;
}

void CEditTrialDisplayDialog::OnBnClickedRadioTrialMovWindTriggerWord()
{
	_dd.movWindowType.useWordTriggers = TRUE;
}

void CEditTrialDisplayDialog::OnBnClickedRadioTrialMovWindTargetChar()
{
	_dd.movWindowType.useWordTargets = FALSE;
}

void CEditTrialDisplayDialog::OnBnClickedRadioTrialMovWindTargetWord()
{
	_dd.movWindowType.useWordTargets = TRUE;
}

void CEditTrialDisplayDialog::OnEnChangeTrialMovWindSize()
{
	const int MAX_LEN = 10;
	int len;
	char buf[MAX_LEN+1];

	len = GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_TRAIL_SIZE)->GetWindowText(buf, MAX_LEN);
	if(len == 0) return;
	else _dd.windowSize.x = atoi(buf);

	len = GetDlgItem(IDC_EDIT_TRIAL_MOV_WIND_LEAD_SIZE)->GetWindowText(buf, MAX_LEN);
	if(len == 0) return;
	_dd.windowSize.y = atoi(buf);

	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::OnBnClickedStaticTrialDisplayChangeRegions()
{
	// TODO: Add your control notification handler code here
}


void CEditTrialDisplayDialog::OnBnClickedDcDistinctRegions()
{
	if(IsDlgButtonChecked(IDC_DC_DISTINCT_REGIONS)){	// then it was previously unchecked
		if(!_dd.isPairedRegionChange && !_dd.isTimedPairedRegionChange){
			_dd.isPairedRegionChange = true;

			_dd.region.clear();
			CListBox &lb = *((CListBox*)GetDlgItem(IDC_LIST_TRIAL_DISPLAY_CHANGE_REGIONS));
			int i = lb.GetCount();
			for (int j = 0; j<i; j++){
				lb.DeleteString(j);
			}
			OnBnClickedDisplayChangeRegion();
			UpdateTrialWnd();

			GetDlgItem(IDC_CHECK_SET_DELAY)->EnableWindow(TRUE);

		}
	}
	else{
		_dd.isPairedRegionChange = false;
		_dd.isTimedPairedRegionChange = false;
		GetDlgItem(IDC_CHECK_SET_DELAY)->EnableWindow(FALSE);
	}
}



void CEditTrialDisplayDialog::OnStnClickedTargetX1Caption()
{
	// TODO: Add your control notification handler code here
}

void CEditTrialDisplayDialog::OnEnChangeEditMwPreviewWordIndex()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}

void CEditTrialDisplayDialog::OnLbnSelchangePreviewWords()
{
	CListBox &lb = *((CListBox *)GetDlgItem(IDC_LIST_TRIAL_MW_PREVIEW_WORDS));
	int idx = lb.GetCurSel();

	char buf[11];
	
	if(idx == LB_ERR)
		return;

	int wIdx = _dd.toggleWords.at(idx).wordIndex;
	int wDur = _dd.toggleWords.at(idx).duration;

	GetDlgItem(IDC_EDIT_MW_PREVIEW_WORD_INDEX)->SetWindowText(_itoa(wIdx, buf, 10));
	GetDlgItem(IDC_EDIT_MW_PREVIEW_DURATION)->SetWindowText(_itoa(wDur, buf, 10));

	return;

}







void CEditTrialDisplayDialog::OnEnChangePairedRegionOnsetDelay()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}

void CEditTrialDisplayDialog::makeRegionsTimed(){
	RectPair rp;
	for(unsigned int i= 0; i<_dd.regionPairs.size(); i++){
		rp = _dd.regionPairs.at(i);
		_dd.timedRectPairs.push_back(TimedRectPair(rp.region1, ToggleData(rp.region2,0,0)));
	}
	_dd.regionPairs.clear();
	_dd.isTimedPairedRegionChange = true;
	_dd.isPairedRegionChange = false;
	return;
}

void CEditTrialDisplayDialog::makeRegionsUntimed(){
	TimedRectPair trp;
	for(unsigned int i = 0; i<_dd.timedRectPairs.size(); i++){
		trp = _dd.timedRectPairs.at(i);
		_dd.regionPairs.push_back(RectPair(trp.r1, trp.r2data.region));
	}
	_dd.timedRectPairs.clear();
	_dd.isPairedRegionChange = true;
	_dd.isTimedPairedRegionChange = false;
}

void CEditTrialDisplayDialog::OnBnClickedCheckSetDelay()
{
	if(IsDlgButtonChecked(IDC_CHECK_SET_DELAY))	{					// then it was previously unchecked
		GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY)->EnableWindow(TRUE);
		//GetDlgItem(IDC_STATIC_ONSET_DELAY)->EnableWindow(TRUE);
		GetDlgItem(IDC_PAIRED_REGION_DURATION)->EnableWindow(TRUE);
		//GetDlgItem(IDC_STATIC_REVERT_DELAY)->EnableWindow(TRUE);
		// move any regions in untimed paired to timed paired w/ zeros 
		if(!_dd.display3.empty() || !_dd.images3.empty()){
			GetDlgItem(IDC_PAIRED_REGION_DURATION_2)->EnableWindow(TRUE);
			GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY_2)->EnableWindow(TRUE);
		}
		makeRegionsTimed();
	}

	else{														// then it was previously checked, just unchecked it
		GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY)->EnableWindow(FALSE);
		GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY)->SetWindowText("");
		//GetDlgItem(IDC_STATIC_ONSET_DELAY)->EnableWindow(FALSE);
		GetDlgItem(IDC_PAIRED_REGION_DURATION)->EnableWindow(FALSE);
		GetDlgItem(IDC_PAIRED_REGION_DURATION)->SetWindowText("");
		//GetDlgItem(IDC_STATIC_REVERT_DELAY)->EnableWindow(FALSE);
		// move any regions in timed-paired to untimed-paired
		makeRegionsUntimed();
	}
	if(!_dd.isTimedPairedRegionChange){

	}
}

void CEditTrialDisplayDialog::OnBnClickedButtonDisplayTrialAddImages1()
{	
	CEditTrialImagesDialog dlg2(this,_dd, 1);
	if(	dlg2.DoModal() == IDOK){
		_dd = dlg2.getDisplayData();
		UpdateTrialWnd();
	}
	//dlg2.Initialize(GetTrial().getDisplay(), false);
	
	/* doModal == false 
	CEditTrialImagesDialog *dlg = new CEditTrialImagesDialog();
	dlg->Create(IDD_EDIT_TRIAL_ADD_IMAGES_DIALOG, this->GetParent());
	dlg->Initialize(_dd, false);
	*/

/*	CTrialWnd *tWnd = new CTrialWnd();
	CBrush brush(RGB(0, 0, 0));
	CString wndClass = AfxRegisterWndClass(CS_BYTEALIGNWINDOW | CS_OWNDC | CS_HREDRAW | CS_VREDRAW, 0, brush, 0);
	int height = ::GetSystemMetrics(SM_CYSCREEN);
	int width = ::GetSystemMetrics(SM_CXSCREEN);

	tWnd->SetWindowColors(_expt->backgroundColor(), _expt->foregroundColor());
	tWnd->CreateEx(WS_EX_APPWINDOW, wndClass, "Trial Preview", WS_VISIBLE | WS_POPUPWINDOW,
				  0, 0, width, height, this->GetSafeHwnd(), NULL);

	//User must be able to interact with both EditTrialDisplayDialog and TrialWnd
	//at same time, so cannot use modal dialog. When EditTrialDisplayDialog::OnOK
	//is clicked, EditTrialDialog::OnEditDisplayDialogOkay is called so data can be
	//passed back.  A bit convoluted, but it works.

	CEditTrialDisplayDialog *dlg = new CEditTrialDisplayDialog();
	TrialType *type = _expt->getTrialType(_typeName);

	dlg->Create(IDD_EDIT_TRIAL_DISPLAY_DIALOG, tWnd);
	if(type->doGazeContingentStart())
		dlg->EnableGCRect();
	if(type->textFormat() != 0)
		dlg->EnableText(*(type->textFormat()));
	dlg->Initialize(_display);

	_editDisplayDlg = dlg;
	// TODO: Add your control notification handler code here*/
}

void CEditTrialDisplayDialog::OnBnClickedButtonDisplayTrialAddImages2()
{
	CEditTrialImagesDialog dlg2(this,_dd, 2);
	if(	dlg2.DoModal() == IDOK){
		_dd = dlg2.getDisplayData();
		UpdateTrialWnd();
	}
	// TODO: Add your control notification handler code here
}

void CEditTrialDisplayDialog::OnBnClickedCancel()
{
	// TODO: Add your control notification handler code here
	OnCancel();
}

void CEditTrialDisplayDialog::OnBnClickedRadioTrialDisplayPreview3()
{
	_useStimulus1 = false;
	_useStimulus = 3;
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3)->EnableWindow(TRUE);
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3)->SetWindowText(_dd.display3.c_str());
	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::OnEnChangeEditTrialDisplayStimulus3()
{
	const int MAX_LEN = 5000;
	char buf[MAX_LEN+1];

	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3)->GetWindowText(buf, MAX_LEN);
	_dd.display3 = FormatForDisplay(buf);
	if(!_dd.display3.empty()){
		EnableThree(TRUE);
	}
	else
		EnableThree(TRUE);
	UpdateTrialWnd();
}

void CEditTrialDisplayDialog::EnableThree(BOOL enable){
	GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_STIMULUS3)->EnableWindow(enable);
	if(IsDlgButtonChecked(IDC_CHECK_SET_DELAY)){
		GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY_2)->EnableWindow(enable);
		GetDlgItem(IDC_PAIRED_REGION_DURATION_2)->EnableWindow(enable);
		if(_dd.display3.empty() && _dd.images3.empty()){
			GetDlgItem(IDC_PAIRED_REGION_ONSET_DELAY_2)->EnableWindow(FALSE);
			GetDlgItem(IDC_PAIRED_REGION_DURATION_2)->EnableWindow(FALSE);
			ZeroChangeTwo();
		}

	}

}

void CEditTrialDisplayDialog::EnableRadio3(bool enable){
	GetDlgItem(IDC_RADIO_TRIAL_DISPLAY_PREVIEW3)->EnableWindow(enable);
}
void CEditTrialDisplayDialog::OnBnClickedButtonDisplayTrialAddImages3()
{
	CEditTrialImagesDialog dlg3(this,_dd, 3);
	if(	dlg3.DoModal() == IDOK){
		_dd = dlg3.getDisplayData();
		UpdateTrialWnd();
	}
	// TODO: Add your control notification handler code here
}

void CEditTrialDisplayDialog::ZeroChangeTwo(){
	if (_dd.isTimedPairedRegionChange){
		for(int i = 0; i< _dd.timedRectPairs.size(); i++){
			_dd.timedRectPairs.at(i).r2data.delay2 = 0;
			_dd.timedRectPairs.at(i).r2data.duration2 = 0;
		}
	}
}
