/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once

#include "DisplayData.h"

class CEditTrialDisplayDialog : public CDialog
{
	DECLARE_DYNAMIC(CEditTrialDisplayDialog)

public:
	enum { IDD = IDD_EDIT_TRIAL_DISPLAY_DIALOG };

	CEditTrialDisplayDialog(CWnd* pParent = NULL);   // standard constructor
	virtual ~CEditTrialDisplayDialog();

	const DisplayData &GetDisplayData() const;

	afx_msg void OnBnClickedAddRegion();
	afx_msg void OnBnClickedBrowseDisplay1();
	afx_msg void OnBnClickedBrowseDisplay2();
	afx_msg void OnBnClickedDisplayChangeNone();
	afx_msg void OnBnClickedDisplayChangeRegion();
	afx_msg void OnBnClickedDisplayPreview1();
	afx_msg void OnBnClickedDisplayPreview2();
	afx_msg void OnBnClickedDisplayTypeImage();
	afx_msg void OnBnClickedDisplayTypeText();
	afx_msg void OnBnClickedRemoveRegion();
	afx_msg void OnBnClickedUpdateRegion();
	afx_msg void OnEnChangeDCRect();
	afx_msg void OnEnChangeGCRect();
	afx_msg void OnEnChangeStimulus1();
	afx_msg void OnEnChangeStimulus2();
	afx_msg void OnLbnSelchangeRegion();

	void EnableGCRect();
	void EnableText(const TextFormat &tf);
	void Initialize(const DisplayData &data);

protected:
	CToolTipCtrl *m_ToolTip;
	static std::string FormatForDisplay(const std::string &str);
	static std::string FormatForEditBox(const std::string &str);

	CRect ReadDCRect();

	void ReadDCRects(CRect *r1, CRect *r2);
	void ReadDCTimes(unsigned int *del, unsigned int *dur, unsigned int *del2, unsigned int *dur2);

	void makeRegionsTimed();
	void makeRegionsUntimed();

	CPoint ReadPreviewMetrics();

	void resetDCdata();

	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	virtual void OnCancel();
	virtual void OnOK();
	virtual void PostNcDestroy();

	void UpdateTrialWnd();

	DECLARE_MESSAGE_MAP();

private:
	DisplayData _dd;
	TextFormat  _tf;
	bool        _allowGCRect;
	bool        _useStimulus1;
	int			_useStimulus;
	void		EnableThree(BOOL enable);
	void		EnableRadio3(bool enable);
	void		ZeroChangeTwo();
public:
	afx_msg void OnBnClickedOk();
	afx_msg void OnBnClickedDisplayChangeMovingWindow();
	afx_msg void OnBnClickedDisplayChangeMovingWindowPreview();
	afx_msg void OnBnClickedStaticTrialDisplayGcrect();
	afx_msg void OnStnClickedStaticTrialDisplayGcx3();
	afx_msg void OnBnClickedRadioTrialMovWindTriggerChar();
	afx_msg void OnBnClickedRadioTrialMovWindTriggerWord();
	afx_msg void OnBnClickedRadioTrialMovWindTargetChar();
	afx_msg void OnBnClickedRadioTrialMovWindTargetWord();
	afx_msg void OnEnChangeTrialMovWindSize();
	afx_msg void OnBnClickedStaticTrialDisplayChangeRegions();
	afx_msg void OnBnClickedDcDistinctRegions();
	afx_msg void OnStnClickedTargetX1Caption();
	afx_msg void OnEnChangeEditMwPreviewWordIndex();
	afx_msg void OnLbnSelchangePreviewWords();
	afx_msg void OnBnClickedWindowPreviewAdd();
	afx_msg void OnBnClickedWindowPreviewUpdate();
	afx_msg void OnBnClickedWindowPreviewRemove();
	afx_msg void OnEnChangePairedRegionOnsetDelay();
	afx_msg void OnBnClickedCheckSetDelay();
	afx_msg void OnBnClickedButtonDisplayTrialAddImages1();
	afx_msg void OnBnClickedButtonDisplayTrialAddImages2();
	afx_msg void OnBnClickedCancel();
	afx_msg void OnBnClickedRadioTrialDisplayPreview3();
	afx_msg void OnEnChangeEditTrialDisplayStimulus3();
	afx_msg void OnBnClickedButtonDisplayTrialAddImages3();
	afx_msg void OnBnClickedDisplayChangeScotoma();
	virtual BOOL OnInitDialog();
	BOOL PreTranslateMessage(MSG* pMsg);
};
