/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "EyeTrack.h"
#include "EditTrialPropertiesDialog.h"
#include ".\edittrialpropertiesdialog.h"

IMPLEMENT_DYNAMIC(CEditTrialPropertiesDialog, CDialog)
CEditTrialPropertiesDialog::CEditTrialPropertiesDialog(CWnd* pParent /*=NULL*/)
	: CDialog(CEditTrialPropertiesDialog::IDD, pParent), 
	  _answer(Tokenizer::NO_BUTTON), _identifier(), _maxTime(0), _isQuestion(false)
{}

CEditTrialPropertiesDialog::~CEditTrialPropertiesDialog()
{}

void CEditTrialPropertiesDialog::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_COMBO_TRIAL_PROPERTY_CONDITION_TYPE, m_comboTypeNames);
	DDX_Control(pDX, IDC_COMBO_TRIAL_PROPERTY_ANSWER, m_comboAnswer);
	DDX_Control(pDX, IDC_COMBO_TRIAL_PROPERTY_ANSWER2, m_comboAnswer2);
}

BEGIN_MESSAGE_MAP(CEditTrialPropertiesDialog, CDialog)
	ON_BN_CLICKED(IDC_CHECK_TRIAL_PROPERTY_DEPENDENT, OnBnClickedDependent)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_PROPERTY_EXPERIMENTAL, OnChangeTrialType)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_PROPERTY_FILLER, OnChangeTrialType)
	ON_BN_CLICKED(IDC_RADIO_TRIAL_PROPERTY_PRACTICE, OnChangeTrialType)
	ON_EN_KILLFOCUS(IDC_EDIT_TRIAL_PROPERTY_CONDITION, OnEnKillfocusEditCondition)
	ON_EN_KILLFOCUS(IDC_EDIT_TRIAL_PROPERTY_ITEM, OnEnKillfocusEditItem)
	ON_EN_KILLFOCUS(IDC_EDIT_TRIAL_PROPERTY_INDEX, OnEnKillfocusEditIndex)
	ON_EN_KILLFOCUS(IDC_EDIT_TRIAL_PROPERTY_TIME, OnEnKillfocusEditTime)
	ON_CBN_SELCHANGE(IDC_COMBO_TRIAL_PROPERTY_CONDITION_TYPE, OnCbnSelchangeConditionType)
	ON_BN_CLICKED(IDC_CHECK_TRIAL_PROPERTY_QUESTION, OnBnClickedQuestion)
	ON_CBN_SELCHANGE(IDC_COMBO_TRIAL_PROPERTY_ANSWER, OnCbnSelchangeAnswer)
	ON_CBN_SELCHANGE(IDC_COMBO_TRIAL_PROPERTY_ANSWER2, OnCbnSelchangeAnswer)
END_MESSAGE_MAP()

int &CEditTrialPropertiesDialog::maxDisplayTime()
{
	return(_maxTime);
}

void CEditTrialPropertiesDialog::OnBnClickedDependent()
{
	CButton *button = (CButton*)GetDlgItem(IDC_CHECK_TRIAL_PROPERTY_DEPENDENT);
	CWnd *indexLabel = GetDlgItem(IDC_STATIC_TRIAL_PROPERTY_INDEX);
	CWnd *indexEditBox = GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_INDEX);

	if(button->GetCheck() == BST_CHECKED) { //dependent checked, enable index box
		indexEditBox->EnableWindow();
		indexEditBox->SetFocus();
		indexLabel->EnableWindow();
	}
	else {                                  //dependent not checked, disable index box
		indexEditBox->SetWindowText("0");
		indexEditBox->EnableWindow(FALSE);
		indexLabel->EnableWindow(FALSE);
	}
}

void CEditTrialPropertiesDialog::OnBnClickedQuestion()
{
	CButton *button = (CButton*)GetDlgItem(IDC_CHECK_TRIAL_PROPERTY_QUESTION);
	CWnd *answerLabel = GetDlgItem(IDC_STATIC_TRIAL_PROPERTY_ANSWER);
	CWnd *fiveButtonAnswerLabel = GetDlgItem(IDC_STATIC_TRIAL_PROPERTY_ANSWER2);

	if(button->GetCheck() == BST_CHECKED) { //question checked, enable answer box
		if(_controlType == Tokenizer::SEVEN_BUTTON){
			m_comboAnswer.EnableWindow();
			m_comboAnswer.SetFocus();
			answerLabel->EnableWindow();
		}
		else{
			m_comboAnswer2.EnableWindow();
			m_comboAnswer2.SetFocus();
			fiveButtonAnswerLabel->EnableWindow();
		}
		_isQuestion = true;
		
	}
	else {                                  //dependent not checked, disable answer box
		m_comboAnswer.SetCurSel(-1);
		m_comboAnswer.EnableWindow(FALSE);
		answerLabel->EnableWindow(FALSE);
		_isQuestion = false;
	}
}

void CEditTrialPropertiesDialog::OnCbnSelchangeAnswer()
{
	int select = m_comboAnswer.GetCurSel();

	if(select != CB_ERR)                    //good selection, set the answer
		_answer = static_cast<Tokenizer::ButtonType>(select+1);
	else _answer = Tokenizer::NO_BUTTON;    //otherwise clear the selection
}

void CEditTrialPropertiesDialog::OnCbnSelchangeConditionType()
{
	int select = m_comboTypeNames.GetCurSel();

	if(select != CB_ERR) {                  //good selection, read the type name
		char buf[100];

		m_comboTypeNames.GetLBText(select, buf);
		_typeName = buf;
	}
	else _typeName.clear();                 //otherwise clear the name
}

void CEditTrialPropertiesDialog::OnChangeTrialType()
{
	TrialId::Type type;
	unsigned short cond = _identifier.condition();
	unsigned short item = _identifier.item();
	unsigned short index = _identifier.index();
	int selection = GetCheckedRadioButton(IDC_RADIO_TRIAL_PROPERTY_EXPERIMENTAL, 
										  IDC_RADIO_TRIAL_PROPERTY_PRACTICE);

	switch(selection) {
		case IDC_RADIO_TRIAL_PROPERTY_EXPERIMENTAL: 
			type = TrialId::EXPERIMENTAL;
			break;
		case IDC_RADIO_TRIAL_PROPERTY_FILLER:
			type = TrialId::FILLER;
			break;
		case IDC_RADIO_TRIAL_PROPERTY_PRACTICE:
			type = TrialId::PRACTICE;
			break;
	}

	_identifier = TrialId(type, cond, item, index, false);
}

void CEditTrialPropertiesDialog::OnEnKillfocusEditCondition()
{
	TrialId::Type type;
	unsigned short cond;
	unsigned short item = _identifier.item();
	unsigned short index = _identifier.index();
	char buf[10];

	GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_CONDITION)->GetWindowText(buf, 10);
	cond = atoi(buf);

	if(_identifier.isExperimental())
		type = TrialId::EXPERIMENTAL;
	else if(_identifier.isFiller())
		type = TrialId::FILLER;
	else type = TrialId::PRACTICE;
                                            //update the trial's identifier
	_identifier = TrialId(type, cond, item, index, false);
}

void CEditTrialPropertiesDialog::OnEnKillfocusEditIndex()
{
	TrialId::Type type;
	unsigned short cond = _identifier.condition();
	unsigned short item = _identifier.item();
	unsigned short index;
	char buf[10];

	GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_INDEX)->GetWindowText(buf, 10);
	index = atoi(buf);

	if(_identifier.isExperimental())
		type = TrialId::EXPERIMENTAL;
	else if(_identifier.isFiller())
		type = TrialId::FILLER;
	else type = TrialId::PRACTICE;
                                            //update trial's identifier
	_identifier = TrialId(type, cond, item, index, false);
}

void CEditTrialPropertiesDialog::OnEnKillfocusEditItem()
{
	TrialId::Type type;
	unsigned short cond = _identifier.condition();
	unsigned short item;
	unsigned short index = _identifier.index();
	char buf[10];

	GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_ITEM)->GetWindowText(buf, 10);
	item = atoi(buf);

	if(_identifier.isExperimental())
		type = TrialId::EXPERIMENTAL;
	else if(_identifier.isFiller())
		type = TrialId::FILLER;
	else type = TrialId::PRACTICE;
                                            //update trial identifier
	_identifier = TrialId(type, cond, item, index, false);
}

void CEditTrialPropertiesDialog::OnEnKillfocusEditTime()
{
	char buf[20];

	GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_TIME)->GetWindowText(buf, 20);
	_maxTime = atoi(buf);                   //update maximum display time
}

BOOL CEditTrialPropertiesDialog::OnInitDialog()
{
	char buf[20];

	CDialog::OnInitDialog();
                                            //if have a valid identifier...
	if(_identifier.condition() > 0 && _identifier.item() > 0) {
		GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_CONDITION)->SetWindowText(_itoa(_identifier.condition(), buf, 10));
		GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_ITEM)->SetWindowText(_itoa(_identifier.item(), buf, 10));
		GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_INDEX)->SetWindowText(_itoa(_identifier.index(), buf, 10));

		if(_identifier.isExperimental())    //check the correct type button
			CheckRadioButton(IDC_RADIO_TRIAL_PROPERTY_EXPERIMENTAL, 
							 IDC_RADIO_TRIAL_PROPERTY_PRACTICE,
							 IDC_RADIO_TRIAL_PROPERTY_EXPERIMENTAL);
		else if(_identifier.isFiller())
			CheckRadioButton(IDC_RADIO_TRIAL_PROPERTY_EXPERIMENTAL, 
							 IDC_RADIO_TRIAL_PROPERTY_PRACTICE,
							 IDC_RADIO_TRIAL_PROPERTY_FILLER);
		else CheckRadioButton(IDC_RADIO_TRIAL_PROPERTY_EXPERIMENTAL, 
			 				  IDC_RADIO_TRIAL_PROPERTY_PRACTICE,
							  IDC_RADIO_TRIAL_PROPERTY_PRACTICE);

		if(_identifier.isDependent()) {     //check dependent button if needed
			CheckDlgButton(IDC_CHECK_TRIAL_PROPERTY_DEPENDENT, BST_CHECKED);
			GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_INDEX)->EnableWindow();
			GetDlgItem(IDC_STATIC_TRIAL_PROPERTY_INDEX)->EnableWindow();
		}
	}
	else {                                  //otherwise just set 0 values
		GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_CONDITION)->SetWindowText("0");
		GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_ITEM)->SetWindowText("0");
		GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_INDEX)->SetWindowText("0");
	}

	if(_answer != Tokenizer::NO_BUTTON) {   //initialize the question answer
		CheckDlgButton(IDC_CHECK_TRIAL_PROPERTY_QUESTION, BST_CHECKED);

		if(_controlType == Tokenizer::FIVE_BUTTON){
			GetDlgItem(IDC_STATIC_TRIAL_PROPERTY_ANSWER)->EnableWindow();
			m_comboAnswer2.EnableWindow();
			m_comboAnswer2.SetCurSel(static_cast<int>(_answer)-1);
		}
		else {
			GetDlgItem(IDC_STATIC_TRIAL_PROPERTY_ANSWER2)->EnableWindow();
			m_comboAnswer.EnableWindow();
			m_comboAnswer.SetCurSel(static_cast<int>(_answer)-1);
		}
			_isQuestion = true;
	}
                                            //set maximum display time
	GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_TIME)->SetWindowText(_itoa(_maxTime, buf, 10));
                                            //load type names into combo box
	for(StringIter i = _typeNameList.begin(); i != _typeNameList.end(); ++i)
		m_comboTypeNames.AddString(i->c_str());

	if(!_typeName.empty())                  //selcet the proper type name (if available)
		m_comboTypeNames.SelectString(0, _typeName.c_str());

	return TRUE;
}

void CEditTrialPropertiesDialog::OnOK()
{
	if(_identifier.condition() == 0) {      //make sure condition number non-zero
		MessageBox("Condition number must be greater than zero.", "Property Error");
		GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_CONDITION)->SetFocus();
		return;
	}

	if(_identifier.item() == 0) {           //make sure item number non-zero
		MessageBox("Item number must be greater than zero.", "Property Error");
		GetDlgItem(IDC_EDIT_TRIAL_PROPERTY_ITEM)->SetFocus();
		return;
	}
                                            //make sure dependents have non-zero index
	if(_identifier.isDependent() && _identifier.index() == 0) {
		MessageBox("Dependent trials must have non-zero index values.", "Property Error");
		return;
	}

	if(_typeName.empty()) {                 //make sure a trial type is selected
		MessageBox("Select a condition type.", "Property Error");
		return;
	}

	if(_isQuestion && _answer == Tokenizer::NO_BUTTON) {
		MessageBox("Select a desired answer for the question.", "Property Error");
		return;
	}

	CDialog::OnOK();
}

Tokenizer::ButtonType &CEditTrialPropertiesDialog::questionAnswer()
{
	return(_answer);
}

Tokenizer::ControlType &CEditTrialPropertiesDialog::controlType()
{
	return(_controlType);
}

TrialId &CEditTrialPropertiesDialog::trialId()
{
	return(_identifier);
}

std::string &CEditTrialPropertiesDialog::typeName()
{
	return(_typeName);
}

void CEditTrialPropertiesDialog::SetTrialTypeNames(const StringList &list)
{
	_typeNameList = list;
}

void CEditTrialPropertiesDialog::OnCbnSelchangeComboTrialPropertyAnswer2()
{
	// TODO: Add your control notification handler code here
}
