/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "EyeTrack.h"
#include "EditTrialTypeDlg.h"
#include "AudioCaptureDlg.h"
#include "textdlg.h"
#include "w32_exptsppt2.h"

#include <stdlib.h>
#include <sstream>
#include ".\edittrialtypedlg.h"

IMPLEMENT_DYNAMIC(CEditTrialTypeDlg, CDialog)

CEditTrialTypeDlg::CEditTrialTypeDlg(CWnd* pParent)
	: CDialog(CEditTrialTypeDlg::IDD, pParent), 
	  _output(Tokenizer::NO_OUTPUT), _trigger(Tokenizer::NO_TRIGGER),
	  _cursor(0), _dcDelay(0), _stimDelay(0), _nButtons(0), _enableAudioCapture(false),
	  _enableText(false), _revertDisplay(false), _controlType(Tokenizer::UNKNOWN_CONTROLLER)
{}

CEditTrialTypeDlg::~CEditTrialTypeDlg()
{}

void CEditTrialTypeDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_LIST_TRIALTYPE_BUTTONS, mListButtons);
}

BEGIN_MESSAGE_MAP(CEditTrialTypeDlg, CDialog)
	ON_BN_CLICKED(IDC_BUTTON_TRIALTYPE_CHANGE_AUDIO, OnBnClickedChangeAudio)
	ON_BN_CLICKED(IDC_BUTTON_TRIALTYPE_CHANGE_TEXT, OnBnClickedChangeText)
	ON_BN_CLICKED(IDC_CHECK_TRIALTYPE_ENABLE_AUDIO, OnBnClickedEnableAudio)
	ON_BN_CLICKED(IDC_CHECK_TRIALTYPE_ENABLE_TEXT, OnBnClickedEnableText)
	ON_BN_CLICKED(IDC_RADIO_TRIALTYPE_OUTPUT_NOSTREAM, OnBnClickedOutputNostream)
	ON_BN_CLICKED(IDC_RADIO_TRIALTYPE_OUTPUT_STREAM, OnBnClickedOutputStream)
	ON_BN_CLICKED(IDC_RADIO_TRIALTYPE_TRIGGER_DRIFTANDGAZE, OnBnClickedTriggerDriftandgaze)
	ON_BN_CLICKED(IDC_RADIO_TRIALTYPE_TRIGGER_DRIFTCORRECT, OnBnClickedTriggerDriftcorrect)
	ON_BN_CLICKED(IDC_RADIO_TRIALTYPE_TRIGGER_GAZE, OnBnClickedTriggerGaze)
	ON_BN_CLICKED(IDC_RADIO_TRIALTYPE_TRIGGER_NOGAZE, OnBnClickedTriggerNogaze)
	ON_BN_CLICKED(IDC_CHECK_TRIALTYPE_REVERT_DISPLAY_CHANGE, OnBnClickedRevertDisplayChange)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_EN_CHANGE(IDC_EDIT_TRIALTYPE_ID, OnEnChangeEditTrialtypeId)
	ON_LBN_SELCHANGE(IDC_LIST_TRIALTYPE_BUTTONS, OnLbnSelchangeListTrialtypeButtons)
	ON_BN_CLICKED(IDC_RADIO_5_BUTTON_CONTROL, OnBnClickedRadio5ButtonControl)
	ON_BN_CLICKED(IDC_RADIO_7_BUTTON_CONTROL, OnBnClickedRadio7ButtonControl)
	ON_EN_CHANGE(IDC_EDIT_TRIALTYPE_DC_DELAY, OnEnChangeEditTrialtypeDcDelay)
END_MESSAGE_MAP()

TrialType CEditTrialTypeDlg::GetTrialType()
{
	TrialType t(_identifier, _button, _output, _trigger, 
		        _cursor, _dcDelay, _stimDelay, _nButtons, _revertDisplay, _controlType);

	if(_enableAudioCapture)
		t.setAudioFormat(_audioFmt);
	if(_enableText)
		t.setTextFormat(_textFmt);
	return(t);
}

void CEditTrialTypeDlg::OnBnClickedChangeAudio()
{
	CAudioCaptureDlg dlg(this);

	if(_audioFmt.path.empty() == false)          //initialize the audio dialog
		dlg.SetCaptureData(_audioFmt);

	if(dlg.DoModal() == IDOK)                    //retrieve data from audio dialog
		_audioFmt = dlg.GetCaptureData();
}

void CEditTrialTypeDlg::OnBnClickedChangeText()
{
	LOGFONT lf, *lfPtr = NULL;

	if(_textFmt.fontName.empty() == false) {     //select the the font
		CClientDC dc(this);

		_textFmt.fontName.copy(lf.lfFaceName, 32);
		lf.lfHeight = -MulDiv(_textFmt.fontHt, dc.GetDeviceCaps(LOGPIXELSY), 72);
		lf.lfWeight = 400;
		lfPtr = &lf;
	}

	CTextDlg textDlg(lfPtr, CF_SCREENFONTS | CF_NOSCRIPTSEL | CF_NOVERTFONTS, NULL, this);

	textDlg.SetTextFormat(_textFmt);             //initialize text dialog
	if(textDlg.DoModal() == IDOK)                //retrieve text data
	_textFmt = textDlg.GetTextFormat();

#ifdef newMode
	CTextDlg textDlg2(lfPtr, CF_SCREENFONTS | CF_NOSCRIPTSEL | CF_NOVERTFONTS, NULL, this);

	textDlg2.SetTextFormat(_textFmt);             //initialize text dialog
	if(textDlg2.DoModal() == IDOK)                //retrieve text data
	_textFmt2 = textDlg2.GetTextFormat();
#endif
}

void CEditTrialTypeDlg::OnBnClickedEnableAudio()
{
	CButton *btn = (CButton*)(GetDlgItem(IDC_CHECK_TRIALTYPE_ENABLE_AUDIO));
	_enableAudioCapture = (btn->GetCheck() == BST_CHECKED);

	if(_enableAudioCapture)
		GetDlgItem(IDC_BUTTON_TRIALTYPE_CHANGE_AUDIO)->EnableWindow(TRUE);
	else
		GetDlgItem(IDC_BUTTON_TRIALTYPE_CHANGE_AUDIO)->EnableWindow(FALSE);
}

void CEditTrialTypeDlg::OnBnClickedEnableText()
{
	CButton *btn = (CButton*)(GetDlgItem(IDC_CHECK_TRIALTYPE_ENABLE_TEXT));
	_enableText = (btn->GetCheck() == BST_CHECKED);

	if(_enableText)
		GetDlgItem(IDC_BUTTON_TRIALTYPE_CHANGE_TEXT)->EnableWindow(TRUE);
	else
		GetDlgItem(IDC_BUTTON_TRIALTYPE_CHANGE_TEXT)->EnableWindow(FALSE);
}

void CEditTrialTypeDlg::OnBnClickedOutputNostream()
{
	_output = Tokenizer::NOSTREAM;
}

void CEditTrialTypeDlg::OnBnClickedOutputStream()
{
	_output = Tokenizer::STREAM;
}

void CEditTrialTypeDlg::OnBnClickedRevertDisplayChange()
{
	CButton *btn = (CButton*)GetDlgItem(IDC_CHECK_TRIALTYPE_REVERT_DISPLAY_CHANGE);

	if(btn->GetCheck() == BST_CHECKED)
		_revertDisplay = true;
	else _revertDisplay = false;
}

void CEditTrialTypeDlg::OnBnClickedTriggerDriftandgaze()
{
	_trigger = Tokenizer::DRIFTANDGAZE;
	GetDlgItem(IDC_STATIC_STIMULUS_DELAY)->EnableWindow(FALSE);
	GetDlgItem(IDC_EDIT_TRIALTYPE_STIMULUS_DELAY)->EnableWindow(FALSE);
	GetDlgItem(IDC_EDIT_TRIALTYPE_STIMULUS_DELAY)->SetWindowText("");
}

void CEditTrialTypeDlg::OnBnClickedTriggerDriftcorrect()
{
	_trigger = Tokenizer::DRIFTCORRECT;
	GetDlgItem(IDC_STATIC_STIMULUS_DELAY)->EnableWindow(FALSE);
	GetDlgItem(IDC_EDIT_TRIALTYPE_STIMULUS_DELAY)->EnableWindow(FALSE);
	GetDlgItem(IDC_EDIT_TRIALTYPE_STIMULUS_DELAY)->SetWindowText("");
}

void CEditTrialTypeDlg::OnBnClickedTriggerGaze()
{
	_trigger = Tokenizer::GAZE;
	GetDlgItem(IDC_STATIC_STIMULUS_DELAY)->EnableWindow(FALSE);
	GetDlgItem(IDC_EDIT_TRIALTYPE_STIMULUS_DELAY)->EnableWindow(FALSE);
	GetDlgItem(IDC_EDIT_TRIALTYPE_STIMULUS_DELAY)->SetWindowText("");
}

void CEditTrialTypeDlg::OnBnClickedTriggerNogaze()
{
	_trigger = Tokenizer::NOGAZE;
	GetDlgItem(IDC_STATIC_STIMULUS_DELAY)->EnableWindow(TRUE);
	GetDlgItem(IDC_EDIT_TRIALTYPE_STIMULUS_DELAY)->EnableWindow(TRUE);
}

BOOL CEditTrialTypeDlg::OnInitDialog()
{
	char buf[20];

	CDialog::OnInitDialog();

	if(_identifier.empty() == false) {           //if trial type set, make ident read only
		SendDlgItemMessage(IDC_EDIT_TRIALTYPE_ID, EM_SETREADONLY, TRUE, 0);
		GetDlgItem(IDC_EDIT_TRIALTYPE_ID)->SetWindowText(_identifier.c_str());
	}

	if(_enableAudioCapture) {                    //audio is enabled
		CButton *btn = (CButton*)(GetDlgItem(IDC_CHECK_TRIALTYPE_ENABLE_AUDIO));
		btn->SetCheck(BST_CHECKED);
		OnBnClickedEnableAudio();
	}

	if(_enableText) {                            //inline text is enabled
		CButton *btn = (CButton*)(GetDlgItem(IDC_CHECK_TRIALTYPE_ENABLE_TEXT));
		btn->SetCheck(BST_CHECKED);
		OnBnClickedEnableText();
	}

	SetCheckBoxes(_output, _trigger, _revertDisplay, _controlType);
	GetDlgItem(IDC_EDIT_TRIALTYPE_CURSOR_SIZE)->SetWindowText(_itoa(_cursor, buf, 10));
	GetDlgItem(IDC_EDIT_TRIALTYPE_DC_DELAY)->SetWindowText(_itoa(_dcDelay, buf, 10));
	if(_trigger == Tokenizer::NOGAZE)
		GetDlgItem(IDC_EDIT_TRIALTYPE_STIMULUS_DELAY)->SetWindowText(_itoa(_stimDelay, buf, 10));
	
	if(_controlType == Tokenizer::FIVE_BUTTON){
		mListButtons.AddString("(1) Blue");          //load buttons into list box
		mListButtons.AddString("(2) Red");
		mListButtons.AddString("(3) Green");
		mListButtons.AddString("(4) Yellow");
		mListButtons.AddString("(5) Purple");
		mListButtons.AddString("Mouse Click");
	}


	else{
		mListButtons.AddString("(1) Y");
		mListButtons.AddString("(2) X");
		mListButtons.AddString("(3) B");
		mListButtons.AddString("(4) A");
		mListButtons.AddString("(5) Toggle");
		mListButtons.AddString("(6) Left Trigger");
		mListButtons.AddString("(7) Right Trigger");
		mListButtons.AddString("Mouse Click");
	}

	for(int i=0; i < _nButtons; ++i) {           //set selected buttons
		int idx = static_cast<int>(_button[i])-1;
		mListButtons.SetSel(idx, true);
	}


	m_ToolTip= new CToolTipCtrl;
	m_ToolTip->Create(this);
	
	
	m_ToolTip->AddTool(GetDlgItem(IDC_EDIT_TRIALTYPE_DC_DELAY), "Enter 0 for change at time of boundary crossing, or enter nonzero number of ms delay between fixation and DC.");
	m_ToolTip->AddTool(GetDlgItem(IDC_RADIO_TRIALTYPE_TRIGGER_GAZE), "Show GC rectangle before each trial starts recording.");
	m_ToolTip->AddTool(GetDlgItem(IDC_RADIO_TRIALTYPE_TRIGGER_DRIFTCORRECT), "Recalibrate before each trial.");
	m_ToolTip->AddTool(GetDlgItem(IDC_RADIO_TRIALTYPE_TRIGGER_DRIFTANDGAZE), "Recalibrate and show GC rect before each trial.");
	m_ToolTip->Activate(TRUE);	

	return TRUE;
}

void CEditTrialTypeDlg::OnOK()
{
	const int maxLen = 200;
	std::ostringstream os;
	char buf[maxLen];
	int button[10];

	os<<"The following fields are required:"<<std::endl;

	if(_identifier.empty()) {                    //read identifier if needed
		GetDlgItem(IDC_EDIT_TRIALTYPE_ID)->GetWindowText(buf, maxLen);
		_identifier = buf;
		if(_identifier.empty() )
			os<<"  * Trial Type Identifier"<<std::endl;
	}

	if(_enableAudioCapture)                      //check presence of audio format
		if(_audioFmt.path.empty())
			os<<"  * Audio Capture Properties"<<std::endl;

	if(_enableText)                              //check presence of text format
		if(_textFmt.fontName.empty())
			os<<"  * Inline Text Properties"<<std::endl;

	if(_output == Tokenizer::NO_OUTPUT)          //make sure output method selected
		os<<"  * Output Type"<<std::endl;

	if(_trigger == Tokenizer::NO_TRIGGER)        //make sure trigger method selected
		os<<"  * Trigger Type"<<std::endl;
	else if(_trigger == Tokenizer::NOGAZE) {     //check for stimulus delay (if needed)
		if(GetDlgItem(IDC_EDIT_TRIALTYPE_STIMULUS_DELAY)->GetWindowText(buf, maxLen) <= 0)
			os<<"  * Stimulus Delay"<<std::endl;
		else _stimDelay = atoi(buf);
	}

	if(GetDlgItem(IDC_EDIT_TRIALTYPE_DC_DELAY)->GetWindowText(buf, maxLen) > 0)
		_dcDelay = atoi(buf);
	else _dcDelay = 0;

	if(GetDlgItem(IDC_EDIT_TRIALTYPE_CURSOR_SIZE)->GetWindowText(buf, maxLen) > 0)
		_cursor = atoi(buf);
	else _cursor = 0;

	_nButtons = mListButtons.GetSelItems(10, button);
	for(int i=0; i < _nButtons; ++i) {           //get button selections
		_button[i] = static_cast<Tokenizer::ButtonType>(button[i]+1);

		if(
			((_controlType == Tokenizer::FIVE_BUTTON && _button[i]+2 == Tokenizer::MOUSE)
			||(_controlType == Tokenizer::SEVEN_BUTTON && _button[i] == Tokenizer::MOUSE))		
			&& _nButtons > 1) {
			os<<"  * Cannot select 'Mouse Click' as a button in conjunction with other buttons."<<std::endl;
			break;
		}
	}

	if(os.str().length() > 40) {
		MessageBox(os.str().c_str(), "Dialog Incomplete");
		return;
	}

	CDialog::OnOK();
}

void CEditTrialTypeDlg::SetCheckBoxes(Tokenizer::OutputType output,  
									  Tokenizer::TriggerType trigger,
									  bool revertDisplay, Tokenizer::ControlType control)
{
	int strmChk = (output == Tokenizer::STREAM) ? BST_CHECKED : BST_UNCHECKED;
	int noStrmChk = (output == Tokenizer::NOSTREAM) ? BST_CHECKED : BST_UNCHECKED;
	int gaze = (trigger == Tokenizer::GAZE) ? BST_CHECKED : BST_UNCHECKED;
	int noGaze = (trigger == Tokenizer::NOGAZE) ? BST_CHECKED : BST_UNCHECKED;
	int driftCor = (trigger == Tokenizer::DRIFTCORRECT) ? BST_CHECKED : BST_UNCHECKED;
	int driftGaze = (trigger == Tokenizer::DRIFTANDGAZE) ? BST_CHECKED : BST_UNCHECKED;
	int revert = revertDisplay ? BST_CHECKED : BST_UNCHECKED;

	int five = (control == Tokenizer::FIVE_BUTTON) ? BST_CHECKED : BST_UNCHECKED;
	int seven = (control == Tokenizer::SEVEN_BUTTON) ? BST_CHECKED : BST_UNCHECKED;

	((CButton*)GetDlgItem(IDC_RADIO_TRIALTYPE_OUTPUT_STREAM))->SetCheck(strmChk);
	((CButton*)GetDlgItem(IDC_RADIO_TRIALTYPE_OUTPUT_NOSTREAM))->SetCheck(noStrmChk);

	((CButton*)GetDlgItem(IDC_RADIO_TRIALTYPE_TRIGGER_GAZE))->SetCheck(gaze);
	((CButton*)GetDlgItem(IDC_RADIO_TRIALTYPE_TRIGGER_NOGAZE))->SetCheck(noGaze);
	((CButton*)GetDlgItem(IDC_RADIO_TRIALTYPE_TRIGGER_DRIFTCORRECT))->SetCheck(driftCor);
	((CButton*)GetDlgItem(IDC_RADIO_TRIALTYPE_TRIGGER_DRIFTANDGAZE))->SetCheck(driftGaze);

	((CButton*)GetDlgItem(IDC_CHECK_TRIALTYPE_REVERT_DISPLAY_CHANGE))->SetCheck(revert);

	((CButton*)GetDlgItem(IDC_RADIO_5_BUTTON_CONTROL))->SetCheck(five);
	((CButton*)GetDlgItem(IDC_RADIO_7_BUTTON_CONTROL))->SetCheck(seven);
}

void CEditTrialTypeDlg::SetTrialType(TrialType *t)
{
	if(t == 0) return;

	if(t->audioFormat()) {
		_audioFmt = *(t->audioFormat());
		_enableAudioCapture = true;
	}

	if(t->textFormat()) {
		_textFmt = *(t->textFormat());
		_enableText = true;
	}

	_identifier = t->identifier();
	_nButtons = t->nButtons();
	for(int i = 0; i < _nButtons; ++i)
		_button[i] = (t->buttons())[i];

	_output = t->output();
	_trigger = t->trigger();
	_cursor = t->cursorSize();
	_dcDelay = t->displayChangeDelay();
	_stimDelay = t->stimulusDelay();
	_revertDisplay = t->doDisplayRevert();
	_controlType = t->controlType();
}

void CEditTrialTypeDlg::OnBnClickedOk()
{
	// TODO: Add your control notification handler code here
	OnOK();
}

void CEditTrialTypeDlg::OnEnChangeEditTrialtypeId()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}

void CEditTrialTypeDlg::OnLbnSelchangeListTrialtypeButtons()
{
	// TODO: Add your control notification handler code here
}

void CEditTrialTypeDlg::OnBnClickedRadio5ButtonControl()
{
	_controlType = Tokenizer::FIVE_BUTTON;
	updateButtonList();
	// TODO: Add your control notification handler code here
}

void CEditTrialTypeDlg::OnBnClickedRadio7ButtonControl()
{
	_controlType = Tokenizer::SEVEN_BUTTON;
	updateButtonList();
	// TODO: Add your control notification handler code here
}

void CEditTrialTypeDlg::updateButtonList()
{
	if(_controlType == Tokenizer::FIVE_BUTTON){
		mListButtons.ResetContent();
		mListButtons.AddString("(1) Blue");          //load buttons into list box
		mListButtons.AddString("(2) Red");
		mListButtons.AddString("(3) Green");
		mListButtons.AddString("(4) Yellow");
		mListButtons.AddString("(5) Purple");
		mListButtons.AddString("Mouse Click");
	}

	else{
		mListButtons.ResetContent();
		mListButtons.AddString("(1) Y");
		mListButtons.AddString("(2) X");
		mListButtons.AddString("(3) B");
		mListButtons.AddString("(4) A");
		mListButtons.AddString("(5) Toggle");
		mListButtons.AddString("(6) Left Trigger");
		mListButtons.AddString("(7) Right Trigger");
		mListButtons.AddString("Mouse Click");
	}

	return;
}

void CEditTrialTypeDlg::OnEnChangeEditTrialtypeDcDelay()
{
	// TODO:  If this is a RICHEDIT control, the control will not
	// send this notification unless you override the CDialog::OnInitDialog()
	// function and call CRichEditCtrl().SetEventMask()
	// with the ENM_CHANGE flag ORed into the mask.

	// TODO:  Add your control notification handler code here
}
BOOL CEditTrialTypeDlg::PreTranslateMessage(MSG* pMsg)
{
    // TODO: Add your specialized code here and/or call the base class

	/*switch (pMsg->message){
		case CB_SETCURSEL:
			break;
	}*/
    if(m_ToolTip != NULL)
       m_ToolTip->RelayEvent(pMsg);


    return CDialog::PreTranslateMessage(pMsg);
}