/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "EyeTrack.h"
#include "ExperimentDlg.h"
#include "InitExperimentDlg.h"
#include "AudioCapture.h"
#include "AudioRender.h"
#include "Eyelink2.h"
#include "Tokenizer.h"
#include <fstream>
#include <sstream>
#include <memory>
#ifdef INCLUDE_EYELINK
#include "w32_exptsppt2.h"
#endif

//#define PIC_WINDOW  // USE THIS FOR MOVING SCOTOMA
//#define USE_SERIAL_DATA

//#define REVERT_TIMED_REGION_CHANGE_ON_EXIT_REGION

//#define DO_OFFSET_DRIFT_CORRECT

#define NEW_STIM_3
//#define ORIG_STIM_3

//#define spacingMod
//#define newMode

IMPLEMENT_DYNAMIC(CExperimentDlg, CDialog)
CExperimentDlg::CExperimentDlg(CWnd* pParent, const CSize &size, COLORREF back, COLORREF fore, Tokenizer::FilterMode fm)
: CDialog(CExperimentDlg::IDD, pParent), _wndSz(0, 0, size.cx, size.cy),
_backColor(back), _foreColor(fore), _filterMode(fm), 
  _isExperiment(false), _isEyelink2(false), _needsInitialization(true)
{
	Create(IDD_EXPERIMENTDLG, pParent);
}

CExperimentDlg::~CExperimentDlg()
{}

void CExperimentDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
}

BEGIN_MESSAGE_MAP(CExperimentDlg, CDialog)
	ON_WM_PAINT()
	ON_WM_SHOWWINDOW()
	ON_WM_CHAR()
	ON_WM_CLOSE()
	ON_WM_KEYDOWN()
END_MESSAGE_MAP()

/*****************************************************************************/

void CExperimentDlg::ClearDisplay(COLORREF color)
{
	CBrush brush(color); 
	CBrush *pOldBrush = _dc.SelectObject(&brush); 
	CBitmap *pOldBitmap = _dc.SelectObject(&_bitmap);

	_dc.PatBlt(_wndSz.left, _wndSz.top, _wndSz.Width(), _wndSz.Height(), PATCOPY);

	_dc.SelectObject(pOldBitmap);
	_dc.SelectObject(pOldBrush);
}

void CExperimentDlg::CreateGCBitmap(CBitmap *bmp, CRect &gazeBox, COLORREF bg, COLORREF fg)
{
	CBrush bgBrush(bg), fgBrush(fg);
	CBrush *pOldBrush = _dc.SelectObject(&bgBrush);
	CBitmap *pOldBitmap;
	CDC *pWndDC = GetDC();

	bmp->CreateCompatibleBitmap(pWndDC, _wndSz.Width(), _wndSz.Height());
	ReleaseDC(pWndDC);

	pOldBitmap = _dc.SelectObject(bmp);
	_dc.PatBlt(_wndSz.left, _wndSz.top, _wndSz.Width(), _wndSz.Height(), PATCOPY);
	_dc.SelectObject(&fgBrush);
	_dc.PatBlt(gazeBox.left, gazeBox.top, gazeBox.Width(), gazeBox.Height(), PATCOPY);

	_dc.SelectObject(pOldBitmap);
	_dc.SelectObject(pOldBrush);
}

CRect CExperimentDlg::DrawStimulus(/*const*/ Trial &t, const TrialType &tc, 
								   CBitmapFile &bmp, bool doDisplay2)
{
	const std::string &display(doDisplay2 ? t.display2() : t.display1());
	CBitmap *pOldBitmap;
	CDC *pDC = GetDC();
	CRect fixationRect(0, 0, 50, 50);
	int displayIdx = (doDisplay2 ? 2 : 1);
                                                 //init the bitmap
	bmp.CreateCompatibleBitmap(pDC, _wndSz.right, _wndSz.bottom);
	//pOldBitmap = _dc.SelectObject(&bmp);
	//_dc.SelectObject(pOldBitmap);


	CBitmapFile bmpPart;
	bmpPart.CreateCompatibleBitmap(pDC, _wndSz.right, _wndSz.bottom);

	CDC memdc, memdc2; // MOD
	memdc.CreateCompatibleDC(pDC); // MOD

		pOldBitmap = memdc.SelectObject(&bmp);

		CBrush bgBrush(_backColor);
		memdc.SelectObject(&bgBrush);
		memdc.PatBlt(_wndSz.left, _wndSz.top, _wndSz.right, _wndSz.bottom, PATCOPY);

	if (!doDisplay2 && !t.displayData().images1.empty() || doDisplay2 ){
		CSize fullsize(_wndSz.Width(), _wndSz.Height());
		CSize zero(0,0);

		std::vector<IMAGEPOS>::const_iterator imageIter, end;

		if(doDisplay2){
			imageIter = t.displayData().images2.begin();
			end = t.displayData().images2.end();
		}
		else { // doing display 1
			imageIter = t.displayData().images1.begin();
			end = t.displayData().images1.end();
		}

		//do {
		while(imageIter != end){
			imagePos ip = imagePos(imageIter->fileName, imageIter->upperLeft);

			CBitmapFile smallBmp;
			CDC small_memdc;

			smallBmp.CreateCompatibleBitmap(pDC, _wndSz.right, _wndSz.bottom);
			small_memdc.CreateCompatibleDC(pDC);

			smallBmp.LoadBitmap(ip.fileName.c_str(), zero);
			small_memdc.SelectObject(smallBmp);
			memdc.BitBlt(ip.upperLeft.x, ip.upperLeft.y,fullsize.cx,fullsize.cy,&small_memdc, 0,0,SRCCOPY);
			
			eyemsg_printf("LOADED IMAGE %i %s AT (%d, %d)", displayIdx, ip.fileName.c_str(), ip.upperLeft.x, ip.upperLeft.y);
			
			smallBmp.DeleteObject();
			small_memdc.DeleteDC();


			imageIter++;
		}
		//} while (imageIter != end);//(imageIter != t.displayData().images1.end());

	}

	ReleaseDC(pDC);
	//_dc.SelectObject(pOldBitmap);
	if(t.isTextDisplay()) {
		const TextFormat &tf = *(tc.textFormat());
		const TextFormat &tf2 = *(tc.textFormat2());	
		TextPassage tp(display, _wndSz, t.getDisplay(), tc.getHColor() );
		tp.f2start = t.startf2;
		tp.f2end = t.endf2;
		if (!doDisplay2 && t.isMovingWindow())
			tp.resetMWregions();

		//TextPassage tp(display, _wndSz, t);
		CBrush bgBrush(_backColor);
		CBrush *pOldBrush = _dc.SelectObject(&bgBrush);
		memdc.SelectObject(&bgBrush); // MOD
		CFont font, *oldFont, font2;

		eyemsg_printf("DISPLAY TEXT %i", displayIdx);
		tp.computeProperties(_dc, *(tc.textFormat()), *(tc.textFormat2()));
		tp.writeToEyelink();

		if(t.gcRect().IsRectNull())              //set fixation region
			fixationRect = tp.gcRect();
		else fixationRect = t.gcRect();
	                                             //generate font
		font.CreateFont(tp.fontHeight(), 0, 0, 0, tc.textFormat()->fontWt, tc.textFormat()->italic,/*FW_NORMAL, FALSE,*/ FALSE, 0, ANSI_CHARSET,
						OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, tf.rendering,
						DEFAULT_PITCH | FF_DONTCARE, tf.fontName.c_str());

		TEXTMETRIC tm;
		_dc.GetTextMetrics(&tm);
		int it = tm.tmAveCharWidth;
		printf("%d", it);

#ifdef spacingMod
		memdc.SetMapMode(MM_HIENGLISH);
		//_dc.SetMapMode(MM_HIENGLISH);
		font2.CreateFont(tp.fontHeight() * 9, 0/*tm.tmAveCharWidth*1*/,0, 0, FW_NORMAL, tf.italic, FALSE, 0, ANSI_CHARSET,
						OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, tf.rendering,
						DEFAULT_PITCH | FF_DONTCARE, tf.fontName.c_str());
		memdc.SetMapMode(MM_TEXT);
#endif
#ifdef newMode
		if(tc.textFormat2()){
		font2.CreateFont(tp.fontHeight() * 1, 0/*tm.tmAveCharWidth*1*/,0, 0, tf2.fontWt, tf2.italic, FALSE, 0, ANSI_CHARSET,
						OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, tf2.rendering,
						DEFAULT_PITCH | FF_DONTCARE, tf2.fontName.c_str());
		}

#endif


		oldFont = memdc.SelectObject(&font);
		memdc.SetBkMode(TRANSPARENT);
		memdc.SetMapMode(MM_TEXT);
		memdc.SetTextColor(_foreColor);

		oldFont = _dc.SelectObject(&font);       //select font	
		_dc.SetBkMode(TRANSPARENT);              //do not draw background behind text
		_dc.SetMapMode(MM_TEXT);                 //mapping mode most useful for text
		_dc.SetTextColor(_foreColor);            //set text color		
												//set entire bitmap to background color
		_dc.PatBlt(_wndSz.left, _wndSz.top, _wndSz.right, _wndSz.bottom, PATCOPY);
		//memdc.PatBlt(_wndSz.left, _wndSz.top, _wndSz.right, _wndSz.bottom, PATCOPY);
		if(doDisplay2){
			//tp.drawText(_dc, 0, false);		// don't need to recompute MW regions if second display,
												// regardless of whether this trial uses MW
		//	memdc.SelectObject(&font2);
			tp.drawText(memdc, 0, false, "", &font2, tf.spacingDiff/*tc.getInterCharSpacing()*/);
			
		}
		else { 
//			tp.drawText(_dc, 0/*, t.getDisplay()*/, t.isMovingWindow());     //draw inline text
			if(!_isExperiment){
				int myint = tc.getInterCharSpacing();
				eyemsg_printf("%d", myint);
				tp.drawText(memdc, 0, t.isMovingWindow(), t.identifier().idString().c_str(), &font2, tf.spacingDiff/*tc.getInterCharSpacing()*/);
			}
			else
				tp.drawText(memdc, 0, t.isMovingWindow(), "", &font2, tf.spacingDiff/*tc.getInterCharSpacing()*/);
 		}
		_dc.SetMapMode(MM_TEXT);
		/* Moving Window Additions */
		if (t.isMovingWindow()) t.setDisplay(tp.getDisplay());
			
		_dc.SelectObject(oldFont);               //deselect font

		_dc.SelectObject(pOldBrush);             //deselect background brush
	}
	else {
		CSize size(_wndSz.Width(), _wndSz.Height());

		bmp.LoadBitmap(display.c_str(), size);
		eyemsg_printf("LOADED IMAGE %i %s", displayIdx, display.c_str());

		if(t.gcRect().IsRectNull()) {            //set fixation region
			CPoint ctr = _wndSz.CenterPoint();
			fixationRect.MoveToXY(ctr.x-25, ctr.y-25);
		}
		else fixationRect = t.gcRect();
	}

	//_dc.SelectObject(pOldBitmap);                //deselect the bitmap
	memdc.SelectObject(pOldBitmap);
	return(fixationRect);
}

#ifdef ORIG_STIM_3  // use this version for fast-priming 3 stim trials
CRect CExperimentDlg::DrawStimulus3(/*const*/ Trial &t, const TrialType &tc, 
								   CBitmapFile &bmp, bool doDisplay2)
{
	//const std::string &display(doDisplay2 ? t.display2() : t.display1());
	const std::string &display = t.display3();
	CBitmap *pOldBitmap;
	CDC *pDC = GetDC();
	CRect fixationRect(0, 0, 50, 50);
	//int displayIdx = (doDisplay2 ? 2 : 1);
	int displayIdx = 3;
                                                 //init the bitmap
	bmp.CreateCompatibleBitmap(pDC, _wndSz.right, _wndSz.bottom);
	pOldBitmap = _dc.SelectObject(&bmp);
	ReleaseDC(pDC);

	if(t.isTextDisplay()) {
		const TextFormat &tf = *(tc.textFormat());
		TextPassage tp(display, _wndSz, t.getDisplay());
		if (!doDisplay2 && t.isMovingWindow())
			tp.resetMWregions();

		//TextPassage tp(display, _wndSz, t);
		CBrush bgBrush(_backColor);
		CBrush *pOldBrush = _dc.SelectObject(&bgBrush);
		CFont font, *oldFont;

		eyemsg_printf("DISPLAY TEXT %i", displayIdx);
		tp.computeProperties(_dc, *(tc.textFormat()));
		tp.writeToEyelink();

		if(t.gcRect().IsRectNull())              //set fixation region
			fixationRect = tp.gcRect();
		else fixationRect = t.gcRect();
	                                             //generate font
		font.CreateFont(tp.fontHeight(), 0, 0, 0, tc.textFormat()->fontWt, tc.textFormat()->italic,/* FW_NORMAL, FALSE,*/ FALSE, 0, ANSI_CHARSET,
						OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, tf.rendering,
						DEFAULT_PITCH | FF_DONTCARE, tf.fontName.c_str());

		oldFont = _dc.SelectObject(&font);       //select font
		
		_dc.SetBkMode(TRANSPARENT);              //do not draw background behind text
		_dc.SetMapMode(MM_TEXT);                 //mapping mode most useful for text
		_dc.SetTextColor(_foreColor);            //set text color
												 //set entire bitmap to background color
		_dc.PatBlt(_wndSz.left, _wndSz.top, _wndSz.right, _wndSz.bottom, PATCOPY);
		if(doDisplay2)
			tp.drawText(_dc, 0, false);		// don't need to recompute MW regions if second display,
											// regardless of whether this trial uses MW
		else 
			tp.drawText(_dc, 0/*, t.getDisplay()*/, t.isMovingWindow());     //draw inline text

		/* Moving Window Additions */
		if (t.isMovingWindow()) t.setDisplay(tp.getDisplay());
			
		_dc.SelectObject(oldFont);               //deselect font

		_dc.SelectObject(pOldBrush);             //deselect background brush
	}
	else {
		CSize size(_wndSz.Width(), _wndSz.Height());

		bmp.LoadBitmap(display.c_str(), size);
		eyemsg_printf("LOADED IMAGE %i %s", displayIdx, display.c_str());

		if(t.gcRect().IsRectNull()) {            //set fixation region
			CPoint ctr = _wndSz.CenterPoint();
			fixationRect.MoveToXY(ctr.x-25, ctr.y-25);
		}
		else fixationRect = t.gcRect();
	}

	_dc.SelectObject(pOldBitmap);                //deselect the bitmap

	return(fixationRect);
}
#endif

#ifdef NEW_STIM_3 // block this out for multi pic develpment
CRect CExperimentDlg::DrawStimulus3(/*const*/ Trial &t, const TrialType &tc, 
								   CBitmapFile &bmp, bool doDisplay2)
{
	const std::string &display = t.display3();
	CBitmap *pOldBitmap;
	CDC *pDC = GetDC();
	CRect fixationRect(0, 0, 50, 50);
	//int displayIdx = (doDisplay2 ? 2 : 1);
	int displayIdx = 3;
                                                 //init the bitmap
	bmp.CreateCompatibleBitmap(pDC, _wndSz.right, _wndSz.bottom);
	//pOldBitmap = _dc.SelectObject(&bmp);

/**************/

	CBitmapFile bmpPart;
	bmpPart.CreateCompatibleBitmap(pDC, _wndSz.right, _wndSz.bottom);

	CDC memdc, memdc2; // MOD
	memdc.CreateCompatibleDC(pDC); // MOD

		pOldBitmap = memdc.SelectObject(&bmp);

		CBrush bgBrush(_backColor);
		memdc.SelectObject(&bgBrush);
		memdc.PatBlt(_wndSz.left, _wndSz.top, _wndSz.right, _wndSz.bottom, PATCOPY);

		if(!doDisplay2 && !t.displayData().images1.empty() || doDisplay2){
	CSize fullsize(_wndSz.Width(), _wndSz.Height());
	CSize zero(0,0);
		std::vector<IMAGEPOS>::const_iterator imageIter, end;

		imageIter = t.displayData().images3.begin();
		end = t.displayData().images3.end();

		while(imageIter != end){
			imagePos ip = imagePos(imageIter->fileName, imageIter->upperLeft);

			CBitmapFile smallBmp;
			CDC small_memdc;

			smallBmp.CreateCompatibleBitmap(pDC, _wndSz.right, _wndSz.bottom);
			small_memdc.CreateCompatibleDC(pDC);

			smallBmp.LoadBitmap(ip.fileName.c_str(), zero);
			small_memdc.SelectObject(smallBmp);
			memdc.BitBlt(ip.upperLeft.x, ip.upperLeft.y,fullsize.cx,fullsize.cy,&small_memdc, 0,0,SRCCOPY);
			
			eyemsg_printf("LOADED IMAGE %i %s AT (%d, %d)", displayIdx, ip.fileName.c_str(), ip.upperLeft.x, ip.upperLeft.y);
			
			smallBmp.DeleteObject();
			small_memdc.DeleteDC();


			imageIter++;
		}
}
/**************/


	ReleaseDC(pDC);

	if(t.isTextDisplay()) {
		const TextFormat &tf = *(tc.textFormat());
		TextPassage tp(display, _wndSz, t.getDisplay());
		if (!doDisplay2 && t.isMovingWindow())
			tp.resetMWregions();

		//TextPassage tp(display, _wndSz, t);
		CBrush bgBrush(_backColor);
		CBrush *pOldBrush = _dc.SelectObject(&bgBrush);
		memdc.SelectObject(&bgBrush);
		CFont font, *oldFont;

		eyemsg_printf("DISPLAY TEXT %i", displayIdx);
		tp.computeProperties(_dc, *(tc.textFormat()), *(tc.textFormat2()));
		tp.writeToEyelink();

		if(t.gcRect().IsRectNull())              //set fixation region
			fixationRect = tp.gcRect();
		else fixationRect = t.gcRect();
	                                             //generate font
		font.CreateFont(tp.fontHeight(), 0, 0, 0, tc.textFormat()->fontWt, tc.textFormat()->italic,/* FW_NORMAL, FALSE,*/ FALSE, 0, ANSI_CHARSET,
						OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, tf.rendering,
						DEFAULT_PITCH | FF_DONTCARE, tf.fontName.c_str());


		oldFont = memdc.SelectObject(&font);
		memdc.SetBkMode(TRANSPARENT);
		memdc.SetMapMode(MM_TEXT);
		memdc.SetTextColor(_foreColor);
		
		oldFont = _dc.SelectObject(&font);       //select font
		
		_dc.SetBkMode(TRANSPARENT);              //do not draw background behind text
		_dc.SetMapMode(MM_TEXT);                 //mapping mode most useful for text
		_dc.SetTextColor(_foreColor);            //set text color
												 //set entire bitmap to background color
		_dc.PatBlt(_wndSz.left, _wndSz.top, _wndSz.right, _wndSz.bottom, PATCOPY);
		if(doDisplay2)
			tp.drawText(memdc, 0, false);		// don't need to recompute MW regions if second display,
											// regardless of whether this trial uses MW
		else {
			if(!_isExperiment)
				tp.drawText(memdc, 0, t.isMovingWindow(), t.identifier().idString().c_str());
			else
				tp.drawText(memdc, 0/*, t.getDisplay()*/, t.isMovingWindow());     //draw inline text
		}
		/* Moving Window Additions */
		if (t.isMovingWindow()) t.setDisplay(tp.getDisplay());
			
		_dc.SelectObject(oldFont);               //deselect font

		_dc.SelectObject(pOldBrush);             //deselect background brush
	}
	else {
		CSize size(_wndSz.Width(), _wndSz.Height());

		bmp.LoadBitmap(display.c_str(), size);
		eyemsg_printf("LOADED IMAGE %i %s", displayIdx, display.c_str());

		if(t.gcRect().IsRectNull()) {            //set fixation region
			CPoint ctr = _wndSz.CenterPoint();
			fixationRect.MoveToXY(ctr.x-25, ctr.y-25);
		}
		else fixationRect = t.gcRect();
	}

	//_dc.SelectObject(pOldBitmap);                //deselect the bitmap
	memdc.SelectObject(pOldBitmap);
	return(fixationRect);
}


#endif
void CExperimentDlg::OnChar(UINT nChar, UINT nRepCnt, UINT nFlags)
{
#ifdef INCLUDE_EYELINK
	if(_isExperiment) {
		const MSG *msg = GetCurrentMessage();    //pass key message through eyelink system
		process_key_messages(msg->hwnd, msg->message, msg->wParam, msg->lParam);   
	}
#endif
	CDialog::OnChar(nChar, nRepCnt, nFlags);
}

void CExperimentDlg::OnClose()
{
#ifdef INCLUDE_EYELINK
	if(_isExperiment) {
		PostQuitMessage(0);
		terminal_break(1);                       //send "exit" msg through eyelink system
	}
#endif
	CDialog::OnClose();
}

void CExperimentDlg::OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags)
{
#ifdef INCLUDE_EYELINK
	if(_isExperiment) {
		const MSG *msg = GetCurrentMessage();    //pass key message through eyelink system
		process_key_messages(msg->hwnd, msg->message, msg->wParam, msg->lParam); 
	}
#endif
	CDialog::OnKeyDown(nChar, nRepCnt, nFlags);
}

void CExperimentDlg::OnPaint()
{
	CPaintDC dc(this);                           //not needed except appears to have an important side-effect
	ShowDisplay(_wndSz, false);                  //draw the bitmap to the screen
}

void CExperimentDlg::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CDialog::OnShowWindow(bShow, nStatus);

	if(_needsInitialization) {
		CDC *pDC = this->GetDC();

		GdiSetBatchLimit(1);                     //disable batching
		_needsInitialization = false;
		_dc.CreateCompatibleDC(pDC);
		_bitmap.CreateCompatibleBitmap(pDC, _wndSz.right, _wndSz.bottom);
		this->ReleaseDC(pDC);

		ClearDisplay(_backColor);
	}
}
int myGetMode ( int which ){
#ifdef JDK_DEBUG
	INT16	sample_rate=-1, 
				crmode=-1, 
				file_filter=-1, 
				link_filter=-1;
	eyelink2_mode_data( &sample_rate, &crmode, &file_filter, &link_filter);

	eyemsg_printf(
			"DEBUG CHECK %d:	sample_rate = %hd, crmode = %hd, file_filter = %hd, link_filter = %hd",
				which, sample_rate, crmode, file_filter, link_filter);
#endif //JDK_DEBUG
	return 0;
}

int CExperimentDlg::sendMode(Tokenizer::FilterMode filtermode){
	int result = 0;
#ifdef INCLUDE_EYELINK
	if(!_edfFileName.IsEmpty()){
		switch(filtermode){
			case(Tokenizer::FILTER_OFF):	result = eyecmd_printf("heuristic_filter = OFF");
			break;
			case(Tokenizer::FILTER_ON):		result = eyecmd_printf("heuristic_filter = ON");
			break;
			case(Tokenizer::NONE_SELECTED):	result = eyecmd_printf("heuristic_filter = ON");
			break;
			default:						result = eyecmd_printf("heuristic_filter = ON");
		}
	}
#endif // INCLUDE_EYELINK
	return result;
}

void CExperimentDlg::RunExperiment(Experiment *expt, IdentifierList &trials, 
								   const char *exptName)
{
	std::ostringstream os;
	IdentifierIter i = trials.begin();
	Sequence *s = 0;
	int idx = 1, nTrials = expt->countTrials(trials);
	int nErrorRepeats = 0;
#ifdef USE_SERIAL_DATA
	Serial sPort;
#endif



	if(exptName != 0) {                          //this is an actual experiment
		CEyelink2::SetTrackMode(true);
		_edfFileName = exptName;
		_edfFileName.Append(".EDF");
		_isExperiment = true;
	}
	else {                                       //this is only a test
		CEyelink2::SetTrackMode(false);
		_edfFileName = "";
	}

	try {
		CEyelink2::Initialize(GetSafeHwnd(), _wndSz, _edfFileName, _foreColor, _backColor, _filterMode, expt->calibrationType(), expt->calibrationHeight());
	} catch(CEyelinkException &ee) {
		MessageBox(ee.what(), "EyeLink Error", MB_ICONSTOP | MB_OK);
		i = trials.end();
	}

	std::ostringstream msg;
	msg<<"Recorded with EyeTrack version: ";
	msg<<VERSION;
	eyemsg_printf((char*)msg.str().c_str());
	
/*	if( sendMode(expt->getFilterMode()) )
	{
		os<<"Failed to set heuristic filter level."<<std::endl;
		throw (CExperimentException(os.str().c_str() ));
	}*/	
	CString identify;

#ifdef NOT_DEFINED
	int result = sPort.OpenPort(identify);
	if(result == TRUE)
		eyemsg_printf("OPENPORT successful");
	else if(result == 10){
		eyemsg_printf("OPENPORT READ/WRITE successful");
		eyemsg_printf("RECIEVED %s", identify);
	}
	else
		eyemsg_printf("OPENPORT failed");
#endif


	_displayType = expt->getDisplayType();

	TrialType *lastTC = 0;
	while((s && !s->atEnd()) || i != trials.end()) {
		Trial *t = 0;
		TrialType *tc = 0;
		int result;

		if(s) {                                  //select trial (within sequence)
			if(s->atEnd() || (s->atAltEnd() && s->altLength() >0)) {
				s = 0;
				if(i == trials.end()) break;
			}
			else {
				if (!s->atEnd())
				t = expt->getTrial(s->nextTrialId());

				int myButtons[3];
				myButtons[0] = s->altSelectButton();
				myButtons[1] = s->altSelectButton();
				myButtons[2] = s->altSelectButton();
				//int lastPressed = CEyelink2::TestForButtonPress(reinterpret_cast<int*>(lastTC->buttons()), 
				//	lastTC->nButtons());
				int lastPressed = lastButton;
				if (!s->atAltEnd() &&
				lastPressed == s->altSelectButton()){
						t = expt->getTrial(s->nextAltId());
					}

				/*if(1==1 && !s->atEnd() 
#ifdef INCLUDE_EYELINK
					&& eyelink_last_button_press(NULL) == Tokenizer::TOGGLE
#endif					
					)
					t = expt->getTrial(s->nextTrialId()); // skip one*/
			}
			
		}

		if(!s) {                                 //select trial (no sequence)
			if(i->isSequence()) {
				s = expt->getSequence(*i);
				s->resetTrialPtr();
				t = expt->getTrial(s->nextTrialId());
				s->nextAltId(); // skips past the independent trial in alt list.
				//int button = t->questionAnswer();
			}
			else
				t = expt->getTrial(*i);
			++i;
		}

		tc = expt->getTrialType(t->typeId());
		if(!tc) {                                //invalid trial condition
			MessageBox("Error initializing trial: Invalid trial condition.\n Experiment aborted.", "Error");
			break;
		}

		while(ShowCursor(FALSE) >= 0);           //turn off mouse cursor

		if(_isExperiment && CEyelink2::TestStatus(false) != TRIAL_OK)
			break;  
                             //lost link or other bail out
		CEyelink2::PrepareForTrial(t->identifier().idString().c_str(), idx, nTrials, tc->doStreamOutput());
		
		try{
			result = RunTrial(*t, *tc, expt->getDCThreshold() );
		} catch (CExperimentException &ee) {
			MessageBox(ee.what(), "EyeLink Error", MB_ICONSTOP | MB_OK);
		}
	

		switch(result) {
			case ABORT_EXPT:                     //experiment aborted or disconnected
				eyemsg_printf("EXPERIMENT ABORTED");
				nErrorRepeats=0;
				break;
			case REPEAT_TRIAL:	                 //trial must restart (from the independent)
				eyemsg_printf("TRIAL REPEATED");
				s = 0;
				--i;
				nErrorRepeats = 0;
				break;
			case SKIP_TRIAL:	                 //skip trial (to next independent)
				eyemsg_printf("TRIAL ABORTED");
				s = 0;
				++idx;
				nErrorRepeats = 0;
				break;
			case TRIAL_OK:                       //successful trial
				eyemsg_printf("TRIAL OK");
				++idx;
				nErrorRepeats = 0;
				break;
			default:                             //some other error
			//	eyemsg_printf("TRIAL ERROR");
			//	s = 0;
			//	++idx;
				// The following sets this trial up to be repeated b/c error -- make 5 attempts
				if( nErrorRepeats < 10 ){
					eyemsg_printf("TRIAL REPEATED");
					nErrorRepeats++;
					s = 0;
					--i;
				}
				else {
					eyemsg_printf("TRIAL ERROR");
					s = 0;
					++idx;
				}
				break;
		}

		if(result == ABORT_EXPT) break;
		lastTC = tc;
	}
	while(ShowCursor(TRUE) < 0);                 //turn on mouse cursor
	CEyelink2::Shutdown(_edfFileName);
	_isExperiment = false;

}


bool contains(std::vector<CRect> targetList, CRect r){
	for( unsigned int i=0; i< targetList.size(); i++){
		if (targetList.at(i) == r)
			return true;
	}
	return false;
}

bool bigStep(CPoint a, CPoint b, int threshold){
	if (b.x == 0 && b.y == 0){
#ifdef DEBUG
		eyemsg_printf("bigStep = TRUE");
		eyemsg_printf("a = (%d, %d), b = (%d, %d)", a.x, a.y, b.x, b.y );
#endif
		return true;
	}
	if (abs(a.x - b.x) > threshold ){
#ifdef DEBUG
		eyemsg_printf("bigStep = TRUE");
		eyemsg_printf("a = (%d, %d), b = (%d, %d)", a.x, a.y, b.x, b.y );
#endif
		return true;
	}
#ifdef DEBUG
	eyemsg_printf("bigStep = FALSE");
	eyemsg_printf("a = (%d, %d), b = (%d, %d)", a.x, a.y, b.x, b.y );
#endif
	return false;
}

void update_samples(SampleBlock *last_ten_samples, CPoint pt, int time){
	for (int i = 0; i<9; i++){
		last_ten_samples->a[i].gx = last_ten_samples->a[i+1].gx;
		last_ten_samples->a[i].gy = last_ten_samples->a[i+1].gy;
		last_ten_samples->a[i].time = last_ten_samples->a[i+1].time;
	}
	last_ten_samples->a[9].gx = pt.x;
	last_ten_samples->a[9].gy = pt.y;
	last_ten_samples->a[9].time = time;
	return;
}

bool withinXofRect(CPoint pt, CRect r, int X){
	if (r.PtInRect(pt))
		return true;
	r.InflateRect(X,X);
	if (r.PtInRect(pt))
		return true;
	return false;
}


bool deviationGreaterThan(CPoint a, CPoint b, CPoint c, CPoint d, int t){
	long xAvg = (a.x + b.x + c.x + d.x)/4;
	long yAvg = (a.y + b.y + c.y + d.y)/4;
	//eyemsg_printf("xAvg = %d, yAvg = %d", xAvg, yAvg);

	if (abs(a.x - xAvg) > t)
		return true;
	if (abs(b.x - xAvg) > t)
		return true;
	if (abs(c.x - xAvg) > t)
		return true;
	if (abs(d.x - xAvg) > t)
		return true;

	if (abs(a.y - yAvg) > t)
		return true;
	if (abs(b.y - yAvg) > t)
		return true;
	if (abs(c.y - yAvg) > t)
		return true;
	if (abs(d.y - yAvg) > t)
		return true;

	return false;

}
bool inFixationMode(SampleBlock ss){
	if (deviationGreaterThan(
			CPoint(ss.a[9].gx,ss.a[9].gy), 
			CPoint(ss.a[8].gx,ss.a[8].gy), 
			CPoint(ss.a[7].gx,ss.a[7].gy),
			CPoint(ss.a[6].gx,ss.a[6].gy), 3))
		return false;
#ifdef DEBUG
	//eyemsg_printf("last 4: %d, %d, %d, %d", ss.a[9].gx, ss.a[8].gx, ss.a[7].gx, ss.a[6].gx); 
#endif
	return true;
}

int CExperimentDlg::RunTrial(/*const*/ Trial &t, const TrialType &tc, int wt) throw (CExperimentException)
{
	CAudioCapture aCapture;
	CAudioRender aRender;
	CBitmapFile bmp1, bmp2, bmp3;
	CRect cursorRect(0, 0, 0, 0), gazeRect(0, 0, 0, 0);
	UINT32 cursorUpdate=0, displayUpdate=0;
	UINT32 curTime, timeout;
	int button, error = 0, result;
	std::ostringstream os;
#ifdef USE_SERIAL_DATA
	Serial sPort;
#endif

	t.resetSingleChangeTRP();
	                                             /** PREPARE FOR TRIAL **/
	CEyelink2::Flush();                          //clear any pending messages

	if(t.questionAnswer() != Tokenizer::NO_BUTTON && t.questionAnswer() != Tokenizer::MOUSE) {
		button = static_cast<int>(t.questionAnswer());
		eyemsg_printf("QUESTION_ANSWER %d", button);
	}
	else 
		button = 0;

/*	if(tc.doDriftCorrection()) {                //PERFORM DRIFT CORRECTION
#ifdef DO_OFFSET_DRIFT_CORRECT
		CRect fakeWndSz = CRect(0,0,
			MulDiv( tc.textFormat()->xOffset *2, _dc.GetDeviceCaps(LOGPIXELSX), 72) + 25,
			MulDiv(	tc.textFormat()->yOffset *2, _dc.GetDeviceCaps(LOGPIXELSY), 72) + 50);
		int result = CEyelink2::DriftCorrect( fakeWndSz );
#endif
#ifndef DO_OFFSET_DRIFT_CORRECT
		int result = CEyelink2::DriftCorrect(_wndSz);
#endif
		if(result > 0) return(ABORT_EXPT);
	}*/

	if(tc.doDelay())                             //perform NOGAZE delay
		CEyelink2::Delay(tc.stimulusDelay());

	int modeResult = TRUE;// = sendMode( _filterMode);
	int warning = 0;
	while( modeResult && warning < 100){
		modeResult = sendMode(_filterMode);
		warning++;
	}
	//eyemsg_printf("DEBUG: warning = %d", warning);
	if( modeResult )				// need to set the link filter 
	{											// before each recording session
		os<<"Failed to set heuristic filter level. Error Code: "<<modeResult<<std::endl;
		throw (CExperimentException(os.str().c_str() ));
	}

	if(tc.doMouseResponse()){
		CPoint pt;
		while(CEyelink2::TestForMouseClick(pt) > 0); // wait for the mouse to be free before starting recording
	}

	while( CEyelink2::TestForButtonPress(reinterpret_cast<int*>(tc.buttons()), 
												   tc.nButtons())); // wait for the buttons to be free

	//bitmap_to_backdrop(_bitmap, 0, 0, 0, 0, 0, 0, BX_MAXCONTRAST | BX_AVERAGE);

                                                 //initialize data recording
	/*result = CEyelink2::StartRecording(tc.doStreamOutput());
	if(result != 0)
		return(result);*/

	gazeRect = DrawStimulus(t, tc, bmp1, false);        //draw stimulus images to bitmaps
	if(t.isFixedDisplay() == false){
		DrawStimulus(t, tc, bmp2, true);
		//if(t.isFastPrime()){
		if(t.hasThree()){
			DrawStimulus3(t, tc, bmp3, true);
		}
	}

#ifdef INCLUDE_EYELINK
	if(_isExperiment){
		set_offline_mode();
		bitmap_to_backdrop(bmp1, 0, 0, 0, 0, 0, 0, BX_MAXCONTRAST | BX_AVERAGE);
	}
#endif


	if(tc.doDriftCorrection()) {                 /** PERFORM DRIFT CORRECTION **/
#ifdef DO_OFFSET_DRIFT_CORRECT
		CRect fakeWndSz = CRect(0,0,
			MulDiv( tc.textFormat()->xOffset *2, _dc.GetDeviceCaps(LOGPIXELSX), 72) + 25,
			MulDiv(	tc.textFormat()->yOffset *2, _dc.GetDeviceCaps(LOGPIXELSY), 72) + 50);
		int result = CEyelink2::DriftCorrect( fakeWndSz );
#endif
#ifndef DO_OFFSET_DRIFT_CORRECT
		int result = CEyelink2::DriftCorrect(_wndSz);
#endif
		if(result > 0) return(ABORT_EXPT);
	}


	result = CEyelink2::StartRecording(tc.doStreamOutput());

#ifdef USE_SERIAL_DATA
	CString returnValue;
	int openStart = GetCurrentTime();
	sPort.OpenPort(returnValue);
	sPort.StartDataRecording(25000);
	int openFinish = GetCurrentTime();
	CEyelink2::Delay(1000);
#endif

	if(result != 0)
		return(result);

	if(tc.doGazeContingentStart()) {             /** GAZE-CONTINGENT TRIAL START **/
		CBitmap gcBmp;
		int result;

		//CreateGCBitmap(&gcBmp, gazeRect, _backColor, _foreColor);

		// set logGazeRect to point coordinated version of gazeRect (which is in pixels)
		CRect logGazeRect;
		logGazeRect.left = MulDiv(gazeRect.left, _dc.GetDeviceCaps(LOGPIXELSX), 72);
		logGazeRect.right = MulDiv(gazeRect.right, _dc.GetDeviceCaps(LOGPIXELSX), 72);
		logGazeRect.top = MulDiv(gazeRect.top, _dc.GetDeviceCaps(LOGPIXELSY), 72);
		logGazeRect.bottom = MulDiv(gazeRect.bottom, _dc.GetDeviceCaps(LOGPIXELSY), 72);

		//float sysRefresh = (float)_dc.GetDeviceCaps(VREFRESH);
		//eyemsg_printf("DEBUG SYSRATE _DC %1.2f Hz.", sysRefresh);

		//if(t.gcRect().IsRectNull())
			CreateGCBitmap(&gcBmp, gazeRect, _backColor, _foreColor);
		//else
		//	CreateGCBitmap(&gcBmp, logGazeRect, _backColor, _foreColor);

		UpdateDisplay(&gcBmp, 0, _wndSz);        //copy gc image to display bitmap
		ShowDisplay(_wndSz, true);               //draw fixation point to screen
		eyemsg_printf("GAZE TARGET ON");

		//result = WaitForFixation(t, tc, gcBmp, gazeRect);
		//if(t.gcRect().IsRectNull())
			result = WaitForFixation(t, tc, gcBmp, gazeRect);
	//	else
			//result = WaitForFixation(t, tc, gcBmp, logGazeRect);
//

#ifdef USE_SERIAL_DATA
		CEyelink2::Delay(2000);
#endif
		ClearDisplay(_backColor);
		ShowDisplay(_wndSz, true);
		eyemsg_printf("GAZE TARGET OFF");

#ifdef USE_SERIAL_DATA
		sPort.getData(); 
		sPort.ClosePort();
		int first = sPort.getFirst();
		int last = sPort.getLast();
#endif

		if(result != TRIAL_OK)
			return(result);                      //abort if no fixation

		CEyelink2::Flush();                      //clear any pending messages
	
	}
	
	if(tc.audioFormat() != 0) {                  /** START AUDIO CAPTURE **/
		aCapture.Initialize(*(tc.audioFormat()));
		aCapture.Start();
		eyemsg_printf("AUDIO CAPTURE ON");
	}

	if(t.hasAudio()) {                           /** PREPARE AUDIO PLAYBACK **/
		aRender.Initialize(const_cast<Trial&>(t).audioData()); //init player
		aRender.Start(CEyelink2::GetCurrentTime());
		if(tc.nButtons() == 0)                   //if no buttons, stop with audio
			timeout = aRender.StopTime();
		else timeout = CEyelink2::GetCurrentTime() + t.maxDisplayTime();
	}
	else timeout = CEyelink2::GetCurrentTime() + t.maxDisplayTime();

	if(tc.cursorSize() > 0) {                    //update cursor marker (if needed)
		cursorUpdate = CEyelink2::GetCurrentTime() + 60;
		gazeRect.SetRect(0, 0, 0, 0);
	}
	else cursorUpdate = timeout + 1000;

	if(tc.doMouseResponse())                     //if a mouse trial, turn on cursor
		while(ShowCursor(TRUE) < 0);

	UpdateDisplay(&bmp1, &bmp2, _wndSz);
	ShowDisplay(_wndSz, true);                   // ***DISPLAY STIMULUS***
	eyemsg_printf("DISPLAY ON");                 //record start time
	eyemsg_printf("SYNCTIME");                   //mark zero-plot time for EDFVIEW
	curTime = CEyelink2::GetCurrentTime();
	CPoint lastGazeLocation = (0,0);
	SampleBlock a;
	SampleBlock *last_ten_samples = &a;
	ToggleEvent currentToggle;
	TimedRectPairEvent currentRectPairEvent;
	TimedRectPairEvent lastRectPairEvent;

	currentToggle.active = false;
	currentRectPairEvent.active = false;
	lastRectPairEvent.active = false;

	for (int i = 0; i<10; i++) {			
		last_ten_samples->a[i].gx=0;
		last_ten_samples->a[i].gy=0;
		last_ten_samples->a[i].time=0;
	}
	bool completed = false, reverted = false, waiting = false, aborted = false;
	CRect started;
	started.SetRect(0,0,0,0);
	bool tryNewMovingWindowStyle = !t.displayData().toggleWords.empty(), newFlicker = true;
	UINT32 revertSecondWord = 0;

	UINT32 lastFixTime = 0, newFlickerTime = 0;
	CPoint lastFixLoc = (0,0);
	CRect prevRect = (0,0,0,0);
	while(curTime < timeout) {                   // ***MAIN PROCESSING LOOP***
		switch(aRender.Poll(curTime)) {          //test audio rendering
		case CAudioRender::RENDER_START:
			eyemsg_printf("AUDIO PLAYBACK ON");
			break;
		case CAudioRender::RENDER_STOP:
			eyemsg_printf("AUDIO PLAYBACK OFF");
			break;
		}

		if(curTime >= cursorUpdate) {            //update the cursor (if needed)
			CRect updateRect(cursorRect);

			cursorRect = UpdateCursor(&bmp1, &bmp2, cursorRect, gazeRect, tc.cursorSize());
			updateRect |= cursorRect;
			ShowDisplay(updateRect, false);
			cursorUpdate = CEyelink2::GetCurrentTime() + 60;
		}

		error = CEyelink2::TestStatus(true);     //check for error or abort
		if(error != TRIAL_OK)
			break;

		if(tc.doButtonResponse()) {              //check if a button was pressed

			button = CEyelink2::TestForButtonPress(reinterpret_cast<int*>(tc.buttons()), 
												   tc.nButtons());

			lastButton = button;
			/*int myButtons[2];
			myButtons[0] = 7;
			myButtons[1] = 8;
			myButtons[2] = 9;
			button = CEyelink2::TestForButtonPress(myButtons, 3);*/
			if(button > 0) {
				eyemsg_printf("ENDBUTTON %d", button);
				break;
			}
			else if(button < 0) {
				error = ABORT_EXPT;
				break;
			}
		}
		
		if(tc.doMouseResponse()) {               //if mouse on, check messages for clicks
			CPoint pt;

			if(CEyelink2::TestForMouseClick(pt) > 0) {
				eyemsg_printf("MOUSECLICK %d %d", pt.x, pt.y);
				button = Tokenizer::MOUSE;
				break;
			}
		}

		if(t.isFixedDisplay() == false) {        //if this is a display change trial
			aborted = false;
			CPoint pt;
			//CRect prevRect = (0,0,0,0);
			if(CEyelink2::GetGazeLocation(pt) > 0 ) {

 				update_samples(last_ten_samples,pt,0); 
				if(inFixationMode(*last_ten_samples) && lastFixTime < (newFlickerTime-4) ){
					lastFixTime = curTime - 4;
					lastFixLoc = pt;
					//eyemsg_printf("in fixation mode at time %u", curTime);
					//HERE
				}

				if(inFixationMode(*last_ten_samples) && currentToggle.active && currentToggle.fixTime == 0){
					currentToggle.fixTime = curTime - 4;
					currentToggle.revertTime = currentToggle.fixTime + currentToggle.td.duration;
					//eyemsg_printf("updated curToggle fixTime %u, %u", currentToggle.fixTime, currentToggle.revertTime);
				}
				bool updated = false;
				bool update_reverted = false;
				CRect r;
				std::vector<CRect> targetList;
				if(!t.isMovingWindow() && !t.isPairedRegionChange() && !t.isTimedPairedRegionChange()){
					r = t.testRegions(pt);
					if (r.bottom < lastFixLoc.y || r.top > lastFixLoc.y || r.left < lastFixLoc.x){
						r = CRect(0,0,0,0);			//Ignore change regions until a fixation is recorded on the line
					}
				}
			
				else if(!t.isMovingWindow() && t.isTimedPairedRegionChange()){

					TIMED_RECT_PAIR trp = t.testTimedRegions(pt);
					r = trp.r1; // always zero-rect when pt not in a timed region

				if ((t.isTimedPairedRegionChange() && 
					currentRectPairEvent.active && 
					currentRectPairEvent.revertTime < curTime &&
					currentRectPairEvent.revertTime != 0 &&
					currentRectPairEvent.written)  || 
					
					// THIS SECOND SET OF CRITERIA CAUSES REVERT ON LEAVING REGION
					(tc.doDisplayRevert() && 
					currentRectPairEvent.active && 
					currentRectPairEvent.written && 
					t.testTimedRegions(pt).r1 != currentRectPairEvent.trp.r1 )){
						if(t.isFastPrime()){
							UpdateDisplay(&bmp3, &bmp2, currentRectPairEvent.trp.r2data.region);
						}
						else UpdateDisplay(&bmp1, &bmp2, currentRectPairEvent.trp.r2data.region);
						update_reverted = true;
						updated = true;
						eyemsg_printf("DISPLAY CHANGE REVERTED");
						currentRectPairEvent.active = false;
						currentRectPairEvent.completed = true;
				}

				if (t.hasThree() &&
					t.isTimedPairedRegionChange() &&
					currentRectPairEvent.active &&
					currentRectPairEvent.secondRevertTime < curTime &&
					currentRectPairEvent.secondRevertTime != 0 &&
					currentRectPairEvent.wrote2){
						UpdateDisplay(&bmp2, &bmp3, currentRectPairEvent.trp.r2data.region);
						update_reverted = true;
						updated = true;
						eyemsg_printf("DISPLAY CHANGE REVERTED s3->s2");
						//currentRectPairEvent.active = false;
						//currentRectPairEvent.completed = true;
					}


				// *****************************
				// ONLY UPDATE THE CRP if we've change rects (r != gazeRect)
				// *****************************
				if( !r.IsRectNull() &&  r != gazeRect && 
					(!currentRectPairEvent.active || (currentRectPairEvent.trp.r1 != t.testTimedRegions(pt).r1)) ){
				//if (!r.IsRectNull() && currentRectPairEvent.trp.r1 != t.testTimedRegions(pt).r1){
						if (currentRectPairEvent.trp.r1 != t.testTimedRegions(pt).r1)
							lastRectPairEvent = currentRectPairEvent;

						currentRectPairEvent.active = true;
#ifdef DEBUG
						eyemsg_printf("set CRPE.active");
#endif
						currentRectPairEvent.trp = trp;
						currentRectPairEvent.completed = false;
						currentRectPairEvent.displayChangeTime = 0;
						currentRectPairEvent.secondDCtime = 0;
						currentRectPairEvent.eventTime = curTime;
						currentRectPairEvent.fixTime = 0;
						currentRectPairEvent.revertTime = 0;
						currentRectPairEvent.secondRevertTime = 0;
						currentRectPairEvent.written = false;
						currentRectPairEvent.wrote2 = false;
					}
				}

				else if(!t.isMovingWindow() && t.isPairedRegionChange() && !t.isTimedPairedRegionChange()){
					r = t.getCorrespondingRegion(pt);
				}
				else if (t.isMovingWindow()){
					targetList = t.getCorrespondingRegions(pt);
					//r = CRect(0,0,0,0) ;
					//r = targetList.back();
					r = t.testRegions(pt);	
				}
				else r = CRect (0,0,0,0);
#ifdef PIC_WINDOW
			/*	if (1==1){
					r = CRect(CPoint(pt.x-50, pt.y-50), CPoint(pt.x+50, pt.y+50));
				}
				r = CRect(CPoint(pt.x-200, pt.y-50), CPoint(pt.x-100, pt.y+50));*/
				if( t.isScotoma() ){
					CRect s = t.getScotomaRegion();
					r = CRect(pt + s.TopLeft(), pt + s.BottomRight());
				}
#endif

				//if (tc.displayChangeDelay() != 0 && !inFixationMode(*last_ten_samples))
				//	r == CRect(0,0,0,0);

				// CAUSES REVERT ON LEAVING HOTZONE FOR lastrectpairevent
				if (tc.doDisplayRevert() && 
					lastRectPairEvent.active &&
					lastRectPairEvent.written &&
					t.testTimedRegions(pt).r1 != lastRectPairEvent.trp.r1){
						UpdateDisplay(&bmp1, &bmp2, lastRectPairEvent.trp.r2data.region);
						update_reverted = true;
						updated = true;
						eyemsg_printf("DISPLAY CHANGE REVERTED");
						lastRectPairEvent.active = false;
						lastRectPairEvent.completed = true;
				}
				
				if (tc.displayChangeDelay()){
					if (!started.IsRectNull() && r != started){
						if (withinXofRect(pt, prevRect, wt)){
							r = prevRect;
#ifdef DEBUG
							eyemsg_printf("DRIFTING -- RESET to old r");
#endif
						}

					}
				}


				BOOL changedRects = (r != gazeRect /*&& bigStep(pt, lastGazeLocation, wt)*//*|| !contains(targetList, gazeRect)*/);



#ifdef REVERT_TIMED_REGION_CHANGE_ON_EXIT_REGION
				if (changedRects && t.isTimedPairedRegionChange() && currentRectPairEvent.active && currentRectPairEvent.revertTime==0){
					currentRectPairEvent.revertTime = curTime-1;
				}
#endif


				//if( !newFlicker && curTime >= revertSecondWord && tryNewMovingWindowStyle){
				// !newFlicker indicates that we're between preview display on and preview display reverting 
				//}



				// best previous flicker
				//if( !newFlicker && curTime >= (lastFixTime + 300) /*&& lastFixTime > newFlickerTime */&& tryNewMovingWindowStyle){
				//	for(unsigned int q = 1; q < targetList.size(); q++){
				//		UpdateDisplay(&bmp1, &bmp2, targetList[q]);
				//			newFlicker = true;
				//			update_reverted = true;
				//			updated = true;
				//	}
				//	eyemsg_printf("reverting at 300ms from last fix time %d", lastFixTime);
				//	
				//} 

		/*		if (currentToggle.active){
					TOGGLEDATA td = currentToggle.td;
					CRect reg = td.region;
					eyemsg_printf("toggle %d, %d, %d, %d active", reg.left, reg.top, reg.right, reg.bottom);
					eyemsg_printf("toggle eventTime: %u", currentToggle.eventTime);
					eyemsg_printf("toggle fixTime: %u", currentToggle.fixTime);
					eyemsg_printf("toggle revertTime: %u", currentToggle.revertTime);
				}*/

				if (currentToggle.active && currentToggle.revertTime < curTime && currentToggle.fixTime != 0){
					for(unsigned int q = 1; q < targetList.size(); q++)
						UpdateDisplay(&bmp1, &bmp2, targetList[q]);
					newFlicker = true;
					update_reverted = true;
					updated = true;
					//CRect revR = currentToggle.td.region;
					//eyemsg_printf("DEBUG reverting at %u from last fix time %u, (%d,%d,%d,%d) ", currentToggle.td.duration, currentToggle.fixTime, revR.left, revR.top, revR.right, revR.bottom);
					currentToggle.active = false;
					currentToggle.completed = true;
				}

				// Forces wait for fixation in TIMED RECT PAIR mode
				if( currentRectPairEvent.active && 
					(inFixationMode(*last_ten_samples) || currentRectPairEvent.trp.r2data.delay == 0) &&  
					currentRectPairEvent.displayChangeTime == 0){
						currentRectPairEvent.fixTime = curTime - 4*(inFixationMode(*last_ten_samples));// if no delay, then "fixTime is set as current time
						currentRectPairEvent.displayChangeTime = currentRectPairEvent.fixTime + currentRectPairEvent.trp.r2data.delay;// and then we do the display change at current time
						currentRectPairEvent.secondDCtime = currentRectPairEvent.fixTime + currentRectPairEvent.trp.r2data.delay2;
						currentRectPairEvent.revertTime = inFixationMode(*last_ten_samples)*(currentRectPairEvent.displayChangeTime + currentRectPairEvent.trp.r2data.duration);
						currentRectPairEvent.secondRevertTime = inFixationMode(*last_ten_samples)*(currentRectPairEvent.secondDCtime + currentRectPairEvent.trp.r2data.duration2);
						// note: revertTime stays = 0 if not in fixationMode	
				}





				// triggered for delay=0 changes where duration is timed from fixation
				// ie, above loop was performed because delay == 0
				if(currentRectPairEvent.active &&
					currentRectPairEvent.displayChangeTime &&  // we've already set the DC time
					!currentRectPairEvent.revertTime && // but the revertTime was left at 0 b/c no fixation detected
					inFixationMode(*last_ten_samples) ){ // and we've now got a fixation
						currentRectPairEvent.revertTime = curTime - 4 + currentRectPairEvent.trp.r2data.duration;
					}

				if(t.hasThree() &&
					currentRectPairEvent.active &&
					currentRectPairEvent.secondDCtime &&  // we've already set the DC time
					!currentRectPairEvent.secondRevertTime && // but the revertTime was left at 0 b/c no fixation detected
					inFixationMode(*last_ten_samples) ){ // and we've now got a fixation
						currentRectPairEvent.secondRevertTime = curTime - 4 + currentRectPairEvent.trp.r2data.duration2;
					}

					


				// set the REVERT TIME and the DISPLAY CHANGE TIME
	/*			if (t.isTimedPairedRegionChange() &&
					currentRectPairEvent.active && 
					currentRectPairEvent.displayChangeTime == 0 ){
						currentRectPairEvent.displayChangeTime = currentRectPairEvent.trp.r2data.delay + curTime;
						currentRectPairEvent.revertTime = currentRectPairEvent.displayChangeTime + currentRectPairEvent.trp.r2data.duration;
#ifdef DEBUG			
						eyemsg_printf("set timed DC time and REVERT time");
#endif
					}*/

				// REVERT the display if we've passed the revert time HERE
				if ((t.isTimedPairedRegionChange() && 
					currentRectPairEvent.active && 
					currentRectPairEvent.revertTime < curTime &&
					currentRectPairEvent.revertTime != 0 &&
					currentRectPairEvent.written)  || 
					
					// this secound set of criteria cuases revert on leaving region
					(tc.doDisplayRevert() && 
					currentRectPairEvent.active && 
					currentRectPairEvent.written && 
					t.testTimedRegions(pt).r1 != currentRectPairEvent.trp.r1 )){
						if(t.isFastPrime()){
							UpdateDisplay(&bmp3, &bmp2, currentRectPairEvent.trp.r2data.region);
						}
						else UpdateDisplay(&bmp1, &bmp2, currentRectPairEvent.trp.r2data.region);
						update_reverted = true;
						updated = true;
						eyemsg_printf("DISPLAY CHANGE REVERTED -- current");
						currentRectPairEvent.active = false;
						currentRectPairEvent.completed = true;
				}

				if ((t.isTimedPairedRegionChange() && 
					lastRectPairEvent.active && 
					lastRectPairEvent.revertTime < curTime &&
					lastRectPairEvent.revertTime != 0 &&
					lastRectPairEvent.written)  || 
					
					// this second set of criteria causes revert on leaving hotzone
					(tc.doDisplayRevert() && 
					lastRectPairEvent.active && 
					lastRectPairEvent.written && 
					t.testTimedRegions(pt).r1 != lastRectPairEvent.trp.r1 )){
						if(t.isFastPrime()){
							UpdateDisplay(&bmp3, &bmp2, lastRectPairEvent.trp.r2data.region);
						}
						else UpdateDisplay(&bmp1, &bmp2, lastRectPairEvent.trp.r2data.region);
						update_reverted = true;
						updated = true;
						eyemsg_printf("DISPLAY CHANGE REVERTED");
						lastRectPairEvent.active = false;
						lastRectPairEvent.completed = true;
				}


				// write the bitmap to the display (not written to screen until if(updated) loop below)
				if (/*changedRects && *//*  r != prevRect &&*/
					t.isTimedPairedRegionChange() &&
					currentRectPairEvent.active && 
					currentRectPairEvent.displayChangeTime != 0 &&
					curTime  > currentRectPairEvent.displayChangeTime &&
					!currentRectPairEvent.written){
						eyemsg_printf("DISPLAY CHANGE STARTED");
#ifdef DEBUG
						eyemsg_printf("1");
#endif
						UpdateDisplay(&bmp2, &bmp1, currentRectPairEvent.trp.r2data.region);
						// this (bool = true) sends message to not allow this particular RP change again
						t.testTimedRegions(currentRectPairEvent.trp.r1.CenterPoint(), true);
						//t.testTimedRegions(currentRectPairEvent.trp.r1.CenterPoint(), false);
						currentRectPairEvent.written = true;
						updated = true;
				}

				if (/*changedRects &&*/ /*r != prevRect &&*/
					t.hasThree() &&
					t.isTimedPairedRegionChange() &&
					currentRectPairEvent.active && 
					currentRectPairEvent.secondDCtime != 0 &&
					curTime  > currentRectPairEvent.secondDCtime &&
					!currentRectPairEvent.wrote2){
						eyemsg_printf("DISPLAY CHANGE STARTED s2->s3");
						UpdateDisplay(&bmp3, &bmp1, currentRectPairEvent.trp.r2data.region);
						currentRectPairEvent.wrote2 = true;
						currentRectPairEvent.trp.written = true;
						updated = true;
				}
				


                //revert bitmap after gaze leaves hot zone
				//here WT is the "windowing threshold" set in the Edit Experiment Dialog as Display Change Threshold
				//Note that we won't revert unless gaze position has moved at least WT pixels since last sample
				if(!started.IsRectNull() && (prevRect != r) && changedRects && tc.doDisplayRevert() && !gazeRect.IsRectNull() && bigStep(pt, lastGazeLocation, wt) && !t.isTimedPairedRegionChange()) {
					UpdateDisplay(&bmp1, &bmp2, _wndSz /*gazeRect*/); // TODO isOK?
					updated = true;
					eyemsg_printf("DISPLAY CHANGE REVERTED");
					started.SetRect(0,0,0,0);
					update_reverted = true;
				}
#ifdef PIC_WINDOW
				if(t.isScotoma() && changedRects){   // Allows smooth pursuit of scotoma, updated every refresh cycle
					UpdateDisplay(&bmp1, &bmp2, _wndSz /*gazeRect*/); // TODO isOK?
					updated = true;
					eyemsg_printf("DISPLAY CHANGE REVERTED");
					started.SetRect(0,0,0,0);
					update_reverted = true;
				}
#endif
				                                //make initial display change to bitmap
				if (!started.IsRectNull() && r != started && tc.displayChangeDelay() && !t.isTimedPairedRegionChange()){
					started.SetRect(0,0,0,0);
					eyemsg_printf("DISPLAY CHANGE ABORTED");
					gazeRect = r;
					//prevRect = r;
					displayUpdate = 0;
					r.SetRect(0,0,0,0);
					waiting = false;
					aborted = true;
				}

				if(!t.isTimedPairedRegionChange() && ((changedRects && !r.IsRectNull() && r != prevRect) || (started.IsRectNull() && waiting)) || tc.displayChangeDelay() ){
					newFlicker = true;
					int delayStart=0;
					curTime = CEyelink2::GetCurrentTime();
					if(displayUpdate == 0 || displayUpdate == -1) {     //set the update time (may be delayed)
						//displayUpdate = curTime + tc.displayChangeDelay();
						CPoint fixAvg = (0,0);

						int timeInFixation = 0;

						if (tc.displayChangeDelay() == 0){
							displayUpdate = curTime;
//							eyemsg_printf("DISPLAY CHANGE STARTED -- NO DELAY");
							eyemsg_printf("DISPLAY CHANGE STARTED");
#ifdef DEBUG
						eyemsg_printf("2");
#endif
							started = r;
							completed = false;
						}
						else if (inFixationMode(*last_ten_samples) && waiting /*&& started.IsRectNull()*/){
							displayUpdate = curTime + tc.displayChangeDelay();
							eyemsg_printf("DISPLAY CHANGE STARTED");
#ifdef DEBUG
						eyemsg_printf("3");
#endif
							started = r;
							completed = false;
							waiting = false;
						}
						
						else if(!r.IsRectNull() && !inFixationMode(*last_ten_samples)) {

							//eyemsg_printf("DC WAITING FOR FIXATION FOR START");
							completed = false;
							waiting = true;
							displayUpdate = -1;
						}
						else waiting = false;
						//eyemsg_printf("DEBUG :: DISPLAY CHANGE RECT = %d,%d,%d,%d",r.left,r.top,r.right,r.bottom);
					/*	else {
							delayStart = CEyelink2::GetFirstFixation(pt);
							//eyemsg_printf("DELAYSTART = %d", delayStart);
						
							if (delayStart > 0) {
								//eyemsg_printf("fixAvg = (%d,%d)", fixAvg.x, fixAvg.y);
								//displayUpdate = curTime + tc.displayChangeDelay() - timeInFixation;
								displayUpdate = curTime + delayStart + tc.displayChangeDelay();
								eyemsg_printf("DISPLAY CHANGE STARTED");
								//eyemsg_printf("DISPLAY CHANGE STARTED -- DELAY = %d, fixTime = %d, update at %d", tc.displayChangeDelay(), delayStart, displayUpdate);
								//eyemsg_printf("TIME IN FIXATION = %d", timeInFixation);
							}
						
							else {
								displayUpdate = curTime;
								eyemsg_printf("DISPLAY CHANGE STARTED");				
								//eyemsg_printf("DISPLAY CHANGE STARTED -- DEFAULT");
							}
						}*/
					}
					if(t.isTimedPairedRegionChange()){

					}
					if(curTime >= displayUpdate && !waiting && !aborted && !started.IsRectNull() && !t.isTimedPairedRegionChange()) { //update bitmap if delay has passed
						if (tryNewMovingWindowStyle && t.isMovingWindow()){
							if(newFlicker){
								TOGGLEDATA td = t.testPreviewRegions(pt);
								if(!td.region.IsRectNull()){
									for (unsigned int q = 0; q < targetList.size(); q++){
										UpdateDisplay(&bmp2, &bmp1, targetList[q]);
									}
									currentToggle.td = td;
									currentToggle.eventTime = curTime;
									currentToggle.revertTime = 0;
									currentToggle.fixTime = 0;
									currentToggle.active = true;
									currentToggle.completed = false;
									currentToggle.written = false;

									newFlicker = false;
									revertSecondWord = curTime + 300;
									newFlickerTime = curTime;

									//eyemsg_printf("DEBUG :: set current toggle to %d, %d, %d, %d", td.region.left, td.region.top, td.region.right, td.region.bottom);
								}
								else {
									UpdateDisplay(&bmp2, &bmp1, targetList[0]);
								}

							}

						}
						else if (t.isMovingWindow()){
							for (unsigned int q = 0; q < targetList.size(); q++){
								UpdateDisplay(&bmp2, &bmp1, targetList[q]);
							}
						}
						else UpdateDisplay(&bmp2, &bmp1, r);
						updated = true;
						waiting = false;
						//eyemsg_printf("DISPLAY CHANGE COMPLETED");
					}
					/*else{

						eyemsg_printf("curTime= %d, displayUpdate = %d", curTime, displayUpdate);

					}*/
				}
				if(updated) {                    //bitmap changed, so update the actual display
					ShowDisplay(_wndSz, true);
					//eyemsg_printf("DISPLAY UPDATE COMPLETED");
					if(!update_reverted)
						eyemsg_printf("DISPLAY CHANGE COMPLETED");
					displayUpdate = 0;
					gazeRect = r;
					lastGazeLocation = pt;		// used to prevent flicker for Moving Windows
												  // and display change trials with revert
					//started.SetRect(0,0,0,0);
					completed = true;
					waiting = false;
					reverted = false;

					if(currentToggle.completed && !currentToggle.active && !currentToggle.written)
						currentToggle.written = true;
				}
				prevRect = r;
			}
#ifdef USE_SERIAL_DATA		
			sPort.ClosePort();
#endif
			
		}
		curTime = CEyelink2::GetCurrentTime();
	}

	ClearDisplay(_backColor);                    //clear subject display
	ShowDisplay(_wndSz, true);
	eyemsg_printf("DISPLAY OFF");

	if(tc.audioFormat()) {                       //stop recording and save audio data
		aCapture.Stop();
		
		bool wavSaved = aCapture.WriteData(tc.audioFormat()->path, t.identifier().idString());
	}
	
	aRender.Stop();

	if(error) return(error);
                                                 //0 = TIMEOUT, otherwise response key/button
	eyemsg_printf("TRIAL_RESULT %d", button);
	//CEyelink2::StopRecording();
	//return(TRIAL_ERROR);
	return(CEyelink2::StopRecording());
}

void CExperimentDlg::ShowDisplay(const CRect &displayRgn, bool updateEyelinkPC)
{
	CBitmap *oldBmp = _dc.SelectObject(&_bitmap);
	CDC *pWndDC = GetDC();

	if(_displayType != Tokenizer::LCD){
		CEyelink2::WaitForRefresh();
	}

	pWndDC->BitBlt(displayRgn.left, displayRgn.top, displayRgn.Width(), displayRgn.Height(),
			       &_dc, displayRgn.left, displayRgn.top, SRCCOPY);
	if(_displayType != Tokenizer::LCD){
		CEyelink2::WaitForRefresh();

	}

	_dc.SelectObject(oldBmp);
	ReleaseDC(pWndDC);

#ifdef INCLUDE_EYELINK
	if(_isExperiment && updateEyelinkPC){
	//	bitmap_to_backdrop(_bitmap, 0, 0, 0, 0, 0, 0, BX_MAXCONTRAST | BX_AVERAGE);

	}
#endif
}


CRect CExperimentDlg::UpdateCursor(CBitmap *bgd, CBitmap *fgd, CRect &oldRect, 
								   CRect &gazeRect, int size)
{
	CPoint newGazePt(0, 0);
	CRect newRect(0, 0, 0, 0);

	if(CEyelink2::GetGazeLocation(newGazePt) > 0) {
		CBrush fgBrush(_foreColor);
		CBrush *pOldBrush = _dc.SelectObject(&fgBrush);
		CBitmap *pOldBitmap = _dc.SelectObject(&_bitmap);
		CDC bmpDC;

		newRect.SetRect(-size/2, -size/2, size/2, size/2);
		newRect.OffsetRect(newGazePt);

		bmpDC.CreateCompatibleDC(&_dc);
		if(gazeRect.PtInRect(newGazePt))
			bmpDC.SelectObject(fgd);
		else
			bmpDC.SelectObject(bgd);

		_dc.BitBlt(oldRect.left, oldRect.top, oldRect.Width(), oldRect.Height(), 
				   &bmpDC, oldRect.left, oldRect.top, SRCCOPY);
		_dc.Ellipse(newRect);                    //render the gaze cursor

		_dc.SelectObject(pOldBitmap);
		_dc.SelectObject(pOldBrush);

		return(newRect);
	}

	return(oldRect);
}

void CExperimentDlg::UpdateDisplay(CBitmap *bgd, CBitmap *fgd, CRect &updateRect)
{
	CDC tmpDC;

	tmpDC.CreateCompatibleDC(&_dc);
	tmpDC.SelectObject(bgd);

	_dc.SelectObject(&_bitmap); // CHANGED
	
	_dc.BitBlt(updateRect.left, updateRect.top, updateRect.Width(), updateRect.Height(),
			   &tmpDC, updateRect.left, updateRect.top, SRCCOPY);

}
/*
void update(CPoint a, CPoint b, CPoint c, CPoint d, float x, float y){
	a.x = b.x;
	a.y = b.y;

	b.x = c.x;
	b.y = c.y;

	c.x = d.x;
	c.y = d.y;
	d.x = static_cast <LONG> (x);
	d.y = static_cast <LONG> (y);

	return;
}

bool deviationGreaterThan(CPoint a, CPoint b, CPoint c, CPoint d, int t){
	long xAvg = (a.x + b.x + c.x + d.x)/4;
	long yAvg = (a.y + b.y + c.y + d.y)/4;
	
	if (abs(a.x - xAvg) > t)
		return true;
	if (abs(b.x - xAvg) > t)
		return true;
	if (abs(c.x - xAvg) > t)
		return true;
	if (abs(d.x - xAvg) > t)
		return true;

	if (abs(a.y - yAvg) > t)
		return true;
	if (abs(b.y - yAvg) > t)
		return true;
	if (abs(c.y - yAvg) > t)
		return true;
	if (abs(d.y - yAvg) > t)
		return true;

	return false;

}*/

/*
int CExperimentDlg::GetFirstFixation(CRect r, Trial &t){
#ifdef INCLUDE_EYELINK
	int time = 0;
	CPoint a=(0,0),b=(0,0),c=(1000,1000),d=(1000,1000);
	while( a.x == 0){ //deviationGreaterThan (a,b,c,d, 1)){
		if(eyelink_newest_float_sample(NULL) > 0){
			ALLF_DATA sample;
			
			eyelink_newest_float_sample(&sample);
			update(a,b,c,d, sample.fs.gx[1], sample.fs.gy[1]);
			eyemsg_printf("A = %d,%d", a.x, a.y);
			eyemsg_printf("B = %d,%d", b.x, b.y);
			eyemsg_printf("C = %d,%d", c.x, c.y);
			eyemsg_printf("D = %d,%d", d.x, d.y);

			time = sample.fs.time;
		}
		else return (-2);
	}
	return time;
#endif
	return (-1);
}
*/

/*int CExperimentDlg::WaitForDCFixation(CRect &fixRgn, int curTime)
{
	int duration;
	CPoint pt(0,0);
	bool gotFixation = false;
	int fixTime = 0;
	curTime = current_msec();
	while (fixTime != -1 ){
		//duration = CEyelink2::TestForFixation(pt);
		fixTime = CEyelink2::GetNextFixUpdate(pt, &duration);
		if (fixTime > 0){

			eyemsg_printf("DEBUG: GOT FIXATION POINT (%d,%d) at time %d", pt.x, pt.y, fixTime);
			eyemsg_printf("DEBUG: LOOKING FOR FIXTIME AFTER %d", curTime);
			eyemsg_printf("DEBUG: USING FIXRGN x=(%d,%d), y=(%d,%d)", 
							fixRgn.left,
							fixRgn.right,
							fixRgn.top,
							fixRgn.bottom);

			gotFixation = true;
			//if(fixRgn.PtInRect(pt)){
				//return 1;
			//else return 0;
		}

		else {
#ifdef DEBUG
			eyemsg_printf("DEBUG: GetNextFixUpdate returned %d", fixTime);
#endif
		}
	}
	//eyemsg_printf("DEBUG: fixRgn.PtInRct(pt) = FALSE");
	return(-1);
}	*/


int CExperimentDlg::WaitForFixation(const Trial &t, const TrialType &tc, CBitmap &bmp, CRect &fixRgn)
{
	CPoint fixCenter(fixRgn.CenterPoint());
	CRect  cursorRect(0, 0, 0, 0);
	unsigned int curTime, cursorUpdate;
	int fixMeanX = 0, fixMeanY = 0, fixDuration = 0;
	int eyeUsed = -1;

	if(_isExperiment == false)                   //turn on mouse pointer if this is a test
		while(ShowCursor(TRUE) < 0);

	curTime = CEyelink2::GetCurrentTime();
	if(tc.cursorSize() > 0)                      //set cursor update time (if needed)
		cursorUpdate = curTime + 60;
	else cursorUpdate = 1073741824;              //2^30

	while(fixDuration < START_GAZE_TRIAL_THRESHOLD) {
		CPoint pt(0, 0);
		int duration, status = CEyelink2::TestStatus(true);

		if(status != TRIAL_OK)
			return(status);

		if(curTime > cursorUpdate) {             //update cursor position (if needed)
			CRect updateRect(cursorRect);

			cursorRect = UpdateCursor(&bmp, 0, cursorRect, CRect(0, 0, 0, 0), tc.cursorSize());
			updateRect |= cursorRect;
			ShowDisplay(updateRect, false);
			cursorUpdate = curTime + 60;
		}

		duration = CEyelink2::TestForFixation(pt);
		if(duration > 0) {
			if(fixRgn.PtInRect(pt)) {
				fixDuration += duration;
				fixMeanX += pt.x * duration;
				fixMeanY += pt.y * duration;
			}
			else {
				fixDuration = 0;
				fixMeanX = fixMeanY = 0;
			}
		}
		curTime = CEyelink2::GetCurrentTime();
	} //end while
	
	fixMeanX /= fixDuration;
	fixMeanY /= fixDuration;
#ifdef DEBUG
	eyemsg_printf("TRIGGERED FIXATION REGION %d %d %d %d %d",
		          fixMeanX, fixMeanY, fixCenter.x, fixCenter.y, fixDuration);
#endif

	while(ShowCursor(FALSE) >= 0);               //make sure mouse ptr off 

	return(TRIAL_OK);
}
