/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once
#pragma warning( disable : 4290 )

#include "BitmapFile.h"
#include "Experiment.hpp"
#include "Tokenizer.h"
#include "TextPassage.hpp"
#include "Serial.h"

#define START_GAZE_TRIAL_THRESHOLD  300

// CExperimentDlg dialog

typedef struct {
	int time;
	LONG gx, gy;
} MySample;

typedef struct {
	MySample a[10];
} SampleBlock;

typedef struct {
	TOGGLEDATA td;
	unsigned int fixTime;
	unsigned int eventTime;
	unsigned int revertTime;
	bool active;
	bool completed;
	bool written;
} ToggleEvent;

typedef struct {
	TIMED_RECT_PAIR trp;
	unsigned int fixTime;
	unsigned int eventTime;
	unsigned int displayChangeTime;
	unsigned int secondDCtime;
	unsigned int revertTime;
	unsigned int secondRevertTime;
	bool active;
	bool completed;
	bool written;
	bool wrote2;
} TimedRectPairEvent;

class CExperimentDlg : public CDialog
{
	DECLARE_DYNAMIC(CExperimentDlg)

public:
	enum { IDD = IDD_EXPERIMENTDLG };

	CExperimentDlg(CWnd* pParent, const CSize &size, COLORREF back, COLORREF fore, Tokenizer::FilterMode fm);
	virtual ~CExperimentDlg();

	afx_msg void OnChar(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnClose();
	afx_msg void OnKeyDown(UINT nChar, UINT nRepCnt, UINT nFlags);
	afx_msg void OnPaint();
	afx_msg void OnShowWindow(BOOL bShow, UINT nStatus);

	void RunExperiment(Experiment *expt, IdentifierList &trials, const char *exptName=0);

protected:
	CRect DrawStimulus(/*const*/ Trial &t, const TrialType &tc, CBitmapFile &bmp, bool doDisplay2=false);
	CRect DrawStimulus3(Trial &t, const TrialType &tc, CBitmapFile &bmp, bool doDisplay2=false);
	CRect UpdateCursor(CBitmap *bgd, CBitmap *fgd, CRect &oldRect, CRect &gazeRect, int size);

	void updateSamples(MySample **m, CPoint pt, int time);
	int RunTrial(/*const*/ Trial &t, const TrialType &tc, int windowThreshold) throw(CExperimentException);
	int WaitForFixation(const Trial &t, const TrialType &tc, CBitmap &bmp, CRect &fixRgn);
	int WaitForDCFixation(CRect &fixRgn, int curTime);
	int GetFirstFixation(CRect r, Trial &t);
	void ClearDisplay(COLORREF color);
	void CreateGCBitmap(CBitmap *bmp, CRect &gazeBox, COLORREF bg, COLORREF fg);
	void ShowDisplay(const CRect &displayRgn, bool updateEyelinkPC);
	void UpdateDisplay(CBitmap *bgd, CBitmap *fgd, CRect &updateRect);

	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	
	DECLARE_MESSAGE_MAP()

private:
	CDC         _dc;
	CBitmapFile _bitmap;                         //bitmap for trial text
	CRect       _wndSz;                          //dimensions of the window
	CString     _edfFileName;	    			 //name of EDF data file
	COLORREF    _backColor;
	COLORREF    _foreColor;
	Tokenizer::FilterMode	_filterMode;
	bool        _isExperiment;
	bool        _isEyelink2;
	bool        _needsInitialization;
	Tokenizer::DisplayType _displayType;
	int			sendMode(Tokenizer::FilterMode fm);
	int			lastButton;
};
