/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <stdarg.h>
#include "stdafx.h"
#include "Eyelink2.h"
#include "Tokenizer.h"

#ifdef INCLUDE_EYELINK
#include "w32_exptsppt2.h"
#else
int eyecmd_printf(char *format, ...)
{return 0;}

int eyemsg_printf(char *format, ...)
{
	va_list ap;
	std::string str;
	double dVal;
	int i = 0, iVal;
	char cVal, buf[100];

	TRACE("%u: ", CEyelink2::GetCurrentTime());  //write time-stamp

	va_start(ap, format);                        //get list of arguments
	for(char *p = format; *p; ++p) {             //process message char by char
		if(*p != '%') {
			buf[i++] = *p;
			continue;
		}
		else {                                   //write plain text
			buf[i] = '\0';
			TRACE("%s", buf);
			i = 0;
		}

		switch(*++p) {                           //write data arguments
		case 'c':                                //write char
			cVal = va_arg(ap, char);
			TRACE("%c", cVal);
			break;
		case 'd':                                //write int
		case 'i':
			iVal = va_arg(ap, int);
			TRACE("%d", iVal);
			break;
		case 'f':                                //write float/double
			dVal = va_arg(ap, double);
			TRACE("%f", dVal);
			break;
		case 's':                                //write string
			str = va_arg(ap, char*);
			TRACE("%s", str.c_str());
			break;
		default:                                 //some other type
			TRACE("%c", *p);
			break;
		}
	}
	va_end(ap);                                  //free arg list

	buf[i] = '\0';                               //write any remaining chars
	TRACE("%s\n", buf);
	return 0;
}
#endif

/****************************************************************************/

CDirectInput CEyelink2::_di;
int          CEyelink2::_eyeUsed = -1;
bool         CEyelink2::_isTrackMode = false;

CEyelink2::CEyelink2()
{}

CEyelink2::~CEyelink2()
{}

void CEyelink2::Delay(int msec)
{
#ifdef INCLUDE_EYELINK
	if(_isTrackMode) {
		eyemsg_printf("DELAY %d MS", msec);
		msec_delay(msec);
		return;
	}
#endif
	if(msec > 10) {
		unsigned int start = GetCurrentTime();
		unsigned int end = start + msec;
		while(GetCurrentTime() < end);
	}
}

int CEyelink2::DriftCorrect(const CRect &wndSz)
{
#ifdef INCLUDE_EYELINK
	if(_isTrackMode) {
		while(do_drift_correct(wndSz.Width()/2, wndSz.Height()/2, 1, 1) == 27)
			if(!eyelink_is_connected())          //drift correct while user presses ESC
				return(ABORT_EXPT);              //exit if tracker has stopped
	}
#endif
	return(TRIAL_OK);
}

void CEyelink2::Flush()
{
	MSG msg;
                                                 //clear any queued Windows messages
	while(PeekMessage(&msg, NULL, 0, 0, PM_REMOVE) > 0) {
		::TranslateMessage(&msg);
		::DispatchMessage(&msg);
	} 

#ifdef INCLUDE_EYELINK
	if(_isTrackMode) {                           //reset all keys & buttons
		flush_getkey_queue();
		eyelink_flush_keybuttons(0);
	}
#endif
}

unsigned int CEyelink2::GetCurrentTime()
{
#ifdef INCLUDE_EYELINK
	if(_isTrackMode)
		return(current_time());
#endif
	static LARGE_INTEGER freq, start;
	static BOOL result1 = QueryPerformanceFrequency(&freq);
	static BOOL result2 = QueryPerformanceCounter(&start);

	LARGE_INTEGER cur_time;

	QueryPerformanceCounter(&cur_time);
	cur_time.QuadPart -= start.QuadPart;
	cur_time.QuadPart /= (freq.QuadPart / 1000);
	return(cur_time.LowPart);
}



/*int dumpEventData(ALLF_DATA event, CString s){
	char* type = (char*)malloc(20);
	switch(event.fs.type){
		case STARTBLINK:
			type = "STARTBLINK";
			break;
		case ENDBLINK:
			type = "ENDBLINK";
			break;
		case STARTSACC:
			type = "STARTSACC";
			break;
		case ENDSACC:
			type = "ENDSACC";
			break;
		case STARTFIX:
			type = "STARTFIX";
			break;
		case ENDFIX:
			type = "ENDFIX";
			break;
		case FIXUPDATE:
			type = "FIXUPDATE";
			break;
		case MESSAGEEVENT:
			type = "MESSAGEEVENT";
			break;
		case BUTTONEVENT:
			type = "BUTTONEVENT";
			break;
		case INPUTEVENT:
			type = "INPUTEVENT";
			break;
		case SAMPLE_TYPE:
			type = "SAMPLE_TYPE";
			break;
		default:
			type = "NO TYPE FOUND";
	}
	eyemsg_printf("%s: event type=%s, time=%d", s, type,
			event.fs.time);

	return 0;
}*/


int CEyelink2::GetGazeLocation(CPoint &pt)
{
	if(_isTrackMode == false) {
		pt = _di.GetCursorLocation();
		SetCursorPos(pt.x, pt.y);
		return(1);
	}

#ifdef INCLUDE_EYELINK
	if(eyelink_newest_float_sample(NULL) > 0) {  //check for incoming data
		ALLF_DATA event;

		eyelink_newest_float_sample(&event);     //get event data
		//dumpEventData(event, "GETGAZELOC--eyelink_newest_float_sample");
		if(_eyeUsed < 0) {                       //which eye is being used?
			_eyeUsed = eyelink_eye_available();
			if(_eyeUsed == BINOCULAR) _eyeUsed = LEFT_EYE;
		}

		pt.x = static_cast<LONG>(event.fs.gx[_eyeUsed]);
		pt.y = static_cast<LONG>(event.fs.gy[_eyeUsed]);
		if(pt.x == MISSING_DATA || pt.y == MISSING_DATA || event.fs.pa[_eyeUsed] <= 0)
			pt.x = pt.y = -1;
		else return(1);
	}
#endif
	return(0);
}

void update(CPoint *a, CPoint *b, CPoint *c, CPoint *d, long x, long y){
	a->x = b->x;
	a->y = b->y;

	b->x = c->x;
	b->y = c->y;

	c->x = d->x;
	c->y = d->y;
	d->x = x;
	d->y = y;

	return;
}

bool DeviationGreaterThan(CPoint a, CPoint b, CPoint c, CPoint d, int t){
	long xAvg = (a.x + b.x + c.x + d.x)/4;
	long yAvg = (a.y + b.y + c.y + d.y)/4;
	//eyemsg_printf("xAvg = %d, yAvg = %d", xAvg, yAvg);

	if (abs(a.x - xAvg) > t)
		return true;
	if (abs(b.x - xAvg) > t)
		return true;
	if (abs(c.x - xAvg) > t)
		return true;
	if (abs(d.x - xAvg) > t)
		return true;

	if (abs(a.y - yAvg) > t)
		return true;
	if (abs(b.y - yAvg) > t)
		return true;
	if (abs(c.y - yAvg) > t)
		return true;
	if (abs(d.y - yAvg) > t)
		return true;

	return false;

}
int CEyelink2::GetFirstFixation(CPoint &pt)
{
	if(_isTrackMode == false) {
		pt = _di.GetCursorLocation();
		SetCursorPos(pt.x, pt.y);
		return(1);
	}
#ifdef INCLUDE_EYELINK
	int time = 0, x = 0;
	CPoint a=(0,0),b=(0,0),c=(1000,1000),d=(1000,1000);
	UINT32 startTime = GetCurrentTime();
	while( DeviationGreaterThan (a,b,c,d, 3)){
		//int available = eyelink_newest_float_sample(NULL);
		//eyemsg_printf("available = %d", available);
		while(eyelink_newest_float_sample(NULL) <= 0){}
		if ( true /*available*/ ){//eyelink_newest_float_sample(NULL) > 0){
			ALLF_DATA sample;
			
			eyelink_newest_float_sample(&sample);
			//dumpEventData(sample, "GETFIRSTFIX--newest_float_sample");
			if(_eyeUsed < 0) {                       //which eye is being used?
				_eyeUsed = eyelink_eye_available();
				if(_eyeUsed == BINOCULAR) _eyeUsed = LEFT_EYE;
			}
			pt.x = static_cast<LONG>(sample.fs.gx[_eyeUsed]);
			pt.y = static_cast<LONG>(sample.fs.gy[_eyeUsed]);
			if(pt.x == MISSING_DATA || pt.y == MISSING_DATA || sample.fs.pa[_eyeUsed] <= 0)
				pt.x = pt.y = -1;
			update(&a,&b,&c,&d, pt.x, pt.y);
			/*eyemsg_printf("A = %d,%d", a.x, a.y);
			eyemsg_printf("B = %d,%d", b.x, b.y);
			eyemsg_printf("C = %d,%d", c.x, c.y);
			eyemsg_printf("D = %d,%d", d.x, d.y);*/
			time = sample.fs.time;
			//return (-1);
		}
	//	else return (-2);
	}
	UINT32 endTime = GetCurrentTime();
	return (endTime - startTime);//time;
#endif
	return (-1);
}


int CEyelink2::GetNextFixUpdate(CPoint &pt, int *duration)
{
	if(_isTrackMode == false) {
		bool result = _di.IsRButtonDown(pt);

		SetCursorPos(pt.x, pt.y);
		return(result ? 1000 : 0);
	}

#ifdef INCLUDE_EYELINK
	int type = eyelink_get_next_data(NULL);
	if(type == FIXUPDATE) {
		ALLF_DATA event;

		if(_eyeUsed < 0) {                       //which eye is being used?
			_eyeUsed = eyelink_eye_available();
			if(_eyeUsed == BINOCULAR) _eyeUsed = LEFT_EYE;
		}

		eyelink_get_float_data(&event);          //get event data
		//dumpEventData(event, "GetNextFixUpdate_A--eyelink_get_next_data");
		if(event.fe.eye == _eyeUsed) {           //make sure have correct eye
			pt.x = static_cast<int>(event.fe.gavx);
			pt.y = static_cast<int>(event.fe.gavy);
			*duration = event.fe.entime - event.fe.sttime;
			return(event.fe.time);
		}
	}
	else if (type == STARTFIX || type == ENDFIX){
		ALLF_DATA event;

		if(_eyeUsed < 0) {                       //which eye is being used?
			_eyeUsed = eyelink_eye_available();
			if(_eyeUsed == BINOCULAR) _eyeUsed = LEFT_EYE;
		}

		eyelink_get_float_data(&event);
		//dumpEventData(event, "GetNextFixUpdate_B--eyelink_get_next_data");
		pt.x = 42; pt.y = 42;
		return (event.fe.time);
	}

	else {
		ALLF_DATA event;
		
		eyelink_get_float_data(&event);
		//dumpEventData(event, "GetNextFixUpdate_C--eyelink_get_next_data");
		if (type == 0)
		return(-1);
	}
	//lse return type;
#endif

	return(0);
}

 /*
#ifdef INCLUDE_EYELINK
	while(eyelink_get_next_data(NULL) != FIXUPDATE){
		ALLF_DATA event;

		eyelink_get_float_data(&event);
		if(_eyeUsed < 0) {
			_eyeUsed = eyelink_eye_available();
			if(_eyeUsed == BINOCULAR) _eyeUsed = LEFT_EYE;
		}
		eyemsg_printf("processing event type %d", static_cast<int>(event.fs.type));
		if (event.fe.type == FIXUPDATE){
			pt.x = static_cast<LONG>(event.fs.gx[_eyeUsed]);
			pt.y = static_cast<LONG>(event.fs.gx[_eyeUsed]);
			if(pt.x == MISSING_DATA || pt.y == MISSING_DATA || event.fs.pa[_eyeUsed] <= 0)
				pt.x = pt.y = -1;
			else return(static_cast<int>(event.fe.entime - event.fe.sttime));
		}
		
	}
	eyemsg_printf("NO NEW DATA ON LINK");
#endif
	return(0);
}*/


void CEyelink2::Initialize(HWND hWnd, CRect &wndSz, CString &edfFile, 
						   COLORREF fg, COLORREF bg, int fm, int calibrationType, int calibrationHeight) throw(CEyelinkException)
{
#ifdef INCLUDE_EYELINK
	if(_isTrackMode) {
		DISPLAYINFO dispinfo;
		int calibTargSz = wndSz.Width() / 60;        //size of calibration and focal targets
		int calibFocalSz = (wndSz.Width() > 600) ? wndSz.Width() / 300 : 2;
		int result;
		bool isEyelink2 = false;

		if(open_eyelink_connection(0))               //abort if cannot connect
			throw(CEyelinkException("Could not open connection to EyeLink"));

		set_offline_mode();                          
		flush_getkey_queue();                        //initialize getkey() system
		if(eyelink_get_tracker_version(NULL) == 2)
			isEyelink2 = true;

		if(init_expt_graphics(hWnd, &dispinfo) != 0) {
			close_eyelink_connection();              //disconnect from tracker
			throw(CEyelinkException("Could not initialize EyeLink graphics system"));
		}

		if(dispinfo.palsize) {                       //color pallate modes: bad
			close_expt_graphics();                   //release window
			close_eyelink_connection();              //disconnect from tracker
			throw(CEyelinkException("EyeTrack Software requires at least a 16 bit color display"));
		}

		if(dispinfo.refresh < 40)                    //refresh rate too slow
			throw(CEyelinkException("Eyetrack Software requires at least 40Hz display refresh rate"));
    
		set_target_size(calibTargSz, calibFocalSz);  //set calibration attributes
		set_calibration_colors(fg, bg);
		set_cal_sounds("", "", "");
		set_dcorr_sounds("", "off", "off");

		result = edit_dialog(hWnd, "Create EDF File", "EyeTrack EDF results file name: ",
							 edfFile.GetBuffer(),32);
		if(!strstr(edfFile.GetBuffer(), ".EDF")){
			//edfFile.Append(".EDF");
			CString test;
			test = "";
			test.Append(edfFile.GetBuffer());
			test.Append(".EDF");
			edfFile = test;
		}

		if(result == -1) {                           //error
			close_expt_graphics();
			close_eyelink_connection();
			throw(CEyelinkException("Error acquiring EDF file name"));
		} else if(result == 1) {                     //Cancelled, no file name
			edfFile.Empty();
			throw(CEyelinkException("EDF file creation cancelled"));
		}

		if(!edfFile.IsEmpty()) {                     //if name given, try to open
				
			result = open_data_file(edfFile.GetBuffer());
			if(result != 0) {                        //if an error
				close_expt_graphics();
				close_eyelink_connection();
				throw(CEyelinkException("Could not open EDF file"));
			}
			else {                                   //OK: write title to preamble
				eyecmd_printf("add_file_preamble_text 'RECORDED BY EYETRACK'");
			}
		}

	/*	INT16	sample_rate=-1, 
				crmode=-1, 
				file_filter=-1, 
				link_filter=-1;
		eyelink2_mode_data( &sample_rate, &crmode, &file_filter, &link_filter);

		eyemsg_printf("PRIOR:	sample_rate = %hd, crmode = %hd, file_filter = %hd, link_filter = %hd",
								sample_rate, crmode, file_filter, link_filter);
*/
		/*int OnResult = -10;
		int OffResult = -10;
		int DefaultResult = -10;

		switch (fm){
			case Tokenizer::FILTER_ON:		OnResult = eyecmd_printf("heuristic_filter = ON");
				break;
			case Tokenizer::FILTER_OFF:		OffResult = eyecmd_printf("heuristic_filter = OFF");
				break;
			default:						DefaultResult = eyecmd_printf("heuristic_filter = OFF");
		}*/
		// too early - mode returns to default on first trial

	/*	sample_rate=-1, 
		crmode=-1, 
		file_filter=-1, 
		link_filter=-1;
		eyelink2_mode_data( &sample_rate, &crmode, &file_filter, &link_filter);

		eyemsg_printf("POST:	sample_rate = %hd, crmode = %hd, file_filter = %hd, link_filter = %hd",
								sample_rate, crmode, file_filter, link_filter);*/
		//eyemsg_printf("ON = %d, OFF = %d, DEF = %d", OnResult, OffResult, DefaultResult);

                                                 //set display properties
		eyecmd_printf("screen_pixel_coords = %ld %ld %ld %ld",
					  dispinfo.left, dispinfo.top, dispinfo.right, dispinfo.bottom);
		//eyecmd_printf("calibration_type = HV9");     //set calibration type
		switch(calibrationType){
			case (Tokenizer::H3):		
				if(calibrationHeight && (calibrationHeight < SCRHEIGHT) && (calibrationHeight > 0)){
					eyecmd_printf("horizontal_target_y = %d", calibrationHeight);
				}
				else{
					eyecmd_printf("horizontal_target_y = %d", SCRHEIGHT/2);
				}
				eyecmd_printf("calibration_type = H3");
				break;
			case (Tokenizer::HV9):		
				eyecmd_printf("calibration_type = HV9");
				break;
			case (Tokenizer::HV13):
				eyecmd_printf("calibration_type = HV13");
				break;
			default:		
				eyecmd_printf("calibration_type = HV9");
				break;	
		}
		                                             //write display properties to EDF
		eyemsg_printf("DISPLAY COORDS %ld %ld %ld %ld",
					  dispinfo.left, dispinfo.top, dispinfo.right, dispinfo.bottom);
		if(dispinfo.refresh > 40)
			eyemsg_printf("FRAMERATE %1.2f Hz.", dispinfo.refresh);

		//float sysRefresh = (float)GetDeviceCaps(NULL,VREFRESH);
		//eyemsg_printf("DEBUG SYSRATE %1.2f Hz.", sysRefresh);

		if(isEyelink2)                               //set parser thresholds (conservative)
			eyecmd_printf("select_parser_configuration 0");
		else {
			eyecmd_printf("saccade_velocity_threshold = 35");
			eyecmd_printf("saccade_acceleration_threshold = 9500");
		}
		                                           //set event & data filters
		eyecmd_printf("file_event_filter = LEFT,RIGHT,FIXATION,SACCADE,BLINK,MESSAGE,BUTTON");
		eyecmd_printf("file_sample_data  = LEFT,RIGHT,GAZE,GAZERES,AREA,STATUS");

			                                         //set link data (used for gaze cursor) 
//		eyecmd_printf("link_event_filter = LEFT,RIGHT,FIXATION,FIXUPDATE,SACCADE,BLINK,BUTTON");

		eyecmd_printf("link_event_filter = LEFT,RIGHT,FIXATION,FIXUPDATE");
		eyecmd_printf("link_sample_data  = LEFT,RIGHT,GAZE,GAZERES,AREA,STATUS");

				                                     //init button #5 for drift correction
		eyecmd_printf("button_function 5 'accept_target_fixation'");

		if(!eyelink_is_connected() || break_pressed()) {
			Shutdown(edfFile);
			throw(CEyelinkException("EyeLink not connected"));
		}

		pump_delay(500);                             //make sure all previous operations are complete

		do_tracker_setup();                          //place EyeLink tracker into Setup mode
	}

#else

	if(_isTrackMode)
		throw(CEyelinkException("Eyelink not available in Mini-Eyetrack."));

#endif

	if(_di.Initialize() == false)
		throw(CEyelinkException("Direct input initialization failed."));
}

void CEyelink2::PrepareForTrial(const char *trialId, int idx, int nTrials, bool streamOutput)
{
#ifdef INCLUDE_EYELINK
	if(_isTrackMode) {
		set_offline_mode();                      //place tracker in idle mode

		if(streamOutput)                         //send regular fixation updates
			eyecmd_printf("file_event_filter = LEFT,RIGHT,FIXUPDATE,FIXATION,SACCADE,BLINK,BUTTON,INPUT,MESSAGE");
		else                                     //do not stream the fixation updates
			eyecmd_printf("file_event_filter = LEFT,RIGHT,FIXATION,SACCADE,BLINK,BUTTON,INPUT,MESSAGE");

		eyecmd_printf("link_event_filter = LEFT,RIGHT,FIXUPDATE, FIXATION");
		eyecmd_printf("fixation_update_interval = 4");
		eyecmd_printf("fixation_update_accumulate = 4");
		                                         //write message on eyelink PC display
		eyecmd_printf("record_status_message 'TRIAL %d/%d     IDENT %s' ", idx, nTrials, trialId);

		eyemsg_printf("TRIALID %s", trialId);    //note trial ID in EDF file

		eyecmd_printf("clear_screen 0");         //clear eyelink PC display
	}
#endif
	_eyeUsed = -1;
	TRACE("TRIALID %s\n", trialId);
}

void CEyelink2::SetTrackMode(bool isTrackMode)
{
	_isTrackMode = isTrackMode;
}

void CEyelink2::Shutdown(CString &edfFile)
{
#ifdef INCLUDE_EYELINK
	if(_isTrackMode) {
		set_offline_mode();                      //make ready to transfer file
		pump_delay(500);                         //delay for tracker
		eyecmd_printf("close_data_file");        //close EDF file

		if(!break_pressed())                     //transfer EDF file
			receive_data_file(edfFile.GetBuffer(), "", 0);

		close_expt_graphics();                   //release window (from eyelink)
		close_eyelink_connection();              //disconnect tracker
	}
#endif

	_di.Terminate();
}

int CEyelink2::StartRecording(bool streamOutput)
{
#ifdef INCLUDE_EYELINK
	if(_isTrackMode) {
		INT16 temp = streamOutput ? 1 : 0;
		INT16 result = start_recording(temp, 1, temp, 1);

		msec_delay(100);
		return(result);
	}
#endif
	return(TRIAL_OK);
}

int CEyelink2::StopRecording()
{
#ifdef INCLUDE_EYELINK
	if(_isTrackMode)
		return(check_record_exit());
#endif
	return(TRIAL_OK);
}

int CEyelink2::TestForButtonPress(const int *button, int nButtons)
{
	int pressed = -1;

	if(_isTrackMode == false) {
		int key = _di.KeyDown();

		if(key == DIK_ESCAPE)
			return(0);

		for(int i=0; i < 8; ++i)                 //check for buttons 1 to 5
			if(key == DIK_1+i) {
				pressed = i+1;
				break;
			}
	}
#ifdef INCLUDE_EYELINK
	else
		pressed = eyelink_last_button_press(NULL);
#endif

	if(pressed) {
		for(int i=0; i < nButtons; ++i)
			if(pressed == button[i])
				return(pressed);
	}
	return(0);
}

int CEyelink2::TestForFixation(CPoint &pt)
{
	if(_isTrackMode == false) {
		bool result = _di.IsRButtonDown(pt);

		SetCursorPos(pt.x, pt.y);
		return(result ? 1000 : 0);
	}

#ifdef INCLUDE_EYELINK
	if(eyelink_get_next_data(NULL) == FIXUPDATE) {
		ALLF_DATA event;

		if(_eyeUsed < 0) {                       //which eye is being used?
			_eyeUsed = eyelink_eye_available();
			if(_eyeUsed == BINOCULAR) _eyeUsed = LEFT_EYE;
		}

		eyelink_get_float_data(&event);          //get event data
		//dumpEventData(event, "TESTFORFIX_A--eyelink_get_next_data");
		if(event.fe.eye == _eyeUsed) {           //make sure have correct eye
			pt.x = static_cast<int>(event.fe.gavx);
			pt.y = static_cast<int>(event.fe.gavy);
			return(event.fe.entime - event.fe.sttime);
		}
	}
	else{
		ALLF_DATA event;
		eyelink_get_float_data(&event);
		//dumpEventData(event, "TESTFORFIX_B--eyelink_get_next_data");
	}
#endif
	return(0);
}



int CEyelink2::TestForMouseClick(CPoint &pt)
{
	bool result = _di.IsLButtonDown(pt);

	SetCursorPos(pt.x, pt.y);
	return(result ? 1 : 0);
}

int CEyelink2::TestStatus(bool isRecording)
{
#ifdef INCLUDE_EYELINK
	if(_isTrackMode) {
		if(break_pressed())
			return(ABORT_EXPT);
		else if(escape_pressed())
			return(SKIP_TRIAL);
		else if(isRecording)
			return(check_recording());
		else if(!eyelink_is_connected())
			return(ABORT_EXPT);
	}
#endif
	int key = _di.KeyDown();

	if(key == DIK_ESCAPE)
		return(ABORT_EXPT);
	return(TRIAL_OK);
}

void CEyelink2::WaitForRefresh()
{
#ifdef INCLUDE_EYELINK
	if(_isTrackMode)
		wait_for_video_refresh();
#endif
}
