/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA

	---------------------------------------------------------------------------

	This code abstracts the functions of the Eyelink2 system so that the Eyetrack
	program may be compiled for machines that are not necessarily attached to 
	an Eyelink hardware system.  All of these functions may be called regardless of
	whether or not the system is attached to Eyelink hardware. Alternate actions
	are coded here for systems that are using the mouse to simulate an Eyelink2.

	Set or clear the #define INLCUDE_EYELINK preprocessor definition in stdafx.h
	to attach to ignore the Eyelink hardware.
*/

#pragma once
#pragma warning( disable : 4290 )

#include "DirectInput.h"
#include "EyelinkException.h"

#ifdef INCLUDE_EYELINK
	#include "w32_exptsppt2.h"
#else
	#define TRIAL_ERROR  -1
	#define DONE_TRIAL   0
	#define TRIAL_OK     0
	#define REPEAT_TRIAL 1
	#define SKIP_TRIAL   2
	#define ABORT_EXPT   3

	int eyecmd_printf(char *format, ...);
	int eyemsg_printf(char *format, ...);
#endif

class CEyelink2 {
public:
	static unsigned int GetCurrentTime();

	static int DriftCorrect(const CRect &wndSz);           //perform drift correction in window with dimens wndSz
	static int GetGazeLocation(CPoint &pt);                //place current gaze location in pt
	static int GetNextFixUpdate(CPoint &pt, int *duration);
	static int GetFirstFixation(CPoint &pt);
	static int StartRecording(bool streamOutput);          //start eyelink data recording
	static int StopRecording();                            //stop eyelink data recording
	static int TestForButtonPress(const int *buttons, int nButtons); //true if one of *buttons was pressed
	static int TestForFixation(CPoint &pt);                //place last fixation location in pt
	static int TestForMouseClick(CPoint &pt);              //place mouse cursor location during right click in pt
	static int TestStatus(bool isRecording);               //test status of eyelink data recorder

	static void Delay(int msec);                           //cause delay of msec milliseconds
	static void Flush();                                   //clears all message, key and button queues
	static void Initialize(HWND hWnd, CRect &wndSz, CString &edfFile, //init the eyelink 2
						   COLORREF fg, COLORREF bg, int fm,
							int calibrationType, int calibrationHeight=0	) throw(CEyelinkException);
	                                                       //perform eyelink setup for an individual trial
	static void PrepareForTrial(const char *trialId, int idx, int nTrials, bool streamOutput);
	static void SetTrackMode(bool isTrackMode);            //place eyelink in track mode (false for tests)
	static void Shutdown(CString &edfFile);                //shutdown eyelink system
	static void WaitForRefresh();                          //wait for video screen to refresh

private:
	CEyelink2();
	~CEyelink2();

	static CDirectInput _di;
	static int          _eyeUsed;
	static bool         _isTrackMode;
};
