/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include <sstream>

#include "EyeTrack.h"
#include "InitExperimentDlg.h"
#include ".\initexperimentdlg.h"

// CInitExperimentDlg dialog

IMPLEMENT_DYNAMIC(CInitExperimentDlg, CDialog)

CInitExperimentDlg::CInitExperimentDlg(CWnd* pParent, int nConds, bool isExp)
: CDialog(CInitExperimentDlg::IDD, pParent), 
	_method(CInitExperimentDlg::NONE), _condition(0), _item(0), isExp(isExp),
	_maxConditions(nConds), _maxExperiments(0)
{
}

CInitExperimentDlg::~CInitExperimentDlg()
{}

void CInitExperimentDlg::DoDataExchange(CDataExchange* pDX)
{
	CDialog::DoDataExchange(pDX);
	DDX_Control(pDX, IDC_EDIT_EXPT_ORDER_FILE, m_editExptFile);
	DDX_Control(pDX, IDC_BUTTON_EXPT_ORDER_FILE, m_buttonBrowse);
	DDX_Control(pDX, IDC_EDIT_EXPT_CONDITION_NUM, m_editCondition);
}


BEGIN_MESSAGE_MAP(CInitExperimentDlg, CDialog)
	ON_BN_CLICKED(IDC_BUTTON_EXPT_ORDER_FILE, OnBnClickedBrowseOrderFile)
	ON_BN_CLICKED(IDC_RADIO_EXPT_ORDER_FILE, OnBnClickedFileOrder)
	ON_BN_CLICKED(IDC_RADIO_EXPT_ORDER_RANDOM_FILLER, OnBnClickedRandomFillerOrder)
	ON_BN_CLICKED(IDC_RADIO_EXPT_ORDER_RANDOM, OnBnClickedRandomOrder)
	ON_BN_CLICKED(IDC_RADIO_EXPT_ORDER_SCRIPT, OnBnClickedScriptOrder)
	ON_BN_CLICKED(IDOK, OnBnClickedOk)
	ON_EN_KILLFOCUS(IDC_EDIT_EXPT_ORDER_FILE, OnEnKillfocusExptOrderFile)
	ON_EN_KILLFOCUS(IDC_EDIT_EXPT_CONDITION_NUM, OnEnKillfocusExptConditionNum)
	ON_EN_CHANGE(IDC_EDIT_EXPT_CONDITION_NUM, OnEnChangeEditExptConditionNum)
	ON_EN_CHANGE(IDC_EDIT_EXPT_ITEM_NUMBER, OnEnChangeEditExptItemNumber)
END_MESSAGE_MAP()


bool CInitExperimentDlg::DoInterleave() const
{
	return(_method == INTERLEAVE);
}

int CInitExperimentDlg::GetCondition() const
{
	return(_condition);
}

int CInitExperimentDlg::GetItem() const
{
	return(_item);
}

const std::string &CInitExperimentDlg::GetOrderFile() const
{
	return(_fileName);
}

CInitExperimentDlg::SelectionMethod CInitExperimentDlg::GetSelectionMethod() const
{
	return(_method);
}

void CInitExperimentDlg::OnBnClickedBrowseOrderFile()
{
	char filterExt[] = "Script Files (*.script)|*.script|All Files (*.*)|*.*||";
	CFileDialog fileDlg(true, "script", NULL, OFN_FILEMUSTEXIST, filterExt, this);

	if(fileDlg.DoModal() == IDOK) {
		CWnd *editBox = GetDlgItem(IDC_EDIT_EXPT_ORDER_FILE);
		editBox->SetWindowText(fileDlg.GetPathName());
		editBox->SetFocus();
	}
}

void CInitExperimentDlg::OnBnClickedFileOrder()
{
	_method = CInitExperimentDlg::FILE;

	m_buttonBrowse.EnableWindow();
	m_editExptFile.EnableWindow();

	m_editCondition.SetWindowText("");
	m_editCondition.EnableWindow(FALSE);

	_item = 0;
	GetDlgItem(IDC_EDIT_EXPT_ITEM_NUMBER)->SetWindowText("");
	GetDlgItem(IDC_EDIT_EXPT_ITEM_NUMBER)->EnableWindow(false);
	_condition = 0;
}

void CInitExperimentDlg::OnBnClickedOk()
{
	if(_method == 0) {
		MessageBox("Select an ordering method for experimental trials.", "Initialization Incomplete");
		return;
	}

	if(_method == CInitExperimentDlg::FILE && _fileName.empty()) {
		MessageBox("Select an ordering file for the experimental trials.", "Initialization Incomplete");
		return;
	}

	if(_method != CInitExperimentDlg::FILE && 
	   (_condition < 1 || _condition > _maxConditions)) {
			std::ostringstream os;
			os<<"Select a condition number between 1 and "<<_maxConditions<<'.'<<std::endl;
			MessageBox(os.str().c_str(), "Initialization Incomplete");
			return;
	}

	OnOK();
}

void CInitExperimentDlg::OnBnClickedRandomFillerOrder()
{
	_method = CInitExperimentDlg::INTERLEAVE;

	m_buttonBrowse.EnableWindow(FALSE);
	m_editExptFile.SetWindowText("");
	m_editExptFile.EnableWindow(FALSE);
	_item = 0;
	GetDlgItem(IDC_EDIT_EXPT_ITEM_NUMBER)->SetWindowText("");
	GetDlgItem(IDC_EDIT_EXPT_ITEM_NUMBER)->EnableWindow(false);
	_fileName.clear();

	m_editCondition.EnableWindow();
}

void CInitExperimentDlg::OnBnClickedRandomOrder()
{
	_method = CInitExperimentDlg::RANDOM;

	m_buttonBrowse.EnableWindow(FALSE);
	m_editExptFile.SetWindowText("");
	m_editExptFile.EnableWindow(FALSE);
	_item = 0;
	GetDlgItem(IDC_EDIT_EXPT_ITEM_NUMBER)->SetWindowText("");
	GetDlgItem(IDC_EDIT_EXPT_ITEM_NUMBER)->EnableWindow(false);
	_fileName.clear();

	m_editCondition.EnableWindow();
}

void CInitExperimentDlg::OnBnClickedScriptOrder()
{
	_method = CInitExperimentDlg::SCRIPT;

	m_buttonBrowse.EnableWindow(FALSE);
	m_editExptFile.SetWindowText("");
	m_editExptFile.EnableWindow(FALSE);
	if(isExp)
		GetDlgItem(IDC_EDIT_EXPT_ITEM_NUMBER)->SetWindowText("");
	int enable = isExp ? TRUE : FALSE;
	GetDlgItem(IDC_EDIT_EXPT_ITEM_NUMBER)->EnableWindow(!enable);
	_fileName.clear();

	m_editCondition.EnableWindow();
}

void CInitExperimentDlg::OnEnKillfocusExptConditionNum()
{
	char buf[250];

	m_editCondition.GetWindowText(buf, 250);
	_condition = atoi(buf);
}

void CInitExperimentDlg::OnEnKillfocusExptOrderFile()
{
	char buf[250];

	m_editExptFile.GetWindowText(buf, 250);
	_fileName = buf;
}

BOOL CInitExperimentDlg::OnInitDialog()
{
	CDialog::OnInitDialog();
	return TRUE;
}

void CInitExperimentDlg::OnEnChangeEditExptConditionNum()
{
	CEdit* edit;
	const int maxlen = 10;
	char buf[maxlen];

	edit = (CEdit*)GetDlgItem(IDC_EDIT_EXPT_CONDITION_NUM);
	if(edit->GetWindowText(buf,maxlen))
		_condition = atoi(buf);
	return;
}

void CInitExperimentDlg::OnEnChangeEditExptItemNumber()
{
	CEdit *edit;
	const int maxlen = 10;
	char buf[maxlen];

	edit = (CEdit*)GetDlgItem(IDC_EDIT_EXPT_ITEM_NUMBER);
	if(edit->GetWindowText(buf, maxlen))
		_item = atoi(buf);
	return;
}
