/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once
#include "afxwin.h"
#include "experiment.hpp"

// CInitExperimentDlg dialog

class CInitExperimentDlg : public CDialog
{
	DECLARE_DYNAMIC(CInitExperimentDlg)
public:
	enum SelectionMethod {
		NONE = 0,
		FILE = IDC_RADIO_EXPT_ORDER_FILE,
		INTERLEAVE = IDC_RADIO_EXPT_ORDER_RANDOM_FILLER,
		RANDOM = IDC_RADIO_EXPT_ORDER_RANDOM,
		SCRIPT = IDC_RADIO_EXPT_ORDER_SCRIPT
	};

public:
	CInitExperimentDlg(CWnd* pParent, int nConds, bool isExp = false);
	virtual ~CInitExperimentDlg();

// Dialog Data
	enum { IDD = IDD_INITEXPERIMENTDLG };

	const std::string &GetOrderFile() const;

	SelectionMethod GetSelectionMethod() const;

	int GetCondition() const;
	int GetItem() const;

	bool DoInterleave() const;

	virtual BOOL OnInitDialog();

	afx_msg void OnBnClickedBrowseOrderFile();
	afx_msg void OnBnClickedDataChar();
	afx_msg void OnBnClickedDataRow();
	afx_msg void OnBnClickedDataSubword();
	afx_msg void OnBnClickedDataWord();
	afx_msg void OnBnClickedFileOrder();
	afx_msg void OnBnClickedOk();
	afx_msg void OnBnClickedRandomFillerOrder();
	afx_msg void OnBnClickedRandomOrder();
	afx_msg void OnBnClickedScriptOrder();
	afx_msg void OnEnKillfocusExptConditionNum();
	afx_msg void OnEnKillfocusExptOrderFile();

protected:
	virtual void DoDataExchange(CDataExchange* pDX);    // DDX/DDV support
	void InitializeDialogControls(BOOL enableCE, BOOL enableN, BOOL enableOrder);

	DECLARE_MESSAGE_MAP()

private:
	CButton m_buttonBrowse;
	CButton m_buttonChar;
	CButton m_buttonSubword;
	CButton m_buttonWord;
	CButton m_buttonRow;
	CEdit   m_editCondition;
	CEdit   m_editExptFile;

	std::string     _fileName;
	SelectionMethod _method;
	int             _condition;
	int				_item;
	int             _maxConditions;
	int             _maxExperiments;
	bool			isExp;
public:
	afx_msg void OnEnChangeEditExptConditionNum();
	afx_msg void OnEnChangeEditExptItemNumber();
};
