/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "afxcview.h"
#include <fstream>
#include <sstream>

#include "EditExperimentDlg.h"
#include "EditSequenceDialog.h"
#include "EditTrialDialog.h"
#include "EditTrialTypeDlg.h"
#include "ExperimentDlg.h"
#include "EyeTrack.h"
#include "InitExperimentDlg.h"
#include "MainFrm.h"

#ifdef _DEBUG
#define new DEBUG_NEW
#endif



IMPLEMENT_DYNAMIC(CMainFrame, CFrameWnd)

BEGIN_MESSAGE_MAP(CMainFrame, CFrameWnd)
	ON_COMMAND(ID_MENU_ADD_SEQUENCE, OnMenuAddSequence)
	ON_COMMAND(ID_MENU_ADD_TRIAL, OnMenuAddTrial)
	ON_COMMAND(ID_MENU_ADD_TRIAL_TYPE, OnMenuAddTrialType)
	ON_COMMAND(ID_MENU_CLOSE, OnMenuClose)
	ON_COMMAND(ID_MENU_EDIT_EXPT, OnMenuEditExperiment)
	ON_COMMAND(ID_MENU_NEW, OnMenuNew)
	ON_COMMAND(ID_MENU_OPEN, OnMenuOpen)
	ON_COMMAND(ID_MENU_RUN_EXPT, OnMenuRunExpt)
	ON_COMMAND(ID_MENU_SAVE, OnMenuSave)
	ON_COMMAND(ID_MENU_SAVEAS, OnMenuSaveAs)
	ON_COMMAND(ID_MENU_TEST_EXPT, OnMenuTestExpt)
	ON_COMMAND(ID_MENU_VALIDATE_EXPT, OnMenuValidateExpt)
	ON_WM_CLOSE()
	ON_WM_CREATE()
	ON_WM_SETFOCUS()
	ON_WM_SIZE()
END_MESSAGE_MAP()

static UINT indicators[] =
{
	ID_SEPARATOR,           // status line indicator
	ID_INDICATOR_CAPS,
	ID_INDICATOR_NUM,
	ID_INDICATOR_SCRL,
};

CMainFrame::CMainFrame()
: _experiment(0)
{
	

}

CMainFrame::~CMainFrame()
{
//	_CrtDumpMemoryLeaks();
	if(_experiment)
		delete _experiment;
	
}

#ifdef _DEBUG
void CMainFrame::AssertValid() const
{
	CFrameWnd::AssertValid();
}

void CMainFrame::Dump(CDumpContext& dc) const
{
	CFrameWnd::Dump(dc);
}

#endif //_DEBUG


void CMainFrame::statusReady(){
/*	CString stat = "Ready";
	m_wndStatusBar.SetWindowText(stat);*/
	return;
}

void CMainFrame::EditSequence(CTreeCtrl &ctrl, HTREEITEM item)
{
	CEditSequenceDialog dialog(this);

	if(item != NULL) {                           //initialize the dialog
		CString name = ctrl.GetItemText(item);
		TrialId tmp(name.GetBuffer());
		Sequence &s = *(_experiment->getSequence(tmp));
		dialog.SetSequence(s);
	}

	dialog.SetTrials(_experiment->getTrialNames(TrialId::INDEPENDENT),
					 _experiment->getTrialNames(TrialId::DEPENDENT));

	if(dialog.DoModal() == IDOK) {
		_experiment->addSequence(dialog.GetSequence());
	
		/*if(item != NULL){
			CString name = ctrl.GetItemText(item);
			TrialId tmp(name.GetBuffer());
			Sequence &temp = *(_experiment->getSequence(tmp));
		}*/

		if(item == NULL)                         //update tree view
			UpdateView();
	}
}

void CMainFrame::EditTrial(CTreeCtrl &ctrl, HTREEITEM item)
{
	CEditTrialDialog dialog(this, _experiment);

	if(item != NULL) {                           //initialize the dialog box
		CString itemName = ctrl.GetItemText(item);
		TrialId tmp(itemName.GetBuffer());
		Trial *t = _experiment->getTrial(tmp);
		dialog.SetTrial(t);
	}

	if(dialog.DoModal() == IDOK) {
		_experiment->addTrial(dialog.GetTrial());
		if(item == NULL)                         //update tree view
			UpdateView();
	}
}

void CMainFrame::EditTrialType(CTreeCtrl &ctrl, HTREEITEM item)
{
	CEditTrialTypeDlg dialog(theApp.GetMainWnd());

	if(item != NULL) {                           //initialize dialog
		CString itemName = ctrl.GetItemText(item);
		TrialType *t = _experiment->getTrialType(itemName.GetBuffer());

		dialog.SetTrialType(t);
	}

	if(dialog.DoModal() == IDOK) {               //update experiment
		_experiment->addTrialType(dialog.GetTrialType());
		if(item == NULL)                         //update tree view
			UpdateView();
	}
}

bool CMainFrame::InitializeExperiment(IdentifierList &trials, bool isExp)
{
	CInitExperimentDlg dlg(theApp.GetMainWnd(), _experiment->nConditions(), isExp);

	if(dlg.DoModal() != IDOK) 
		return(false);

	try {


		if(dlg.GetSelectionMethod() == CInitExperimentDlg::FILE) //use ordering in file
			trials = _experiment->selectTrials(dlg.GetOrderFile());
		else if(dlg.GetSelectionMethod() == CInitExperimentDlg::SCRIPT) //script ordering
			trials = _experiment->selectTrials(EXPT_ALL, dlg.GetCondition(), dlg.GetItem());
		else                                                     //randomize trials
			trials = _experiment->selectTrials(dlg.GetCondition(), dlg.DoInterleave());
	} catch(CExperimentException &ee) {
		MessageBox(ee.what(), "Experiment Initialization Error", MB_ICONSTOP | MB_OK);
		return(false);
	}
	return(true);
}

void CMainFrame::OnClose()
{
	if(_experiment && !_experiment->isSaved()) {
		int result = MessageBox("Save changes to current experiment?", "Save Changes?", MB_YESNOCANCEL | MB_ICONQUESTION);

		if(result == IDYES)                      //Save and continue
			OnMenuSave();
		else if(result == IDCANCEL)              //Cancel the "Open"
			return;
	}

	CFrameWnd::OnClose();
}

BOOL CMainFrame::OnCmdMsg(UINT nID, int nCode, void* pExtra, AFX_CMDHANDLERINFO* pHandlerInfo)
{
	// let the view have first crack at the command
	if (m_wndSplt.OnCmdMsg(nID, nCode, pExtra, pHandlerInfo))
		return TRUE;

	// otherwise, do default handling
	return CFrameWnd::OnCmdMsg(nID, nCode, pExtra, pHandlerInfo);
}

int CMainFrame::OnCreate(LPCREATESTRUCT lpCreateStruct)
{
	if (CFrameWnd::OnCreate(lpCreateStruct) == -1)
		return -1;
/*
	if (!m_wndStatusBar.Create(this) ||
		!m_wndStatusBar.SetIndicators(indicators,
		  sizeof(indicators)/sizeof(UINT)))
	{
		TRACE0("Failed to create status bar\n");
		return -1;      // fail to create
	}
*/
	return 0;
}

BOOL CMainFrame::OnCreateClient(LPCREATESTRUCT lpcs, CCreateContext* pContext)
{
	BOOL bCreateSpltr = m_wndSplt.CreateStatic(this, 1, 3);

	m_wndSplt.CreateView(0, 0, RUNTIME_CLASS(CTreeView), CSize(200,0), pContext);
	m_wndSplt.CreateView(0, 1, RUNTIME_CLASS(CTreeView), CSize(200,0), pContext);
	m_wndSplt.CreateView(0, 2, RUNTIME_CLASS(CTreeView), CSize(200,0), pContext);

	return (bCreateSpltr);
}

void CMainFrame::OnMenuAddSequence()
{
	if(_experiment) {
		CTreeView *tView = static_cast<CTreeView*>(m_wndSplt.GetPane(0, 2));
		CTreeCtrl &ctrl = tView->GetTreeCtrl();

		EditSequence(ctrl, NULL);
	}
	else
		MessageBox("There is currently no active experiment.\nOpen a new or existing experiment file first.", "EyeTrack Error", MB_OK | MB_ICONERROR);
}

void CMainFrame::OnMenuAddTrial()
{
	if(_experiment) {
		CTreeView *tView = static_cast<CTreeView*>(m_wndSplt.GetPane(0, 1));
		CTreeCtrl &ctrl = tView->GetTreeCtrl();

		EditTrial(ctrl, NULL);
	}
	else
		MessageBox("There is currently no active experiment.\nOpen a new or existing experiment file first.", "EyeTrack Error", MB_OK | MB_ICONERROR);
}

void CMainFrame::OnMenuAddTrialType()
{
	if(_experiment) {
		CTreeView *tView = static_cast<CTreeView*>(m_wndSplt.GetPane(0, 0));
		CTreeCtrl &ctrl = tView->GetTreeCtrl();

		EditTrialType(ctrl, NULL);
	}
	else
		MessageBox("There is currently no active experiment.\nOpen a new or existing experiment file first.", "EyeTrack Error", MB_OK | MB_ICONERROR);
}

void CMainFrame::OnMenuClose()
{
	if(_experiment && !_experiment->isSaved()) {
		int result = MessageBox("Save changes to current experiment before closing?", "Save Experiment?", MB_YESNOCANCEL | MB_ICONQUESTION);

		if(result == IDYES) {                    //Save and continue
			if(_expFileName.empty())             //file unspecified
				OnMenuSaveAs();
			else {                               //save to specified file
				std::ofstream os(_expFileName.c_str(), std::ios::out);
				_experiment->save(os);
				os.close();
			}
		}
		else if(result == IDCANCEL)              //Cancel the "Close"
			return;
	}

	if(_experiment) {
		delete _experiment;                      //clear old experiment
		_experiment = 0;
	}

	_expFileName.clear();                        //clear old file name
	_expFileTitle.clear();
	UpdateView();
}

void CMainFrame::OnMenuEditExperiment()
{
	if(_experiment) {
		CEditExperimentDlg dlg(theApp.GetMainWnd());

		dlg.SetColors(_experiment->backgroundColor(), _experiment->foregroundColor());
		dlg.SetConditions(_experiment->nConditions(), _experiment->nExperiments());
		dlg.SetHeader(_experiment->header(true));
		dlg.SetFilterMode(_experiment->filterMode());
		dlg.SetCalibrationType(_experiment->calibrationType());
		dlg.SetDisplayType(_experiment->getDisplayType());
		dlg.SetDCThreshold(_experiment->getDCThreshold());
		int ibuf[20];
		_experiment->getExpConditions(ibuf);
		dlg.SetExpConditions(ibuf);

		if(dlg.DoModal() == IDOK) { 
			_experiment->setColors(dlg.GetBackground(), dlg.GetForeground());
			_experiment->setConditions(dlg.GetNConditions(), dlg.GetNExperiments());
			_experiment->setHeader(dlg.GetHeader());
			_experiment->setFilterMode(dlg.GetFilterMode());
			_experiment->setCalibrationType(dlg.GetCalibrationType());
			_experiment->setDisplayType(dlg.GetDisplayType());
			_experiment->setDCThreshold(dlg.GetDCThreshold());
			int econds[20];
			for (int i=0; i<20; i++)
				econds[i] =1;
			dlg.getExpConditions(econds);
			_experiment->setExpConditions(econds);
		}
	}
	else
		MessageBox("There is currently no active experiment.\nOpen a new or existing experiment file first.", "EyeTrack Error", MB_OK | MB_ICONERROR);
}

void CMainFrame::OnMenuNew()
{
	if(_experiment && !_experiment->isSaved()) {
		int result = MessageBox("Save changes to current experiment?", "Save Experiment?", MB_YESNOCANCEL | MB_ICONQUESTION);

		if(result == IDYES) {               //Save and continue
			if(_expFileName.empty())        //  file unspecified
				OnMenuSaveAs();
			else {                          //  save to specified file
				std::ofstream os(_expFileName.c_str(), std::ios::out);
				_experiment->save(os);
				os.close();
			}
		}
		else if(result == IDCANCEL) {       //Cancel the "New"
			return;
		}
	}

	if(_experiment)
		delete _experiment;                 //clear old experiment
	_expFileName.clear();                   //clear old file name
	_expFileTitle.clear();

	_experiment = new Experiment();
	UpdateView();
	statusReady();
}

void CMainFrame::OnMenuOpen()
{
	char filterExt[] = "Script Files (*.script)|*.script|All Files (*.*)|*.*||";
	CFileDialog fileDlg(true, "script", NULL, OFN_FILEMUSTEXIST, filterExt, this);

	if(_experiment && !_experiment->isSaved()) {
		int result = MessageBox("Save changes to current experiment?", "Save Experiment?", MB_YESNOCANCEL | MB_ICONQUESTION);

		if(result == IDYES)                 //Save and continue
			OnMenuSave();
		else if(result == IDCANCEL)         //Cancel the "Open"
			return;
	}

	if(fileDlg.DoModal() == IDOK) {         //select file to open
		if(_experiment) {                   //clear old experiment
			delete _experiment;
			_experiment = 0;
		} 

 		_experiment = new Experiment();
		_expFileName = fileDlg.GetPathName();
		_expFileTitle = fileDlg.GetFileTitle();

		try {                               //attempt to load experiment
			_experiment->load(_expFileName);
			UpdateView();
		} catch(CExperimentException &ee) { //error: clear experiment
			MessageBox(ee.what(), "Experiment Loading Error", MB_OK | MB_ICONSTOP); 
			delete _experiment;
			_experiment = 0;
			_expFileName.clear();
			_expFileTitle.clear();

		}
	}
}

void CMainFrame::OnMenuRunExpt()
{
	if(_experiment) {
		IdentifierList trials;

		if(InitializeExperiment(trials) == true) {
			CSize wndSz(::GetSystemMetrics(SM_CXSCREEN), ::GetSystemMetrics(SM_CYSCREEN));
			CExperimentDlg eDlg(this, wndSz, _experiment->backgroundColor(), _experiment->foregroundColor(), _experiment->filterMode());

			eDlg.ShowWindow(SW_SHOWMAXIMIZED);
			eDlg.RunExperiment(_experiment, trials, _expFileTitle.c_str());
			eDlg.DestroyWindow();
		}
	}
	else MessageBox("There is currently no active experiment.\nOpen a new or existing experiment file first.", "EyeTrack Error", MB_OK | MB_ICONERROR);
}

void CMainFrame::OnMenuSave()
{
/*	if(!_experiment || _experiment->isSaved()){
		CString stat = "Experiment matched saved version";
		m_wndStatusBar.SetWindowText(stat);			
		return;
	}*/

	if(_expFileName.empty())
		OnMenuSaveAs();
	else {
		std::ofstream os(_expFileName.c_str(), std::ios::out);
		_experiment->save(os);
		os.close();
	}
/*	CString stat = "Experiment saved";
	m_wndStatusBar.SetWindowText(stat);
	return;*/
}

void CMainFrame::OnMenuSaveAs()
{
	char filterExt[] = "Script Files (*.script)|*.script|All Files (*.*)|*.*||";
	CFileDialog fileDlg(false, "script", _expFileName.c_str(),
						OFN_HIDEREADONLY | OFN_OVERWRITEPROMPT, filterExt, this);

	if(!_experiment) {
		MessageBox("Nothing to save.", "EyeTrack Warning", MB_OK | MB_ICONWARNING);
		return;
	}

	if(fileDlg.DoModal() == IDOK) {
		std::ofstream os;

		_expFileName = fileDlg.GetPathName();
		_expFileTitle = fileDlg.GetFileTitle();
		os.open(_expFileName.c_str(), std::ios::out);
		_experiment->save(os);
		os.close();
	}
}

void CMainFrame::OnMenuTestExpt()
 {
	if(_experiment) {
		IdentifierList trials;

		if(InitializeExperiment(trials) == true) {
			CSize wndSz(::GetSystemMetrics(SM_CXSCREEN), ::GetSystemMetrics(SM_CYSCREEN));
			CExperimentDlg eDlg(this, wndSz, _experiment->backgroundColor(), _experiment->foregroundColor(), _experiment->filterMode());

			eDlg.ShowWindow(SW_SHOWMAXIMIZED);

			//IdentifierIter trialIter;
			//trialIter = trials.begin();
			//while(trialIter != trials.end() && trialIter._Ptr->_Myval.idString != CString("E1I2D0"))
			//	trialIter++;
			//eDlg.RunExperiment(_experiment, trialIter);
			eDlg.RunExperiment(_experiment, trials);
			eDlg.DestroyWindow();
		}
	}
	else MessageBox("There is currently no active experiment.\nOpen a new or existing experiment file first.", "EyeTrack Error", MB_OK | MB_ICONERROR);
}

void CMainFrame::OnMenuValidateExpt()
{
	if(_experiment) {
		std::ostringstream os;

		os<<"The following errors were found in the active experiment:\n"<<std::endl;
		if(_experiment->validate(os))
			MessageBox("The active experiment was validated successfully.", "Validation Complete");
		else
			MessageBox(os.str().c_str(), "Validation Complete");
	}
	else MessageBox("There is currently no active experiment.\nOpen a new or existing experiment file first.", "EyeTrack Error", MB_OK | MB_ICONERROR);
}

BOOL CMainFrame::OnNotify(WPARAM wParam, LPARAM lParam, LRESULT* pResult)
{
	if(_experiment) {
		NMHDR *nmhdr = (NMHDR*)(lParam);
		int col, row;
		CTreeView *active = static_cast<CTreeView*>(m_wndSplt.GetActivePane(&row, &col));

		if(m_wndSplt.GetActivePane(&row, &col) != 0) {
			CTreeCtrl &ctrl = active->GetTreeCtrl();
			HTREEITEM item = ctrl.GetSelectedItem();
			std::ostringstream os;
			switch(nmhdr->code) {
			case NM_DBLCLK:                      //double click, open the edit dialog
				if(col == 0 && item != _typeTreeRoot)
					EditTrialType(ctrl, item);    
				else if(col == 1 && item != _trialTreeRoot)
					EditTrial(ctrl, item);
				else if(col == 2 && item != _sequenceTreeRoot)
					EditSequence(ctrl, item);
				break;
		//	case NM_RCLICK:
		//		os<<"Got Right Click on: "<< ctrl.GetItemText(item).GetBuffer();
		//		MessageBox(os.str().c_str(), "Right Click Message");
		//		break;
			case TVN_KEYDOWN:                    //check for delete key
				if(reinterpret_cast<NMTVKEYDOWN*>(nmhdr)->wVKey == 46) { 
					CString itemText = ctrl.GetItemText(item);
					ctrl.DeleteItem(item);
					if(col == 0)
						_experiment->deleteTrialType(itemText.GetBuffer());
					else if(col == 1)
						_experiment->deleteTrial(itemText.GetBuffer());
					else {
						_experiment->deleteSequence(itemText.GetBuffer());
						UpdateView();
					}
				}
			}
		}
	}

	return(CFrameWnd::OnNotify(wParam, lParam, pResult));
}

void CMainFrame::OnSetFocus(CWnd* /*pOldWnd*/)
{
	m_wndSplt.SetFocus();                        //forward focus to the view window
}

void CMainFrame::OnSize(UINT nType, int cx, int cy)
{
	CRect rect;

	CFrameWnd::OnSize(nType, cx, cy);
	GetWindowRect(&rect);

	m_wndSplt.SetColumnInfo(0, rect.Width()/3, 10);
	m_wndSplt.SetColumnInfo(1, rect.Width()/3, 10);
	m_wndSplt.SetColumnInfo(2, rect.Width()/3, 10);
	m_wndSplt.RecalcLayout();
}

BOOL CMainFrame::PreCreateWindow(CREATESTRUCT& cs)
{
	if( !CFrameWnd::PreCreateWindow(cs) )
		return FALSE;

	cs.dwExStyle &= ~WS_EX_CLIENTEDGE;
	cs.lpszClass = AfxRegisterWndClass(0);
	return TRUE;
}

void CMainFrame::UpdateView()
{
	CTreeCtrl &typeCtrl = static_cast<CTreeView*>(m_wndSplt.GetPane(0, 0))->GetTreeCtrl();
	CTreeCtrl &trialCtrl = static_cast<CTreeView*>(m_wndSplt.GetPane(0, 1))->GetTreeCtrl();
	CTreeCtrl &sequenceCtrl = static_cast<CTreeView*>(m_wndSplt.GetPane(0, 2))->GetTreeCtrl();
	//CTreeCtrl &dudCtrl = static_cast<CTreeView*>(m_wndSplt.GetPane(0,3))->GetTreeCtrl();

	sequenceCtrl.DeleteAllItems();
	trialCtrl.DeleteAllItems();
	typeCtrl.DeleteAllItems();

	_sequenceTreeRoot = sequenceCtrl.InsertItem("Trial Sequences");
	_trialTreeRoot = trialCtrl.InsertItem("Trials");
	_typeTreeRoot = typeCtrl.InsertItem("Trial Types");

	sequenceCtrl.SetItemState(_sequenceTreeRoot, TVIS_BOLD, TVIS_BOLD);
	trialCtrl.SetItemState(_trialTreeRoot, TVIS_BOLD, TVIS_BOLD);
	typeCtrl.SetItemState(_typeTreeRoot, TVIS_BOLD, TVIS_BOLD);

	if(_experiment) {
		IdentifierList sequenceList = _experiment->getSequenceNames();
		IdentifierList trialList = _experiment->getTrialNames(TrialId::ALL);
		StringList typeList = _experiment->getTrialTypeNames();

		for(IdentifierIter i = sequenceList.begin(); i != sequenceList.end(); ++i)
			trialCtrl.InsertItem(i->idString().c_str(), _sequenceTreeRoot);

		for(IdentifierIter i = trialList.begin(); i != trialList.end(); ++i)
			trialCtrl.InsertItem(i->idString().c_str(), _trialTreeRoot);

		for(StringIter i = typeList.begin(); i != typeList.end(); ++i)
			typeCtrl.InsertItem(i->c_str(), _typeTreeRoot);
	}

	sequenceCtrl.Expand(_sequenceTreeRoot, TVE_EXPAND);
	trialCtrl.Expand(_trialTreeRoot, TVE_EXPAND);
	typeCtrl.Expand(_typeTreeRoot, TVE_EXPAND);
}
