#include <sstream>
#include "Sequence.h"
#include "Eyelink2.h"

#ifdef INCLUDE_EYELINK
#include "w32_exptsppt2.h"
#endif


Sequence::Sequence(const IdentifierList &seq, const IdentifierList &altSeq, const std::string &ident, Tokenizer::ButtonType b)
  : _sequence(seq), _identifier(ident), _altSequence(altSeq)
{
	_sPtr = _sequence.end();
	_altPtr = _altSequence.end();
	_altSelectButton = b;
}

Sequence::Sequence(const IdentifierList &seq, const IdentifierList &altSeq, const std::string &ident)
  : _sequence(seq), _identifier(ident), _altSequence(altSeq), _altSelectButton(1)
{
	_sPtr = _sequence.end();
	_altPtr = _altSequence.end();
}

Sequence::Sequence(const IdentifierList &seq, const std::string &ident)
  : _sequence(seq), _identifier(ident), _altSequence(0), _altSelectButton(1)
{	_sPtr = _sequence.end();
	//_altSequence.pop_back();
	//_altPtr = _altSequence.end();
}

Sequence::Sequence(Tokenizer &tok) : _sequence(0), _altSequence(0), _altSelectButton(1)
{
	std::string str = tok.readString();
	bool alt;

	_identifier = TrialId(str);
	str = tok.readString();
	while(str.compare("end") != 0 && str.compare("alt") != 0) {
		TrialId temp(str);
		_sequence.push_back(temp);
		//_altSequence.push_back(temp);
		str = tok.readString();
	}
	if (str.compare("alt") == 0){
		str = tok.readString();

		Tokenizer::ControlType c = Tokenizer::FIVE_BUTTON;
		//c = _sequence.front()._type
		_altSelectButton =	tok.readButton(&c);
		

	str = tok.readString();  // get the first trial in alt;

	while(str.compare("end")) {
		TrialId temp(str);
		_altSequence.push_back(temp);
		//_altSequence.push_back(temp);
		str = tok.readString();
	}
	}
	


	str = tok.readString();
	if(str.compare(_identifier.idString()) != 0) {
		std::ostringstream os;
		os<<"Error reading sequence '"<<_identifier.idString()
		  <<"', end tag does not match sequence name"<<std::endl;
		throw(CExperimentException(os.str().c_str()));
	}

	_sPtr = _sequence.end();
	//_altSequence.pop_back();
	_altPtr = _altSequence.end();
}

Sequence::Sequence(const Sequence &seq)
{
	_sequence   = seq._sequence;
	_altSequence = seq._altSequence;
	_sPtr       = _sequence.end();
	_altPtr		= seq._altSequence.end();
	_identifier = seq._identifier;
	_altSelectButton = seq._altSelectButton;
}


Sequence::~Sequence()
{}

void Sequence::appendToList(IdentifierList &list) const
{
	for(IdentifierIter i = _sequence.begin(); i != _sequence.end(); ++i)
		list.push_back(*i);
}

bool Sequence::atEnd() const
{
	return(_sPtr == _sequence.end()/* || _altPtr == _altSequence.end()*/);  // if no alt sequence, this returns false until at seq.end()
}

bool Sequence::atAltEnd() const
{
	return(_altPtr == _altSequence.end()); 
}

const TrialId &Sequence::identifier() const
{
	return(_identifier);
}

int Sequence::length() const
{
	return(static_cast<int>(_sequence.size()));
}

int Sequence::altLength() const
{
	return(static_cast<int>(_altSequence.size()));
}

int Sequence::altSelectButton() const
{
	return(static_cast<int>(_altSelectButton));
}

const TrialId &Sequence::nextTrialId()
{
	const TrialId &sTmp = *_sPtr;

	if(_sPtr != _sequence.end())
		++_sPtr;
	int buttons[1];
	buttons[0] = Tokenizer::TOGGLE;
	/*if(CEyelink2::TestForButtonPress(buttons, _sequence.begin()))
		++_sPtr;*/
	return(sTmp);
}

const TrialId &Sequence::nextAltId()
{
	const TrialId &sTmp = *_altPtr;

	if(_altPtr != _altSequence.end())
		++_altPtr;
//	int buttons[1];
//	buttons[0] = Tokenizer::TOGGLE;
	/*if(CEyelink2::TestForButtonPress(buttons, _sequence.begin()))
		++_sPtr;*/
	return(sTmp);
}
const Sequence &Sequence::operator=(const Sequence &rhs)
{
	_sequence   = rhs._sequence;
	_altSequence = rhs._altSequence;
	_sPtr       = _sequence.end();
	_altPtr		= _altSequence.end();
	_identifier = rhs._identifier;
	_altSelectButton = rhs._altSelectButton;
	return(*this);
}

void Sequence::resetTrialPtr()
{
	_sPtr = _sequence.begin();
	_altPtr = _altSequence.begin();
}

void Sequence::save(std::ostream &os) const
{
	os<<"sequence "<<_identifier.idString()<<std::endl;
	for(IdentifierIter i = _sequence.begin(); i != _sequence.end(); ++i)
		os<<"  "<<i->idString()<<std::endl;
	if(!_altSequence.empty()){
		os<<"alt "<<_identifier.idString()<<" "<<Tokenizer::decode5((Tokenizer::ButtonType)_altSelectButton)<<std::endl;
	for(IdentifierIter i = _altSequence.begin(); i != _altSequence.end(); ++i)
		os<<"  "<<i->idString()<<std::endl;
	}
	os<<"end "<<_identifier.idString()<<'\n'<<std::endl;
}

