/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include "EyeTrack.h"
#include "TextDlg.h"
#include ".\textdlg.h"
#include "math.h"

IMPLEMENT_DYNAMIC(CTextDlg, CFontDialog)
CTextDlg::CTextDlg(LPLOGFONT lplfInitial, DWORD dwFlags, CDC* pdcPrinter, CWnd* pParentWnd) : 
	CFontDialog(lplfInitial, dwFlags, pdcPrinter, pParentWnd)
{
	m_cf.Flags |= CF_ENABLETEMPLATE;
	m_cf.hInstance = AfxGetInstanceHandle();
	m_cf.lpTemplateName = MAKEINTRESOURCE(IDD_TEXT_FORMAT_DLG);

	_format.lineSpace = 0;
	_format.xOffset = 0;
	_format.yOffset = 0;
	_format.rendering = NONANTIALIASED_QUALITY;
	_format.italic = FALSE;
	_format.fontWt = FW_NORMAL;

	points = false;
	loaded = false;
}

CTextDlg::~CTextDlg()
{}

BEGIN_MESSAGE_MAP(CTextDlg, CFontDialog)
		ON_EN_CHANGE(IDC_EDIT_XOFFSET_PIX, OnEnChangeEditOffsetXPix)
		ON_EN_CHANGE(IDC_EDIT_XOFFSET, OnEnChangeEditOffsetXPt)
		ON_EN_CHANGE(IDC_EDIT_YOFFSET_PIX, OnEnChangeEditOffsetYPix)
		ON_EN_CHANGE(IDC_EDIT_YOFFSET, OnEnChangeEditOffsetYPt)
		ON_BN_CLICKED(IDC_RADIO_PT, OnEnChangePts)
		ON_BN_CLICKED(IDC_RADIO_PIX, OnEnChangePix)
END_MESSAGE_MAP()

const TextFormat &CTextDlg::GetTextFormat()
{
	return(_format);
}

BOOL CTextDlg::OnInitDialog()
{
	CFontDialog::OnInitDialog();

	if(/*_format.fontName.empty() == false*/1==1) {
		CButton *button;
		char buf[40];

		GetDlgItem(IDC_EDIT_TEXT_LINE_SPACE)->SetWindowText(_itoa(_format.lineSpace, buf, 10));
		GetDlgItem(IDC_EDIT_TEXT_SPACING_DIFF)->SetWindowText(_itoa(_format.spacingDiff, buf, 10));
		GetDlgItem(IDC_EDIT_XOFFSET)->SetWindowText(_itoa(_format.xOffset, buf, 10));
		GetDlgItem(IDC_EDIT_YOFFSET)->SetWindowText(_itoa(_format.yOffset, buf, 10));

		GetDlgItem(IDC_EDIT_XOFFSET_PIX)->SetWindowText(itoa(_format.xOffset * 96/72, buf, 10));
		GetDlgItem(IDC_EDIT_YOFFSET_PIX)->SetWindowText(itoa(_format.yOffset * 96/72, buf, 10));

		loaded = true;
		
		switch(_format.rendering) {
			case ANTIALIASED_QUALITY:
				button = (CButton*)GetDlgItem(IDC_RADIO_TEXT_ANTIALIASED);
				break;
			case NONANTIALIASED_QUALITY:
				button = (CButton*)GetDlgItem(IDC_RADIO_TEXT_NONANTIALIASED);
				break;
			case CLEARTYPE_QUALITY:
				button = (CButton*)GetDlgItem(IDC_RADIO_TEXT_CLEARTYPE);
				break;
			default:
				button = (CButton*)GetDlgItem(IDC_RADIO_TEXT_NONANTIALIASED);
		}

		if(button)
			button->SetCheck(BST_CHECKED);
		CComboBox *combo;

		combo = (CComboBox*)GetDlgItem(IDC_COMBO_TEXT_FONTNAME);
		combo->SetWindowText(_format.fontName.c_str());
		combo->SelectString(0,_format.fontName.c_str());

		combo = (CComboBox*)GetDlgItem(IDC_COMBO_TEXT_FONT_STYLE);
		std::string a;
//BOLDSTUFF
		if (_format.fontWt == FW_NORMAL && _format.italic == FALSE)
			a = "Regular";
		else if(_format.fontWt == FW_BOLD && _format.italic == FALSE)
			a = "Bold";
		else if(_format.fontWt == FW_NORMAL && _format.italic == TRUE)
			a = "Italic";
		else if(_format.fontWt == FW_BOLD && _format.italic == TRUE)
			a = "Bold Italic";
		else a = "Regular";
		combo->SelectString(0,a.c_str());

		combo = (CComboBox*)GetDlgItem(IDC_COMBO_TEXT_FONTSZ);
		combo->SetWindowText( itoa( _format.fontHt,buf,10 ));
		combo->SelectString(0,itoa(_format.fontHt,buf,10));

	}
	return TRUE;
}

int convertFontWt(CString buf){
	if (buf.Compare("Regular") == 0) 
		return FW_NORMAL;
	else if(buf.Compare("Bold") == 0 || buf.Compare("Bold Italic") == 0)
		return FW_BOLD;
	else return FW_NORMAL;
}

BYTE convertFontItalic(CString buf){
	if (buf.Compare("Italic") == 0 || buf.Compare("Bold Italic") == 0)
		return 1;
	else return 0;
}



void CTextDlg::OnOK()
{
	const int maxLen = 40;
	CComboBox *combo;
	char buf[maxLen];
	int checked;

	combo = (CComboBox*)GetDlgItem(IDC_COMBO_TEXT_FONTNAME);
	combo->GetLBText(combo->GetCurSel(), buf);
	_format.fontName = buf;

	combo = (CComboBox*)GetDlgItem(IDC_COMBO_TEXT_FONTSZ);
	combo->GetLBText(combo->GetCurSel(), buf);
	_format.fontHt = atoi(buf);

	combo = (CComboBox*)GetDlgItem(IDC_COMBO_TEXT_FONT_STYLE);
	combo->GetLBText(combo->GetCurSel(), buf);
//BOLDSTUFF
	_format.fontWt = convertFontWt(buf);	
	_format.italic = convertFontItalic(buf);



	checked = GetCheckedRadioButton(IDC_RADIO_TEXT_ANTIALIASED, IDC_RADIO_TEXT_CLEARTYPE);
	switch(checked) {
		case IDC_RADIO_TEXT_ANTIALIASED:
			_format.rendering = ANTIALIASED_QUALITY;
			break;
		case IDC_RADIO_TEXT_NONANTIALIASED:
			_format.rendering = NONANTIALIASED_QUALITY;
			break;
		case IDC_RADIO_TEXT_CLEARTYPE:
			_format.rendering = CLEARTYPE_QUALITY;
	}

	GetDlgItem(IDC_EDIT_TEXT_LINE_SPACE)->GetWindowText(buf, maxLen);
	_format.lineSpace = atoi(buf);

	
	GetDlgItem(IDC_EDIT_TEXT_SPACING_DIFF)->GetWindowText(buf, maxLen);
	_format.spacingDiff = atoi(buf);

	GetDlgItem(IDC_EDIT_XOFFSET)->GetWindowText(buf, maxLen);
	_format.xOffset = atoi(buf);

	GetDlgItem(IDC_EDIT_YOFFSET)->GetWindowText(buf, maxLen);
	_format.yOffset = atoi(buf);

	CFontDialog::OnOK();
}

void CTextDlg::OnEnChangeEditOffsetXPt(){
	if(points && loaded){
	int maxLen = 40;
	char buf[40];
	
	GetDlgItem(IDC_EDIT_XOFFSET)->GetWindowText(buf, maxLen);
	_format.xOffset = atoi(buf);

	GetDlgItem(IDC_EDIT_XOFFSET_PIX)->SetWindowText(itoa(_format.xOffset * 96/72, buf, 10));
	}

}

void CTextDlg::OnEnChangeEditOffsetYPt(){
	if(points && loaded){
	int maxLen = 40;
	char buf[40];
	
	GetDlgItem(IDC_EDIT_YOFFSET)->GetWindowText(buf, maxLen);
	_format.yOffset = atoi(buf);

	GetDlgItem(IDC_EDIT_YOFFSET_PIX)->SetWindowText(itoa(_format.yOffset * 96/72, buf, 10));
	}

}

void CTextDlg::OnEnChangeEditOffsetXPix(){
	if(!points && loaded){
	int maxLen = 40;
	char buf[40];
	
	GetDlgItem(IDC_EDIT_XOFFSET_PIX)->GetWindowText(buf, maxLen);
	_format.xOffset = ceil((float)atoi(buf) * 72/96);

	GetDlgItem(IDC_EDIT_XOFFSET)->SetWindowText(itoa(_format.xOffset, buf, 10));
	}
}

void CTextDlg::OnEnChangeEditOffsetYPix(){
	if(!points && loaded){
	int maxLen = 40;
	char buf[40];
	
	GetDlgItem(IDC_EDIT_YOFFSET_PIX)->GetWindowText(buf, maxLen);
	_format.yOffset = ceil((float)atoi(buf) * 72/96);

	GetDlgItem(IDC_EDIT_YOFFSET)->SetWindowText(itoa(_format.yOffset, buf, 10));
	}
}

void CTextDlg::OnEnChangePts(){
	points = true;
	GetDlgItem(IDC_EDIT_XOFFSET_PIX)->EnableWindow(FALSE);
	GetDlgItem(IDC_EDIT_YOFFSET_PIX)->EnableWindow(FALSE);
	GetDlgItem(IDC_EDIT_XOFFSET)->EnableWindow(TRUE);
	GetDlgItem(IDC_EDIT_YOFFSET)->EnableWindow(TRUE);

}

void CTextDlg::OnEnChangePix(){
	points = false;
	GetDlgItem(IDC_EDIT_XOFFSET_PIX)->EnableWindow(TRUE);
	GetDlgItem(IDC_EDIT_YOFFSET_PIX)->EnableWindow(TRUE);
	GetDlgItem(IDC_EDIT_XOFFSET)->EnableWindow(FALSE);
	GetDlgItem(IDC_EDIT_YOFFSET)->EnableWindow(FALSE);
}

void CTextDlg::SetTextFormat(const TextFormat &format)
{
	_format = format;
}
