/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <cctype>
#include <sstream>
#include "tokenizer.h"




Tokenizer::Tokenizer()
{}

Tokenizer::~Tokenizer()
{
	_ifstrm.close();
}

void Tokenizer::close()
{
	_ifstrm.close();
}
std::string Tokenizer::decode(BYTE b)
{
	switch(b) {
	case ANTIALIASED_QUALITY:    return("antialiased");
	case CLEARTYPE_QUALITY:      return("cleartype");
	case NONANTIALIASED_QUALITY: return("nonantialiased");
	default:                     return("<bad text quality type>");
	}
}

std::string Tokenizer::decodeWeight(int weight)
{
	switch(weight) {
		case FW_NORMAL:			return("normal");
		case FW_BOLD:			return("bold");
		default:				return("normal");//return("<bad text weight type>");
	}
}

std::string Tokenizer::decodeItalic(BYTE italic)
{
	switch(italic) {
		case TRUE:				return("italic");
		case FALSE:				return("non-italic");
		default:				return("non-italic");//return("<bad text italic value>");
	}
}


std::string Tokenizer::decode5(ButtonType b)
{
	switch(b) {
	case BLUE:   return("blue");
	case RED:    return("red");
	case GREEN:  return("green");
	case YELLOW: return("yellow");
	case PURPLE: return("purple");
	case 6:  return("mouse");
	default:     return("<bad button type>");
	}
}

std::string Tokenizer::decode7(ButtonType b)
{
	switch(b) {
	case Y:				return("Y");
	case X:				return("X");
	case B:				return("B");
	case A:				return("A");	
	case TOGGLE:		return("toggle");
	case LEFT_TRIGGER:	return("leftTrigger");
	case RIGHT_TRIGGER:	return("rightTrigger");
	case MOUSE:			return("mouse");
	default:			return("<bad button type>");
	}
}

std::string Tokenizer::decode(Keyword k)
{
	switch(k) {
	case ALT:				return("alt");
	case AUDIO_FORMAT:     return("audio_format");
	case BACKGROUND:       return("background");
	case BUTTON:           return("button");
	case CALIBRATION_TYPE: return("calibration_type");
	case CALIBRATION_HEIGHT: return("calibration_height");
	case CONDITIONS:       return("conditions");
	case CURSOR_SIZE:      return("cursor_size");
	case DC_DELAY:		   return("dc_delay");
	case DISPLAY_TYPE:	   return("display_type");
	case END:              return("end");
	case EXPERIMENTS:      return("experiments");
	case EXPCONDITIONS:		return("expConditions");
	case FILTERMODE:	   return("filterMode");
	case FOREGROUND:       return("foreground");
	case IMAGE:            return("image");
	case INLINE:		   return("inline");
	case INTER_CHAR:	   return("inter_char");
	case MAX_DISPLAY_TIME: return("max_display_time");
	case OUTPUT:           return("output");
	case REGION:           return("region");
	case REGIONPAIR:	   return("region_pair");
	case REVERT:		   return("revert");
	case SCOTOMA:		   return("scotoma");
	case SEQUENCE:         return("sequence");
	case SET:              return("set");
	case STIMULUS_DELAY:   return("stimulus_delay");
	case TEXT_FORMAT:      return("text_format");
	case TEXT_WEIGHT:	   return("text_weight");
	case TRIAL:            return("trial");
	case TRIALTYPE:        return("trial_type");
	case TRIGGER:          return("trigger");
	case WORDS:				return("words");
	case WINDOW:			return("window");
	case WINDOW_THRESHOLD:	return("windowThreshold");
	default:               return("<unknown keyword>");
	}
}

std::string Tokenizer::decode(OutputType o)
{
	switch(o) {
	case NOSTREAM: return("nostream");
	case STREAM:   return("stream");
	default:       return("<bad output type>");
	}
}


std::string Tokenizer::decode(FilterMode f)
{
	switch(f) {
		case NONE_SELECTED: return("none_selected");
		case FILTER_ON:		return("filter_on");
		case FILTER_OFF:	return("filter_off");
		default:			return("<bad filter mode>");
	}
}


std::string Tokenizer::decode(CalibrationType c)
{
	switch(c) {
		case H3:				return("H3");
		case HV9:				return("HV9");
		case HV13:				return("HV13");
		case NO_CALIBRATION:	return("<bad calibration type>");
		default:				return("<bad calibration type>");
	}
}	
std::string Tokenizer::decode(TriggerType t)
{
	switch(t) {
	case GAZE:         return("gaze");
	case NOGAZE:       return("nogaze");
	case DRIFTCORRECT: return("driftcorrect");
	case DRIFTANDGAZE: return("driftandgaze");
	default:           return("<bad trigger type>");
	}
}
std::string Tokenizer::decode(DisplayType d)
{
	switch(d) {
		case LCD:		return("LCD");
		case CRT:		return("CRT");
		default:		return("<bad display type>");
	}
}

bool Tokenizer::isOkay() const
{
	return(_ifstrm.is_open() && _ifstrm.fail() == false);
}

void Tokenizer::open(const std::string &filename) throw(CExperimentException)
{
	_ifstrm.open(filename.c_str(), std::ios::in);
	if(_ifstrm.is_open() == false) {
		std::ostringstream os;
		os<<"Could not open script file '"<<filename;
		os<<"'."<<std::endl;
		throw(CExperimentException(os.str().c_str()));
	}
}

char Tokenizer::peekChar()
{
	if(_badKeyword.empty()) 
		return(_ifstrm.peek());
	else {
		_ifstrm.setstate(std::ios::failbit, true);
		return('\0');
	}
}

int Tokenizer::readAudioChannels()
{
	if(_badKeyword.empty()) {
		std::string str;

		_ifstrm>>str;
		if(str.compare("mono") == 0)        return(1);
		else if(str.compare("stereo") == 0) return(2);
	}

	_ifstrm.setstate(std::ios::failbit, true);
	return(0);
}

Tokenizer::ButtonType Tokenizer::readButton(Tokenizer::ControlType *c)
{
	if(_badKeyword.empty()){
		std::string str;
		bool sevenButton = false, fiveButton = false;
		_ifstrm>>str;
		if (*c == UNKNOWN_CONTROLLER){
			sevenButton = true;
			fiveButton = true;
		}
		if (*c == FIVE_BUTTON){
			fiveButton = true;
		}
		if (*c == SEVEN_BUTTON){
			sevenButton = true;
		}

		if(str.compare("X") == 0 && sevenButton)					{*c = Tokenizer::SEVEN_BUTTON;return(X);}
		else if(str.compare("Y") == 0 && sevenButton)				{*c = Tokenizer::SEVEN_BUTTON;return(Y);}
		else if(str.compare("A") == 0 && sevenButton)				{*c = Tokenizer::SEVEN_BUTTON;return(A);}
		else if(str.compare("B") == 0 && sevenButton)				{*c = Tokenizer::SEVEN_BUTTON;return(B);}
		else if(str.compare("toggle") == 0 && sevenButton)			{*c = Tokenizer::SEVEN_BUTTON;return(TOGGLE);}
		else if(str.compare("leftTrigger") == 0 && sevenButton)		{*c = Tokenizer::SEVEN_BUTTON;return(LEFT_TRIGGER);}
		else if(str.compare("rightTrigger") == 0 && sevenButton)	{*c = Tokenizer::SEVEN_BUTTON;return(RIGHT_TRIGGER);}
		else if(str.compare("mouse") == 0 && sevenButton)			{*c = Tokenizer::SEVEN_BUTTON;return(MOUSE);}

		else if(str.compare("blue") == 0 && fiveButton)				{*c = Tokenizer::FIVE_BUTTON; return(BLUE);}
		else if(str.compare("red") == 0 && fiveButton)				{*c = Tokenizer::FIVE_BUTTON; return(RED);}
		else if(str.compare("green") == 0 && fiveButton)			{*c = Tokenizer::FIVE_BUTTON; return(GREEN);}
		else if(str.compare("yellow") == 0 && fiveButton)				{*c = Tokenizer::FIVE_BUTTON; return(YELLOW);}
		else if(str.compare("purple") == 0 && fiveButton)				{*c = Tokenizer::FIVE_BUTTON; return(PURPLE);}
		else if(str.compare("mouse") == 0 && fiveButton)				{*c = Tokenizer::FIVE_BUTTON; return(MOUSE);}
	}

	_ifstrm.setstate(std::ios::failbit, true);
	return(NO_BUTTON);

}

/*
Tokenizer::ButtonType Tokenizer::readButton()
{
	if(_badKeyword.empty()) {
		std::string str;

		_ifstrm>>str;
		if(str.compare("blue") == 0)        return(BLUE);
		else if(str.compare("red") == 0)    return(RED);
		else if(str.compare("green") == 0)  return(GREEN);
		else if(str.compare("yellow") == 0) return(YELLOW);
		else if(str.compare("purple") == 0) return(PURPLE);
		else if(str.compare("mouse") == 0)  return(MOUSE);
	}

	_ifstrm.setstate(std::ios::failbit, true);
	return(NO_BUTTON);
}*/

char Tokenizer::readChar()
{
	if(_badKeyword.empty()) {
		char ch;
		_ifstrm>>ch;
		return(ch);
	}
	else {
		_ifstrm.setstate(std::ios::failbit, true);
		return('\0');
	} 
}

Tokenizer::FilterMode Tokenizer::readFilterMode()
{
	if(_badKeyword.empty()){
		int i;
		_ifstrm>>i;
		return (Tokenizer::FilterMode) i;
	}
	else {
		_ifstrm.setstate(std::ios::failbit, true);
		return((Tokenizer::FilterMode)0);
	}
}


Tokenizer::CalibrationType Tokenizer::readCalibrationType()
{
	if(_badKeyword.empty()){
		int i;
		_ifstrm>>i;
		return (Tokenizer::CalibrationType) i;
	}
	else {
		_ifstrm.setstate(std::ios::failbit, true);
		return((Tokenizer::CalibrationType)0);
	}
}

Tokenizer::DisplayType Tokenizer::readDisplayType()
{
	if(_badKeyword.empty()) {
		std::string str;
		
		_ifstrm>>str;
		if(str.compare("LCD") == 0)		return (LCD);
		else if(str.compare("CRT") == 0)	return (CRT);
	}

	_ifstrm.setstate(std::ios::failbit, true);
	return(NO_DISPLAY_TYPE);
}

COLORREF Tokenizer::readColor()
{
	if(_badKeyword.empty()) {
		COLORREF color;
		_ifstrm>>color;
		return(color);
	}
	else {
		_ifstrm.setstate(std::ios::failbit, true);
		return(0);
	}
}

std::string Tokenizer::readHeader() throw(CExperimentException)
{
	if(_badKeyword.empty()) {
		std::string header;
		std::string str;

		getline(_ifstrm, str);
		if(str.compare("%BeginHeader") != 0)
			throw(CExperimentException("Missing '%BeginHeader' at start of script file."));

		getline(_ifstrm, str);
		while(!_ifstrm.eof() && str.compare("%EndHeader") != 0) {
			header.append(str);
			header.append(1, '\n');
			getline(_ifstrm, str);
		} 

		if(_ifstrm.eof())
			throw(CExperimentException("Missing '%EndHeader' near start of script file."));
		return(header);
	}
	else {
		_ifstrm.setstate(std::ios::failbit, true);
		return("");
	}
}

int Tokenizer::readInteger()
{
	if(_badKeyword.empty()) {
		int i;
		_ifstrm>>i;
		return(i);
	}
	else {
		_ifstrm.setstate(std::ios::failbit, true);
		return(0);
	}
}

Tokenizer::Keyword Tokenizer::readKeyword()
{
	std::string str;

	_ifstrm>>str;
	if(str.compare("audio_format") == 0)          return(AUDIO_FORMAT);
	else if(str.compare("alt") == 0)			return(ALT);
	else if(str.compare("background") == 0)       return(BACKGROUND);
	else if(str.compare("button") == 0)           return(BUTTON);
	else if(str.compare("calibration_type") == 0) return(CALIBRATION_TYPE);
	else if(str.compare("calibration_height") == 0) return(CALIBRATION_HEIGHT);
	else if(str.compare("conditions") == 0)       return(CONDITIONS);
	else if(str.compare("cursor_size") == 0)      return(CURSOR_SIZE);
	else if(str.compare("dc_delay") == 0)         return(DC_DELAY);
	else if(str.compare("display_type") == 0)	  return(DISPLAY_TYPE);
	else if(str.compare("end") == 0)              return(END);
	else if(str.compare("experiments") == 0)      return(EXPERIMENTS);
	else if(str.compare("expConditions") == 0)	return(EXPCONDITIONS);
	else if(str.compare("filterMode") == 0)		  return(FILTERMODE);
	else if(str.compare("windowThreshold") == 0)  return(WINDOW_THRESHOLD);
	else if(str.compare("foreground") == 0)       return(FOREGROUND);
	else if(str.compare("gc_rect") == 0)          return(GC_RECT);
	else if(str.compare("highlight_color") == 0)  return(HIGHLIGHT_COLOR);
	else if(str.compare("image") == 0)            return(IMAGE);
    else if(str.compare("inline") == 0)           return(INLINE);
	else if(str.compare("inter_char") == 0)		  return(INTER_CHAR);
	else if(str.compare("max_display_time") == 0) return(MAX_DISPLAY_TIME);
	else if(str.compare("output") == 0)           return(OUTPUT);
	else if(str.compare("region") == 0)           return(REGION);
	else if(str.compare("region_pair") == 0)	  return(REGIONPAIR);
	else if(str.compare("revert") == 0)           return(REVERT);
	else if(str.compare("scotoma") == 0)		return(SCOTOMA);
	else if(str.compare("sequence") == 0)         return(SEQUENCE);
	else if(str.compare("set") == 0)              return(SET);
	else if(str.compare("stimulus_delay") == 0)   return(STIMULUS_DELAY);
	else if(str.compare("text_format") == 0)      return(TEXT_FORMAT);
	else if(str.compare("text_weight") == 0)	  return(TEXT_WEIGHT);
	else if(str.compare("trial") == 0)            return(TRIAL);
	else if(str.compare("trial_type") == 0)       return(TRIALTYPE);
	else if(str.compare("trigger") == 0)          return(TRIGGER);
	else if(str.compare("words") == 0)			  return(WORDS);
	else if(str.compare("window") == 0)				return(WINDOW);
	else if(str.compare("window_threshold") == 0)	return(WINDOW_THRESHOLD);

	_badKeyword = str;
	return(UNKNOWN);
}

Tokenizer::OutputType Tokenizer::readOutput()
{
	if(_badKeyword.empty()) {
		std::string str;

		_ifstrm>>str;
		if(str.compare("stream") == 0)        return(STREAM);
		else if(str.compare("nostream") == 0) return(NOSTREAM);
	}

	_ifstrm.setstate(std::ios::failbit, true);
	return(NO_OUTPUT);
}

CPoint Tokenizer::readPoint()
{
	CPoint pt(-1,-1);
	char ch;

	readWhitespace();
	ch = _ifstrm.peek();
	if(ch == '(') {
		_ifstrm>>ch;
		_ifstrm>>pt.x;
		_ifstrm>>pt.y;
		_ifstrm>>ch;

		if(ch != ')')
			_ifstrm.setstate(std::ios::failbit, true);
	}
	return( pt );
}
		
CRect Tokenizer::readRectangle()
{
	CRect r(0, 0, 0, 0);
	char ch;

	readWhitespace();
	ch = _ifstrm.peek();
	if(ch == '(') {
		_ifstrm>>ch;
		_ifstrm>>r.top;
		_ifstrm>>r.left;
		_ifstrm>>r.bottom;
		_ifstrm>>r.right;
		_ifstrm>>ch;

		if(ch != ')')
			_ifstrm.setstate(std::ios::failbit, true);
	}
	return(r);
}
TIMED_RECT_PAIR Tokenizer::readTimedRectPair()
{
	CRect r1(0,0,0,0), r2(0,0,0,0);
	char ch;
	ToggleData td;
	td.region = r2;

	readWhitespace();
	ch = _ifstrm.peek();
	if (ch == '[') {
		_ifstrm>>ch;
		r1 = readRectangle();
		_ifstrm>>ch;
		if (ch != ','){
			_ifstrm.setstate(std::ios::failbit, true);
			return TimedRectPair(r1, td);
		}
		td.region = readRectangle();
		_ifstrm>>ch;
		if (ch != ','){
			_ifstrm.setstate(std::ios::failbit, true);
			return TimedRectPair(r1, td);
		}
		CPoint p = readPoint();
		td.delay = static_cast <unsigned int> (p.x);
		td.duration = static_cast <unsigned int> (p.y);
		_ifstrm>>ch;
	
		if (ch == ','){
			CPoint p2 = readPoint();
			td.delay2 = static_cast <unsigned int> (p2.x);
			td.duration2 = static_cast <unsigned int> (p2.y);
			_ifstrm>>ch;
		}
		else{
			td.delay2 = 0;
			td.duration2 = 0;
		}

		if (ch != ']'){
			_ifstrm.setstate(std::ios::failbit, true);
		}	
	}
	return TimedRectPair(r1, td);

}
RECTPAIR Tokenizer::readRectPair()
{
	CRect r1(0,0,0,0), r2(0,0,0,0);
	char ch;

	readWhitespace();
	ch = _ifstrm.peek();
	if (ch == '{') {
		_ifstrm>>ch;
		r1 = readRectangle();
		_ifstrm>>ch;
		if (ch != ','){
			_ifstrm.setstate(std::ios::failbit, true);
			return RectPair(r1, r2);
		}
		r2 = readRectangle();
		_ifstrm>>ch;
		if (ch != '}'){
			_ifstrm.setstate(std::ios::failbit, true);
		}

	}
	return RectPair(r1, r2);
}

void Tokenizer::readExpConditions(int* expC)
{
	return;
}

std::string Tokenizer::readLine()
{
	if(_badKeyword.empty()) {
		std::string str;
		char ch = '\n';

		readWhitespace();
		getline(_ifstrm, str, ch);
		return(str);
	}
	else {
		std::string temp = _badKeyword;
		_badKeyword.clear();
		return(temp);
	}
}

std::string Tokenizer::readString()         //read quoted or unquoted strings
{                                           //  quoted: read from start to end quote
	if(_badKeyword.empty()) {               //  unquoted: read one string between whitespaces
		std::string str;
		char ch;

		readWhitespace();
		ch = _ifstrm.peek();
		if(ch == '\'' || ch == '\"') {
			_ifstrm.ignore();
			getline(_ifstrm, str, ch);
		}
		else _ifstrm>>str;
		return(str);
	}
	else {
		std::string temp = _badKeyword;
		_badKeyword.clear();
		return(temp);
	}
}

std::string Tokenizer::readText()           //read inline text
{
	if(_badKeyword.empty()) {
		std::string str;
		std::string::size_type idx;

		getline(_ifstrm, str);
		idx = str.find('\\');
		while(idx != std::string::npos) {
			if(str[idx+1] == 'n')
				str[idx] = '\n';
			else if(str[idx+1] == '_')
				str[idx] = '\a';
			str.erase(idx+1, 1);

			idx = str.find('\\', idx);
		}
		return(str);
	}

	_ifstrm.setstate(std::ios::failbit, true);
	return("");
}

BYTE Tokenizer::readTextQuality()
{
	if(_badKeyword.empty()) {
		std::string str;

		_ifstrm>>str;
		if(str.compare("antialiased") == 0)         return(ANTIALIASED_QUALITY);
		else if(str.compare("cleartype") == 0)      return(CLEARTYPE_QUALITY);
		else if(str.compare("nonantialiased") == 0) return(NONANTIALIASED_QUALITY);
	}

	_ifstrm.setstate(std::ios::failbit, true);
	return(0);
}

BYTE Tokenizer::readTextItalic()
{
	if(_badKeyword.empty()) {
		std::string str;

		_ifstrm>>str;
		if(str.compare("italic") == 0)         return(TRUE);
		else if(str.compare("non-italic") == 0)      return(FALSE);
		else return(FALSE);
	}

	//_ifstrm.setstate(std::ios::failbit, true);
	return(FALSE);
}

int Tokenizer::readTextWeight()
{
	if(_badKeyword.empty()) {
		std::string str;

		_ifstrm>>str;
		if(str.compare("normal") == 0)         return(FW_NORMAL);
		else if(str.compare("bold") == 0)      return(FW_BOLD);
		else return(FW_NORMAL);
	}

	//_ifstrm.setstate(std::ios::failbit, true);
	return(FW_NORMAL);
}


Tokenizer::TriggerType Tokenizer::readTrigger()
{
	if(_badKeyword.empty()) {
		std::string str;

		_ifstrm>>str;
		if(str.compare("gaze") == 0)              return(GAZE);
		else if(str.compare("nogaze") == 0)       return(NOGAZE);
		else if(str.compare("driftcorrect") == 0) return(DRIFTCORRECT);
		else if(str.compare("driftandgaze") == 0) return(DRIFTANDGAZE);
	}

	_ifstrm.setstate(std::ios::failbit, true);
	return(NO_TRIGGER);
}

void Tokenizer::readWhitespace()
{
	char ch = _ifstrm.peek();

	while(std::isspace(ch)) {
		_ifstrm.ignore();
		ch = _ifstrm.peek();
	}
}



//const windowType &operator=(const windowType &wt);