	/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once
#pragma warning( disable : 4290 )

#include <fstream>
#include <string>
#include "resource.h"
#include "afxwin.h"
#include "ExperimentException.h"
#include "DisplayData.h"

typedef unsigned char BYTE;                      //windows type, redefined here for simplicity



class Tokenizer {
public:
	/*enum ButtonType {                            //Eyetrack button pad key enumeration
		NO_BUTTON = 0,
		BLUE = 1,
		RED = 2,
		GREEN = 3,
		YELLOW = 4,
		PURPLE = 5,
		MOUSE = 6
	} ;*/

	enum ControlType {
		SEVEN_BUTTON,
		FIVE_BUTTON,
		UNKNOWN_CONTROLLER
	};

	enum ButtonType {
		NO_BUTTON,
		Y = 1,
		BLUE = 1,
		X = 2,
		RED = 2,
		B = 3,
		GREEN = 3,
		A = 4,
		YELLOW = 4,
		TOGGLE = 5,
		PURPLE = 5, 
		LEFT_TRIGGER =6,
		RIGHT_TRIGGER = 7,
		MOUSE = 8
	};

	enum Keyword {                               //Keywords in the script file
		ALT,
		AUDIO_FORMAT,
		BACKGROUND,
		BUTTON,
		CALIBRATION_TYPE,
		CALIBRATION_HEIGHT,
		CONDITIONS,
		CURSOR_SIZE,
		DC_DELAY,
		DISPLAY_TYPE,
		END,
		EXPERIMENTS,
		EXPCONDITIONS,
		FILTERMODE,
		FOREGROUND,
		GC_RECT,
		IMAGE,
		INLINE,
		INTER_CHAR,
		MAX_DISPLAY_TIME,
		OUTPUT,
		REGION,
		REGIONPAIR,
		REVERT,
		SCOTOMA,
		SEQUENCE,
		SET,
		STIMULUS_DELAY,
		TEXT_FORMAT,
		TEXT_WEIGHT,
		TRIAL,
		TRIALTYPE,
		TRIGGER,
		WORDS,
		WINDOW,
		WINDOW_THRESHOLD,
		UNKNOWN,
		HIGHLIGHT_COLOR
	};

	enum OutputType {                            //Eyetrack output type enumeration
		NO_OUTPUT,
		NOSTREAM,
		STREAM
	};

	enum TriggerType {                           //Eyetrack stimulus trigger types
		NO_TRIGGER,
		DRIFTANDGAZE,
		DRIFTCORRECT,
		GAZE,
		NOGAZE
	};

	enum FilterMode {
		NONE_SELECTED,
		FILTER_ON,
		FILTER_OFF
	};

	enum CalibrationType {
		H3,
		HV9,
		HV13,
		NO_CALIBRATION
	};

	enum DisplayType {
		LCD,
		CRT,
		NO_DISPLAY_TYPE
	};

	enum {
		CHAR_TRIGGER,
		WORD_TRIGGER
	};

	enum {
		CHAR_TARGET,
		WORD_TARGET
	};

public:
	Tokenizer();
	~Tokenizer();

	static std::string decode(BYTE b);
	static std::string decode5(ButtonType b);
	static std::string decode7(ButtonType b);
	static std::string decode(Keyword k);
	static std::string decode(OutputType o);
	static std::string decode(TriggerType t);
	static std::string decode(FilterMode f);
	static std::string decode(CalibrationType c);
	static std::string decode(DisplayType dt);
	static std::string decodeItalic(BYTE b);
	static std::string decodeWeight(int i);
	std::string readHeader() throw(CExperimentException);
	std::string readString();
	std::string readLine();
	std::string readText();

	CRect readRectangle();
	CPoint readPoint();
	RECTPAIR readRectPair();
	TIMED_RECT_PAIR readTimedRectPair();

	void readExpConditions(int*);

	COLORREF readColor();
	Tokenizer::FilterMode readFilterMode();
	Tokenizer::CalibrationType readCalibrationType();
	Tokenizer::DisplayType readDisplayType();

	ButtonType readButton(Tokenizer::ControlType *c);
//	ButtonTypeSideWinder readButtonSideWinder();
	Keyword readKeyword();
	OutputType readOutput();
	TriggerType readTrigger();

	int readAudioChannels();
	int readInteger();

	BYTE readTextQuality();

	BYTE readTextItalic();
	int readTextWeight();

	char peekChar();
	char readChar();

	bool isOkay() const;

	void close();
	void open(const std::string &filename) throw(CExperimentException);
	void readWhitespace();

private:
	std::ifstream _ifstrm;
	std::string   _badKeyword;
};
