/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once
#pragma warning( disable : 4290 )

#include <list>
#include <string>
#include "ExperimentException.h"

class TrialId {
public:
	enum Type {
		PRACTICE,
		EXPERIMENTAL,
		FILLER,
		ALL,
		DEPENDENT,
		INDEPENDENT
	};

public:
	TrialId();
	TrialId(Type type, unsigned short cond, unsigned short item, 
			unsigned short index, bool seq);
	TrialId(const std::string &identifier) throw(CExperimentException);
	TrialId(const TrialId &id);
	~TrialId();

	const TrialId &operator=(const TrialId &rhs);

	std::string idString() const;

	Type type() const;

	int compare(const TrialId &id) const;

	unsigned short condition() const;
	unsigned short index() const;
	unsigned short item() const;

	bool isDependent() const;
	bool isExperimental() const;
	bool isFiller() const;
	bool isPractice() const;
	bool isSequence() const;

private:
	Type           _type;
	unsigned short _condition;
	unsigned short _item;
	unsigned short _index;
	bool           _isSequence;

};

typedef std::list<TrialId>                 IdentifierList;
typedef std::list<TrialId>::const_iterator IdentifierIter; 
