/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "stdafx.h"
#include <sstream>

#include "EyeTrack.h"
#include "TrialWnd.h"
#include "textPassage.hpp"
#include ".\trialwnd.h"

//#define spacingMod

// CTrialWnd

IMPLEMENT_DYNAMIC(CTrialWnd, CWnd)
CTrialWnd::CTrialWnd()
: _backColor(0xffffff), _foreColor(0x0), _upperLeft(CPoint(0,0)), _lowerRight(CPoint(0,0)), _rButtonDown(false)
{}

CTrialWnd::~CTrialWnd()
{}

BEGIN_MESSAGE_MAP(CTrialWnd, CWnd)
	ON_WM_LBUTTONDOWN()
	ON_WM_RBUTTONDOWN()
	ON_WM_RBUTTONUP()
	ON_WM_PAINT()
	ON_WM_SHOWWINDOW()
	ON_WM_MOUSEMOVE()
END_MESSAGE_MAP()

void CTrialWnd::DrawBackground(const std::string &image)
{
	if(image.empty()) {                          //draw plain background
		CBitmap *pOldBitmap;
		CBrush brush(_backColor), *pOldBrush; 
		CDC dc, *pWndDC = GetDC();

		dc.CreateCompatibleDC(pWndDC);
		ReleaseDC(pWndDC);

		pOldBitmap = dc.SelectObject(&_bitmap);
		pOldBrush = dc.SelectObject(&brush); 

		dc.PatBlt(_wndSz.left, _wndSz.top, _wndSz.Width(), _wndSz.Height(), PATCOPY);
		dc.SelectObject(pOldBrush);
		dc.SelectObject(pOldBitmap);
	}
	else {                                       //load image as background
		CSize size(_wndSz.Width(), _wndSz.Height());

		if(!_bitmap.LoadBitmap(image.c_str(), size))
			MessageBox("Unable to load bitmap file.", "Preview Error");
	}
}

void CTrialWnd::DrawDCRect(const CRect &r, bool isSelected)
{
	CBitmap *pOldBitmap;
	CBrush brush;
	CDC dc, *pWndDC = GetDC();

	dc.CreateCompatibleDC(pWndDC);
	ReleaseDC(pWndDC);

	if(isSelected)
		brush.CreateSolidBrush(0x0000FF);
	else
		brush.CreateSolidBrush(0x00FF00);

	pOldBitmap = dc.SelectObject(&_bitmap);
	dc.FrameRect(&r, &brush);
	dc.SelectObject(pOldBitmap);
}

void CTrialWnd::DrawGCRect(const CRect &r)
{
	CBitmap *pOldBitmap;
	CBrush brush(0x00FF00), *pOldBrush; 
	CDC dc, *pWndDC = GetDC();

	dc.CreateCompatibleDC(pWndDC);
	ReleaseDC(pWndDC);

	pOldBitmap = dc.SelectObject(&_bitmap);
	pOldBrush = dc.SelectObject(&brush); 

	CRect lr;
	lr.left = MulDiv(r.left, dc.GetDeviceCaps(LOGPIXELSX), 72);
	lr.right = MulDiv(r.right, dc.GetDeviceCaps(LOGPIXELSX), 72);
	lr.top = MulDiv(r.top, dc.GetDeviceCaps(LOGPIXELSY), 72);
	lr.bottom = MulDiv(r.bottom, dc.GetDeviceCaps(LOGPIXELSY), 72);

	//dc.PatBlt(lr.left, lr.top, lr.Width(), lr.Height(), PATCOPY);
	dc.PatBlt(r.left, r.top, r.Width(), r.Height(), PATCOPY);
	dc.SelectObject(pOldBrush);
	dc.SelectObject(pOldBitmap);
}


void CTrialWnd::DrawText(const std::string &text, const TextFormat &fmt, DisplayData dd, int doDisplay, int spacingDiff)
{
	if(text.empty() && ((dd.images1.empty() && doDisplay == 1) || dd.images2.empty() && doDisplay == 2 || dd.images3.empty() && doDisplay == 3))
		return;

	CBitmap *pOldBitmap;
	CBrush brush(0xFF0000); 
	CDC /*dc,*/ *pWndDC = GetDC();
	CFont font, *pOldFont, font2;                   //font for text, temporary
	TextPassage tp(text, _wndSz);

	CDC memdc; //, memdc2; // MOD
	memdc.CreateCompatibleDC(pWndDC); // MOD
	//memdc.SelectObject(&_bitmap);
	//memdc2.CreateCompatibleDC(pWndDC);

	ReleaseDC(pWndDC);

//***	dc.CreateCompatibleDC(pWndDC);
//***	ReleaseDC(pWndDC);

	int fontHt = -MulDiv(fmt.fontHt, memdc.GetDeviceCaps(LOGPIXELSY), 72);
	int lineSpace = MulDiv(fmt.lineSpace, memdc.GetDeviceCaps(LOGPIXELSY), 72);
	int xOffset = MulDiv(fmt.xOffset, memdc.GetDeviceCaps(LOGPIXELSX), 72);
	int yOffset = MulDiv(fmt.yOffset, memdc.GetDeviceCaps(LOGPIXELSY), 72);

	//BOLDSTUFF
	font.CreateFont(fontHt, 0, 0, 0, fmt.fontWt, fmt.italic, FALSE, 0, ANSI_CHARSET,
					OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, fmt.rendering,
					DEFAULT_PITCH | FF_DONTCARE, fmt.fontName.c_str());
#ifdef spacingMod
	memdc.SetMapMode(MM_HIENGLISH);
		//_dc.SetMapMode(MM_HIENGLISH);
	font2.CreateFont(tp.fontHeight(), 0/*tm.tmAveCharWidth*1*/,0, 0, fmt.fontWt, fmt.italic, FALSE, 0, ANSI_CHARSET,
						OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, fmt.rendering,
						DEFAULT_PITCH | FF_DONTCARE, fmt.fontName.c_str());
	memdc.SetMapMode(MM_TEXT);
#endif
//***	pOldBitmap = dc.SelectObject(&_bitmap);  //draw onto the bitmap

	//pOldFont = dc.SelectObject(&font);       //select font

/***** image_text additions START ***/
//	int displayIdx = (doDisplay2 ? 2 : 1);
	int displayIdx = doDisplay;
	CBitmapFile bmpPart;
	bmpPart.CreateCompatibleBitmap(pWndDC, _wndSz.right, _wndSz.bottom);



//	memdc2.SelectObject(bmpPart);
	//memdc2.CreateCompatibleDC(pDC);

		CSize fullsize(_wndSz.Width(), _wndSz.Height());
		CSize zero(0,0);

//***		_bitmap.LoadBitmap(dd.images1.back().fileName.c_str(), fullsize);
		pOldBitmap = memdc.SelectObject(&_bitmap);

		CBrush bgBrush(_backColor);
		memdc.SelectObject(&bgBrush);
		memdc.PatBlt(_wndSz.left, _wndSz.top, _wndSz.right, _wndSz.bottom, PATCOPY);



		std::vector<IMAGEPOS>::iterator imageIter, end;

		if(doDisplay == 3){
			imageIter = dd.images3.begin();
			end = dd.images3.end();
		}
		else if(doDisplay == 2){
			imageIter = dd.images2.begin();
			end = dd.images2.end();
		}
		else if(doDisplay == 1 ){ // doing display 1
			imageIter = dd.images1.begin();
			end = dd.images1.end();
		}
		else {
			imageIter == dd.images3.begin();
			end = dd.images3.end();
		}

		 while (imageIter != end){
			//if (imageIter == NULL || end == NULL)
			//	break;
			imagePos ip = imagePos(imageIter->fileName, imageIter->upperLeft);

			CBitmapFile smallBmp;
			CDC small_memdc;

			smallBmp.CreateCompatibleBitmap(pWndDC, _wndSz.right, _wndSz.bottom);
			small_memdc.CreateCompatibleDC(pWndDC);

			smallBmp.LoadBitmap(ip.fileName.c_str(), zero);
			small_memdc.SelectObject(smallBmp);
			memdc.BitBlt(ip.upperLeft.x, ip.upperLeft.y,fullsize.cx,fullsize.cy,&small_memdc, 0,0,SRCCOPY);
			
			//eyemsg_printf("LOADED IMAGE %i %s AT (%d, %d)", displayIdx, ip.fileName.c_str(), ip.upperLeft.x, ip.upperLeft.y);
			
			smallBmp.DeleteObject();
			small_memdc.DeleteDC();


			imageIter++;
		}//(imageIter != t.displayData().images1.end());

	

	/***** image_text additions END ***/

	pOldFont = memdc.SelectObject(&font);

	memdc.SetBkMode(TRANSPARENT);               //do not draw background behind text
	memdc.SetMapMode(MM_TEXT);                  //mapping mode most useful for text
	memdc.SetTextColor(_foreColor);             //black text

#ifndef spacingMod
	tp.computeRegion(memdc, CPoint(xOffset, yOffset), CSize(0, lineSpace/2), &font);
	tp.drawText(memdc, &brush, tp.getDisplay().isMovingWindow, "", &font);
#endif 

#ifdef spacingMod

	tp.computeRegion(memdc, CPoint(xOffset, yOffset), CSize(0, lineSpace/2), &font2);
	memdc.SelectObject(&font2);
	tp.drawText(memdc, &brush, tp.getDisplay().isMovingWindow, "", &font2, fmt.spacingDiff);
#endif
	//tp.drawText(dc, &brush, TRUE);
	


	memdc.SelectObject(pOldBitmap);
	memdc.SelectObject(pOldFont);
	//dc.SelectObject(pOldBitmap);
	//dc.SelectObject(pOldFont);
}


void CTrialWnd::OnMouseMove(UINT nFlags, CPoint point){

	if(_rButtonDown){
		
		_lowerRight.x = point.x; _lowerRight.y = point.y;
		Invalidate(FALSE);
	}
	CWnd::OnMouseMove(nFlags, point);
}

void CTrialWnd::OnRButtonDown(UINT nFlags, CPoint point)
{
	_upperLeft.x = point.x; _upperLeft.y = point.y;
	_lowerRight.x = point.x; _lowerRight.y = point.y;

	_rButtonDown = true;
	
	Invalidate(FALSE);
	CWnd::OnRButtonDown(nFlags, point);
}

void CTrialWnd::OnRButtonUp(UINT nFlags, CPoint point)
{
	_lowerRight.x = point.x; _lowerRight.y = point.y;

	_rButtonDown = false;
	Invalidate(FALSE);
	CWnd::OnRButtonUp(nFlags, point);
}

void CTrialWnd::OnLButtonDown(UINT nFlags, CPoint point)
{
	_mousePt.x = point.x;
	_mousePt.y = point.y;


	Invalidate(FALSE);

	char buf[10];
	CString cs;
	//GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_X1)->GetWindowText(buf, 10); 
	//->SetWindowText(_itoa(point.x, buf, 10));
	//GetDlgItem(IDC_EDIT_TRIAL_DISPLAY_CHANGE_Y1)->SetWindowText(_itoa(point.y, buf, 10));
	
	CWnd::OnLButtonDown(nFlags, point);
}

void CTrialWnd::OnPaint()
{
	CBitmap *pOldBmp;                            //temporary pointer
	CDC bmpDC;                                   //drawing context for the bitmap
	CPaintDC dc(this);                           //device context for painting
	CPen pen(PS_SOLID, 1, 0x0000FF), *pOldPen;   //pen for marking points
	std::ostringstream os, os2;

	bmpDC.CreateCompatibleDC(&dc);               //create drawing context for the bitmap

	pOldBmp = bmpDC.SelectObject(&_bitmap);      //select the bitmap into context

	dc.BitBlt(0, 0, _wndSz.right, _wndSz.bottom, //blast bitmap to window
			  &bmpDC, 0, 0, SRCCOPY);

	bmpDC.SelectObject(pOldBmp);                 //deselect bitmap

	pOldPen = dc.SelectObject(&pen);             //draw a marked point
	dc.MoveTo(_mousePt.x, _mousePt.y-10);
	dc.LineTo(_mousePt.x, _mousePt.y+10);
	dc.MoveTo(_mousePt.x+10, _mousePt.y);
	dc.LineTo(_mousePt.x-10, _mousePt.y);

		dc.MoveTo(_upperLeft.x, _upperLeft.y);
		dc.LineTo(_upperLeft.x, _lowerRight.y);
		dc.LineTo(_lowerRight.x, _lowerRight.y);
		dc.LineTo(_lowerRight.x, _upperLeft.y);
		dc.LineTo(_upperLeft.x, _upperLeft.y);


		dc.SelectObject(pOldPen);

		dc.SetBkMode(TRANSPARENT);
		dc.SetTextColor(0x0000FF);
		os<<'('<<_upperLeft.x<<','<<_upperLeft.y<<')';   //draw the coordinates
		dc.TextOut(_upperLeft.x-10, _upperLeft.y-20, 
			os.str().c_str(), static_cast<int>(os.str().size()));

		os.flush();

		os.clear();
		os2<<'('<<_lowerRight.x<<','<<_lowerRight.y<<')';   //draw the coordinates
		dc.TextOut(_lowerRight.x+10, _lowerRight.y+10, 
			os2.str().c_str(), static_cast<int>(os2.str().size()));

		os2.flush();
	

	//dc.SelectObject(pOldPen);


	os<<'('<<_mousePt.x<<','<<_mousePt.y<<')';   //draw the coordinates
	dc.SetBkMode(TRANSPARENT);
	dc.SetTextColor(0x0000FF);
	dc.TextOut(_mousePt.x-10, _mousePt.y+10, 
			   os.str().c_str(), static_cast<int>(os.str().size()));
}

void CTrialWnd::OnShowWindow(BOOL bShow, UINT nStatus)
{
	CWnd::OnShowWindow(bShow, nStatus);

	_wndSz.top = _wndSz.left = 0;
	_wndSz.right = ::GetSystemMetrics(SM_CXSCREEN);
	_wndSz.bottom = ::GetSystemMetrics(SM_CYSCREEN);

	CDC *pDC = GetDC();
	_bitmap.CreateCompatibleBitmap(pDC, _wndSz.right, _wndSz.bottom);
	ReleaseDC(pDC);
}

void CTrialWnd::PostNcDestroy()
{
	CWnd::PostNcDestroy();
	delete this;
}

void CTrialWnd::SetWindowColors(COLORREF backColor, COLORREF foreColor)
{
	_backColor = backColor;
	_foreColor = foreColor;
}

void CTrialWnd::setEditDisplayCwnd(CEditTrialDisplayDialog edit){
	_editWnd = &edit;
}

CPoint CTrialWnd::getUpperLeft(){
	return _upperLeft;
}

CPoint CTrialWnd::getLowerRight(){
	return _lowerRight;
}

