/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once
#pragma warning( disable : 4290 )

#include "stdafx.h"
#include "sequence.h"
#include "trial.hpp"
#include "trial-type.hpp"

#define EXPT_ALL      0
#define EXPT_FILLER   -1
#define EXPT_PRACTICE -2

class Random;                                    //predeclaration

class Experiment {
public:
	enum OrderType {                             //trial ordering types
		FILE,
		NONE,
		RANDOM,
		RANDOM_INTER,
		SCRIPT
	};

public:
	Experiment();
	~Experiment();

	IdentifierList getSequenceNames() const;
	IdentifierList getTrialNames(TrialId::Type) const;
	IdentifierList selectTrials(const std::string &fileName) const 
		throw(CExperimentException);
	IdentifierList selectTrials(int expId, int groupID, int itemNum = 0) const;
	IdentifierList selectTrials(int groupID, bool interleave) const 
		throw(CExperimentException);

	StringList getTrialTypeNames() const;

	Sequence *getSequence(const TrialId &identifier) const;

	Trial *getTrial(const TrialId &identifier) const;
	Trial *getTrial(const TrialId &identifier);

	TrialType *getTrialType(const std::string &identifier) const;

	const std::string header(bool formatForEditBox) const;

	COLORREF backgroundColor() const;
	COLORREF foregroundColor() const;

	int countTrials(const IdentifierList &l);
	int nConditions() const;
	int nExperiments() const;
	int nTrials() const;
	Tokenizer::FilterMode filterMode() const;

	bool addSequence(const Sequence &s);
	bool addTrial(const Trial &t);
	bool addTrialType(TrialType &t);
	bool hasInlineText() const;
	bool isSaved() const;
	bool validate(std::ostream &os) const;

	void deleteSequence(const std::string &identifier);
	void deleteTrial(const std::string &identifier);
	void deleteTrialType(const std::string &identifier);
	void load(const std::string &fileName) throw(CExperimentException);
	void save(std::ostream &os);
	void setColors(COLORREF bkgd, COLORREF fgd);
	void setConditions(int nConds, int nExp);
	void setHeader(const std::string &header);
	void setFilterMode(const Tokenizer::FilterMode fMode);
	void setCalibrationType(const Tokenizer::CalibrationType cType);
	void setDisplayType(const Tokenizer::DisplayType dType);
	void setDCThreshold(const unsigned int DCthresh);
	Tokenizer::FilterMode getFilterMode();
	Tokenizer::CalibrationType calibrationType();
	Tokenizer::DisplayType getDisplayType();
	int calibrationHeight();
	unsigned int getDCThreshold();
	Tokenizer::ControlType getControlType() const;
	void setExpConditions(int _expC[20]);
	void getExpConditions(int buf[20]);


protected:
	void randomize(IdentifierList &l, Random &rng) const;

private:
	SequenceList  _sequences;                    //list of trial sequences
	TrialList     _trials;                       //list of trials
	TrialTypeList _trialTypes;                   //list of trial conditions
	std::string   _header;
	COLORREF      _backColor;
	COLORREF	  _foreColor;
	int  _nConditions;
	int  _nExperiments;
	int  _expConditions[20];
	int  _nTrials;
	int _dcThreshold;
	Tokenizer::FilterMode _filterMode;
	Tokenizer::CalibrationType	 _calibrationType;
	int _calibrationHeight;
	Tokenizer::DisplayType _displayType;
	bool _isSaved;
	Tokenizer::ControlType _controlType;
	int sum(int *e, int t);
};
