/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include "random.hpp"
#include <time.h>

/* Random number generator implementation.
   Based on algorithm in Numerical Recipes in C (p. 280)
*/

const float Random::AM = (1.0f / IM);
const float Random::RNMX = (1.0f - FLT_EPSILON);

Random::Random()
  : iy(0)
{
	idnum = static_cast<long>(time(0));

	if(idnum < 1) idnum = 1;
	for(int i=NTAB+7; i>=0; i--) {
		long k = idnum / IQ;
		idnum = IA * (idnum - k*IQ) - IR*k;
		if(idnum < 0) idnum += IM;
		if(i < NTAB) iv[i] = idnum;
	}
	iy = iv[0];
}

Random::Random(long seed)
  : iy(0)
{
	if(seed < 1) idnum = 1;
	else idnum = seed;

	for(int i=NTAB+7; i>=0; i--) {
		long k = idnum / IQ;
		idnum = IA * (idnum - k*IQ) - IR*k;
		if(idnum < 0) idnum += IM;
		if(i < NTAB) iv[i] = idnum;
	}
	iy = iv[0];
}

Random::~Random()
{}

float Random::generateRandom()
{
	float temp;
	long k;
	int j;

	k = idnum / IQ;
	idnum = IA * (idnum - k*IQ) - IR*k;
	if(idnum < 0) idnum += IM;
	j = iy / NDIV;
	iy = iv[j];
	iv[j] = idnum;
	temp = AM*iy;
	if(temp > RNMX) return(RNMX);
	return(temp);
}

float Random::randomFloat(float min, float range)
{
	float temp = generateRandom() * range;
	return(temp + min);
}

int Random::randomInt(int min, int range)
{
	float temp = generateRandom() * float(range);
	return(int(temp) + min);
}

void Random::reseed(long seed)
{
	if(seed < 1) idnum = static_cast<long>(time(0));
	else idnum = seed;

	for(int i=NTAB+7; i>=0; i--) {
		long k = idnum / IQ;
		idnum = IA * (idnum - k*IQ) - IR*k;
		if(idnum < 0) idnum += IM;
		if(i < NTAB) iv[i] = idnum;
	}
	iy = iv[0];
}
