/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#ifndef RANDOM_NUMBER_GENERATOR
#define RANDOM_NUMBER_GENERATOR

#include <cfloat>

/* Random number generator class. Based on Park & Miller with
   Bays-Durham shuffle and other added safeguards. Random numbers
   generated range between 0.0 and 1.0 (exclusive of the endpoints).
   Do not alter the initialization value between calls to the generator.
*/

class Random {
public:
	Random();
	Random(long seed);
	~Random();

	float randomFloat(float min, float range);

	int randomInt(int min, int range);

	void reseed(long seed = 0);

private:
	float generateRandom();

	long idnum;
	long iy;
	long iv[32];

	const static long IA = 16807;
	const static long IM = 2147483647;
	const static long IQ = 127773;
	const static long IR = 2836;
	const static long NTAB = 32;
	const static long NDIV = (1+(IM-1)/NTAB);
	  
	const static float AM;
	const static float RNMX;
};

#endif
