/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <list>
#include "StdAfx.h"
#include "Eyelink2.h"
#include "DisplayData.h"
#include "TextPassage.hpp"
//#define spacingMod
//#define newMode


/*****************************************************************************/

TextPassage::Character::Character(int idx, char c)
  : textRgn(0, 0, 0, 0), trackRgn(0, 0, 0, 0), index(idx), ch(c)
{}

TextPassage::Character::Character(const Character &c)
{
	textRgn  = c.textRgn;
	trackRgn = c.trackRgn;
	index    = c.index;
	ch       = c.ch;
}

const TextPassage::Character &
TextPassage::Character::operator=(const TextPassage::Character &rhs)
{
	textRgn  = rhs.textRgn;
	trackRgn = rhs.trackRgn;
	index    = rhs.index;
	ch       = rhs.ch;
	return(*this);
}

/*****************************************************************************/
TextPassage::TextPassage(const std::string &text, const CRect &wndSz)
 : _line(0), _gcRect(0, 0, 0, 0), _trackRgn(wndSz), _fontHt(0), _nLines(1), f2start(15), f2end(25), _hColor(0)
{
	int lineIdx = 0;
	int wordIdx = 0;

	for(unsigned int i=0; i < text.size(); ++i)  //count number of ext lines
		if(text[i] == '\n') ++_nLines;

	_line = new std::list<TextPassage::Character>[_nLines];


	for(unsigned int i=0; i < text.size(); ++i) { //break text into lines
		if(text[i] == '\a')
			continue;
		if(text[i] == '\n') {
			++lineIdx;
			continue;
		}
		_line[lineIdx].push_back(Character(i, text[i]));
	}
}

TextPassage::TextPassage(const std::string &text, const CRect &wndSz, DisplayData &dd, COLORREF hColor)
//TextPassage::TextPassage(const std::string &text, const CRect &wndSz, Trial t)
: _line(0), _gcRect(0, 0, 0, 0), _trackRgn(wndSz), _fontHt(0), _nLines(1), _display(), f2start(15), f2end(25)
{

	_hColor = hColor;
	int lineIdx = 0;
	int wordIdx = 0;
//	int state = IN_WHITESPACE;
//	char x = 't' | 'p';

	_display = dd;

	for(unsigned int i=0; i < text.size(); ++i)  //count number of ext lines
		if(text[i] == '\n') ++_nLines;

	_line = new std::list<TextPassage::Character>[_nLines];
	
	for(unsigned int i=0; i < text.size(); ++i) { //break text into lines
		if(text[i] == '\a')
			continue;
		if(text[i] == '\n') {
			++lineIdx;
		//	state = IN_WHITESPACE;
			continue;
		}
		_line[lineIdx].push_back(Character(i, text[i]));
	}
}

TextPassage::~TextPassage(void)
{
	if(_line)
		delete [] _line;
}

void TextPassage::computeProperties(CDC &dc, const TextFormat &tf, const TextFormat &tf2)
{
	int lineSpace, xOffset, yOffset;             //size and space measurements
	CFont font, *oldFont;                        //font for text, temporary
	CFont font2;

	dc.SetMapMode(MM_TEXT);                      //mapping mode most useful for text
	                                             //compute text properties
	_fontHt = -MulDiv(tf.fontHt, dc.GetDeviceCaps(LOGPIXELSY), 72);
	lineSpace = MulDiv(tf.lineSpace, dc.GetDeviceCaps(LOGPIXELSY), 72);
	xOffset = MulDiv(tf.xOffset, dc.GetDeviceCaps(LOGPIXELSX), 72);
	yOffset = MulDiv(tf.yOffset, dc.GetDeviceCaps(LOGPIXELSY), 72);

                                                 //generate font
	font.CreateFont(_fontHt, 0, 0, 0, tf.fontWt, tf.italic/*, FW_NORMAL, FALSE*/, FALSE, 0, ANSI_CHARSET,
					OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, tf.rendering,
					DEFAULT_PITCH | FF_DONTCARE, tf.fontName.c_str());
#ifdef newMode
	font2.CreateFont(_fontHt, 0, 0, 0, tf2.fontWt, tf2.italic/*, FW_NORMAL, FALSE*/, FALSE, 0, ANSI_CHARSET,
					OUT_DEFAULT_PRECIS, CLIP_DEFAULT_PRECIS, tf2.rendering,
					DEFAULT_PITCH | FF_DONTCARE, tf2.fontName.c_str());
#endif

	oldFont = dc.SelectObject(&font);             //select font
	                                              //compute bounding regions for passage
	computeRegion(dc, CPoint(xOffset, yOffset), CSize(0, lineSpace/2), &font2);

	dc.SelectObject(oldFont);                     //deselect font
}


void TextPassage::computeRegionOrig(CDC &dc, const CPoint &offset, const CSize &padding)

{
	CPoint upperLeft(offset);
	TEXTMETRIC tm;
	int lineHt, whiteSpace = 0;

	if (_display.isMovingWindow) {
		whiteSpace = 0;
	}


	dc.GetTextMetrics(&tm);
	lineHt = tm.tmHeight + 2*padding.cy + 10;

	_trackRgn.SetRect(0, 0, 0, 0);
	for(int i=0; i < _nLines; ++i) {
		std::list<TextPassage::Character> &l = _line[i];
		std::list<TextPassage::Character>::iterator ci;

		for(ci = l.begin(); ci != l.end(); ++ci) {
			char c = ci->ch;
			int a = static_cast<int>(c);
			CSize chSz;		
			if (_ismbblead(c)){
				char tmp[3];
				tmp[0] = ci->ch;
				if (ci++ != l.end() && _ismbbtrail(ci->ch)) {
					tmp[1] = ci->ch;
					tmp[2] = '\0';
					chSz = dc.GetTextExtent(tmp, 2);
					ci--;
					ci->textRgn.SetRect(upperLeft, upperLeft+chSz);
					ci->textRgn.OffsetRect(padding);
					upperLeft.x += ci->textRgn.Width();             //shift location right




					if(_gcRect.IsRectNull() && ci->ch != ' ' && ci->ch != '\t' && ci->ch != '\n')
						_gcRect = ci->textRgn;
	
					ci->trackRgn = ci->textRgn;
					ci->trackRgn.InflateRect(CSize(padding.cx, whiteSpace + padding.cy));              //trackRgn is textRgn plus padding
					_trackRgn |= ci->trackRgn;

					// fill in blanks for the trail byte in MBCS
					ci++;
					ci->textRgn.SetRect(upperLeft, upperLeft);
					ci->textRgn.OffsetRect(padding);
					ci->trackRgn = ci->textRgn;
					//ci->trackRgn.InflateRect(CSize(padding.cx, whiteSpace + padding.cy));              //trackRgn is textRgn plus padding
					//_trackRgn |= ci->trackRgn;

	
				}
				else {
					tmp[1] = '\0';
					ci--;
					chSz = dc.GetTextExtent(tmp, 1);
					ci->textRgn.SetRect(upperLeft, upperLeft+chSz);
					ci->textRgn.OffsetRect(padding);

					if(_gcRect.IsRectNull() && ci->ch != ' ' && ci->ch != '\t' && ci->ch != '\n')
						_gcRect = ci->textRgn;
	
					ci->trackRgn = ci->textRgn;
					ci->trackRgn.InflateRect(CSize(padding.cx, whiteSpace + padding.cy));              //trackRgn is textRgn plus padding
					_trackRgn |= ci->trackRgn;

					upperLeft.x += ci->textRgn.Width();             //shift location right
				}
			}

			else{ //not lead byte
				chSz = dc.GetTextExtent(&(ci->ch), 1);
	
				ci->textRgn.SetRect(upperLeft, upperLeft+chSz); //compute rect enclosing char
				ci->textRgn.OffsetRect(padding);                //shift rect down by 1/2 line-space

				if(_gcRect.IsRectNull() && ci->ch != ' ' && ci->ch != '\t' && ci->ch != '\n')
					_gcRect = ci->textRgn;
	
				ci->trackRgn = ci->textRgn;
				ci->trackRgn.InflateRect(CSize(padding.cx, whiteSpace + padding.cy));              //trackRgn is textRgn plus padding
				_trackRgn |= ci->trackRgn;

				upperLeft.x += ci->textRgn.Width();             //shift location right
			}


		}
		upperLeft.x = offset.x;                             //move to beginning of next line
		upperLeft.y += lineHt;
	}

	if(_gcRect.Height() < 25)
		_gcRect.bottom = _gcRect.top + 25;

	if(_gcRect.Width() < 25)
		_gcRect.right = _gcRect.left + 25;
}



void TextPassage::computeRegion(CDC &dc, const CPoint &offset, const CSize &padding, CFont *font2)

{
	bool inFont2 = false;
	CFont *oldFt;
	CPoint upperLeft(offset);
	TEXTMETRIC tm;
	int lineHt, whiteSpace = 0;

	if (_display.isMovingWindow) {
		whiteSpace = 0;
	}


	dc.GetTextMetrics(&tm);
	lineHt = tm.tmHeight + 2*padding.cy + 10;

	_trackRgn.SetRect(0, 0, 0, 0);
	for(int i=0; i < _nLines; ++i) {
		std::list<TextPassage::Character> &l = _line[i];
		std::list<TextPassage::Character>::iterator ci;

		for(ci = l.begin(); ci != l.end(); ++ci) {
			if(ci->index == f2start ){
				oldFt = dc.SelectObject(font2);
				inFont2 = true;
			}
			if(ci->index == f2end && inFont2){
				dc.SelectObject(oldFt);
				inFont2 = false;
			}
			char c = ci->ch;
			int a = static_cast<int>(c);
			CSize chSz;		
			if (_ismbblead(c)){
				char tmp[3];
				tmp[0] = ci->ch;
				if (ci++ != l.end() && _ismbbtrail(ci->ch)) {
					tmp[1] = ci->ch;
					tmp[2] = '\0';
					chSz = dc.GetTextExtent(tmp, 2);
					ci--;
					ci->textRgn.SetRect(upperLeft, upperLeft+chSz);
					ci->textRgn.OffsetRect(padding);
					upperLeft.x += ci->textRgn.Width();             //shift location right




					if(_gcRect.IsRectNull() && ci->ch != ' ' && ci->ch != '\t' && ci->ch != '\n')
						_gcRect = ci->textRgn;
	
					ci->trackRgn = ci->textRgn;
					ci->trackRgn.InflateRect(CSize(padding.cx, whiteSpace + padding.cy));              //trackRgn is textRgn plus padding
					_trackRgn |= ci->trackRgn;

					// fill in blanks for the trail byte in MBCS
					ci++;
					ci->textRgn.SetRect(upperLeft, upperLeft);
					ci->textRgn.OffsetRect(padding);
					ci->trackRgn = ci->textRgn;
					//ci->trackRgn.InflateRect(CSize(padding.cx, whiteSpace + padding.cy));              //trackRgn is textRgn uplus padding
					//_trackRgn |= ci->trackRgn;

	
				}
				else {
					tmp[1] = '\0';
					ci--;
					chSz = dc.GetTextExtent(tmp, 1);
					ci->textRgn.SetRect(upperLeft, upperLeft+chSz);
					ci->textRgn.OffsetRect(padding);

					if(_gcRect.IsRectNull() && ci->ch != ' ' && ci->ch != '\t' && ci->ch != '\n')
						_gcRect = ci->textRgn;
	
					ci->trackRgn = ci->textRgn;
					ci->trackRgn.InflateRect(CSize(padding.cx, whiteSpace + padding.cy));              //trackRgn is textRgn plus padding
					_trackRgn |= ci->trackRgn;

					upperLeft.x += ci->textRgn.Width();             //shift location right
				}
			}

			else{ //not lead byte
				chSz = dc.GetTextExtent(&(ci->ch), 1);
	
				ci->textRgn.SetRect(upperLeft, upperLeft+chSz); //compute rect enclosing char
				ci->textRgn.OffsetRect(padding);                //shift rect down by 1/2 line-space

				if(_gcRect.IsRectNull() && ci->ch != ' ' && ci->ch != '\t' && ci->ch != '\n')
					_gcRect = ci->textRgn;
	
				ci->trackRgn = ci->textRgn;
				ci->trackRgn.InflateRect(CSize(padding.cx, whiteSpace + padding.cy));              //trackRgn is textRgn plus padding
				_trackRgn |= ci->trackRgn;

				upperLeft.x += ci->textRgn.Width();             //shift location right


			}


		}
		upperLeft.x = offset.x;                             //move to beginning of next line
		upperLeft.y += lineHt;
	}

	if(_gcRect.Height() < 25)
		_gcRect.bottom = _gcRect.top + 25;

	if(_gcRect.Width() < 25)
		_gcRect.right = _gcRect.left + 25;
}

bool isWhiteSpace( char c ){
	if (c == '\f' || c == '\n' || c == '\r' || c == '\t' || c == ' ')
		return true;
	return false;
}

bool TextPassage::isWordStart( char prev, char c, bool lineBegin )
{
	if (lineBegin && !isWhiteSpace(c))
		return true;
	if (isWhiteSpace(prev) && !(isWhiteSpace(c)))
		return true;
	return false;
}

bool TextPassage::isWordEnd( char c, char next, bool lineEnd )
{
	if (lineEnd && !isWhiteSpace(c))
		return true;
	if (isWhiteSpace(next) && !(isWhiteSpace(c)))
		return true;
	return false;
}

unsigned int TextPassage::isToggle(int index){
	for (int i=0; i<_display.toggleWords.size(); i++){
		if(_display.toggleWords.at(i).wordIndex == index)
			return _display.toggleWords.at(i).duration;
	}
	return 0;
}

int sigma(INT *spaces, int start, int end){
	int sum = 0;
	for (int i = start; i<=end; i++){
		sum += spaces[i];
	}
	return sum;
}

void TextPassage::drawTextLetterSpacingMods(CDC &dc, CRect line_rect, char* tmp_str, int tmp_i){
		line_rect.MoveToX(line_rect.left + 171);
		//CFont* oldft = dc.SelectObject(ft);
		INT spaces[100];
			int j = 0;
			for(int i=0; i<100;i++){
				spaces[i] = 81;
				if(i+1 < tmp_i && tmp_str[i+1] == ' '){
					//spaces[i] = 81 + ((j-1) * 11);
					spaces[i] = (81 * (i+1)) - sigma(spaces, 0, i-1) + 5;
					j=0;
				}
				else{
				spaces[i] = 76; //MM_HIENGLISH
				}
				j++;
				//spaces[i] = 9.9;		//MM_TEXT
			}

			//dc.SetMapMode(MM_TWIPS);
			dc.SetMapMode(MM_HIENGLISH);
			dc.ExtTextOut(2400, -2300, 0, CRect(100,100,3000,3000), tmp_str, tmp_i, spaces);
			//dc.ExtTextOut(267, 254, 0, CRect(100,100,3000,3000), tmp_str1, tmp_i_1, spaces);

			line_rect.MoveToX(line_rect.left+45);
			//dc.SelectObject(oldft);
			dc.SetMapMode(MM_TEXT);
}

doublePoint TextPassage::getMultiplier(CPoint rez){
	doublePoint mult;
	switch(rez.x){
		case 1024: mult.x = 12.3; break;
		case 1280: mult.x = 9.85; break;
		default: mult.x = 9.85; break;
	}

	switch(rez.y){
		case 768: mult.y = -12.3; break;
		case 1024: mult.y = -9.85; break;
		default: mult.y = -9.85; break;
	}

	return mult;
}


void TextPassage::drawText(CDC &dc, CBrush *pBrush/*&, DisplayData *dd*/, bool isMovWind, CString ID, CFont *ft, int interCharSpacing)
{	
		CBrush tBrush(_hColor);
		CBrush oldBrush;
		
		

	if( !isMovWind ){	
		if(pBrush)
			dc.FrameRect(_trackRgn, pBrush);
	
		for(int i=0; i < _nLines; ++i) {
			std::list<TextPassage::Character> &l = _line[i];
			std::list<TextPassage::Character>::iterator ci;

			char tmp_str[500];
			char tmp_str1[500];
			char tmp_str2[500];
			char tmp_str3[500];
			int tmp_i = 0; 
			int tmp_i_1 = 0;
			int tmp_i_2 = 0;
			int tmp_i_3 = 0;
			CRect line_rect = CRect(0,0,0,0), line1_rect = CRect(0,0,0,0), line2_rect = CRect(0,0,0,0), line3_rect = CRect(0,0,0,0);
 
			for(ci = l.begin(); ci != l.end(); ++ci) {
				if (tmp_i < 500) {
					tmp_str[tmp_i++] = ci->ch;
					if (ci->index < f2start){
						tmp_str1[tmp_i_1++] = ci->ch;
						line1_rect |= ci->textRgn;
					}
					else if( ci->index < f2end){
						tmp_str2[tmp_i_2++] = ci->ch;
						line2_rect |= ci->textRgn;
					}
					else{
						tmp_str3[tmp_i_3++] = ci->ch;
						line3_rect |= ci->textRgn;
					}
						
					line_rect |= ci->textRgn;
					if(pBrush){	
						dc.FrameRect(ci->trackRgn, pBrush);
					}		
				}

		/*		if( isleadbyte(atoi(&(ci->ch))))
					dc.DrawText(&(ci->ch), 1 , &(ci->textRgn), DT_LEFT | DT_TOP | DT_NOCLIP | DT_NOPREFIX);
				else 		
					dc.DrawText(&(ci->ch), 1 , &(ci->textRgn), DT_LEFT | DT_TOP | DT_NOCLIP | DT_NOPREFIX);
				if(pBrush)
					dc.FrameRect(ci->trackRgn, pBrush); */
			}
			tmp_str[tmp_i] = '\0';
			tmp_str1[tmp_i_1] = '\0';
			tmp_str2[tmp_i_2] = '\0';
#ifndef newMode
#ifndef spacingMod
		 	dc.DrawText(tmp_str, tmp_i, line_rect, DT_LEFT | DT_TOP | DT_NOCLIP | DT_NOPREFIX);
#endif
#endif
			//dc.DrawText(tmp_str, 19, line_rect, DT_LEFT | DT_TOP | DT_NOCLIP | DT_NOPREFIX);

#ifdef newMode
			dc.DrawText(tmp_str1, tmp_i_1, line1_rect, DT_LEFT | DT_TOP | DT_NOCLIP | DT_NOPREFIX);
			//line_rect.MoveToX(line_rect.left + 171);
			CFont* oldft = dc.SelectObject(ft);

			dc.SelectObject(&tBrush);
			dc.PatBlt(line2_rect.TopLeft().x,
									line2_rect.TopLeft().y,
									line2_rect.Width(),
									line2_rect.Height(),PATCOPY);
			dc.DrawText(tmp_str2, tmp_i_2, line2_rect, DT_LEFT | DT_TOP | DT_NOCLIP | DT_NOPREFIX);


			dc.SelectObject(oldft);
			dc.DrawText(tmp_str3, tmp_i_3, line3_rect, DT_LEFT | DT_TOP | DT_NOCLIP | DT_NOPREFIX);
			//dc.DrawText(tmp_str+19, 5, line_rect, DT_LEFT | DT_TOP | DT_NOCLIP | DT_NOPREFIX);

			//dc.SelectObject(oldft);

			//drawTextLetterSpacingMods(dc,line_rect, tmp_str,tmp_i);
#endif
#ifdef spacingMod
			dc.SetMapMode(MM_HIENGLISH);
			CFont* oldft = dc.SelectObject(ft);
			TEXTMETRIC tm;
			dc.GetTextMetrics(&tm);

			int width = tm.tmAveCharWidth;
			int diff = ((interCharSpacing - 100) * width ) / 100;
			int mmTextX = dc.GetDeviceCaps(LOGPIXELSX);
			int mmTextY = dc.GetDeviceCaps(LOGPIXELSY);
			int mmHrez = dc.GetDeviceCaps(HORZRES);
			int mmVrez = dc.GetDeviceCaps(VERTRES);


			INT spaces[500];
			INT orig[500];
			CSize cz;
			int j = 0;


			int CHARWIDTH = tm.tmMaxCharWidth;
			eyemsg_printf("%d", CHARWIDTH);
			int wordLen = 0;

			// constant width fonts only
		/*	for(int i=0; i<500;i++){
				spaces[i] = width;
				if(i+1 < tmp_i && tmp_str[i] == ' ' && false){
					spaces[i] = ( width * (wordLen+1)) - sigma(spaces, i-(wordLen), i-1) - diff;
					j=0;
				}
				else{
				wordLen++;
				spaces[i] = width + diff; //MM_HIENGLISH
				}
				j++;
			}*/

			for(int i=0; i<500;i++){
				cz = dc.GetTextExtent(&(tmp_str[i]), 1);
				spaces[i] = width;
				if(i+1 < tmp_i && tmp_str[i] == ' '){
					//spaces[i] = ( width * (wordLen+1)) - sigma(spaces, i-(wordLen), i-1) - diff;
					spaces[i] = sigma(orig, 0, i-1) -  sigma(spaces, 0, i-1) + cz.cx;
					orig[i] = cz.cx;
					j=0;
				}
				else{
				wordLen++;
				orig[i] = cz.cx;
				spaces[i] = cz.cx + diff; ; //MM_HIENGLISH
				}
				j++;
			}

			int mmHiEnglishX = dc.GetDeviceCaps(LOGPIXELSX);
			int mmHiEnglishY = dc.GetDeviceCaps(LOGPIXELSY);
		    mmHrez = dc.GetDeviceCaps(HORZRES);
			mmVrez = dc.GetDeviceCaps(VERTRES);
			
			doublePoint multiplier = getMultiplier(CPoint(mmHrez, mmVrez));
			int localXoffset = multiplier.x * (line_rect.TopLeft().x * mmHiEnglishX) / mmTextX;
			int localYoffset = multiplier.y * (line_rect.TopLeft().y * mmHiEnglishY) / mmTextY;
			
			// THIS version was for debuggin, align the original in place above it
			//dc.ExtTextOut(line_rect.TopLeft().x * 9, line_rect.TopLeft().y * -9.5, /* 2400, -2400,*/ 0,0/* CRect(100,100,101,101)*/, tmp_str, tmp_i, spaces);
		
			//dc.ExtTextOut(line_rect.TopLeft().x * 9.85, line_rect.TopLeft().y * -9.85, /* 2400, -2400,*/ 0,0/* CRect(100,100,101,101)*/, tmp_str, tmp_i, spaces);
			dc.ExtTextOut(localXoffset, localYoffset,  0,0, tmp_str, tmp_i, spaces);
			dc.SelectObject(oldft);
			dc.SetMapMode(MM_TEXT);

#endif
#ifdef newMode
			//line_rect.MoveToX(line_rect.left+45);

#endif
		//	dc.DrawText(tmp_str+24, tmp_i-25, line_rect, DT_LEFT | DT_TOP | DT_NOCLIP | DT_NOPREFIX);

			
		//ci = l.begin();
		//dc.DrawText(&(ci->ch), 2, _trackRgn, DT_LEFT | DT_TOP | DT_NOCLIP | DT_NOPREFIX);
		}


		dc.DrawText(ID, ID.StringLength(ID), CRect(0,0,100,100), DT_LEFT | DT_TOP | DT_NOCLIP | DT_NOPREFIX);
	return;
	}


	int j = 0,k = 0;

	//if(isMovWind) _display.region.clear();

	if(pBrush)
		dc.FrameRect(_trackRgn, pBrush);
	std::vector<CPoint> wordUpperLefts;
	std::vector<CPoint> wordLowerRights;
	std::vector<CPoint> wordStartIndex;
	std::vector<CPoint> wordEndIndex;
	//std::vector<std::list<Character>::iterator> wordStartIter;
	//std::vector<std::list<Character>::iterator> wordEndIter;
	//std::vector<CPoint> charGroupUpperLefts;
//	std::vector<CPoint> charGroupLowerRights;
	for(int i=0; i < _nLines; ++i) {

		int w = 0, c =0, charGroup=0;

		//std::vector<CPoint> charUpperLefts;
		//std::vector<CPoint> charLowerRights;
		std::vector<CRect> charRects;

		std::list<TextPassage::Character> &l = _line[i];
		std::list<TextPassage::Character>::iterator ci;
		std::list<TextPassage::Character>::iterator b;
		std::list<TextPassage::Character>::iterator prev;	
		std::list<TextPassage::Character>::iterator next;
		b = l.begin();
		prev = l.begin();
		next = l.begin();
		for(ci = l.begin(); ci != l.end(); ++ci) {
			
		dc.DrawText(&(ci->ch), 1, &(ci->textRgn), DT_LEFT | DT_TOP | DT_NOCLIP | DT_NOPREFIX);
			if(pBrush)
				dc.FrameRect(ci->trackRgn, pBrush);

			if (!isWhiteSpace(ci->ch)){
				charRects.push_back( ci->trackRgn );
				charGroup++;
				if (charGroup >= _display.windowSize.y) {
					//charGroupLowerRights.push_back( ci->trackRgn.BottomRight() );
					//charGroup = 0;
				}
			}
			if (isWordStart(prev->ch, ci->ch, ci==l.begin())){		  
				wordUpperLefts.push_back( prev->trackRgn.TopLeft() );		// when ci == l.begin prev == ci
				wordStartIndex.push_back( CPoint(i,ci->index));
				//charGroup = 0;
				//charGroupUpperLefts.push_back( ci->trackRgn.TopLeft() );	// using prev tacks white space 
																			// onto word following it
#ifdef MOV_WIND_DEBUG
				eyemsg_printf("pushed upperLefts (%d, %d)", 
					ci->trackRgn.TopLeft().x,
					ci->trackRgn.TopLeft().y);
#endif
			}

			//if (isWordEnd(ci->ch, next->ch, ci==l.end()))
			//	wordLowerRights.push_back( ci->trackRgn.BottomRight());
			
			if (next != l.end()){
				next++;
				// fix this re last word or handle in isWordEnd
				if (isWordEnd(ci->ch, next->ch, next==l.end())){
#ifdef MOV_WIND_DEBUG
					eyemsg_printf("next:pushed lowerRights (%d, %d) at char %c", 
						next->trackRgn.BottomRight().x, 
						next->trackRgn.BottomRight().y, 
						ci->ch); 
#endif

					wordLowerRights.push_back( ci->trackRgn.BottomRight());
					wordEndIndex.push_back( CPoint(i, ci->index));
				}
			}
			else {
				if (!isWhiteSpace(ci->ch)){
#ifdef MOV_WIND_DEBUG
					eyemsg_printf("ci:pushed lowerRights (%d, %d) at char %c", 
						ci->trackRgn.BottomRight().x, 
						ci->trackRgn.BottomRight().y, 
						ci->ch); 
#endif

					wordEndIndex.push_back( CPoint( i, ci->index));
					wordLowerRights.push_back( ci->trackRgn.BottomRight());
	//				charGroupLowerRights.push_back( ci->trackRgn.BottomRight());
				}
			}
			// if(/*_display &&*/ isMovWind){
			//	if (distance( b, ci) == (_display.wordTriggers)[j])		
			//		_display.region[j] = ci->textRgn;
			//		if (distance( b, ci) == (_display.wordTargets)[j])
			//			_display.target[j] = ci->textRgn;
			// }
		if (ci!= l.begin())
			prev++;	
		}
		// at end of line
		// DEBUG must verify array bounds for Triggers/Targets w/in word list
	}
	//std::vector<rectVector> temp;

	// TODO fix dummy trigger 
	//if(!_display.region.empty())

	//	_display.region.pop_back();
   if(_display.movWindowType.useWordTriggers == TRUE && _display.movWindowType.useWordTargets == TRUE){
	for (unsigned int i = 0; i< _display.wordTriggers.size(); i++){
		_display.region.push_back(
			CRect(	wordUpperLefts.at(_display.wordTriggers[i]),
					wordLowerRights.at(_display.wordTriggers[i])));
		//	CRect(	wordUpperLefts.at(_display.wordTriggers[i]),
			//			CPoint( wordUpperLefts.at(_display.wordTriggers[i] + 1).x,
			//					wordLowerRights.at(_display.wordTriggers[i] + 1).y)));
	//	if( _display.wordTriggers[i] == _display.toggleWord.x ){	
		if( isToggle(_display.wordTriggers[i] )){ 
			TOGGLEDATA td;
			td.delay = 0;
			td.duration = isToggle(_display.wordTriggers[i]);//_display.toggleWord.y;
			td.region = CRect(	wordUpperLefts.at(_display.wordTriggers[i]),
								wordLowerRights.at(_display.wordTriggers[i]));
			_display.toggleRegions.push_back( td );
		/*	TOGGLEDATA td2;
			td2.delay = 0;
			td2.duration = _display.toggleWord.y;
			td2.region = CRect( wordUpperLefts.at(_display.wordTriggers[i+4]),
								wordLowerRights.at(_display.wordTriggers[i+4]));
			_display.toggleRegions.push_back( td2 );*/
		}
		std::vector<CPoint> ul;
		std::vector<CPoint> lr;
		int X = _display.wordTargetRange[i].x;
		int Y = _display.wordTargetRange[i].y;
//		int size = _display.wordTriggers.size();
		/*for ( int j = 0; j < Y-X; j++){
				ul.push_back(wordUpperLefts.at(X+j));
				lr.push_back(wordLowerRights.at(X+j));
			}*/
		
		/*	int max = 0;
			for (unsigned int j = 0; j < ul.size(); j++){
				if (ul.at(j) >= ul.at(max))
					max = j;
			}*/

//			adds full region to target list iff region does not span multiple lines
//			_display.target.push_back(
//				CRect(	wordUpperLefts.at(_display.wordTargetRange[i].x),
//					wordLowerRights.at(_display.wordTargetRange[i].y)));
	
		std::vector<CRect> multi;
		
		for (int j = 0; j<=Y-X; j++){
			CPoint aaa = wordUpperLefts.at(X+j);
			CPoint bbb = wordLowerRights.at(X+j);
#ifdef MOV_WIND_DEBUG
			eyemsg_printf("word %d triggers upperleft(%d, %d) to lowerright(%d, %d)",
				i, aaa.x, aaa.y, bbb.x, bbb.y);
#endif //MOV_WIND_DEBUG
			if (_display.movWindowType.useWordTargets)
				multi.push_back( CRect( wordUpperLefts.at(X+j), wordLowerRights.at(X+j)) );
			else  { // word targets, char windows
/*				CPoint cLast_before = wordStartIndex.at(j);
				std::list<Character>::iterator cFirst_before 
					= getCharGroupStart (cLast_before, _display.windowSize.x);
				multi.push_back( 
					CRect( 
						cFirst_before->trackRgn.TopLeft(), 
						cFirst_before->trackRgn.BottomRight()));

*/
				//CPoint cFirst_after = wordEndIndex.at(j);
				//CPoint cLast_after = getCharGroupEnd (cFirst_after, _display.windowSize.y);
				multi.push_back( CRect( wordUpperLefts.at(X+j), wordLowerRights.at(X+j)));
			}	
		}
		//temp.push_back(multi);
		_display.multiTarget.push_back(multi);

		/*tPair tempPair;
		tempPair.trigger = CRect(	wordUpperLefts.at(_display.wordTriggers[i]),
					wordLowerRights.at(_display.wordTriggers[i]));
		tempPair.target = multi;
		_display.tPairs.push_back(tempPair);*/
		
		}
   }
//#ifdef NOT_DEFINED
	else if(_display.movWindowType.useWordTriggers && !_display.movWindowType.useWordTargets){
		for (unsigned int j = 0; j< _display.wordTriggers.size(); j++){
			
			CPoint x = wordUpperLefts.at(_display.wordTriggers[j]);
			CPoint y = wordLowerRights.at(_display.wordTriggers[j]);
			_display.region.push_back(
			CRect(	wordUpperLefts.at(_display.wordTriggers[j]),
					wordLowerRights.at(_display.wordTriggers[j])));

			CPoint cLast_before, cFirst_after;
	
			size_t startI, endI;	
			if( _display.wordTriggers[j] >= wordStartIndex.size() )
				startI = wordStartIndex.size();
			else startI = _display.wordTriggers[j];
			if( _display.wordTriggers[j] >= wordEndIndex.size() )
				endI = wordEndIndex.size();
			else endI = _display.wordTriggers[j];
			cLast_before = wordStartIndex.at(startI);
			cFirst_after = wordEndIndex.at(endI);
			
			/*cLast_before = wordStartIndex.at( _display.wordTriggers[j]);
			cFirst_after = wordEndIndex.at( _display.wordTriggers[j]);*/
			
		/*	std::list<Character>::iterator cFirst_before_iter 
				= getCharGroupStart (cLast_before, _display.windowSize.x);
			std::list<Character>::iterator cFirst_after_iter 
				= getCharGroupEnd(cFirst_after, 1);
			std::list<Character>::iterator cLast_after_iter 
				= getCharGroupEnd(cFirst_after, _display.windowSize.y);
			std::list<Character>::iterator cLast_before_iter 
				= getCharGroupStart(cLast_before, 1);*/
	
			// add the characters in the "before word" window
			std::vector<CRect> multi;
			for (int k = 0; k<_display.windowSize.x; k++){
				std::list<Character>::iterator ci = getCharGroupStart(cLast_before, _display.windowSize.x - k);
				multi.push_back( 
					CRect( 
						ci->trackRgn.TopLeft(),
						ci->trackRgn.BottomRight()));
				//cFirst_before_iter++;
			}


			/*	multi.push_back( 
				CRect( 
					cFirst_before_iter->trackRgn.TopLeft(),
					cFirst_before_iter->trackRgn.BottomRight()));
			multi.push_back(
				CRect(
					cLast_before_iter->trackRgn.TopLeft(),
					cLast_before_iter->trackRgn.BottomRight()));*/
			/*for(std::list<Character>::iterator ci = cFirst_before_iter; ci != cLast_before_iter; ci++){
				multi.push_back(
					CRect(
						ci->trackRgn.TopLeft(),
						ci->trackRgn.BottomRight()));
			}*/


			// add the characters in triggering word
			multi.push_back(
				CRect( wordUpperLefts.at(_display.wordTriggers.at(j)),
						wordLowerRights.at(_display.wordTriggers.at(j))));

			// add the characters in the "after word" window
			for (int k = _display.windowSize.y; k>0; k--){
				std::list<Character>::iterator ci = getCharGroupEnd_B(cFirst_after, _display.windowSize.y - k);
				multi.push_back( 
					CRect( 
						ci->trackRgn.TopLeft(),
						ci->trackRgn.BottomRight()));
				//cFirst_before_iter++;
			}




			/*multi.push_back(
				CRect( cLast_after_iter->trackRgn.TopLeft(),
						cLast_after_iter->trackRgn.BottomRight()));*/

			//temp.push_back(multi);
			_display.multiTarget.push_back(multi);
		}
	}

	else if(!_display.movWindowType.useWordTriggers && !_display.movWindowType.useWordTargets){
		std::list<Character>::iterator cc;
		for (unsigned int j = 0; j< _display.display1.size(); j++){
			/*std::list<Character>::iterator*/ cc = getCharGroupEnd_B(CPoint(0,j), 0);
			_display.region.push_back( cc->trackRgn );
			std::vector<CRect> multi;
			multi.clear();
			multi.push_back(cc->trackRgn);
			for (int k = 0; k<_display.windowSize.x; k++){
				std::list<Character>::iterator ci = getCharGroupStart(CPoint(0,j), _display.windowSize.x - k);
			 multi.push_back( 
					CRect( 
						ci->trackRgn.TopLeft(),
						ci->trackRgn.BottomRight()));
			}

			// add the characters in the "after word" window

			for (int k = _display.windowSize.y; k>0; k--){
				if(j + k < _display.display1.size() ){
					//if (cc == _line[0].end()) break;
					cc++;
					multi.push_back( 
					CRect( 
						cc->trackRgn.TopLeft(),
						cc->trackRgn.BottomRight()));
				}
				
			}
				/*for (int k = _display.windowSize.y-1; k>=0; k--){
					//std::list<Character>::iterator ci = getCharGroupEnd_B(CPoint(0,j), 1);
					std::list<Character>::iterator ci = getCharGroupEnd_B(CPoint(0,j), _display.windowSize.y - k);
					if(j+k<_display.display1.size()){
						multi.push_back( 
							CRect( 
								ci->trackRgn.TopLeft(),
  								ci->trackRgn.BottomRight()));
					}
					
				}*/
			

			_display.multiTarget.push_back(multi);
		}
	}
			
			/*while (!temp.empty()){								
		_display.multiTarget.push_back(temp.back());	// reverse the order again so consistent w/ trigger region
		temp.pop_back();
	}*/
//#endif // NOT_DEFINED


}






/*
std::list<TextPassage::Character>::iterator TextPassage::getCharGroupStart(CPoint EndIndex, int size)
{
	std::list<Character>::iterator temp;
	temp = _line[EndIndex.x].begin();
	long offset = EndIndex.y - (_line[EndIndex.x].begin())->index - size;
	if (offset <= 0) return temp;
	else while (--offset >=0 && temp != (_line[EndIndex.x].end())){
		temp++;
	}
	return temp;
}*/

std::list<TextPassage::Character>::iterator TextPassage::getCharGroupStart(CPoint EndIndex, int size)
{
	int start = 0;
	std::list<Character>::iterator temp;

	while((start < (_nLines-1))  && !_line[start].empty() && (_line[start].back().index < (EndIndex.y - size)) )
	{
		start++;
	}
	int IND;
	temp = _line[start].begin();
	do{
		//temp++;
		if (temp->index >= (EndIndex.y - size)){
			IND = temp->index; 
			return temp;
		}
		temp++;
	}while (temp != _line[start].end());
	IND = temp->index;
	return _line[start].begin();
	return temp;
}

std::list<TextPassage::Character>::iterator TextPassage::getCharGroupEnd_B (CPoint StartIndex, int size)
{
	int start = 0;
	std::list<Character>::iterator temp;

	while((start < (_nLines-1)) && !_line[start].empty() && (_line[start].back().index < (StartIndex.y + size)) )
	{
		start++;
	}
	int IND;
	temp = _line[start].begin();
	do{
		if (temp->index >= (StartIndex.y + size)){
			IND = temp->index;
			return temp;
		}
		temp++;
	} while (temp != _line[start].end() && temp->index < _display.display1.size());
	IND = temp->index;
	return _line[start].end();
	return temp;
}



/*std::list<TextPassage::Character>::iterator
TextPassage::getCharGroupEnd(CPoint StartIndex, int size)
{
	int start = -1;
	std::list<Character>::iterator temp;

	do {
		start++;
	}while((start < (_nLines-1) && 
		!_line[start].empty() && 
		(_line[start].begin()->index <= (StartIndex.y + size)));
*/

std::list<TextPassage::Character>::iterator 
TextPassage::getCharGroupEnd(CPoint StartIndex, int size)
{
//	return _line[0].begin();
	std::list<Character>::iterator temp;
	int current = -1;
	size_t i = 0;

//	std::list<Character>::iterator temp = _line[0].begin();

	int goal = StartIndex.y + size;
	do { 
		++current;
		if (current >= 2)
			return _line[0].begin();
	} while (current < (_nLines - 1) 
			&& !_line[current].empty() 
			&& _line[current].back().index < goal );

	if (_line[current].empty())
		return (++(_line[0].begin()));
	temp = _line[current].begin();
	do{
		temp++;
		if (temp->index >= goal){
			//IND = temp->index; 
			return temp;
		}
	}while (temp != _line[current].end());
	//IND = temp->index;
	return temp;

	// END

	//temp = _line[StartIndex.x].begin();
	long offset = StartIndex.y -(_line[StartIndex.x].begin())->index + size;
/*	if(offset > _line[StartIndex.x].size() )
		return _line[StartIndex.x].end();
		else*/ 
	int X = StartIndex.x;
	/*
	#define ORIG
	while(temp != _line[X].end() && --offset >= 0){
		temp++;
	}
	*/

	int startLine = 0;
	int stop = FALSE;
	while(_line[startLine].begin()->index <= (StartIndex.y + size) && !stop){
		startLine++;
//		int p = _line->size();
		if (startLine >= _nLines){
			stop = TRUE;
			startLine--;
			break;
		}
		if (_line[startLine].begin()->index > (StartIndex.y + size) ){
			startLine--;
			stop = TRUE;
			break;
		}
	}

	temp = _line[startLine].begin();
	while(temp != _line[startLine].end()){
		if (temp->index == StartIndex.y + size)
			return temp;
		else {
			temp++;
			if (temp == _line[startLine].end() && startLine < (_nLines-1))
				startLine++;
		}
	}
	return temp;
	/* 
#define ORIG
	while (--offset >= 0 && X < _line->size()){
		if (temp == _line[X].end())
			temp = _line[X+1].begin();
		else temp++;
	}*/
	return temp;
}

/*std::list<Character>::iterator TextPassage::getCharGroupEnd(CPoint StartIndex, int size)
{
	CPoint temp;
	temp.x = StartIndex.x;
	temp.y = StartIndex + size;
	if (temp.y > _line[StartIndex.x].end()) temp.y = _line[StartIndex.x].end();
	return temp;

	std::list<Character>::iterator temp;
	temp = _line[Endindex.x].begin();
	while(ci = temp; ci != _line[StartIndex.x].end(); ci++){

}*/
void TextPassage::drawText(CDC &dc, CBrush *pBrush) const
{
	if(pBrush)
		dc.FrameRect(_trackRgn, pBrush);

	for(int i=0; i < _nLines; ++i) {
		std::list<TextPassage::Character> &l = _line[i];
		std::list<TextPassage::Character>::iterator ci;

		for(ci = l.begin(); ci != l.end(); ++ci) {
			dc.DrawText(&(ci->ch), 1, &(ci->textRgn), DT_LEFT | DT_TOP | DT_NOCLIP | DT_NOPREFIX);
			if(pBrush)
				dc.FrameRect(ci->trackRgn, pBrush);
		}
	}
	
}

int TextPassage::fontHeight() const
{
	return(_fontHt);
}

CRect TextPassage::gcRect() const
{
	return(_gcRect);
}

CRect TextPassage::trackingRegion() const
{
	return(_trackRgn);
}

DisplayData TextPassage::getDisplay()
{
	return(_display);
}

void TextPassage::writeToEyelink() const
{
	for(int i=0; i < _nLines; ++i) {
		std::list<TextPassage::Character> &l = _line[i];
		std::list<TextPassage::Character>::iterator ci;

		for(ci = l.begin(); ci != l.end(); ++ci) {
			const CRect &r = ci->trackRgn;

			if(ci->ch < 0 || ci->ch > 127) {       //if Chinese, write whitespace
				if(_ismbblead(ci->ch) && _ismbbtrail((++ci)->ch)){
					char mbstr[3];
					mbstr[0] = (--ci)->ch;
					mbstr[1] = (++ci)->ch;
					mbstr[2] = '\0';
					eyemsg_printf("REGION CHAR %d 1 %s %d %d %d %d", ci->index, mbstr, r.left, r.top, r.right, r.bottom);
				}
				else {
					eyemsg_printf("REGION CHAR %d 1   %d %d %d %d", ci->index, r.left, r.top, r.right, r.bottom);
				}
			}
			else                                 //otherwise write the character
				eyemsg_printf("REGION CHAR %d 1 %c %d %d %d %d", ci->index, ci->ch, r.left, r.top, r.right, r.bottom);
			eyecmd_printf("draw_box %d %d %d %d 14", r.left, r.top, r.right, r.bottom);
			CEyelink2::Delay(1);                 //avoid flooding the link
		}
	}
}

void TextPassage::resetMWregions()
{
	_display.region.clear();
	while(!_display.multiTarget.empty()){
		rectVector rv = _display.multiTarget.back();
		rv.clear();
		_display.multiTarget.pop_back();
	}
	return;
}
