/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once
#include <vector>

enum {IN_WORD, IN_WHITESPACE};

class doublePoint {
public:
	double x;
	double y;
};

class TextPassage {
public:
	TextPassage(const std::string &text, const CRect &wndSz, DisplayData &dd, COLORREF hColor = 0);
	TextPassage(const std::string &text, const CRect &wndSz);
//	TextPassage(const std::string &text, const CRect &wndSz, Trial t);
	~TextPassage(void);

	CRect gcRect() const;
	CRect trackingRegion() const;
	DisplayData getDisplay();

	int fontHeight() const;

	void computeProperties(CDC &dc, const TextFormat &tf, const TextFormat &tf2);
	void computeRegion(CDC &dc, const CPoint &topLeft, const CSize &padding, CFont *cf = 0); 
	void computeRegionOrig(CDC &dc, const CPoint &topLeft, const CSize &padding); 
	void drawText(CDC &dc, CBrush *pBrush = 0/*, DisplayData *dd = 0*/, bool isMovWind = true, CString ID = "", CFont *ft = new CFont(), int interCharSpacing = 100);
	void drawText(CDC &dc, CBrush *pBrush = 0) const;
	void drawTextLetterSpacingMods(CDC &dc, CRect lineRect, char *str, int i);
	void writeToEyelink() const;

	void resetMWregions();
	
protected:
	struct Character {
		Character(int idx, char c);
		Character(const Character &c);

		const Character &operator=(const Character &rhs);

		CRect textRgn;                           //rect enclosing character
		CRect trackRgn;                          //rect enclosing char plus any extra space
		int   index;                             //character index (zero'd from start of passage)
		char  ch;
	};

private:
	bool isWordStart( char prev, char c, bool linebegin);
	bool isWordEnd( char c, char next, bool lineEnd);
	std::list<Character>::iterator getCharGroupStart (CPoint EndIndex, int size);
	std::list<Character>::iterator getCharGroupEnd (CPoint StartIndex, int size);
	std::list<Character>::iterator getCharGroupEnd_B (CPoint StartIndex, int size);
//	CPoint getCharGroupEnd (CPoint StartIndex, int size);
	unsigned int isToggle(int index);
	std::vector<CRect>	  wordTriggerRegion;
	std::vector<CRect>	  wordTargetRegion;
	std::vector<CRect>	  charTriggerRegion;
	std::vector<CRect>	  charTargetRegion;
	DisplayData			  _display;
	std::list<Character> *_line;
	CRect                 _gcRect;
	CRect                 _trackRgn;
	int                   _fontHt;
	int                   _nLines;
	doublePoint getMultiplier(CPoint rez);
public:
	int						f2start;
	int						f2end;
	COLORREF				_hColor;
};
