/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once
#pragma warning( disable : 4290 )

#include <list>
#include "AudioCapture.h"
#include "DisplayData.h"
#include "Tokenizer.h"

/*****************************************************************************/

typedef std::list<std::string>                 StringList;
typedef std::list<std::string>::const_iterator StringIter;

class TrialType {
public:                                          //class member functions
	TrialType(const TrialType &tc);

	TrialType(Tokenizer &tok) throw(CExperimentException);

	TrialType(const std::string &ident, Tokenizer::ButtonType btn[], 
			  Tokenizer::OutputType out, Tokenizer::TriggerType trig,
			  int cursor, int dcDelay, int stimDelay, int nButtons, 
			  bool revertDisplay, Tokenizer::ControlType ctrl);

	~TrialType();

	const TrialType &operator=(const TrialType &tc);

	const std::string &identifier() const;

	const AudioCaptureData *audioFormat() const;

	const TextFormat *textFormat() const;
	const TextFormat *textFormat2() const;

	Tokenizer::ButtonType *buttons() const;
	Tokenizer::ControlType controlType() const;
	Tokenizer::OutputType output() const;
	Tokenizer::TriggerType trigger() const;

	int cursorSize() const;
	int displayChangeDelay() const;
	int nButtons() const;
	int stimulusDelay() const;

	bool doButtonResponse() const;
	bool doDelay() const;
	bool doDisplayRevert() const;
	bool doDriftCorrection() const;
	bool doGazeContingentStart() const;
	bool doMouseResponse() const;
	bool doStreamOutput() const;

	void save(std::ostream &os) const;
	void setAudioFormat(const AudioCaptureData &format);
	void setTextFormat(const TextFormat &format, bool setFormat2 = false);

	int getInterCharSpacing() const;
	COLORREF getHColor() const;

private:                                         //class member data
	std::string _identifier;

	AudioCaptureData *_audioFmt;
	TextFormat       *_textFmt;
	TextFormat		 *_textFmt2;
	Tokenizer::ButtonType  *_button;
	Tokenizer::OutputType   _output;
	Tokenizer::TriggerType  _trigger;
	Tokenizer::ControlType _controlType;

	int _cursorSize;
	int _nButtons;
	int _stimulusDelay;
	int _dcDelay;
	bool _revertDisplay;
	int _interCharSpacing;
	COLORREF _hColor;
};

/*******************************************************************************************/

typedef std::list<TrialType*>                  TrialTypeList;
typedef std::list<TrialType*>::iterator        TrialTypeIter;
typedef std::list<TrialType*>::const_iterator  constTrialTypeIter;
