/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#include <sstream>
#include "trial.hpp"
//#define MOV_WIND_DEBUG

Trial::Trial(const Trial &t) throw()
{
	(*this) = t;                                 //calls operator =
}

Trial::Trial(Tokenizer &tok) throw(CExperimentException)
: _audio(), _display(), _answer(Tokenizer::NO_BUTTON), _controlType(Tokenizer::UNKNOWN_CONTROLLER)
{                                                //read trial from script file
	std::ostringstream os;                       //error messages
	std::string temp;
	CRect region[100];
	Tokenizer::Keyword keyword;
	bool haveInline = false, haveImage = false;

	CHANGE_ONCE_MODE = true;

	_trialId = TrialId(tok.readString());
	os<<"Error loading trial '"<<_trialId.idString()<<"'."<<std::endl;

	keyword = tok.readKeyword();
	TIMED_RECT_PAIR tttt;
	while(keyword != Tokenizer::END && keyword != Tokenizer::UNKNOWN) {
		tok.readChar();                          //read the '=' sign
		tok.readWhitespace();                    //read intervening whitespace
		switch(keyword) {
		case Tokenizer::AUDIO_FORMAT:
			_audio.path = tok.readString();
			_audio.startDelay = tok.readInteger();
			_audio.stopDelay = tok.readInteger();
			break;
		case Tokenizer::BUTTON:{
			Tokenizer::ControlType c = _controlType;
			_answer = tok.readButton(&c);
			if (_controlType == Tokenizer::UNKNOWN_CONTROLLER)
				_controlType = c;
			break;
							   }
		case Tokenizer::GC_RECT:
			_display.gcRect = tok.readRectangle();
			if(!tok.isOkay()) {
				os<<"Error reading gaze contingent rectangle."<<std::endl;
				throw(CExperimentException(os.str().c_str()));
			}
			break;
		case Tokenizer::IMAGE:
			if(isTextDisplay()/*haveInline*/) {
				if(_display.display2.empty()){
					do{
						CPoint pt = tok.readPoint();
						if (pt.x == -1){
							if(_display.display1.empty()) {
								_display.display1 = tok.readString();
								_display.isTextDisplay = false;
								haveImage = true;
								break;
							}
							else{
								_display.display2 = tok.readString();
								_display.isFixedDisplay = false;
								_display.isScotoma = false;
								break;
							}

						}
						else{
							std::string fName = tok.readString();
							_display.images1.push_back(imagePos(fName, pt));
						}
					}while (!_display.images1.back().fileName.compare("") && tok.isOkay());
				}
				else if(_display.display3.empty()){
					do{
						CPoint pt = tok.readPoint();
						std::string fName = tok.readString();
						_display.images2.push_back(imagePos(fName, pt));
					}while(!_display.images2.back().fileName.compare("") && tok.isOkay());
				}
				else /*if(_display.images3.empty())*/{
					do{	
						CPoint pt = tok.readPoint();
						std::string fName = tok.readString();
						_display.images3.push_back(imagePos(fName, pt));
					}while(!_display.images3.back().fileName.compare("") && tok.isOkay());
				}
/*
				if(_display.display2.empty()){
					do{
						CPoint pt = tok.readPoint();
						std::string fName = tok.readString();
						_display.images1.push_back(imagePos(fName, pt));
					}while (!_display.images1.back().fileName.compare("") && tok.isOkay());
				}
				else {
					do{
						CPoint pt = tok.readPoint();
						std::string fName = tok.readString();
						_display.images2.push_back(imagePos(fName, pt));
					}while(!_display.images2.back().fileName.compare("") && tok.isOkay());
				}*/
			



				//os<<"Display change trial contains both inline text and image file."<<std::endl;
				//throw(CExperimentException(os.str().c_str()));
			}
			else{
				if(_display.display1.empty()) {
					_display.display1 = tok.readString();
					_display.isTextDisplay = false;
					haveImage = true;
				}
				else if(_display.display2.empty()) {
					_display.display2 = tok.readString();
					_display.isFixedDisplay = false;
					_display.isScotoma = false;
				}
				else {
					os<<"Display change trial contains more than two display images."<<std::endl;
					throw(CExperimentException(os.str().c_str()));
				}
			}
			break;
		case Tokenizer::INLINE:
			if(haveImage) {
				//os<<"Display change trial contains both inline text and image file."<<std::endl;
				//throw(CExperimentException(os.str().c_str()));
			}
			if(_display.display1.empty()) {
				_display.display1 = tok.readText();
				_display.display1.erase(0, 1);
				//int i = pullTags(_display.display1, stripped);
			
				std::string::iterator i = findTags(_display.display1, true);
				_display.display1.erase(startf2,1);
				std::string::iterator j = findTags(_display.display1, false);
				_display.display1.erase(endf2, 1);

				_display.isTextDisplay = true;
				haveInline = true;
			}
			else if(_display.display2.empty()) {
				_display.display2 = tok.readText();
				_display.display2.erase(0, 1);
				_display.isFixedDisplay = false;
							_display.isScotoma = false;
			}
			else if(_display.display3.empty()) {
				_display.display3 = tok.readText();
				_display.display3.erase(0, 1);
			}
			else {
				os<<"Display change trial contains more than two inline text displays."<<std::endl;
				throw(CExperimentException(os.str().c_str()));
			}
			break;
		case Tokenizer::MAX_DISPLAY_TIME:
			_maxDisplayTime = tok.readInteger();
			break;
		case Tokenizer::REGION: // TODO: syntax in REGION field
			do {
				_display.region.push_back(tok.readRectangle());
				//_display.target.push_back(tok.readRectangle());
				//std::vector<CRect> empty;
				//empty.push_back(CRect(0,0,0,0));
				//_display.multiTarget.push_back(empty);
			} while(!_display.region.back().IsRectNull() && tok.isOkay());
			if(!tok.isOkay()) {
				os<<"Error reading display change regions."<<std::endl;
				throw(CExperimentException(os.str().c_str()));
			}
			_display.region.pop_back();
			//_display.target.pop_back(); // TODO 
			//_display.multiTarget.pop_back();
			_display.isFixedDisplay = false;
			_display.isScotoma = false;
			_display.isMovingWindow = false;
			_display.isPairedRegionChange = false;
			_display.isTimedPairedRegionChange = false;
			break;
		case Tokenizer::SCOTOMA: // TODO: syntax in REGION field
			do {
				_display.region.push_back(tok.readRectangle());
				//_display.target.push_back(tok.readRectangle());
				//std::vector<CRect> empty;
				//empty.push_back(CRect(0,0,0,0));
				//_display.multiTarget.push_back(empty);
			} while(!_display.region.back().IsRectNull() && tok.isOkay());
			if(!tok.isOkay()) {
				os<<"Error reading display change regions."<<std::endl;
				throw(CExperimentException(os.str().c_str()));
			}
			_display.region.pop_back();
			//_display.target.pop_back(); // TODO 
			//_display.multiTarget.pop_back();
			_display.isFixedDisplay = false;
			_display.isScotoma = true;
			_display.isMovingWindow = false;
			_display.isPairedRegionChange = false;
			_display.isTimedPairedRegionChange = false;
			break;
		case Tokenizer::REGIONPAIR:
			do {
				_display.regionPairs.push_back(tok.readRectPair());
			} while(!_display.regionPairs.back().region1.IsRectNull() && tok.isOkay());
			do {
				_display.timedRectPairs.push_back(tok.readTimedRectPair());
			} while(!_display.timedRectPairs.back().r1.IsRectNull() && tok.isOkay());
			
			if (!tok.isOkay()){
				os<<"Error reading display change region pairs."<<std::endl;
				throw(CExperimentException(os.str().c_str()));
			}
			_display.isScotoma = false;
			_display.isFixedDisplay = false;
			_display.isMovingWindow = false;
			_display.regionPairs.pop_back();
			_display.timedRectPairs.pop_back();
			tttt = _display.timedRectPairs.back();
			if(_display.timedRectPairs.empty()){
				_display.isPairedRegionChange = true;
				_display.isTimedPairedRegionChange = false;
			}
			else{
				_display.isTimedPairedRegionChange = true;
				_display.isPairedRegionChange = false;
			}

			break;
		case Tokenizer::WORDS:
			do { 
				CPoint _p = tok.readPoint();  // TODO: parse check
				_display.wordTriggers.push_back(_p.x);
				//_display.wordTargets.push_back(_p.y);
				CPoint _p2 = tok.readPoint();
				if (_p2.x <= _p2.y) 
					_display.wordTargetRange.push_back( CPoint(_p2.x, _p2.y)); // TODO: TP range check
				else {	
					os<<"Moving window word range not well defined. ("<<_p2.x<< ","<<_p2.y<<")"<<std::endl;
					throw(CExperimentException(os.str().c_str()));
				}
			} while(_display.wordTriggers.back() != -1 && tok.isOkay());
			if( !tok.isOkay()) {
				os<<"Error reading word change regions."<<std::endl;
				throw(CExperimentException(os.str().c_str()));
			}
			_display.wordTriggers.pop_back();
			_display.wordTargetRange.pop_back(); // was .wordTargets
			_display.isFixedDisplay = false;
			_display.isMovingWindow = false;
			_display.isScotoma = false;

			break;
		case Tokenizer::WINDOW:
			do { 
				//_display.region.clear();
				//_display.target.clear();
				CPoint _p = tok.readPoint();  // x = trailing and 
											  // y = leading window size. TODO: parse check
				if(_p.x == -1)	break;
				CPoint _p2 = tok.readPoint(); // The range of word indecies to which window applies
				if(_p2.x == -1) break;

				CPoint _types = tok.readPoint();
				if (_types.x == -1) break;

				//CPoint funnyWord = tok.readPoint();
				
				do {
					CPoint funnyWord = tok.readPoint();
					TOGGLEDATA td;
					td.delay = 0;
					td.duration = funnyWord.y;
					td.hasRegion = false;
					td.wordIndex = funnyWord.x;
					_display.toggleWords.push_back(td);
				} while (_display.toggleWords.back().duration != -1);
				_display.toggleWords.pop_back();

				_display.movWindowType.useWordTriggers = _types.x;
				_display.movWindowType.useWordTargets = _types.y;

				if (_p2.x <= _p2.y) {
					_display.windowSize = CPoint(_p);
					_display.windowRange = _p2;
					//_display.wordTargetRange.push_back( CPoint(_p2.x, _p2.y)); // TODO: TP range check

					if (_types.x == Tokenizer::WORD_TRIGGER && _types.y == Tokenizer::WORD_TARGET){
						_display.movWindowType.useWordTriggers = TRUE;
						_display.movWindowType.useWordTargets = TRUE;
						for (int i=_p2.x; i<=_p2.y; i++){	
							_display.wordTriggers.push_back(i);
							_display.wordTargetRange.push_back( 
								fitRange( _p.x, _p.y, i, _p2.x, _p2.y));
						}
						/*if(funnyWord.x != -1 && _p2.x < funnyWord.x && funnyWord.x <= _p2.y){
							_display.toggleWord.SetPoint(funnyWord.x, funnyWord.y);
						}
						else 
							_display.toggleWord.x = -1;*/
					}
					if (_types.x == Tokenizer::WORD_TRIGGER && _types.y == Tokenizer::CHAR_TARGET){
						_display.movWindowType.useWordTriggers = TRUE;
						_display.movWindowType.useWordTargets = FALSE;
						for (int i=_p2.x; i<=_p2.y; i++){
							_display.wordTriggers.push_back(i);
						}
					}
					if (_types.x == Tokenizer::CHAR_TRIGGER && _types.y == Tokenizer::CHAR_TARGET){
						_display.movWindowType.useWordTriggers = FALSE;
						_display.movWindowType.useWordTargets = FALSE;
					}
				}
				else {	
					os<<"Moving window range not well defined. ("<<_p2.x<< ","<<_p2.y<<")"<<std::endl;
					throw(CExperimentException(os.str().c_str()));
				}
			} while(tok.isOkay());
			if( !tok.isOkay()) {
				os<<"Error reading word change regions."<<std::endl;
				throw(CExperimentException(os.str().c_str()));
			}
			//_display.wordTriggers.pop_back();
			//_display.wordTargetRange.pop_back();
			_display.isFixedDisplay = false;
			_display.isMovingWindow = true;
			_display.isPairedRegionChange = false;
			break;
		case Tokenizer::TRIALTYPE:
			_typeId = tok.readLine();

			break;
		default:
			os<<"Illegal data field '"<<Tokenizer::decode(keyword)<<"'."<<std::endl;
			throw(CExperimentException(os.str().c_str()));
		}

		if(!tok.isOkay()) {                          //check for errors
			os<<"Unable to read data for field '"<<Tokenizer::decode(keyword)<<"'."<<std::endl;
			throw(CExperimentException(os.str().c_str()));
		}

		keyword = tok.readKeyword();
	}

	temp = tok.readString();
	if(_trialId.compare(temp) != 0) {
		os<<"'end "<<temp<<"' does not match trial definition."<<std::endl;
		throw(CExperimentException(os.str().c_str()));
	}

	if(_typeId.empty()) {
		os<<"Trial does not specify a trial type."<<std::endl;
		throw(CExperimentException(os.str().c_str()));
	}

	if(_display.display1.empty() && _display.images1.empty()) {
		os<<"Trial does not specify an image file or inline text."<<std::endl;
		throw(CExperimentException(os.str().c_str()));
	}

	if(!_display.display2.empty() && _display.region.size() == 0 && !_display.isMovingWindow && !_display.isPairedRegionChange && !_display.isTimedPairedRegionChange ) {
		os<<"Trial is display change, but specifies no change regions."<<std::endl;
		throw(CExperimentException(os.str().c_str()));
	}

	CHANGE_ONCE_MODE = this->hasThree();
}

Trial::Trial(const TrialId &trialId, 
			 const std::string &typeId,
			 const DisplayData &display,
			 Tokenizer::ButtonType answer,
			 int maxDisplayTime, Tokenizer::ControlType controlType) throw(CExperimentException)
  : _trialId(trialId), _typeId(typeId), _audio(), _display(display),
    _answer(answer), _maxDisplayTime(maxDisplayTime), _controlType(controlType)
{}

Trial::~Trial()
{}

AudioRenderData &Trial::audioData()
{
	return(_audio);
}

const std::string &Trial::display1() const
{
	return(_display.display1);
}

const std::string &Trial::display2() const
{
	return(_display.display2);
}

const std::string &Trial::display3() const
{
	return(_display.display3);
}

const DisplayData &Trial::displayData() const
{
	return(_display);
}

const CRect &Trial::gcRect() const
{
	return(_display.gcRect);
}

bool Trial::hasAudio() const
{
	return(_audio.path.empty() == false);
}

const TrialId &Trial::identifier() const
{
  return(_trialId);
}

bool Trial::isFixedDisplay() const
{
	return(_display.isFixedDisplay);
}

bool Trial::isScotoma() const
{
	return(_display.isScotoma);
}

bool Trial::isMovingWindow() const
{
	return(_display.isMovingWindow);
}

bool Trial::isPairedRegionChange() const
{
	return(_display.isPairedRegionChange);
}

bool Trial::isTimedPairedRegionChange() const
{
	return(_display.isTimedPairedRegionChange);
}

bool Trial::isTextDisplay() const
{
	return(_display.isTextDisplay);
}

int Trial::maxDisplayTime() const
{
	return(_maxDisplayTime);
}

std::string Trial::name() const
{
	return(_trialId.idString());
}

const Trial &Trial::operator=(const Trial &t)
{
	_trialId = t._trialId;
	_typeId = t._typeId;
	_audio = t._audio;
	_display = t._display;
	_answer = t._answer;
	_controlType = t._controlType;
	startf2 = t.startf2;
	endf2 = t.endf2;
	_maxDisplayTime = t._maxDisplayTime;
	CHANGE_ONCE_MODE = t.CHANGE_ONCE_MODE;
	return(*this);
}

Tokenizer::ButtonType Trial::questionAnswer() const
{
	return(_answer);
}

Tokenizer::ControlType Trial::controlType() const
{
	return(_controlType);
}

void Trial::setControlType(Tokenizer::ControlType c)
{
	_controlType = c;
	return;
}

void Trial::save(std::ostream &os) const
{                                                //saves trial to a script file
	os<<"trial "<<_trialId.idString()<<std::endl;

	if(!_audio.path.empty()) {
		os<<"  audio_format =     '"<<_audio.path<<"' ";
		os<<_audio.startDelay<<' '<<_audio.stopDelay<<std::endl;
	}

	if(_answer != Tokenizer::NO_BUTTON){
		if (_controlType == Tokenizer::FIVE_BUTTON)	
			os<<"  button =           "<<Tokenizer::decode5(_answer)<<std::endl;
		else 
			os<<"  button =           "<<Tokenizer::decode7(_answer)<<std::endl;			
	}
	os<<"  gc_rect =          ("<<_display.gcRect.top<<' '<<_display.gcRect.left
	  <<' '<<_display.gcRect.bottom<<' '<<_display.gcRect.right<<')'<<std::endl;

	if(_display.isTextDisplay) {
		int len = static_cast<int>(_display.display1.length());
		os<<"  inline =           |";
		if (len == 0 && _display.isScotoma){
			os<<" ";
		}
		for(int i=0; i < len; ++i) {             //write first inline display
			switch(_display.display1[i]) {
			case '\a': os<<"\\_";   break;
			case '\n': os<<"\\n";   break; 
			default:   os<<_display.display1[i];
			}
		}

		os<<std::endl;

		std::vector<IMAGEPOS>::const_iterator imageIter, imagesEnd;
	
		if(!_display.images1.empty()){
			imageIter = _display.images1.begin();
			imagesEnd = _display.images1.end();

			do{
				os<<"  image =		( ";
				os<<imageIter->upperLeft.x;
				os<<" ";
				os<<imageIter->upperLeft.y;
				os<<" ) '";
				os<<imageIter->fileName;
				os<<"'";
				os<<std::endl;
				imageIter++;
			} while (imageIter != imagesEnd);
		}

		if(_display.display2.empty() == false) {
			len = static_cast<int>(_display.display2.length());
			os<<"  inline =           |";
			for(int i=0; i < len; ++i) {         //write second inline display (if any)
				switch(_display.display2[i]) {
				case '\a': os<<"\\_";   break;
				case '\n': os<<"\\n";   break; 
				default:   os<<_display.display2[i];
				}
			}
			os<<std::endl;

			if(!_display.images2.empty()){
				imageIter = _display.images2.begin();
				imagesEnd = _display.images2.end();

				do{
					os<<"  image =		( ";
					os<<imageIter->upperLeft.x;
					os<<" ";
					os<<imageIter->upperLeft.y;
					os<<" ) '";
					os<<imageIter->fileName;
					os<<"'";
					os<<std::endl;
					imageIter++;
				} while (imageIter != imagesEnd);
			}

		}
		if(_display.display3.empty() == false) {
					len = static_cast<int>(_display.display3.length());
			os<<"  inline =           |";
			for(int i=0; i < len; ++i) {         //write second inline display (if any)
				switch(_display.display3[i]) {
				case '\a': os<<"\\_";   break;
				case '\n': os<<"\\n";   break; 
				default:   os<<_display.display3[i];
				}
			}
			os<<std::endl;

			if(!_display.images3.empty()){
				imageIter = _display.images3.begin();
				imagesEnd = _display.images3.end();

				do{
					os<<"  image =		( ";
					os<<imageIter->upperLeft.x;
					os<<" ";
					os<<imageIter->upperLeft.y;
					os<<" ) '";
					os<<imageIter->fileName;
					os<<"'";
					os<<std::endl;
					imageIter++;
				} while (imageIter != imagesEnd);
			}
		} 
	}
	else {
		os<<"  image =            '"<<_display.display1<<'\''<<std::endl;
		if(_display.display2.empty() == false)
			os<<"  image =            '"<<_display.display2<<'\''<<std::endl;
	}

	os<<"  max_display_time = "<<_maxDisplayTime<<std::endl;

	if(_display.isFixedDisplay == false && !_display.isMovingWindow && !_display.isPairedRegionChange && !_display.isTimedPairedRegionChange) {
		if(_display.isScotoma){
			os<<"  scotoma =		";
		}
		else{
			os<<"  region =          ";
		}
		for(unsigned int i=0; i < _display.region.size(); ++i) {
			const CRect &r = _display.region[i];
			os<<" ("<<r.top<<' '<<r.left<<' '<<r.bottom<<' '<<r.right<<')';
		}
		os<<std::endl;
	}
#ifdef MOV_WIND_DEBUG
	if(!_display.isFixedDisplay && !_display.isPairedRegionChange && !_display.isTimedPairedRegionChange){
		os<<"DB "<<_display.region.size()<< " "<<_display.display1.size();

		os<<"  region =          ";
		for(unsigned int i=0; i < _display.region.size(); ++i) {
			const CRect &r = _display.region[i];
			os<<" ("<<r.top<<' '<<r.left<<' '<<r.bottom<<' '<<r.right<<')';
		}
		os<<std::endl;
		if(_display.isMovingWindow == true){
			os<<_display.region.size();
			os<<"  target =          ";
			//const CRect &r = (_display.multiTarget[0]).at(0);
			//	os<<" ("<<r.top<<' '<<r.left<<' '<<r.bottom<<' '<<r.right<<')';
			for(unsigned int i=0; i< _display.multiTarget.size(); ++i){
				for (unsigned int j = 0; j< _display.multiTarget.at(i).size(); j++){
					const CRect &r = _display.multiTarget.at(i).at(j);
					os<<" ("<<r.top<<' '<<r.left<<' '<<r.bottom<<' '<<r.right<<')';
				}
				os<<'-'<<std::endl;
			}
			os<<std::endl;
			for( int i=0; i<13; i++){
				//os<<flipped[i]<<'.';
			}
			os<<std::endl;
		}
	}
#endif  //MOV_WIND_DEBUG

	if(_display.isPairedRegionChange && !_display.isTimedPairedRegionChange) {
		os<<"  region_pair =   ";
		for(unsigned int i=0; i<_display.regionPairs.size(); ++i){
			const CRect r1 = _display.regionPairs.at(i).region1;
			const CRect r2 = _display.regionPairs.at(i).region2;
			os<<"{("<<r1.top<<' '<<r1.left<<' '<<r1.bottom<<' '<<r1.right<<"),(";
			os		<<r2.top<<' '<<r2.left<<' '<<r2.bottom<<' '<<r2.right<<")} ";
		}
		os<<std::endl;
	}

	if(_display.isTimedPairedRegionChange) {
		os<<"  region_pair =   ";
		for(unsigned int i=0; i<_display.timedRectPairs.size(); ++i){
			const CRect r1 = _display.timedRectPairs.at(i).r1;
			const CRect r2 = _display.timedRectPairs.at(i).r2data.region;
			int delay =		_display.timedRectPairs.at(i).r2data.delay;
			int duration =	_display.timedRectPairs.at(i).r2data.duration;
			int delay2=		_display.timedRectPairs.at(i).r2data.delay2;
			int duration2 = _display.timedRectPairs.at(i).r2data.duration2;
			const CPoint pt(delay,duration);
			os<<"[("<<r1.top<<' '<<r1.left<<' '<<r1.bottom<<' '<<r1.right<<"),(";
			os		<<r2.top<<' '<<r2.left<<' '<<r2.bottom<<' '<<r2.right<<"),(";
			//if(_display.timedRectPairs.at(i).r2data.delay2 
			//	== _display.timedRectPairs.at(i).r2data.duration2 
			//	== 0){
			if( duration2 == 0){
					os<<delay<<' '<<duration<<")] ";
				}
			else {
					os<<delay<<' '<<duration<<"),("<<delay2<<' '<<duration2<<")] ";
			}

		}
		os<<std::endl;
	}

	if(_display.isMovingWindow) {
		os<<"  window =          ";
		const CPoint p1 = _display.windowSize;
		const CPoint p2 = _display.windowRange;
		int trig = _display.movWindowType.useWordTriggers;
		int targ = _display.movWindowType.useWordTargets;
		const CPoint p3 = CPoint(trig, targ);
		//const CPoint p4 = CPoint(_display.toggleWords.at(0).wordIndex, _display.toggleWords.at(0).duration);
		os<<" ("<<p1.x<<' '<<p1.y<<") ("<<p2.x<<' '<<p2.y<<") ("<<p3.x<<' '<<p3.y<<')';// ("<<p4.x<<' '<<p4.y<<')';
		//os<<std::endl;
		for(std::vector<TOGGLEDATA>::const_iterator i = _display.toggleWords.begin(); i != _display.toggleWords.end(); i++){
			os<<" ("<<i->wordIndex<<' '<<i->duration<<')';
		}
		os<<std::endl;
	}

	os<<"  trial_type =       "<<_typeId<<std::endl;
	os<<"end "<<_trialId.idString()<<'\n'<<std::endl;
}

CRect Trial::testRegions(CPoint &pt) const
{
	for(unsigned int i=0; i < _display.region.size(); ++i) {
		const CRect &r = _display.region[i];
		if(r.top < pt.y && r.left < pt.x && r.bottom >= pt.y && r.right >= pt.x)
			return(r);
	}
	return(CRect(0, 0, 0, 0));
}

CRect Trial::getScotomaRegion() const 
{
	if(!_display.region.empty())
		return(_display.region[0]);
	return (CRect(0,0,0,0));
}


TOGGLEDATA Trial::testPreviewRegions(CPoint &pt) const
{
	for(unsigned int i=0; i < _display.toggleRegions.size(); ++i){
		const CRect &r = _display.toggleRegions[i].region;
		if(r.top < pt.y && r.left < pt.x && r.bottom >= pt.y && r.right >= pt.x){
			//return(r);
			return (_display.toggleRegions.at(i));
		}
	}
	TOGGLEDATA td;
	td.region = CRect(0,0,0,0);
	return(td);
}

TIMED_RECT_PAIR Trial::testTimedRegions(CPoint &pt, bool set)
{
	for(unsigned int i=0; i< _display.timedRectPairs.size(); ++i){
		const TIMED_RECT_PAIR  &trp = _display.timedRectPairs.at(i);
		if(	trp.r1.top < pt.y &&
			trp.r1.bottom >= pt.y &&
			trp.r1.left < pt.x &&
			trp.r1.right >= pt.x /*&&
			!trp.written*/){   // if we want possibility of change happening multiple times, must leave written == false;
				if(set){
					_display.timedRectPairs.at(i).written = CHANGE_ONCE_MODE;
				}
				return(trp);
			}
	}

	TIMED_RECT_PAIR trp;
	trp.r1 = CRect(0,0,0,0);
	return trp;
}

void Trial::resetSingleChangeTRP(){
	for(unsigned int i=0; i< _display.timedRectPairs.size(); ++i){
		_display.timedRectPairs.at(i).written = false;
	}
	return;
}

		



const std::string &Trial::typeId() const
{
	return(_typeId);
}

DisplayData Trial::getDisplay() 
{
	return(_display);
}

void Trial::setDisplay(const DisplayData dd) 
{
	_display = dd;
	return;
}
CRect Trial::getCorrespondingRegion(CPoint &pt) const
{
	if( !_display.isPairedRegionChange )
		return testRegions( pt );

	// else we're doing a paired lookup
	for(unsigned int i=0; i < _display.regionPairs.size(); ++i){
		const CRect &r = _display.regionPairs[i].region1;
		if(		r.top < pt.y 
			&&	r.left < pt.x
			&&	r.bottom >= pt.y
			&&	r.right >= pt.x)
			return _display.regionPairs[i].region2;
	}
	return(CRect(0,0,0,0));
}

std::vector<CRect> Trial::getCorrespondingRegions(CPoint &pt)
{
	std::vector<CRect> zeroRect;
	/*for (unsigned int i=0; i< _display.tPairs.size(); ++i){
		const CRect &r = _display.tPairs[i].trigger;
		if(		r.top < pt.y 
			&&	r.left < pt.x
			&&	r.bottom > pt.y
			&&	r.right > pt.x )
		return _display.tPairs[i].target;*/
	for(unsigned int i=0; i < _display.region.size(); ++i){
		const CRect &r = _display.region[i];
			if(		r.top < pt.y 
			&&	r.left < pt.x
			&&	r.bottom >= pt.y
			&&	r.right >= pt.x){
#ifdef MOV_WIND_DEBUG
				//flipped[i-1] = 1;
#endif
				return _display.multiTarget[i];
			}
	}
	zeroRect.push_back(CRect(0,0,0,0));
	return(zeroRect);
}

CPoint Trial::fitRange(int numLeft, int numRight, int c, int leftLimit, int rightLimit){
	int tmpX, tmpY;
	if (c-numLeft < leftLimit)
		tmpX = leftLimit;
	else tmpX = c-numLeft;

	if (c+numRight > rightLimit)
		tmpY = rightLimit;
	else tmpY = c+numRight;

	return	CPoint(tmpX, tmpY);
}

bool Trial::isFastPrime(){
	//if(_display.display3.empty())
	//	return false;
	//return true;

	return false;
}

bool Trial::hasThree(){
	if(_display.display3.empty() && _display.images3.empty())
		return false;
	return true;
}

int Trial::pullTags(std::string &str, std::string &tmp){

	std::string::iterator i = str.begin(), ti = tmp.begin();

	while(i<str.end()){
		if(i != "9"){
			ti = i;
			ti++; i++;
		}
		else{
			i++;
		}
	}
	return 4;
}

std::string::iterator Trial::findTags(std::string str, bool startTag){
	std::string::iterator i = str.begin();
	if(startTag)
		startf2 = 0;
	else
		endf2 = 0;
	while(i<str.end()){
		if( startTag &&  i[0] == '<' ){
			return i;
		}
		else if (i[0] == '>' ){
			return i;
		}
		i++;
		if(startTag)
			startf2++;
		else 
			endf2++;
	}
}
