/*  Eyetracking software creates and executes experiments using 
    Eyelink II hardware and libraries by SR Research

    Copyright (C) 2004  David J. Stracuzzi, University of Massachusetts

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation; either version 2 of the License, or
    (at your option) any later version.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
*/

#pragma once
#pragma warning( disable : 4290 )

#include <list>
#include "AudioRender.h"
#include "DisplayData.h"
#include "Tokenizer.h"
#include "TrialId.h"

class Trial {
public:
	Trial(const Trial &t) throw();

	Trial(Tokenizer &tok) throw(CExperimentException);

	Trial(const TrialId &trialId, 
		  const std::string &typeId,
		  const DisplayData &display, 
		  Tokenizer::ButtonType answer,
		  int maxDisplayTime, 
		  Tokenizer::ControlType controlType) throw(CExperimentException);

	~Trial();

	AudioRenderData &audioData();
	
	const DisplayData &displayData() const;

	const Trial &operator=(const Trial &t);

	const TrialId &identifier() const;

	std::string  name() const;

	CRect testRegions(CPoint &pt) const;
	TOGGLEDATA testPreviewRegions(CPoint &pt) const;
	TIMED_RECT_PAIR testTimedRegions(CPoint &pt, bool set=false);
	CRect getCorrespondingRegion (CPoint &pt) const;	// for use in moving window version
														// returns the region which should be updated
														// when fixation falls in given target region
	CRect getScotomaRegion() const;
	void resetSingleChangeTRP();
	std::vector<CRect> getCorrespondingRegions (CPoint &pt) ;
	const std::string &display1() const;
	const std::string &display2() const;
	const std::string &display3() const;
	const std::string &typeId() const;
	DisplayData getDisplay();
	void setDisplay(DisplayData dd);

	const CRect &gcRect() const;
		
	Tokenizer::ButtonType questionAnswer() const;
	Tokenizer::ControlType controlType() const;
	void setControlType (Tokenizer::ControlType c);
	int maxDisplayTime() const;

	bool hasAudio() const;
	bool isFixedDisplay() const;
	bool isScotoma() const;
	bool isMovingWindow() const;
	bool isTextDisplay() const;
	bool isPairedRegionChange() const;
	bool isTimedPairedRegionChange() const;
	bool isFastPrime();
	bool hasThree();

	void save(std::ostream &os) const;
	int		_junk;
#ifdef MOV_WIND_DEBUG
	int		flipped[20];
#endif
private:
	TrialId				  _trialId;
	std::string			  _typeId;               //trial type identifier

	AudioRenderData       _audio;                //audio playback data
	DisplayData           _display;              //display data
	Tokenizer::ButtonType _answer;               //correct answer to question (if any)
	Tokenizer::ControlType _controlType;		// 7 or 5 button controller (if knonw)
//	Tokenizer::ButtonTypeSideWinder _answer_7button;
	int					  _maxDisplayTime;       //maximum trial display time
	CDC					  _dc;
	CPoint fitRange(int numLeft, int numRight, int center, int rangeLeft, int rangeRight);
	bool				CHANGE_ONCE_MODE;

	int pullTags(std::string &in, std::string &out);
	std::string::iterator findTags(std::string str, bool startTag);

public:
	int startf2;
	int endf2;
};

typedef std::list<Trial*>                 TrialList;         //list of trial pointers
typedef std::list<Trial*>::iterator       TrialIter;         //iterator for list
typedef std::list<Trial*>::const_iterator constTrialIter;    //iterator for list
